@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2021 Zoltan Turanyi
+	Copyright (C) 2008-2022 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -48,7 +48,7 @@ public:
     static const std::vector<std::string> spec_block_names;
     using SpecBlockPtrs = std::array<const BlockBlock*, 5>;
     bool valid = true;              ///<Set to false, if constructor fails, the arrow endpoints dont exist or its path cannot be found. Drop this instruction in PostParseProcess()
-    BlockChart    &chart;           ///<The block chart this element belongs to.  
+    BlockChart    &chart;           ///<The block chart this element belongs to.
     const std::string name_prefix;  ///<The prefix of our parent (without the dot). Empty only at top level.
     BlockStyle     style;           ///<The style of this element
     Label          parsed_label;    ///<The label processed and parsed into lines. Set in PostParseProcess()
@@ -70,7 +70,7 @@ public:
     virtual void AddToDrawOrder(const BlockElement *blk = nullptr, bool before = true);
     /** Add to the drawing list (at the end or before a set of elements). */
     virtual void AddToDrawOrder(const StringWithPosList &list, bool before = true);
-    /** Add an element with its content to the drawing list (at the end or before 
+    /** Add an element with its content to the drawing list (at the end or before
      * another element). The content is inserted *after* the element.*/
     void AddToDrawOrderWithContent(const BlockInstrList & content, const BlockElement * blk, bool before);
     /** Add an element with its content to the drawing list (at the end or before
@@ -121,9 +121,9 @@ public:
 struct Variable {
     std::string name;      ///<The name of the variable. Strictly for debugging.
     const BlockBlock *const block;
-    struct Mapping { 
-        unsigned var = 0; ///<The number of the GLPK column (variable) to map to
-        double off = 0;   ///<Add this much to the value of the GLPK column
+    struct Mapping {
+        unsigned var = 0;  ///<The number of the GLPK column (variable) to map to
+        double off = 0;    ///<Add this much to the value of the GLPK column
     } mapped_to;           ///<The GLPK variable this external variable is mapped to (and the offset to add to that)
     double      value = 0; ///<The (computed) value of the variable. Ready after layout.
     /** Get the value of the variable.*/
@@ -160,7 +160,7 @@ struct Constraint
     /** Enumerates the possible Constraint types.*/
     enum EType
     {
-        EQUAL,         ///<Equality, s1==s2 
+        EQUAL,         ///<Equality, s1==s2
         INEQUAL,       ///<Greater than, s1>=s2
         VEC_MIN_EQUAL, ///<Minimum. s1=minimum of the variables in 'vec'. s1 must be SINGLE and no offset. 's2' is ignored.
         VEC_MAX_EQUAL, ///<Maximum. s1=maximum of the variables in 'vec'. s1 must be SINGLE and no offset. 's2' is ignored.
@@ -174,8 +174,8 @@ struct Constraint
         Pruned,     ///<A constraint that has a stronger constraint before it - we can ignore it and assume it holding
     };
     /** Enumerates the comparison of two constraints how can they hold or be violated.*/
-    enum class ERelation 
-    { 
+    enum class ERelation
+    {
         Unrelated, ///<The two constraints are not related. Any of them can hold or conflict independently.
         Stronger,  ///<The first constraint is stronger: whenever it holds, the second holds, too
         Weaker,    ///<The first constraint is weaker: whenever the second holds, the first one holds, too
@@ -186,7 +186,7 @@ struct Constraint
     EAlignPrio prio; ///<What is the priority of the attribute resulting in us. This can be changed and is used to sort constraints.
     Side       s1;   ///<First side
     Side       s2;   ///<Second side
-    std::vector<VariableOffset> vec;  ///<In case of min/max constraints, the list of variables, with offset
+    std::vector<VariableOffset> vec;    ///<In case of min/max constraints, the list of variables, with offset
 
     const BlockBlock *block = nullptr;  ///<The block generated this constraint.
     FileLineCol       attr_pos;         ///<The position of the attribute that generated this constraint. May be invalid for default or auto attributes from default styles.
@@ -221,7 +221,7 @@ struct Constraint
     FileLineCol GenerateError(MscError &error, EAlignPrio report_level);
     /** Generate an error if this constraint is found to cause a conflict for
      * another constraint.
-     * @param error The place to report the error to. 
+     * @param error The place to report the error to.
      * @param [in] l The location of the other constraint.*/
     void GenerateErrorForAlternative(MscError &error, const FileLineCol &l);
     /** Compares us (first constraint) with another one (second constraint).
@@ -330,22 +330,22 @@ struct CloneAction
     std::string                       before;
     FileLineColRange                  before_pos;
     std::unique_ptr<CloneActionList>  modifiers;
-    explicit CloneAction(const FileLineColRange &l, gsl::owner<BlockInstruction *> inst, 
+    explicit CloneAction(const FileLineColRange &l, gsl::owner<BlockInstruction *> inst,
                          std::string_view bef = {}, const FileLineColRange bef_pos = {}) :
         action(ECloneContentAdjustment::ADD), action_pos(l), instr(std::make_unique<BlockInstrList>()),
         before(bef), before_pos(bef_pos) {instr->Append(inst);}
-    explicit CloneAction(const FileLineColRange &l, gsl::owner<BlockInstrList *> inst, 
+    explicit CloneAction(const FileLineColRange &l, gsl::owner<BlockInstrList *> inst,
                          std::string_view bef = {}, const FileLineColRange bef_pos = {}) :
         action(ECloneContentAdjustment::ADD), action_pos(l), instr(inst), before(bef), before_pos(bef_pos) {}
-    explicit CloneAction(const FileLineColRange &l, gsl::owner<StringWithPosList*> instrs, 
+    explicit CloneAction(const FileLineColRange &l, gsl::owner<StringWithPosList*> instrs,
                          std::string_view bef, const FileLineColRange &bef_pos = {}) :
-        action(ECloneContentAdjustment::MOVE), action_pos(l), before(bef), before_pos(bef_pos) 
+        action(ECloneContentAdjustment::MOVE), action_pos(l), before(bef), before_pos(bef_pos)
         {if (instrs) blocks.swap(*instrs); delete instrs;}
-    explicit CloneAction(const FileLineColRange &l, std::string_view blk, const FileLineColRange &blk_pos, 
+    explicit CloneAction(const FileLineColRange &l, std::string_view blk, const FileLineColRange &blk_pos,
                          gsl::owner<BlockInstruction *> inst) :
-        action(ECloneContentAdjustment::REPLACE), action_pos(l), instr(std::make_unique<BlockInstrList>()), 
+        action(ECloneContentAdjustment::REPLACE), action_pos(l), instr(std::make_unique<BlockInstrList>()),
         blocks({{blk, blk_pos}}) {instr->Append(inst);}
-    explicit CloneAction(const FileLineColRange &l, std::string_view blk, const FileLineColRange &blk_pos, 
+    explicit CloneAction(const FileLineColRange &l, std::string_view blk, const FileLineColRange &blk_pos,
                          gsl::owner<BlockInstrList *> inst) :
         action(ECloneContentAdjustment::REPLACE), action_pos(l), instr(inst), blocks({{blk, blk_pos}}) {}
     explicit CloneAction(const FileLineColRange &l, gsl::owner<StringWithPosList*> instrs) :
@@ -353,7 +353,7 @@ struct CloneAction
     explicit CloneAction(const FileLineColRange &l, gsl::owner<StringWithPosList*> instrs, const FileLineColRange &inst_pos,
                          gsl::owner<AttributeList*> a, gsl::owner<CloneActionList *>mod) :
         action(ECloneContentAdjustment::UPDATE), action_pos(l), attr(a), modifiers(mod) {if (instrs) blocks.swap(*instrs); delete instrs;}
-    
+
     CloneAction(CloneAction &&) = default;
 };
 
@@ -368,7 +368,7 @@ public:
     {
         bool justify_with_neighbour = false; ///<True if we stick to a neighbour and the parent has content_align=justify
         double max_to_imargin = 0;           ///<The maximum coordinate used by arrows relative to the the inner min margin of the parent
-        double max_to_visible = 0;           ///<The maximum coordinate used by arrows relative to the min outer edge of the parent 
+        double max_to_visible = 0;           ///<The maximum coordinate used by arrows relative to the min outer edge of the parent
         AddConstraintResultOneDim & operator +=(const AddConstraintResultOneDim &o) {
             justify_with_neighbour |= o.justify_with_neighbour;
             max_to_imargin = std::max(max_to_imargin, o.max_to_imargin);
@@ -411,7 +411,7 @@ public:
     virtual void ResolveAlignmentAttributes(const BlockBlock *first, const BlockBlock *prev, const BlockBlock *next, const BlockBlock *last) { }
     virtual void AddVariableIndices(ConstraintSet &constraints, unsigned from, unsigned to) const {}
     virtual AddConstraintResult
-        AddConstraints(Canvas & canvas, ConstraintSet &constraints, unsigned from, unsigned to, 
+        AddConstraints(Canvas & canvas, ConstraintSet &constraints, unsigned from, unsigned to,
                        unsigned parent_index_justify[2], const InternalAlignment *parent_attr_justify[2]) const
         {return {};}
     virtual bool CompareLayouts(ConstraintSet &constraints, unsigned from, unsigned to) { return false; }
@@ -490,7 +490,7 @@ protected:
     BlockBlock(BlockChart &chart, EBlockType t, EBlockType c_t, int sh, const FileLineColRange &l, std::string_view name);
     BlockBlock(const BlockBlock &o, const FileLineColRange &l, std::string_view name, std::string_view prefix);
     static AlignmentAttr *
-        TranslatePre(BlockChart &chart, AlignModifier c1, AlignModifier c2, 
+        TranslatePre(BlockChart &chart, AlignModifier c1, AlignModifier c2,
                      const FileLineCol &l1, const FileLineCol &l2,
                      const StringWithPosList *xblocks, const StringWithPosList *yblocks,
                      EdgePos xmin, EdgePos xmax, EdgePos ymin, EdgePos ymax,
@@ -518,11 +518,11 @@ public:
           std::string  name_display;    ///<The human-readable name of the block. Only has line numbers for disambiguation, never empty. We disambiguate in PostParseProcess(), when we have all blocks added.
 
     bool               track = true;    ///<If set to false, we do not add ourselves to the tracking rectangles (a child of multi)
-    unsigned           level;           ///<Our depth in the block hierarchy. Top level main node is 0.
-    unsigned           rank;            ///<The position among our parent's children. Used to break alignment attr ties. Set in SetContent() or ApplyCloneModifiers() of the parent 
+    unsigned           level = 0;       ///<Our depth in the block hierarchy. Top level main node is 0.
+    unsigned           rank = 0;        ///<The position among our parent's children. Used to break alignment attr ties. Set in SetContent() or ApplyCloneModifiers() of the parent
     BlockInstrList     content;         ///<The content of the block, if any
     StringWithPosList  around;          ///<We should be around these blocks
-    bool               has_visible_content;///<True if we have any blocks within 'content' or 'around'. False if only other instructions (arrows, joins, etc.) are there. Set in SetContent().
+    bool    has_visible_content = false;///<True if we have any blocks within 'content' or 'around'. False if only other instructions (arrows, joins, etc.) are there. Set in SetContent().
 
     mutable unsigned index_out_min[2] = {};     ///<The GLPK variable index for left and top outer_line ignoring margin
     mutable unsigned index_out_max[2] = {};     ///<The GLPK variable index for right and bottom outer_line ignoring margin
@@ -532,7 +532,7 @@ public:
     mutable unsigned index_content_max[2] = {}; ///<The GLPK variable index for right/bottom of content inside. Valid only if we have content blocks.
     mutable unsigned index_label_min[2] = {};   ///The GLPK variable index for left top of label inside. Valid only if we have both content and label. If we only have label, we do things separately.
     mutable unsigned index_label_max[2] = {};   ///The GLPK variable index for right/bottom of label inside. Valid only if we have both content and label. If we only have label, we do things separately.
-    mutable unsigned index_justify[2] = {};     ///<If our content is justified, this is the variable for the spacing 
+    mutable unsigned index_justify[2] = {};     ///<If our content is justified, this is the variable for the spacing
 
     mutable Block outer_line;       ///<The outer bounding box of the block
     mutable Block inner_line;       ///<The inner margin of the block
@@ -551,8 +551,8 @@ public:
      * so elements later defined in the file may come before others, so we need the rank mechanism.
      * Attributes of blocks *earlier* in the list/file have higher precedence,
      * hence this "less" function returns true if our rank is higher.*/
-    bool less_for_alignment_ties(const BlockBlock &o) const { 
-        return parent==o.parent ? rank>o.rank : file_pos.start>o.file_pos.start; 
+    bool less_for_alignment_ties(const BlockBlock &o) const {
+        return parent==o.parent ? rank>o.rank : file_pos.start>o.file_pos.start;
     }
 
     void AddToDrawOrder(const BlockElement *blk = nullptr, bool before = true) override
@@ -590,22 +590,22 @@ public:
     void MakeTemplate();
     void AdjustMultiAttributes(unsigned seq, const XY &offset);
     /** Translate one 'below' 'left', etc. modifiers to an AlignmentAttr.*/
-    static AlignmentAttr *TranslatePre(BlockChart &chart, 
-                                       AlignModifier c, const FileLineCol &l, 
-                                       gsl::owner<StringWithPosList *> blocks = nullptr) 
+    static AlignmentAttr *TranslatePre(BlockChart &chart,
+                                       AlignModifier c, const FileLineCol &l,
+                                       gsl::owner<StringWithPosList *> blocks = nullptr)
         { return TranslatePre(chart, c, c, l, l, blocks); }
-    static AlignmentAttr *TranslatePre(BlockChart &chart, 
-                                       AlignModifier c1, AlignModifier c2, 
-                                       const FileLineCol &l1, const FileLineCol &l2, 
+    static AlignmentAttr *TranslatePre(BlockChart &chart,
+                                       AlignModifier c1, AlignModifier c2,
+                                       const FileLineCol &l1, const FileLineCol &l2,
                                        gsl::owner<StringWithPosList *> blocks = nullptr);
     /** Translates one 'below' 'left', etc. modifiers from a coordinate to an AlignmentAttr. */
-    static AlignmentAttr *TranslatePre(BlockChart &chart, 
-                                       AlignModifier c, const FileLineCol &l, 
+    static AlignmentAttr *TranslatePre(BlockChart &chart,
+                                       AlignModifier c, const FileLineCol &l,
                                        std::string_view coord, const FileLineCol &lc)
          { return TranslatePre(chart, c, c, l, l, coord, lc); }
-    static AlignmentAttr *TranslatePre(BlockChart &chart, 
-                                       AlignModifier c1, AlignModifier c2, 
-                                       const FileLineCol &l1, const FileLineCol &l2, 
+    static AlignmentAttr *TranslatePre(BlockChart &chart,
+                                       AlignModifier c1, AlignModifier c2,
+                                       const FileLineCol &l1, const FileLineCol &l2,
                                        std::string_view coord, const FileLineCol &lc);
     void ApplyPre(gsl::owner<AlignmentAttr *>local_alignment);
 	virtual void ApplyFurtherAttributes(const AttributeList *al);
@@ -644,12 +644,12 @@ public:
     XY GetLabelSize() const
     {
         XY ret = parsed_label.getTextWidthHeight();
-        if (IsHorizontal(style.label_orient.GetValue(EDirection::Above)))
+        if (IsHorizontal(style.label_orient.value_or(EDirection::Above)))
             ret.SwapXY();
         return ret;
     }
     AddConstraintResult
-        AddConstraints(Canvas & canvas, ConstraintSet &constraints, unsigned from, unsigned to, 
+        AddConstraints(Canvas & canvas, ConstraintSet &constraints, unsigned from, unsigned to,
                        unsigned parent_index_justify[2], const InternalAlignment *parent_attr_justify[2]) const override;
     void LayoutFromVariables(GLPKSolver &constraints, unsigned from, unsigned to) override;
     bool LayoutWithGLPK(Canvas &canvas, unsigned from, unsigned to, size_t progress_units) override;
@@ -700,7 +700,7 @@ class MultiBlock : public BlockBlock
     std::unique_ptr<BlockBlock> original_block; ///<The block, originally used to create us.
 protected:
     MultiBlock(const MultiBlock &o, const FileLineColRange &l,
-               std::string_view name, std::string_view prefix) : 
+               std::string_view name, std::string_view prefix) :
         BlockBlock(o, l, name, prefix), count(o.count) {}
 
 public:
