@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2021 Zoltan Turanyi
+	Copyright (C) 2008-2022 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -92,9 +92,10 @@ BlockChart::~BlockChart()
 }
 
 
-BlockInstrList *BlockChart::PopContext()
+std::unique_ptr<BlockInstrList> BlockChart::PopContext()
 {
-    if (Contexts.size()<2) return nullptr;
+    std::unique_ptr<BlockInstrList> ret;
+    if (Contexts.size()<2) return ret;
     const bool full = MyCurrentContext().IsFull();
     const size_t old_size = full ? MyCurrentContext().numberingStyle.Size() : 0;
     ChartBase::PopContext();
@@ -102,12 +103,12 @@ BlockInstrList *BlockChart::PopContext()
     //context used to specify a design), no action will be needed
     //in PostParseProcess, so we do not generate any Command arcs.
     if (!full || old_size == MyCurrentContext().numberingStyle.Size())
-        return nullptr;
+        return ret;
     //if number of levels is less in the outer context, we will need to trim the Numbering list
     //during PostParseProcess after processing all arcs in the inner context, so we insert
     //a SetNumbering, which first trims the Numbering list to the new size and then increments
     //the last number, so after 1.2.2 comes 1.3
-    auto ret = new BlockInstrList;
+    ret = std::make_unique<BlockInstrList>();
     if (old_size > MyCurrentContext().numberingStyle.Size())
         ret->Append(std::make_unique<SetNumbering>(*this, MyCurrentContext().numberingStyle.Size(), 
                                                    MyCurrentContext().numberingStyle.Last().increment));
@@ -123,27 +124,28 @@ bool BlockChart::AddCommandLineArg(const std::string & /*arg*/)
     return false;
 }
 
-void BlockChart::AddCommandLineOption(const Attribute & /*a*/)
-{
+void BlockChart::AddCommandLineOption(const Attribute & a)
+{    
+    AddAttribute(a);
 }
 
 /** Adds a chart option. */
-BlockBlock *BlockChart::AddAttribute(const Attribute &a)
+bool BlockChart::AddAttribute(const Attribute &a)
 {
     if (a.Is("numbering.append")) {
         std::vector<NumberingStyleFragment> nsfs;
         if (NumberingStyleFragment::Parse(this, a.linenum_value.start, a.value.c_str(), nsfs))
             MyCurrentContext().numberingStyle.Push(nsfs);
-        return nullptr;
+        return true;
     }
     if (a.Is("numbering")) {
         MyCurrentContext().running_style_blocks.AddAttribute(Attribute("number", a), this);
         MyCurrentContext().running_style_arrows.AddAttribute(Attribute("number", a), this);
-        return nullptr;
+        return true;
     }
     if (a.Is("conflict_report")) {
         if (!a.EnsureNotClear(Error, EStyleType::OPTION))
-            return nullptr;
+            return true;
         if (CaseInsensitiveEqual(a.value, "full"))
             conflict_report = EAlignPrio::Default;
         else if (CaseInsensitiveEqual(a.value, "default"))
@@ -154,29 +156,29 @@ BlockBlock *BlockChart::AddAttribute(const Attribute &a)
             conflict_report = EAlignPrio::Explicit;
         else
             Error.Error(a, false, "Unrecognized value. Use one of 'full', 'default', 'style' or 'off'. Ignoring option.");
-        return nullptr;
+        return true;
     }
-    if (AddDesignAttribute(a)) return nullptr;
+    if (AddDesignAttribute(a)) return true;
     //handle 'text.*' as the default text format
     if (a.StartsWith("text"))
         if (MyCurrentContext().text.AddAttribute(a, this, EStyleType::OPTION))
-            return nullptr;
+            return true;
 
     if (a.Is("file.url")) {
-        if (!a.EnsureNotClear(Error, EStyleType::OPTION)) return nullptr;
+        if (!a.EnsureNotClear(Error, EStyleType::OPTION)) return true;
         file_url = a.value;
-        return nullptr;
+        return true;
     }
     if (a.Is("file.info")) {
-        if (!a.EnsureNotClear(Error, EStyleType::OPTION)) return nullptr;
+        if (!a.EnsureNotClear(Error, EStyleType::OPTION)) return true;
         if (file_info.length())
             file_info.append("\n").append(a.value);
         else
             file_info = a.value;
-        return nullptr;
+        return true;
     }
     Error.Error(a.linenum_attr.start, "Unrecognized chart option. Ignoring it.");
-    return nullptr; //you may return an instruction if the option results in one
+    return false;
 }
 
 
@@ -200,17 +202,15 @@ bool BlockChart::AddDesignAttribute(const Attribute &a)
             return true;
     }
     if (a.Is("numbering.pre")) {
-        MyCurrentContext().numberingStyle.pre.is_set = true;
-        MyCurrentContext().numberingStyle.pre.value = a.value;
-        StringFormat::ExpandReferences(MyCurrentContext().numberingStyle.pre.value, this,
+        MyCurrentContext().numberingStyle.pre = a.value;
+        StringFormat::ExpandReferences(MyCurrentContext().numberingStyle.pre.value(), this,
                                        a.linenum_value.start, nullptr,
                                        false, true, StringFormat::LABEL, true);
         return true;
     }
     if (a.Is("numbering.post")) {
-        MyCurrentContext().numberingStyle.post.is_set = true;
-        MyCurrentContext().numberingStyle.post.value = a.value;
-        StringFormat::ExpandReferences(MyCurrentContext().numberingStyle.post.value, this,
+        MyCurrentContext().numberingStyle.post = a.value;
+        StringFormat::ExpandReferences(MyCurrentContext().numberingStyle.post.value(), this,
                                        a.linenum_value.start, nullptr,
                                        false, true, StringFormat::LABEL, true);
         return true;
@@ -240,7 +240,7 @@ bool BlockChart::AddDesignAttribute(const Attribute &a)
 		if (!a.EnsureNotClear(Error, EStyleType::OPTION))
 			return true;
 		if (a.CheckType(EAttrType::BOOL, Error))
-			MyCurrentContext().pedantic = { true, a.yes };
+			MyCurrentContext().pedantic = a.yes;
 		return true;
 	}
 	return false; //return true if you have recognized & handled the chart option
@@ -531,7 +531,7 @@ BlockChart::AutoCreateBlock(std::string_view name, const FileLineColRange & l,
 							const AttributeList *al, const AlignmentAttr * align)
 {
 	if (SkipContent() || !BlockNameToCreatePart(name)) return {};
-	const int shape = MyCurrentContext().running_style_blocks.shape.GetValue(-1);
+	const int shape = MyCurrentContext().running_style_blocks.shape.value_or(-1);
 	auto b = shape<0 ?
 		std::make_unique<BlockBlock>(*this, EBlockType::Box, l, name) :
 		std::make_unique<BlockBlock>(*this, shape, l, name);
@@ -593,7 +593,7 @@ void BlockChart::StashArrows(const StringWithPosList* l1, ArrowType t, const Str
         ah.bl->remove(nullptr);
     }
 	//If auto define is allowed, we create a block using the default running style (and the shape in it)
-	if (!MyCurrentContext().pedantic.GetValue(false))
+	if (!MyCurrentContext().pedantic.value_or(false))
 		for (auto l :{ l1, l2 })
 			if (l)
                 for (auto &bn: *l) {
@@ -793,7 +793,7 @@ BlockChart::UpdateBlock(std::string_view name, const FileLineColRange & l,
 	}
 	std::unique_ptr<BlockBlock> ret;
 	if (!b[0]) {
-		if (!MyCurrentContext().pedantic.GetValue(false))
+		if (!MyCurrentContext().pedantic.value_or(false))
 			//OK, define this block
 			ret = AutoCreateBlock(name, l, al, align);
 		else
@@ -994,7 +994,7 @@ bool BlockChart::LayoutWithGLPK(Canvas &canvas, unsigned from, unsigned to)
 /** After block layout, convert us to an actual coordinate.
  * If the blocks contain a single "parent", we calculate based on 'parent'
  * If the blocks are empty of some blocks are invalid or nonexistent
- * We return is_set==false.*/
+ * We return empty.*/
 OptAttr<double> BlockChart::GetCoord(const AlignTo & a, bool y, const BlockBlock *parent) const
 {
     Range r(false);
@@ -1007,12 +1007,12 @@ OptAttr<double> BlockChart::GetCoord(const AlignTo & a, bool y, const BlockBlock
         _ASSERT(b[0] && !b[1]);
         if (b[0] && !b[1]) {
             const Range &o = b.front()->outer_line[y];
-            r += o.from - (a.edge.margin ? b.front()->style.min_margin[y].GetValue(0) : 0);
-            r += o.till + (a.edge.margin ? b.front()->style.max_margin[y].GetValue(0) : 0);
+            r += o.from - (a.edge.margin ? b.front()->style.min_margin[y].value_or(0) : 0);
+            r += o.till + (a.edge.margin ? b.front()->style.max_margin[y].value_or(0) : 0);
         }
     }
-    if (r.IsInvalid()) return {false, 0};
-    return {true, a.edge.pos*r.till + (1-a.edge.pos)*r.from + a.edge.offset};
+    if (r.IsInvalid()) return {};
+    return a.edge.pos*r.till + (1-a.edge.pos)*r.from + a.edge.offset;
 }
 
 
@@ -1068,10 +1068,10 @@ Overlaps(iter1 first1, iter1 last1,
     const iter2 first2_save = first2;
     bool overlap;
     for (unsigned u = 0; first1!=last1; ++u, ++first1)
-        if (!visible_only || first1->visible) {
+        if (!visible_only || first1->IsVisible()) {
             first2 = first2_save;
             for (unsigned v = 0; first2!=last2; ++v, ++first2)
-                if (!visible_only || first2->visible) {
+                if (!visible_only || first2->IsVisible()) {
                     const int num = first1->Crossing(*first2, false, xy, my_pos, other_pos, &overlap);
                     if (!overlap) continue;
                     if (num!=2) {
@@ -1227,9 +1227,10 @@ std::vector<Arrow*> BlockChart::CheckArrowOverlap()
             for (auto &ae : pA->overlaps[end])
                 group.emplace_back(ae.first, end ^ !ae.second.same_ends);
             //Now sort the group based on how they depart
-            std::sort(group.begin(), group.end(), [this](auto&a, auto&b)->bool {
-                auto i = a.first->overlaps[a.second].find(b.first);
-                auto j = b.first->overlaps[b.second].find(a.first);
+            std::ranges::sort(group, [this](const auto&a, const auto&b)->bool {
+                if (&a==&b) return false; //self comparison
+                const auto i = a.first->overlaps[a.second].find(b.first);
+                const auto j = b.first->overlaps[b.second].find(a.first);
                 bool reverse = false;
                 if (i!=a.first->overlaps[a.second].end() &&
                     j!=b.first->overlaps[b.second].end()) {
@@ -1271,8 +1272,8 @@ std::vector<Arrow*> BlockChart::CheckArrowOverlap()
             for (unsigned u = 1; u<group.size(); u++) {
                 const double d = group[u-1].first->style.line.LineWidth()/2
                     + group[u].first->style.line.LineWidth()/2
-                    + std::max(group[u-1].first->style.route_arrow_distance.value,
-                               group[u].first->style.route_arrow_distance.value);
+                    + std::max(group[u-1].first->style.route_arrow_distance.value(),
+                               group[u].first->style.route_arrow_distance.value());
                 dist.push_back(d);
                 total_offset += d;
             }
@@ -1298,10 +1299,10 @@ std::vector<Arrow*> BlockChart::CheckArrowOverlap()
                 }
                 //We just need to set the direction if we are curvy or polygon or straight;
                 //Not a good idea. Just creates a lot of complex changes to arrow routing.
-                //if (!g.first->style.routing.is_set ||
-                //    g.first->style.routing.value == EArrowRouting::Polygon ||
-                //    g.first->style.routing.value == EArrowRouting::Straight ||
-                //    g.first->style.routing.value == EArrowRouting::Curvy)
+                //if (!g.first->style.routing ||
+                //    g.first->style.routing.value() == EArrowRouting::Polygon ||
+                //    g.first->style.routing.value() == EArrowRouting::Straight ||
+                //    g.first->style.routing.value() == EArrowRouting::Curvy)
                 //    g.first->ends[g.second].dir.number = dir_number;
                 //If g is horizontal or vertical, it should be appropriate stil....
                 //We do this to fix the direction, so that it does not change due to we shifting one endpoint
@@ -1311,10 +1312,10 @@ std::vector<Arrow*> BlockChart::CheckArrowOverlap()
 }
 
 
-void BlockChart::CompleteParse(Canvas::EOutputType ot, bool avoidEmpty, bool autoPaginate, bool addHeading,
+void BlockChart::CompleteParse(bool autoPaginate, bool addHeading,
                                XY pageSize, bool fitWidth, bool collectLinkInfo)
 {
-    Canvas canvas(ot);
+    Canvas canvas(Canvas::Empty::Query);
     pageBreakData.clear();
     //Consider the copyright text
     StringFormat sf;
@@ -1395,6 +1396,7 @@ void BlockChart::CompleteParse(Canvas::EOutputType ot, bool avoidEmpty, bool aut
         }
         Blocks.outer_line.RoundWider();
         total = Blocks.outer_line;
+        total += Block(0, 1, 0, 1); //have at least this size
         if (pageBreakData.size()==0)
             pageBreakData.emplace_back(XY(0, 0), total.Spans()); //we *must* have one page
         bkFill.emplace_back(Blocks.style.fill, total);
