@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -281,7 +281,7 @@ void BlockCsh::AddOptionsToHints()
 }
 
 
-void BlockCsh::BeforeYaccParse(std::string_view input, int cursor_p)
+void BlockCsh::BeforeYaccParse(const std::string& input, int cursor_p)
 {
     Csh::BeforeYaccParse(input, cursor_p);
     parent_name.clear();
@@ -302,13 +302,13 @@ void BlockCsh::BeforeYaccParse(std::string_view input, int cursor_p)
  * @param [in] input The chart text
  * @param [in] cursor_p The current position of the cursor.
  * @param [in] pedantic The initial value of the pedantic chart option.*/
-void BlockCsh::ParseText(std::string_view input, int cursor_p, bool pedantic)
+void BlockCsh::ParseText(const std::string& input, int cursor_p, bool pedantic)
 {
     //initialize data struct
     BeforeYaccParse(input, cursor_p);
     Context2.back().pedantic = pedantic;
     //call parsing
-    BlockCshParse(*this, input_text.c_str(), unsigned(input_text.length()));
+    BlockCshParse(*this, input_text.c_str(), unsigned(input_text.length()+1)); //include terminating zero
     //Tidy up afterwards
     AfterYaccParse();
 }
@@ -320,12 +320,12 @@ void BlockCsh::ParseText(std::string_view input, int cursor_p, bool pedantic)
  * @param [in] aname The name of the attribute or option, we assign the value to.
  * @param [in] avalue The value of the attribute or option, we assign.
  * @param [in] rewrite_source We rewrite the source of the hint to this.
- * @returns true, if we found good hints. False if we can check for other kinds of hints in the attr value.*/
+ * @returns true, if we found good hints we set hint status to READY.
+ *          False if we can check for other kinds of hints in the attr value.*/
 bool BlockCsh::HandleBlockNamePlus(std::string_view aname, std::string_view avalue,
                                    EHintSourceType rewrite_source)
 {
     if (hintStatus != HINT_FILLING) return false;
-    hintStatus = HINT_LOCATED; //So that additional hints can be added later
     hintSource = rewrite_source;
     if (hintSource==EHintSourceType::ATTR_VALUE)
         hintAttrName = aname;
@@ -336,9 +336,11 @@ bool BlockCsh::HandleBlockNamePlus(std::string_view aname, std::string_view aval
     if (add_num)
         addEntityNamesAtEnd = "AddEntities";
     if (addEntityNamesAtEnd=="AddEntities") {
+        hintStatus = HINT_LOCATED; //So that additional hints can be added later, like attr values
         //OK, we just need to fill in the right description
         if (CaseInsensitiveEqual(aname, "generic_coord")) {
             addEntityNamesAtEnd = "Use one side of block %s to specify a coordinate.";
+            hintStatus = HINT_READY; //Coordinates are not attributes, so nothing comes: finalize hints
         } else if (CaseInsensitiveEqual(aname, "via")) {
             if (avalue.length() && avalue.front()=='(')
                 addEntityNamesAtEnd = "Add block %s to a waypoint specification this arrow must go through.";
@@ -362,6 +364,7 @@ bool BlockCsh::HandleBlockNamePlus(std::string_view aname, std::string_view aval
             addEntityNamesAtEnd = "Align to block %s (or a group of blocks containing it).";
         } else {
         instead:
+            hintStatus = HINT_LOCATED; //So that additional hints can be added later
             addEntityNamesAtEnd.clear();
             return false;
         }
@@ -411,6 +414,9 @@ bool BlockCsh::HandleBlockNamePlus(std::string_view aname, std::string_view aval
             AlignmentAttr::SideValuesAfterAt(*this, true, "Align vertically to ");
         } else
             goto instead;
+        //So that no additional hints can be added later.
+        //Here we are after a @ and we do not want to add hints for the whole attribute value
+        hintStatus = HINT_READY;
     } else
         goto instead;
     return true;
@@ -520,10 +526,10 @@ bool dot_hierarchy_ends_with(std::string_view name, std::string_view suffix) {
 /** Returns the number of chars before the first dot in 'a' and 'b' before which 'a' and 'b' are the same.
  * If 'a' and 'b' does not have such a common prefix, we return zero. If a==b we return their length.
  * If one of them is a prefix of the other, we return the length of that.
- * So 
+ * So
  * - a.b, a.c.d => 1
  * - a.b, a.b => 3
- * - a.b, b.b => 0 
+ * - a.b, b.b => 0
  * - a.b, a.b.c => 3*/
 unsigned dot_hierarchy_common_prefix(std::string_view a, std::string_view b) {
     unsigned ret = 0, i;
@@ -586,7 +592,7 @@ void BlockCsh::AfterYaccParse()
     }
     Csh::AfterYaccParse();
     //Resolve Block forward references
-    for (auto &bm : BlockMentions) 
+    for (auto &bm : BlockMentions)
         if (bm.prefix) {
             std::string_view match; //The full name of the entity we match
             bool precise_match = false; //True if the match was fully specified and not something we added a prefix for
@@ -686,7 +692,7 @@ void BlockCsh::AddCSH_LineBeginSoloString(CshPos pos, std::string_view string) {
         if (ent==-2) { was_partial = true; return COLOR_ENTITYNAME; }
         return COLOR_ERROR;
     }();
-    if (color==COLOR_ERROR) 
+    if (color==COLOR_ERROR)
         AddCSH_Error(pos, "Unrecognized keyword, block name, option or attribute name.");
     else {
         if (full_name.size())
@@ -747,28 +753,28 @@ int BlockCsh::AddCSH_BlockNameOrNew(const CshPos&pos, std::string_view name, ECo
 
 /** This is called when we realized that an arrow_end is standing at the beginning of a line.
  * At this point we have already colored it.
- * If this is a simple string, then it may also be a partial or full keyword 
- * and its color is a new block or an error depending on whether we have pedantic 
- * off or on. 
+ * If this is a simple string, then it may also be a partial or full keyword
+ * and its color is a new block or an error depending on whether we have pedantic
+ * off or on.
  * In this functions we adjust the coloring of the arrow_end if it is a single string.
  * If the string is a compound string (a coordinate or \<a>+\<b>, etc), we do nothing. */
 void BlockCsh::UpdateCSH_ArrowEndAtLineBegin(const CshPos& pos, std::string_view arrow_end) {
     if (arrow_end.find_first_of("@,()+-")!=std::string_view::npos)
         return;
     _ASSERT(CshList.size());
-    _ASSERT(CshList.back() == pos);
-    //If the last CSH entry is this arrow_end, but it is not a block mention
+    //Find the previous color entry, if any
+    const auto i = std::ranges::find(CshList, pos);
+    //If the previous CSH entry at this pos is this arrow_end, but it is not a block mention
     //(which may happen if pedantic is on and this is not an already existing block)
     //then we re-color it as a line begin string (keywords, options, existing blocks, etc.)
-    if (CshList.size() && CshList.back() == pos 
-            && (BlockMentions.empty() || BlockMentions.back().pos != pos)) {
-        CshList.pop_back();
+    if (i!=CshList.end() && std::ranges::find(BlockMentions, pos, &BlockMention::pos)!=BlockMentions.end()) {
+        CshList.erase(i);
         AddCSH_LineBeginSoloString(pos, arrow_end);
     }
 }
 
-/** Marks a part of the text as a block name and registers it among 
- * BlockMentions. Called for block names as attribute values. 
+/** Marks a part of the text as a block name and registers it among
+ * BlockMentions. Called for block names as attribute values.
  * We try to look up the Block among already defined
  * blocks, but forward references are also allowed.*/
 void BlockCsh::AddCSH_BlockNameAsAttrValue(const CshPos &pos, std::string_view name) {
@@ -779,13 +785,10 @@ void BlockCsh::AddCSH_BlockNameAsAttrValue(const CshPos &pos, std::string_view n
 void BlockCsh::AddCSH_AttrValue_CheckAndAddEscapeHint(const CshPos& pos, std::string_view value, std::string_view name)
 {
     //for alignment attributes it is only called if there is no + sign or @ symbol
-    if (AlignmentAttr::AttributeNameDir(name) ||
-            CaseInsensitiveEqual(name, "via") ||
-            CaseInsensitiveEqual(name, "cross") ||
-            CaseInsensitiveEqual(name, "cross_all") ||
-            CaseInsensitiveEqual(name, "distance")) 
-        return;
-    //Not an attribute that takes block names
+    if (AlignmentAttr::AttributeNameDir(name)) return;
+    //These attributes take block names
+    static const char* anames[] = {"via", "cross", "cross_all", "distance", "height", "Width", "size"};
+    if (std::ranges::any_of(anames, [name](const char* n) {return CaseInsensitiveEqual(name, n); })) return;
     //Remove potentially added coloring
     RemoveAllTouchingCSH(pos);
     //remove anything potentially added hints
@@ -846,8 +849,8 @@ void BlockCsh::AddCSH_NameList(const CshStringWithPosList * l, const CshPos & po
 
 std::pair<std::string_view, size_t> BlockCsh::EntityNameUnder(long pos) const {
     auto i = std::find_if(BlockMentions.begin(), BlockMentions.end(),
-                          [pos](const BlockMention &p) { return p.pos.IsWithin(pos); }); 
-    if (i==BlockMentions.end()) 
+                          [pos](const BlockMention &p) { return p.pos.IsWithin(pos); });
+    if (i==BlockMentions.end())
         return {{}, 0};
     auto [entity_in_src, pos_in_src_in_entity] = Csh::EntityNameUnder(pos);
     //the full block name must end with what is in the text file.
@@ -862,8 +865,8 @@ std::pair<std::string_view, size_t> BlockCsh::EntityNameUnder(long pos) const {
     if (bytes<0) return {{}, 0};
     const size_t dot_after = i->name.find('.', bytes);
     const size_t limit_len_to = dot_after == string::npos ? i->name.length() : bytes+dot_after;
-    //2. Adjust the click pos by adding the num of chars in the prefix not written in the 
-    //source file to it (If the source file has B.C and we clicked 'C' and the full name 
+    //2. Adjust the click pos by adding the num of chars in the prefix not written in the
+    //source file to it (If the source file has B.C and we clicked 'C' and the full name
     // of the entity is A.B.C then pos_in_src_in_entity==2 and we now add len('A.') to it.
     return {std::string_view(i->name).substr(0, prefix_len+limit_len_to),
             pos_in_src_in_entity + UTF8len(std::string_view(i->name).substr(0, prefix_len))};
@@ -932,7 +935,7 @@ std::string BlockCsh::ReplaceEntityName(std::string_view full_entity, int pos,
         // - A.B.C.X or B.C.X, where we know it refers to A.B.C.X: here we need to find 'C'
         // - X (where we know it refers to A.B.C.X): nothing to replace here.
         //What is certain fname ends with src_name - and start with full_name (we filtered BlockMentions above).
-        //Find what position the source text 
+        //Find what position the source text
         _ASSERT(fname.length()>=src_name.length() && fname.substr(fname.length()-src_name.length())==src_name);
         const ptrdiff_t start_at = ptrdiff_t(replace_pos_in_full_name) - ptrdiff_t(fname.length()-src_name.length());
         if (start_at<0) {
