@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -37,7 +37,7 @@ enum class EArrowStyle : unsigned char
     DOUBLE   ///<Double line '=>'
 };
 
-/** The complete description of an arrow. Its style and whether it has 
+/** The complete description of an arrow. Its style and whether it has
  * start & end arrowheads. Use it during parsing.*/
 struct ArrowType
 {
@@ -84,7 +84,7 @@ constexpr EDirection Opposite(EDirection t) {
 }
 
 /** True if the direction is Above or Below.*/
-constexpr bool IsVertical(EDirection t) 
+constexpr bool IsVertical(EDirection t)
 { return t==EDirection::Above || t==EDirection::Below; }
 
 /** True if the direction is Left or Right.*/
@@ -110,27 +110,19 @@ class BlockChart;
 
 struct StringWithPosNumber : public StringWithPos
 {
-    double number = 0; 
+    double number = 0;
     using StringWithPos::StringWithPos;
     StringWithPosNumber() = default;
     StringWithPosNumber(StringWithPos &&sp, double num) :StringWithPos(std::move(sp)), number(num) {}
 };
 using StringWithPosNumberList = std::vector<StringWithPosNumber>;
 
-///** Whether the left/top, center or right/bottom side of a block.*/
-//enum class EPosType : unsigned char
-//{
-//    Min,   ///<Left or top side
-//    Mid,   ///<Centerline
-//    Max    ///<Right or bottom side
-//};
-
 /** Specifies one part of a block to align to.
  * Does not include whether horizontal or vertical.*/
 struct EdgePos
 {
     double pos;   ///<Whether the left/top (0.0), center (0.5) or right/bottom side (1.0) - or anything in between and outside
-    bool margin;  ///<Whether we align to the margin (true) or to the visible edge (false). 
+    bool margin;  ///<Whether we align to the margin (true) or to the visible edge (false).
     double offset;///<Pixel offset in addition to pos
     EdgePos() noexcept = default;
     EdgePos(double p, bool m, double o=0) : pos(p), margin(m), offset(o) {}
@@ -254,8 +246,8 @@ struct AlignmentAttr
     OptAlignAttr &GetMid(bool y) noexcept { return y ? YMiddle : XCenter; }
     /** Get the attribute governing right (y false) or bottom (y true) alignment.*/
     OptAlignAttr &GetMax(bool y) noexcept { return y ? Bottom : Right; }
-    /** Get one of the attributes: 0=left, 1=top, 2=xcenter, 3=ymiddle, 4=right, 5=bottom. 
-     * Useful to cycle through the attributes. 
+    /** Get one of the attributes: 0=left, 1=top, 2=xcenter, 3=ymiddle, 4=right, 5=bottom.
+     * Useful to cycle through the attributes.
      * Note that (0,1) are mins, (2,3) are mids, (4,5) are maxes. Also even are x coords, odd are y coords.*/
     OptAlignAttr &Get(unsigned u) noexcept { return u/2==2 ? GetMax(bool(u&1)) : u/2 ? GetMid(bool(u&1)) : GetMin(bool(u&1)); }
     /** Get the attribute governing left (y false) or top (y true) alignment.*/
@@ -288,16 +280,17 @@ struct InternalAlignment
     FileLineCol        file_pos; ///<The actual position of the attribute
 };
 
-/** An optional size attribute (for width and height) with a priority */
-struct OptSizeAttr : OptAttr<double>
-{
+/** A size attribute (for width and height) with a priority
+ * And perhaps a list of blocks to take as value.*/
+struct SizeAttr {
+    double size;               ///<Pixel size specified by the user. If 'blocks' is non-empty, it is a multiplier of the size of the bounding box of those blocks.
     EAlignPrio prio;           ///<The priority of this attribute (stye, explicit, etc.)
     StringWithPosList blocks;  ///<If this is non-empty, use the specific double value as a multiplier
-    OptSizeAttr() = default;
-    OptSizeAttr(bool f, unsigned s, EAlignPrio p) : 
-        OptAttr(f,s), prio(p) {}
 };
 
+/** An optional size attribute (for width and height) with a priority */
+using OptSizeAttr = std::optional<SizeAttr>;
+
 /** Describes how to interpret the 'size' attribute. */
 enum class ELabelMode
 {
@@ -319,7 +312,7 @@ enum class EArrowRouting
     Manhattan  ///<Only 90 degree edges
 };
 
-/** Further differentiator for BlockStyles of type EStyleType::STYLE to express 
+/** Further differentiator for BlockStyles of type EStyleType::STYLE to express
  * how do we use the style.*/
 enum class EBlockStyleType
 {
@@ -334,21 +327,21 @@ enum class EBlockStyleType
 class BlockStyle : public SimpleStyleWithArrow<TwoArrowHeads>
 {
     friend class BlockContext;
-    BlockStyle(EStyleType tt, EBlockStyleType btt, EColorMeaning cm, bool a, 
-               bool t, bool l, bool f, bool s, bool nu, bool shp, 
+    BlockStyle(EStyleType tt, EBlockStyleType btt, EColorMeaning cm, bool a,
+               bool t, bool l, bool f, bool s, bool nu, bool shp,
                bool lab, bool al, bool c, bool m, bool r, bool mkr); ///Control which components are used
     bool f_label;           ///<Flag specifying if this style has labels, label.pos, label.align, label.orient
     bool f_alignment;       ///<Flag specifying if this style has alignment attributes "top", "bottom" etc,
     bool f_content;         ///<Flag specifying if this style has attributes governing children alignemnt "content.*" and "allow_routing"
     bool f_margin;          ///<Flag specifying if this style has margin attributes "margin.*" "imargin.*", "width", "height", "size" and "multi.*"
-    bool f_routing;         ///<Flag specifying if this style has 'routing' attribute          
+    bool f_routing;         ///<Flag specifying if this style has 'routing' attribute
     bool f_marker;          ///<Flag specifying if this style has 'marker' attributes
 public:
     EBlockStyleType            block_style_type;      ///<If type==EStyleType::STYLE, this specifies exactly what kind of style we are. Should be set to Unspecified if type!=STYLE
     OptAttrLine<std::string>   label;                 ///<The label of this element
     AlignmentAttr              alignment;             ///<Our alignment
     OptAttr<InternalAlignment> XChildAlign;           ///<Describes how our content shall be aligned within us horizontally
-    OptAttr<InternalAlignment> YChildAlign;           ///<Describes how our content shall be aligned within us vertically 
+    OptAttr<InternalAlignment> YChildAlign;           ///<Describes how our content shall be aligned within us vertically
     OptAttr<bool>              content_margin;        ///<If true, we respect the margin of the content when trying to encircle them. Else not.
     OptAttr<double>            min_margin[2];         ///<Margin on the left ([0]) and top ([1]) side outside the block
     OptAttr<double>            max_margin[2];         ///<Margin on the left ([0]) and top ([1]) side outside the block
@@ -357,7 +350,7 @@ public:
     OptAttr<bool>              allow_arrows;          ///<True if we allow arrows to cross us
     OptSizeAttr                size[2];               ///<Specific width ([0]) and height ([1])
     OptAttrLine<ELabelMode>    label_mode;            ///<How to interpret the size attribute for label-only blocks
-    OptAttr<double>            multi_offset[2];       ///<Specifies what offset to apply with the "multi" keyword  
+    OptAttr<double>            multi_offset[2];       ///<Specifies what offset to apply with the "multi" keyword
     OptAttrLine<EDirection>    label_pos;             ///<Position of the label of a box relative to its content
     OptAttr<double>            label_align;           ///<left/center/right alignment of label
     OptAttr<EDirection>        label_orient;          ///<How to orient the label. Above is normal...
@@ -389,7 +382,7 @@ public:
     bool AddAttribute(const Attribute &a, Chart *) override;
     void AttributeNames(Csh &csh) const override;
     bool AttributeValues(std::string_view attr, Csh &csh) const override;
-	void Push(const FileLineCol &l); 
+	void Push(const FileLineCol &l);
 };
 
 /** Context for Graphs. Just a styleset and colors, really.*/
@@ -403,14 +396,14 @@ public:
 	OptAttr<bool> pedantic;          ///<If false, we allow block definitions via arrows
 
     /** This constructor is used to create an empty context or one set to plain.
-     * Used an initializing the first context at parse, when starting to record 
-     * a design, storing a procedure or moving a newly defined design to the design 
+     * Used an initializing the first context at parse, when starting to record
+     * a design, storing a procedure or moving a newly defined design to the design
      * store.
      * @param [in] f If true, the context contains a value for all styles and attributes (Full)
      * @param [in] p Tells us what components to observe and how to behave during parsing.
      * @param [in] t Tells us with what content to create the context. It should not be 'COPY
      * @param [in] l The first character of the context in the input file.*/
-    BlockContext(bool f, EContextParse p, EContextCreate t, const FileLineCol &l) : 
+    BlockContext(bool f, EContextParse p, EContextCreate t, const FileLineCol &l) :
         ContextBase<BlockStyle>(f, p, EContextCreate::CLEAR, l),
         running_style_blocks(EStyleType::STYLE, EBlockStyleType::Running, EColorMeaning::FILL, false,
                              true, true, true, true, true, true, true, true, true, true, false, false), //no arrow, routing, marker
@@ -419,14 +412,14 @@ public:
     { switch (t) {
         default: _ASSERT(0); FALLTHROUGH;
         case EContextCreate::PLAIN: Plain(); break;
-        case EContextCreate::EMPTY: Empty(); break; 
+        case EContextCreate::EMPTY: Empty(); break;
         case EContextCreate::CLEAR: background.Empty();}}
-    /** This constructor is used when a context needs to be duplicated due to 
+    /** This constructor is used when a context needs to be duplicated due to
      * the opening of a new scope. You can change the parse mode, e.g., when
      * parsing the not-selected branch of an ifthenelse.*/
     BlockContext(const BlockContext &o, EContextParse p, const FileLineCol &l) :
-        ContextBase<BlockStyle>(o, p, l), 
-        running_style_blocks(o.running_style_blocks), running_style_arrows(o.running_style_arrows), 
+        ContextBase<BlockStyle>(o, p, l),
+        running_style_blocks(o.running_style_blocks), running_style_arrows(o.running_style_arrows),
         blockname_prefix(o.blockname_prefix), background(o.background), pedantic(o.pedantic) {}
     /** Semi copy operator: copy only user-defined context content. If o is full, we overwrite us, else just merge.*/
     void ApplyContextContent(const BlockContext &o);
