@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2021 Zoltan Turanyi
+	Copyright (C) 2008-2022 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -37,6 +37,11 @@
 #undef max
 #undef min
 #endif
+#ifdef CAIRO_HAS_FC_FONT
+#include <fontconfig/fontconfig.h>
+#endif
+#include "ppt.h"
+#include "generic_shapes.h"
 #include "error.h"
 #include "style.h"
 #include "element.h"
@@ -45,13 +50,6 @@ class LanguageCollection;
 
 namespace pngutil {
 
-struct PNGChartText
-{
-    std::array<int, 3> version = { -1,-1,-1 };
-    std::string language;
-    std::string text;
-};
-
 enum class EPNGResult
 {
     OK,
@@ -61,17 +59,15 @@ enum class EPNGResult
     INVALID_PNG
 };
 
-std::pair<EPNGResult, std::vector<PNGChartText>>
+std::pair<EPNGResult, std::vector<EmbedChartData>>
 ReadPNG(FILE *png_file);
 
-EPNGResult AddToPNG(FILE *read_file, FILE *write_file,
-                    std::string_view language, std::string_view chart_text);
+EPNGResult AddToPNG(FILE *read_file, FILE *write_file, const EmbedChartData &chart);
 
-std::pair<std::string, std::string>
-Select(FILE *in, const LanguageCollection &languages, 
-       MscError &Error, const FileLineCol &pos=FileLineCol());
-}
+EmbedChartData Select(FILE *in, const LanguageCollection &languages,
+                      MscError& Error, const FileLineCol& pos = FileLineCol());
 
+} //namespace pngutil
 
 extern std::mutex cairo_text_mutex;
 
@@ -81,7 +77,8 @@ inline void thread_safe_cairo_show_text(cairo_t *cr, const char *utf8)
 #ifdef WIN32
     std::lock_guard<std::mutex> lock(cairo_text_mutex);
 #endif // CAIRO_HAS_WIN32_SURFACE
-    cairo_show_text(cr, utf8);
+    if (cr)
+        cairo_show_text(cr, utf8);
 }
 
 inline void thread_safe_cairo_text_path(cairo_t *cr, const char *utf8)
@@ -89,7 +86,8 @@ inline void thread_safe_cairo_text_path(cairo_t *cr, const char *utf8)
 #ifdef WIN32
     std::lock_guard<std::mutex> lock(cairo_text_mutex);
 #endif // CAIRO_HAS_WIN32_SURFACE
-    cairo_text_path(cr, utf8);
+    if (cr)
+        cairo_text_path(cr, utf8);
 }
 
 inline void thread_safe_cairo_text_extents(cairo_t *cr, const char *utf8,
@@ -98,25 +96,25 @@ inline void thread_safe_cairo_text_extents(cairo_t *cr, const char *utf8,
 #ifdef WIN32
     std::lock_guard<std::mutex> lock(cairo_text_mutex);
 #endif // CAIRO_HAS_WIN32_SURFACE
-    cairo_text_extents(cr, utf8, extents);
+    if (cr)
+        cairo_text_extents(cr, utf8, extents);
 }
 
 /***************************************************************************
  ***************************************************************************/
 
-/** Structure to hold page offsets and othe data.
- * Extensible for each chart type. 
- * The model is that here we store a potential heading (that comes to the page to be 
+/** Structure to hold page offsets and other data.
+ * Extensible for each chart type.
+ * The model is that here we store a potential heading (that comes to the page to be
  * displayed by the chart and a "lefting" some area that is to be drawn to the left of
  * the page. Today, we do not have the ability to draw a "righting" and the footer is
  * strictly just the copyright text.*/
-struct PageBreakData
-{
+struct PageBreakData {
     XY xy;                 ///<The top-left position in chart space
     XY wh;                 ///<The size of the page
     XY headingLeftingSize; ///<The size of the automatic space on the left or top of the page
     bool manual;           ///<True if the page break is inserted manually by the user
-    PageBreakData(const XY &_xy, const XY &_wh) : 
+    PageBreakData(const XY &_xy, const XY &_wh) :
         xy(_xy), wh(_wh), headingLeftingSize(0,0), manual(false) {}
 };
 
@@ -126,43 +124,47 @@ using PBDataVector = std::vector<const PageBreakData*> ;
 
 /** cm to points conversion.
  * see http://www.asknumbers.com/CentimetersToPointsConversion.aspx */
-#define PT_PER_CM 28.3464567 
+#define PT_PER_CM 28.3464567
 
-/** A class having only static members, providing page size conversion capabilities.*/
-class PageSizeInfo
-{
-public:
-    /** Describes fixed page output page sizes.*/
-    enum EPageSize {
-        NO_PAGE=1, ///<This value indicates no fixed page output
-        A0P, A0L,
-        A1P, A1L,
-        A2P, A2L,
-        A3P, A3L,
-        A4P, A4L,
-        A5P, A5L,
-        A6P, A6L,
-        LETTER_P, LETTER_L,
-        LEGAL_P, LEGAL_L,
-        LEDGER, TABLOID,
-        MAX_PAGE ///<Not used, the max value.
-    };
-    /** Converts a string to a fixed page size*/
-    static EPageSize ConvertPageSize(std::string_view);
-    /** Converts a fixed page size to a textual description*/
-    static std::string ConvertPageSize(EPageSize);
-    /** Converts a fixed page size to a verbose textual description*/
-    static std::string ConvertPageSizeVerbose(EPageSize);
-    /** Returns the physical size of `ps` in points (1/72 inch)*/
-    static XY GetPhysicalPageSize(EPageSize ps);
+/** Describes fixed page output page sizes.
+ * Odd values portrait, even values landscape.*/
+enum class EPageSize {
+    NO_PAGE=1, ///<This value indicates no fixed page output
+    A0P, A0L,
+    A1P, A1L,
+    A2P, A2L,
+    A3P, A3L,
+    A4P, A4L,
+    A5P, A5L,
+    A6P, A6L,
+    LETTER_P, LETTER_L,
+    LEGAL_P, LEGAL_L,
+    LEDGER, TABLOID,
+    WIDESCREEN_P, WIDESCREEN_L, 
+    ON_SCREEN_4_3_P, ON_SCREEN_4_3_L, 
+    ON_SCREEN_16_9_P, ON_SCREEN_16_9_L, 
+    ON_SCREEN_16_10_P, ON_SCREEN_16_10_L,
+    MAX_PAGE ///<Not used, the max value.
 };
 
+constexpr EPageSize DEFAULT_PPT_PAGE_SIZE = EPageSize::WIDESCREEN_L;
+
+/** Converts a string to a fixed page size*/
+EPageSize ConvertPageSize(std::string_view);
+/** Converts a fixed page size to a textual description*/
+std::string_view ConvertPageSize(EPageSize);
+/** Converts a fixed page size to a verbose textual description*/
+std::string_view ConvertPageSizeVerbose(EPageSize);
+/** Returns the physical size of `ps` in points (1/72 inch)*/
+XY GetPhysicalPageSize(EPageSize ps);
+
 namespace msc {
     //forward declare so that it can be friend of Canvas
-    class Box;  
-    class Pipe; 
+    class Box;
+    class Pipe;
 }
 
+class PptFile;
 /** A class encapsulating a cairo drawing conext.
  * Besides relaying drawing calls to the cario context Canvas has the following functionality.
  * - Create and manage various target surfaces (PDF, PNG, EPS, EMF, WMF, recording surface)
@@ -178,12 +180,12 @@ namespace msc {
  *   pages (printing or -p PDF output).
  *
  * A few notes on output formats.
- * - WMF internally uses an Enhanced Metafile with limited features and 
+ * - WMF internally uses an Enhanced Metafile with limited features and
  * converts to a MetaFile at closure. This is because a WMF DC cannot be given
  * to cairo, since it fails. For a list of tweaks for WMF see SetLowLevelParams().
  * - PRINTER is internally equal to EMF
- * 
- * The term *chart coordinates* refer to the coordinates the ArcBase::Layout() and 
+ *
+ * The term *chart coordinates* refer to the coordinates the ArcBase::Layout() and
  * ArcBase::Draw() and all Msc functions see. In this space the chart begins
  * at the origin and spans left-down towards growing x and y coordinates.
  * (Thus y coordinate grows downward, compatible with contour assumptions.)
@@ -193,10 +195,11 @@ namespace msc {
  * will translate to one pixel. Similar in PDF and EPS, one unit will translate
  * to one point (1/72 inch in PostScript and PDF).
  * However, the user can ask for scaling (in Canvas::Canvas) and also, for WMF
- * we introduce other scaling mechanisms, so the output may be dawn to a 
+ * we introduce other scaling mechanisms, so the output may be dawn to a
  * space different from the chart coordinate space.
  */
-class Canvas 
+
+class Canvas
 {
 public:
     /** Describes graphics output type */
@@ -210,21 +213,12 @@ public:
         EMFWMF,  ///<EMF output, but only uses features compatible with a WMF
         WIN,     ///<Bitmap output to a Windows window DC
         PRINTER, ///<Output to a Windows printer DC
-        ISMAP    ///<Server-side image map - just an empty file
+        ISMAP,   ///<Server-side image map - just an empty file
+        PPT,     ///<PowerPoint generated with external JavaScript
     };
     constexpr bool IsWindowsOutput(EOutputType t) { return t==EMF || t==WMF || t==EMFWMF || t==WIN || t==PRINTER; }
-    /** Describes the error status of Canvas */
-    enum EErrorType {
-        ERR_OK=0,       ///<No error
-        ERR_FILE,       ///<Error in opening the output file.
-        ERR_CANVAS,     ///<Some error in cairo.
-        ERR_CANVAS_MEM, ///<Out-of memory error from cairo.
-        ERR_PARAM,      ///<Bad parameters
-        ERR_MARGIN,     ///<Bad margins
-        ERR_DONE        ///<We have already closed this canvas.
-    };
     static XY DetermineScaling(const Block &total, const std::vector<XY> &scale,
-        const XY &pageSize, const double margins[4], 
+        const XY &pageSize, const double margins[4],
         double ctexth, const PBDataVector *pageBreakData);
     /** Checks if 'face' is available, or not.*
     * If it is not available, either an empty string or a substitute is returned in 'face'
@@ -233,8 +227,31 @@ public:
     * the -F command line option. FOR NOW WE ALWAYS TRUE.*/
     static bool HasFontFace(string & /*face*/, bool /*italics*/ = false, bool /*bold*/ = false) { return true; }
 
+    /** Returns true if the output format is graphics, and the Line(), Fill() etc
+     * drawing primitives can be called and there is a cairo context available.
+     * Returns false if we shall output higher-level shapes, e.g., for a PPT. */
+    bool does_graphics() const noexcept { return outType!=PPT; }
+
+    /** Return true if the canvas is not properly set up for drawing.
+     * This can be used for 3 things:
+     * 1. to spare drawing operations for dummy canvases;
+     * 2. to avoid drawing when we do not do graphics, just collect shapes (e.g., PPT); and
+     * 3. to prevent drawing on a nullptr cairo context.
+     * @param [in] assert_if_candraw_false We assert if candraw is false. This happens
+     *             when the canvas failed to open or is already closed. We never assert
+     *             for dummy contexts - there all operations always succeed, but have no
+     *             effect.
+     * @return true if this is a dummy canvas, is already closed or failed to open well.
+     *         For non-dummy canvases it ASSERTs that we can do graphics. */
+    bool cannot_draw(bool assert_if_candraw_false = true) const noexcept {
+        if (dummy) return true;
+        if (!does_graphics()) return true;
+        _ASSERT(candraw || !assert_if_candraw_false);
+        return !candraw || !cr || error.size();
+    }
+
 protected:
-    /** @name Low-level compatibility options 
+    /** @name Low-level compatibility options
      * @{ */
     bool     white_background;        /** Draw a white background before chart drawing.*/
     double   fake_scale;              /** Final rendering should be scaled like this. Used for WMF, since WMF coordinates are 16-bit integers. We are better off if the chart is scaeld up (less rounding), but not more than 32K.*/
@@ -256,21 +273,24 @@ protected:
     bool     avoid_transparency;      /** Pipes and gradient fills will be flattened before output. */
     /** @} */
 
-    /** @name Context data 
+    /** @name Context data
      * @{ */
     string           fileName;         /** The name of the output file (if any) in UTF-8*/
     FILE*            outFile;          /** The opened output file (if opened in constructor). If we have to add iTXt chunks to a PNG file, this is a temp file.*/
     FILE*            finalOutFile;     /** The eventual filename. Used if we add iTXt chunks to a PNG file.*/
     cairo_surface_t *surface;          /** The cairo surface we draw to */
     cairo_t         *cr;               /** The cairo context we draw to */
-    EErrorType       status;           /** The status of the Canvas */
+    PptFile          ppt;              /** The PPT file, in case of PPT output */
+    std::vector<std::vector<SimpleTransform>> tr_stack; /** In case of non-graphics (PPT) output this is the transform stack. */
+    std::string      error;            /** Empty if the Canvas is OK, else contains an error. */
+    bool             fatal = true;     /** If set, any error above will be reported as fatal. */
     bool             candraw;          /** False, if Canvas cannot be drawn on. Used only in debug asserts.*/
+    bool             dummy = false;    /** If true, we ignore any drawing operations silently. */
     double           fake_dash_offset; /** Status of fake dashes */
-    std::string      iTXt_language;    /** The name of the language to be added as an iTXt chunk to PNG files.*/
-    std::string_view iTXt_chart_text;  /** The UTF-8 chart text to be added as an iTXt chunk to PNG files.*/
+    EmbedChartData   iTXt_data;        /** The chart data (language, source text, etc.) to be added as an iTXt chunk to PNG files.*/
     /** @}*/
 
-    /** @name Values recorded at construction 
+    /** @name Values recorded at construction
      * @{ */
     const EOutputType outType;            ///<The output type
     const Block       total;              ///<The full chart area (in unscaled chart coordinates, all pages) excluding copyright, but including notes and comments
@@ -280,34 +300,33 @@ protected:
     Block             raw_page_clip;      ///<Used with fix-page outoput only. Contains the raw page size in points (pixels) minus margins. All zero if not a fixed page output.
     const int         h_alignment;        ///<Used with fix-page outoput only. Contains -1/0/+1 for left/center/right page alignment.
     const int         v_alignment;        ///<Used with fix-page outoput only. Contains -1/0/+1 for up/center/down page alignment.
-    unsigned          surface_size_x;     ///<The attempted size of surface (for error reporting)
-    unsigned          surface_size_y;     ///<The attempted size of surface (for error reporting)
     /** @} */
 
-    /** @name Logic 
+    /** @name Logic
      * @{ */
     void SetLowLevelParams();
-    static void GetPagePosition(const Block &total, const PBDataVector *pageBreakData, 
-                                unsigned page, XY &offset, XY &size, 
+    static void GetPagePosition(const Block &total, const PBDataVector *pageBreakData,
+                                unsigned page, XY &offset, XY &size,
                                 const PageBreakData **relevant = nullptr);
-    EErrorType CreateSurface(const XY &size); 
-    EErrorType CreateContext(const XY &origSize, const XY &origOffset,const XY &headingLeftingSize);
-    EErrorType CreateNilContext() { return CreateContext(XY(0, 0), XY(0, 0), XY(0, 0)); }
+    void CreateSurface(const XY &size);
+    void CreateContext(const XY &origSize, const XY &origOffset,const XY &headingLeftingSize);
+    void CreateNilContext() { return CreateContext(XY(0, 0), XY(0, 0), XY(0, 0)); }
     /** @} */
 
+friend class Chart; //for GetPagePosition
 friend class StringFormat; //for all sorts of text manipulation
 friend class msc::Box;  //for exotic line joints
 friend class msc::Pipe;  //for exotic line joints
-     /** @name Cairo context attribute handing. 
+     /** @name Cairo context attribute handing.
      * @{ */
     /** Set dash */
     void SetDash(const LineAttr &);
     /** Set the font face and bold/italics */
-    void SetFontFace(const char*face, bool italics, bool bold);
+    void SetFontFace(const std::string &face, const std::string& lang, bool italics, bool bold);
     /** Set the size of the font */
-    void SetFontSize(double size) {cairo_set_font_size (cr, size);}
+    void SetFontSize(double size) {if (cr) cairo_set_font_size (cr, size);}
     /** Set the source to a single color */
-    void SetColor(ColorType); 
+    void SetColor(ColorType);
     /** Set the source to a linear gradient.*/
     void linearGradient(ColorType from, ColorType to, const Block &b, double angle, EIntGradType type, double frac);
     /** Set the source to a radial gradient.*/
@@ -340,23 +359,31 @@ friend class msc::Pipe;  //for exotic line joints
     /** @} */
 
 public:
-    Canvas(EOutputType ot);
-    Canvas(EOutputType, const Block &tot, double copyrightTextHeight, 
+    /** Ways to open an empty canvas */
+    enum class Empty {
+        Dummy, ///<All operations succeed on this canvas with absolutely no effect. Uses minimal resources.
+        Query  ///<We create a minimal PNG surface to be able to query font sizes. Drawing to this results in asserts.
+    };
+    explicit Canvas(Empty e);
+    Canvas(EOutputType, const Block &tot, double copyrightTextHeight,
            std::string_view fn, const XY &scale=XY(1.,1.),
            const PBDataVector *pageBreakData=nullptr, unsigned page=0);
     Canvas(EOutputType, const Block &tot, std::string_view fn,
-           const XY &scale, const XY &pageSize, 
-           const double margins[4],  int ha, int va, 
+           const XY &scale, const XY &pageSize,
+           const double margins[4],  int ha, int va,
            double copyrightTextHeight, const PBDataVector *pageBreakData);
-    Canvas(EOutputType ot, cairo_surface_t *surf, const Block &tot, 
+    Canvas(EOutputType ot, cairo_surface_t *surf, const Block &tot,
            double copyrightTextHeight, const XY &scale=XY(1.,1.),
            const PBDataVector *pageBreakData=nullptr, unsigned page=0);
     Canvas(EOutputType ot, cairo_surface_t *surf, const Block &tot);
     /** Returns the state of the Canvas*/
-    EErrorType Status() const {return status;}
-    bool TurnPage(const PBDataVector *pageBreakData, unsigned next_page, 
-                  MscError *error=nullptr);
-    bool ErrorAfterCreation(MscError *error,  const PBDataVector *pageBreakData, bool fatal);
+    bool TurnPage(const PBDataVector *pageBreakData, unsigned next_page,
+                  MscError *pError=nullptr);
+    bool ErrorAfterCreation(MscError *pError,  const PBDataVector *pageBreakData);
+#ifdef CAIRO_HAS_FC_FONT
+protected:
+    FcPattern* fc_pattern = nullptr; //Destroy this pattern in CloseOutput()
+#endif
 #ifdef CAIRO_HAS_WIN32_SURFACE
 protected:
     /** @name Windows-related members
@@ -369,38 +396,36 @@ protected:
     Contour stored_fallback_image_places;///<When rendering on WMF or EMF, store the location of fallback images in chart space in CloseOutput()
     /** @} */
 public:
-    Canvas(EOutputType, HDC hdc, const Block &tot=Block(0,0,0,0), 
-           double copyrightTextHeight=0, const XY &scale=XY(1.,1.), 
+    Canvas(EOutputType, HDC hdc, const Block &tot=Block(0,0,0,0),
+           double copyrightTextHeight=0, const XY &scale=XY(1.,1.),
            const PBDataVector *pageBreakData=nullptr, unsigned page=0);
     /** When rendering on WMF or EMF, return the size of the resulting file. Valid only after CloseOutput().*/
-    size_t GetMetaFileSize() const {_ASSERT(outType==WMF||outType==EMF||outType==EMFWMF||outType==PRINTER); return stored_metafile_size;} 
+    size_t GetMetaFileSize() const {_ASSERT(outType==WMF||outType==EMF||outType==EMFWMF||outType==PRINTER); return stored_metafile_size;}
     /** When rendering on WMF or EMF, return the location of fallback images in chart space. Valid only after in CloseOutput()*/
-    Contour &GetFallbackImagePlaces() {_ASSERT(outType==WMF||outType==EMF||outType==EMFWMF||outType==PRINTER); return stored_fallback_image_places;} 
+    Contour &GetFallbackImagePlaces() {_ASSERT(outType==WMF||outType==EMF||outType==EMFWMF||outType==PRINTER); return stored_fallback_image_places;}
     static Contour FallbackImages(HENHMETAFILE hemf, LPRECT lpRECT);
     static void ExpandStrokePathRects(HDC hDC, HENHMETAFILE hemf, LPRECT lpRECT, size_t amount);
     HENHMETAFILE CloseAndGetEMF();
 #endif
 public:
     /** Set the fallback image resolution.*/
-    void SetFallbackImageResolution(double res) {if (status==ERR_OK) cairo_surface_set_fallback_resolution(surface, res/fake_scale, res/fake_scale);}
+    void SetFallbackImageResolution(double res) {if (error.empty()) cairo_surface_set_fallback_resolution(surface, res/fake_scale, res/fake_scale);}
     /** Switches to the second drawing situation allowing to draw header (inserted by e.g., automatic pagination) and footer (copyrgight text).*/
     void PrepareForHeaderFoorter();
     /** Instruct to add the chart text in an iTXt chunk to a PNG file at the end.
      * The chart_text is expected to remain valid until CloseOutput().
      * Returns false if we are not a PNG output or in error.*/
-    bool AddiTXtChunk(const std::string &language, std::string_view chart_text);
+    bool AddiTXtChunk(EmbedChartData &&chart);
     /** Flush output, write to file and close canvas.*/
     void CloseOutput();
-    ~Canvas() {if (status==ERR_OK) CloseOutput();}
+    ~Canvas() {if (error.empty()) CloseOutput();}
 
     /** Return our output type*/
     EOutputType GetOutputType() const {return outType;}
-    /** Return the current cairo context for direct manipulation*/
+    /** Return the current cairo context for direct manipulation. It may return nullptr.*/
     cairo_t *GetContext() const {return cr;}
     /** Our current total size*/
     const Block &GetSize() const {return total;}
-    /** Set color, line with and dash */
-    void SetLineAttr(const LineAttr&);
 
     /** True if the target does not support complex clipping */
     bool NeedsArrowFix() const {return needs_arrow_fix;}
@@ -411,6 +436,13 @@ public:
     /** True if we should avoid transparency when drawing */
     bool AvoidTransparency() const {return avoid_transparency;}
 
+    /** Adds a generic shape if we do not do graphichs, but collect shapes. */
+    bool Add(GenericShape&& shape);
+    /** Adds several generic shapes if we do not do graphichs, but collect shapes. */
+    bool AddMore(std::ranges::range auto&& shapes);
+
+    /** Set color, line with and dash */
+    void SetLineAttr(const LineAttr&);
     /** Set the line join style & return previous one.*/
     cairo_line_join_t SetLineJoin(cairo_line_join_t t);
     /** Set the line cap style & return previous one.*/
@@ -432,7 +464,7 @@ public:
     /** Clip to the inverse of a rectangle (anything outside `area` but inside `total`*/
     void ClipInverse(const Contour &area);
     /** Undo a previous clip operation*/
-    void UnClip() {cairo_restore(cr);}
+    void UnClip() { if (cannot_draw()) return; cairo_restore(cr);}
     void Transform_Rotate(const XY &center, double radian);
     void Transform_Rotate(const XY &center, double cos, double sin);
     void Transform_Rotate90(double s, double d, bool clockwise);
@@ -441,7 +473,7 @@ public:
     /** Transform the canvas by flipping it along the horizontal line at y. */
     void Transform_FlipHorizontal(double y);
     /** Restore the original canvas after a transformation.*/
-    void UnTransform() {cairo_restore(cr);}
+    void UnTransform() { if (!does_graphics()) tr_stack.pop_back(); else if (cannot_draw()) return; else cairo_restore(cr); }
 
     /** Draw a line along `edge` using the line style in `line`*/
     void Line(const Edge& edge, const LineAttr &line);
@@ -470,17 +502,17 @@ public:
     /**Draw shadow to a shape. `angle_radian` is the clockwise rotation*/
     void Shadow(const Contour &area, const ShadowAttr &shadow, double angle_radian=0);
     /** Draws a shape with label and shadow */
-    void Entity(const Block &outer_edge, const Label &label, const SimpleStyle &style, const ShapeCollection &shapes, int shape = -1);
+    void Entity(const Block &outer_edge, const Label &label, const SimpleStyle &style, const ShapeCollection &shapes, int shape = -1, bool align_top = true);
 };
 
-inline void Canvas::Clip(double sx, double dx, double sy, double dy) {cairo_save(cr); RectanglePath(sx, dx, sy, dy); cairo_clip(cr);}
-inline void Canvas::Clip(double sx, double dx, double sy, double dy, const LineAttr &line) {cairo_save(cr); RectanglePath(sx, dx, sy, dy, line); cairo_clip(cr);}
-inline void Canvas::Clip(const XY &s, const XY &d) {cairo_save(cr); RectanglePath(s.x, d.x, s.y, d.y); cairo_clip(cr);}
-inline void Canvas::Clip(const XY &s, const XY &d, const LineAttr &line) {cairo_save(cr); RectanglePath(s.x, d.x, s.y, d.y, line); cairo_clip(cr);}
-inline void Canvas::Clip(const Block &b) {cairo_save(cr); RectanglePath(b.x.from, b.x.till, b.y.from, b.y.till); cairo_clip(cr);}
+inline void Canvas::Clip(double sx, double dx, double sy, double dy) { if (cannot_draw()) return;  cairo_save(cr); RectanglePath(sx, dx, sy, dy); cairo_clip(cr); }
+inline void Canvas::Clip(double sx, double dx, double sy, double dy, const LineAttr &line) { if (cannot_draw()) return; cairo_save(cr); RectanglePath(sx, dx, sy, dy, line); cairo_clip(cr);}
+inline void Canvas::Clip(const XY &s, const XY &d) { if (cannot_draw()) return; cairo_save(cr); RectanglePath(s.x, d.x, s.y, d.y); cairo_clip(cr);}
+inline void Canvas::Clip(const XY &s, const XY &d, const LineAttr &line) { if (cannot_draw()) return; cairo_save(cr); RectanglePath(s.x, d.x, s.y, d.y, line); cairo_clip(cr);}
+inline void Canvas::Clip(const Block &b) { if (cannot_draw()) return; cairo_save(cr); RectanglePath(b.x.from, b.x.till, b.y.from, b.y.till); cairo_clip(cr);}
 //inline void Canvas::Clip(const Block &b, const LineAttr &line); not inline
 //void Clip(const EllipseData &ellipse); not inline
-inline void Canvas::Clip(const Contour &area) {cairo_save(cr); area.CairoPath(cr, true); cairo_clip(cr);}
+inline void Canvas::Clip(const Contour &area) { if (cannot_draw()) return; cairo_save(cr); area.CairoPath(cr, true); cairo_clip(cr);}
 
 
 inline void Canvas::Fill(const XY &s, const XY &d, const FillAttr &fill) {Fill(Block(s, d), fill);}
@@ -491,7 +523,7 @@ inline void Canvas::Fill(const Block &b, const LineAttr &line, const FillAttr &f
 inline void Canvas::Fill(const Contour &area, const FillAttr &fill) {Clip(area); Fill(area.GetBoundingBox(), fill); UnClip();}
 
 //This needs to be in this file for a definition of 'Canvas'
-/** Draws the arrowheads and the path together. 
+/** Draws the arrowheads and the path together.
  * If the canvas NeedsArrowFix() then we chop the path else we clip it.
  * We only draw the arrowheads at the end of the path, no mid-arrowheads
  * even if the supplied 'ah' argument has them.
@@ -514,7 +546,7 @@ void DrawPathWithArrows(Canvas &canvas, ArrowHeads &ah, const Path &path, const
 }
 
 //This needs to be in this file for a definition of 'Canvas'
-/** Draws the arrowheads and the path together using an arrowhead instance class. 
+/** Draws the arrowheads and the path together using an arrowhead instance class.
  * If the canvas NeedsArrowFix() then we chop the path else we clip it.
  * We only draw the arrowheads at the end of the path, no mid-arrowheads
  * even if the supplied 'ah' argument has them.
@@ -526,15 +558,27 @@ void DrawPathWithArrows(Canvas &canvas, ArrowHeads &ah, const Path &path, const
 template <class ArrowHeadsInstance, class ArrowHeads>
 void DrawPathWithArrows(Canvas &canvas, const ArrowHeadsInstance &ahi, ArrowHeads &ah, const Path &path, const LineAttr &line)
 {
-    if (canvas.NeedsArrowFix()) {
-        Path p = ahi.ClipLine(ah, p, line);
-        canvas.Line(p, line);
-    } else {
-        canvas.ClipInverse(ahi.ClipForLine(ah, path, line));
-        canvas.Line(path, line);
-        canvas.UnClip();
-    }
-    ahi.Draw(ah, canvas, path, line);
+    if (canvas.does_graphics()) {
+        if (canvas.NeedsArrowFix()) {
+            Path p = ahi.ClipLine(ah, p, line);
+            canvas.Line(p, line);
+        } else {
+            canvas.ClipInverse(ahi.ClipForLine(ah, path, line));
+            canvas.Line(path, line);
+            canvas.UnClip();
+        }
+        ahi.Draw(ah, canvas, path, line);
+    } else
+        canvas.Add(GSPath(path, line, &ah.GetArrowHead(false, EArrowEnd::END), &ah.GetArrowHead(false, EArrowEnd::START)));
+}
+
+
+bool Canvas::AddMore(std::ranges::range auto&& shapes) {
+    if (does_graphics()) return false;
+    if (!ppt.is_ok()) return false;
+    for (auto& shape : shapes)
+        if (!Add(std::move(shape).get())) return false;
+    return true;
 }
 
 #ifdef _DEBUG
