@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -32,7 +32,7 @@ class Canvas;
 enum class ESingleArrowType
 {
     NONE,         ///<No arrowhead
-    NORMAL,       ///<Line: a triangle, Block: wider than the body 
+    NORMAL,       ///<Line: a triangle, Block: wider than the body
     DIAMOND,      ///<Line: A diamond shape, Block: diamond on targetline wider than body
     NSDIAMOND,    ///<Line: A diamond shape, that is not symmetric, Block: diamond beyond targetline, not wider than body; drawn circle when midtype
     DOT,          ///<Line: A circle, Block: circle on targetline wider than body
@@ -50,7 +50,7 @@ enum class ESingleArrowType
     ICURVE,       ///<Like a '-(' sign (from graphviz)
     TICK,         ///<A bit thick line just on the target line (symmetric). Good for when there is no target line...
     JUMPOVER,     ///<jump over line: used for skip
-    //Block arrows only 
+    //Block arrows only
     STRIPES,      ///<Block only: Three rectangles of increasing size
     TRIANGLE_STRIPES ///<Block only: Three triangles of increasing size
 };
@@ -112,7 +112,7 @@ constexpr static ESingleArrowType _convert_earrow_to_block_esinglearrow[unsigned
 };
 
 /** Convert from EArrowType to ESingleArrowType.
- * In case of double or triple variants, we return their single version. 
+ * In case of double or triple variants, we return their single version.
  * 'block' tells us if the conversion is for a block arrow or a line arrow. */
 constexpr ESingleArrowType ConvertArrowType(EArrowType t, bool block) noexcept { return unsigned(t)<=unsigned(EArrowType::TRIANGLE_STRIPES) ? (block ? _convert_earrow_to_block_esinglearrow : _convert_earrow_to_esinglearrow)[unsigned(t)] : ESingleArrowType::NONE;}
 /* Returns the side of an EArrowType */
@@ -136,7 +136,7 @@ class SingleArrowHead
     bool are_we_part_of_a_half_line_series(const SingleArrowHead * front, const SingleArrowHead * back) const;
     enum EClipLineType
     {
-        CLIP_NO = 0,            ///<In the clip do not include any part of the arrow line. 
+        CLIP_NO = 0,            ///<In the clip do not include any part of the arrow line.
         CLIP_LEFT = 1,          ///<In the clip include the left side of the arrow line, when looking towards its end.
         CLIP_RIGHT = 2,         ///<In the clip include the right side of the arrow line, when looking towards its end.
         CLIP_ALL = 3,           ///<Include the full width of the arrow line in the clip.
@@ -149,7 +149,7 @@ class SingleArrowHead
     Contour curvy_line_cover(const Path &path, const PathPos &pos, bool forward, bool flip,
                              unsigned num, double *gap1, double *gap2, double *length) const;
 public:
-    /** @name Horizontal width & vertical depth of the arrow heads at EArrowType::NORMAL. 
+    /** @name Horizontal width & vertical depth of the arrow heads at EArrowType::NORMAL.
      * @{ */
     static constexpr double baseArrowWidth = 17;   ///<X-size of triangle for 100% size
     static constexpr double baseArrowHeight = 10;  ///<half of the Y-size of triangle for 100% size
@@ -162,9 +162,9 @@ public:
     static constexpr double SHARP_MUL_2 = 0.7;     ///<What percentage of the width of a sharp arrow is at the line
     static constexpr double VEE_MUL_1 = 1;         ///<How much wider is vee arrow than a normal one
     static constexpr double VEE_MUL_2 = 0.5;       ///<What percentage of the width of a vee arrow is at the line
-                                                   /** @} */
+    /** @} */
 
-    ESingleArrowType type;  ///<The type of the arrowhead. 
+    ESingleArrowType type;  ///<The type of the arrowhead.
     ESingleArrowSide side;  ///<Whether we are cropped to one side only
     bool             empty; ///<If true we are hollow inside. Ignored for block arrows.
     XY               scale; ///<Compared to 'normal' size
@@ -176,20 +176,20 @@ public:
 
     /** @name Functions and structures for line arrowheads
      * We provide two sets of functions. One computes/draws the single arrowhead on a straight line
-     * (assuming left-to-right direction along the x-axis, so you will have to rotate if another 
+     * (assuming left-to-right direction along the x-axis, so you will have to rotate if another
      * direction is needed); and the other computes/draws the arrowhead on an arbitrary path that
      * may be curvy or may bend. Note that the latter is a lot more resource intensive and works
      * only if the radius of curves and the angle of bends is large enough for the arrowhead.
      * @{ */
-    /** Structure to return the dimensions of a single line arowhead (an arrowhead segment). 
+    /** Structure to return the dimensions of a single line arowhead (an arrowhead segment).
      *  Here are two examples: SOLID and DIAMOND:
      * @verbatim
      *         |   ___                          |     ___
-     *      \  |    |half                      /:\     | half 
+     *      \  |    |half                      /:\     | half
      *       \ |    |height                   / : \    | height
      *  ------>|   ---               --------<  :  >  ---
      *       / |                              \ : /
-     *      /  |   after_x_extent=0            \:/ 
+     *      /  |   after_x_extent=0            \:/
      *         |                                |
      *     |<->|                            |<->|<->|
      *   before_x_extent               before_*   after_x_extent
@@ -201,7 +201,7 @@ public:
         double before_x_extent; ///<How wide the arrow (x dimension if arrow is horizontal) is before the target line
         double after_x_extent;  ///<How wide the arrow (x dimension if arrow is horizontal) is after the target line. 0 for non-symmetric arrowheads
     };
-    /** Structure to return detailed dimensions of a single line arrowhead. 
+    /** Structure to return detailed dimensions of a single line arrowhead.
      * Offset tells where the tip of next arrowhead element shall be compared to our tip.
      * Usually it equals to before_x_extent, but not for LINE and HALF_LINE arrowheads, where it is smaller
      * if the next element is also a line to typeset DOUBLE_LINE closer.*/
@@ -209,26 +209,26 @@ public:
     {
         double offset; ///<How much to offset along the arrow line to the next arrowhead. Small for LINE and HALF.
     };
-    WHORetval WidthHeightOffset(const LineAttr &mainline_before, const LineAttr &mainline_after, 
+    WHORetval WidthHeightOffset(const LineAttr &mainline_before, const LineAttr &mainline_after,
                                 const SingleArrowHead *front=nullptr,  const SingleArrowHead *back=nullptr) const;
-    Contour Cover(const XY &xy, bool flip, const LineAttr &mainline_before, const LineAttr &mainline_after, 
+    Contour Cover(const XY &xy, bool flip, const LineAttr &mainline_before, const LineAttr &mainline_after,
                   const SingleArrowHead *front=nullptr,  const SingleArrowHead *back=nullptr) const;
-    Contour ClipLine(const XY &xy, bool flip, bool ignore_opaque, const LineAttr &mainline_before, const LineAttr &mainline_after, 
+    Contour ClipLine(const XY &xy, bool flip, bool ignore_opaque, const LineAttr &mainline_before, const LineAttr &mainline_after,
                      const SingleArrowHead *front=nullptr,  const SingleArrowHead *back=nullptr) const;
-    void Draw(Canvas &canvas, const XY &xy, bool flip, 
-              const LineAttr &mainline_before, const LineAttr &mainline_after, 
+    void Draw(Canvas &canvas, const XY &xy, bool flip,
+              const LineAttr &mainline_before, const LineAttr &mainline_after,
               const SingleArrowHead *front=nullptr, const SingleArrowHead *back=nullptr) const;
 
     Contour Cover(const Path &path, const PathPos &pos, bool forward, bool flip,
-                  const LineAttr &mainline_before, const LineAttr &mainline_after, 
+                  const LineAttr &mainline_before, const LineAttr &mainline_after,
                   const SingleArrowHead *front=nullptr,  const SingleArrowHead *back=nullptr) const;
     Contour ClipLine(const Path &path, const PathPos &pos, bool forward, bool flip, bool ignore_opaque,
-                     const LineAttr &mainline_before, const LineAttr &mainline_after, 
-                     const SingleArrowHead *front=nullptr,  const SingleArrowHead *back=nullptr, 
+                     const LineAttr &mainline_before, const LineAttr &mainline_after,
+                     const SingleArrowHead *front=nullptr,  const SingleArrowHead *back=nullptr,
                      const Contour *cached_cover=nullptr) const;
     void Draw(Canvas &canvas, const Path &path, const PathPos &pos, bool forward, bool flip,
-              const LineAttr &mainline_before, const LineAttr &mainline_after, 
-              const SingleArrowHead *front=nullptr,  const SingleArrowHead *back=nullptr, 
+              const LineAttr &mainline_before, const LineAttr &mainline_after,
+              const SingleArrowHead *front=nullptr,  const SingleArrowHead *back=nullptr,
               const Contour *cached_cover = nullptr) const;
     /** @} */
 
@@ -236,7 +236,7 @@ public:
     /** @name Functions for block arrowheads
      * @{ */
 
-     /** Structure to return detailed dimensions of a block arrowhead. 
+     /** Structure to return detailed dimensions of a block arrowhead.
       * Here are a few examples:
       * @verbatim
       *
@@ -320,7 +320,7 @@ struct SingleArrowHeadCoverCache
     Contour cover;   ///<The cover of the arrowhead - non-drawing edges are invisible
 };
 
-/** A vector holding the computed cover info of a given (multi-segment) arrowhead. 
+/** A vector holding the computed cover info of a given (multi-segment) arrowhead.
  * The first element is embedded as 99% of arrowheads are single-element.
  * Supplying this will make it easier to calculate clip for line and target & drawing. */
 struct ArrowHeadCoverCache {
@@ -335,8 +335,8 @@ struct ArrowHeadCoverCache {
  *
  * Here is alternative No. 1 how arrowheads are meant to work (this method is
  * suitable when you primarily draw arrow to straight lines):
- * - You should create an Attribute class, where you allow your language to specify 
- *   arrowheads. There you can use OptAttr<> elements. You can in the end 
+ * - You should create an Attribute class, where you allow your language to specify
+ *   arrowheads. There you can use OptAttr<> elements. You can in the end
  *   allow or disallow arrowhead series, inlcude/exclude block arrows, allow
  *   scaling for arrowheads (or just allow built in sizes), allow or disallow
  *   individual sizing or coloring of arrowheads in a series.
@@ -347,11 +347,11 @@ struct ArrowHeadCoverCache {
  *   You can specify dimensioning and drawing interfaces for your class.
  *   Make your class the descendant for the above Attribute class.
  * - Add your attribute class to each chart element where you support arrowheads.
- *   This can be made via SimpleStyleWithArrow<> (found in styles.h) by specifying 
+ *   This can be made via SimpleStyleWithArrow<> (found in styles.h) by specifying
  *   your arrow Attribute class as template parameter. Then your language will have
- *   styles where it is possible to specify arrowheads via the attributes you have 
+ *   styles where it is possible to specify arrowheads via the attributes you have
  *   defined in the Attribute class described above.
- * - When you have finished parsing and all attributes of your elements are gathered, 
+ * - When you have finished parsing and all attributes of your elements are gathered,
  *   you can create the actual ArrowHeads objects from the attributes of the elements.
  *   Use CreateArrowHeads() function as an example for this.
  *
@@ -362,7 +362,7 @@ struct ArrowHeadCoverCache {
  *   OneArrowHead/TwoArrowHeads is sufficient). Then you shall create an instance class
  *   associeated that has ArrowHeadCoverCache objects for all arrowheads of the line
  *   e.g., start and end. OneArrowHeadInstance/TwoArrowHeadsInstance are examples.
- * - Then, when you have finished parsing create the instance objects for all actual 
+ * - Then, when you have finished parsing create the instance objects for all actual
  *   arrowheads that you want to draw. These will then store the cover of each arrowhead
  *   making clip line calculation, drawing and re-drawing fast. */
 class ArrowHead
@@ -376,6 +376,7 @@ protected:
     const SingleArrowHead *fsah() const { return series.size() ? &series.front() : nullptr; }
     const SingleArrowHead *lsah() const { return series.size() ? &series.back() : nullptr; }
 public:
+    static const ArrowHead noneArrowHead; ///<A pre-established 'NONE' arrowhead, that can be returned, when needed.
     using WHRetval = SingleArrowHead::WHRetval;
     explicit ArrowHead(ESingleArrowType t = ESingleArrowType::NONE, ESingleArrowSide s = ESingleArrowSide::BOTH, bool o = false, const XY &sc = XY(ArrowScale(EArrowSize::SMALL), ArrowScale(EArrowSize::SMALL)), const LineAttr &l = LineAttr()) :
         single(t, s, o, sc, l) {}
@@ -475,7 +476,7 @@ public:
                               const ColorSet *colors=nullptr, FileLineCol line_pos= FileLineCol(), Chart *chart=nullptr,
                               ArrowHead *arrow=nullptr, std::string *replaceto= nullptr);
 
-    /** @name Functions for line arrowheads. 
+    /** @name Functions for line arrowheads.
      * @{ */
     WHRetval WidthHeight(const LineAttr &mainline_before, const LineAttr &mainline_after) const;
     /** True if the arrowhead starts with (or entirely consists of) a symmetric element or is totally empty.*/
@@ -500,7 +501,7 @@ public:
     void Draw(Canvas &canvas, const XY &xy, const XY &from, bool flip, const LineAttr &mainline_before, const LineAttr &mainline_after, bool mirror = false) const;
 
     /** Return the area for which the target (e.g., an entity line) shall not be drawn.
-     * This version of the function assumes the arrowhead is drawn to 
+     * This version of the function assumes the arrowhead is drawn to
      * the end (or start) of an arbitrary path.
      * @param [in] path The tip of the arrow is at the end/start of this path.
      * @param [in] flip If true, the arrowhead needs to be flipped to the arrow line.
@@ -514,7 +515,7 @@ public:
     Contour TargetCover(const Path &path, bool flip, const LineAttr &mainline_before, const LineAttr &mainline_after, bool at_end = true, bool mirror = false, const ArrowHeadCoverCache *cache = nullptr) const
         { return TargetCover(path, PathPos(at_end ? path.size()-1 : 0, at_end ? 1 : 0), at_end, flip, mainline_before, mainline_after, mirror, cache); }
     /** Return the area where the arrow line shall not be drawn.
-     * This version of the function assumes the arrowhead is drawn to 
+     * This version of the function assumes the arrowhead is drawn to
      * the end (or start) of an arbitrary path.
      * @param [in] path The tip of the arrow is at the end/start of this path.
      * @param [in] flip If true, the arrowhead needs to be flipped to the arrow line.
@@ -531,7 +532,7 @@ public:
     Contour ClipForLine(const Path &path, bool flip, bool ignore_opaque, const LineAttr &mainline_before, const LineAttr &mainline_after, bool at_end = true, bool mirror = false, const ArrowHeadCoverCache *cache = nullptr) const
         { return ClipForLine(path, PathPos(at_end ? path.size()-1 : 0, at_end ? 1 : 0), at_end, flip, ignore_opaque, mainline_before, mainline_after, mirror, cache); }
     /** Return the area covered by the arrowhead.
-     * This version of the function assumes the arrowhead is drawn to  
+     * This version of the function assumes the arrowhead is drawn to
      * the end (or start) of an arbitrary path.
      * @param [in] path The tip of the arrow is at the end/start of this path.
      * @param [in] flip If true, the arrowhead needs to be flipped to the arrow line.
@@ -540,13 +541,13 @@ public:
      * @param [in] mainline_after If the line continues beyond, that line style.
      * @param [in] at_end If true the arrow is drawn at the end of the path, else the beginning (in reverse).
      * @param [in] mirror Set it to true if this is a mirrored version of an already drawn arrowhead.
-     * @param [out] cache If a non-nullptr cache object is given, it is filled with individual covers and 
+     * @param [out] cache If a non-nullptr cache object is given, it is filled with individual covers and
      *                    positions to be used by TargetCover(), ClipForLine() and Draw().
      * @returns The area, covered by the arrowhead.*/
     Contour Cover(const Path &path, bool flip, const LineAttr &mainline_before, const LineAttr &mainline_after, bool at_end = true, bool mirror = false, ArrowHeadCoverCache *cache = nullptr) const
         { return Cover(path, PathPos(at_end ? path.size()-1 : 0, at_end ? 1 : 0), at_end, flip, mainline_before, mainline_after, mirror, cache); }
     /** Draws the arrowhead.
-     * This version of the function assumes the arrowhead is drawn to 
+     * This version of the function assumes the arrowhead is drawn to
      * the end (or start) of an arbitrary path.
      * @param canvas The canvas to draw onto.
      * @param [in] path The tip of the arrow is at the end/start of this path.
@@ -566,8 +567,8 @@ public:
     void Draw(Canvas &canvas, const Path &path, const PathPos &pos, bool forward, bool flip, const LineAttr &mainline_before, const LineAttr &mainline_after, bool mirror = false, const ArrowHeadCoverCache *cache = nullptr) const;
     /** @} */
 
-    /** @name Functions for block arrowheads. 
-     * Since they can only contain a single segment, usually just forwarding to the 
+    /** @name Functions for block arrowheads.
+     * Since they can only contain a single segment, usually just forwarding to the
      * corresponding function in SingleArrowHead.
      * @{ */
      /** True if the arrowhead consists of a symmetric element and is not totally empty.*/
@@ -589,24 +590,25 @@ class OneArrowHead : public OneArrowAttr
 public:
     mutable ArrowHead endArrowHead;   ///<The actual ending arrowhead (calculated from size, xmul, ymul and endType)
     using OneArrowAttr::OneArrowAttr;
-    void CreateArrowHeads(double scale = 1) const { 
-        _ASSERT(OneArrowAttr::IsComplete()); 
-        if (dirty) 
-        { CreateAnArrowHead(endArrowHead, scale, endType, gvEndType); dirty = false; } 
+    void CreateArrowHeads(double scale = 1) const {
+        _ASSERT(OneArrowAttr::IsComplete());
+        if (dirty)
+        { CreateAnArrowHead(endArrowHead, scale, endType, gvEndType); dirty = false; }
     }
+    const ArrowHead& GetArrowHead(bool bidir, EArrowEnd which) const noexcept { _ASSERT(!dirty); return which==EArrowEnd::END ? endArrowHead : ArrowHead::noneArrowHead; }
 
     /** Returns what area we cover from the target at the end (e.g., an entity line).
      * (Used only for symmetric empty arrowheads, others return empty.)
      * @param [in] path The path with the arrowhead at its end.
      * @param [in] line The line style of the path.
      * @returns the area, where background should show instead of the target.*/
-    Contour TargetCoverEnd(const Path &path, const LineAttr &line) const 
+    Contour TargetCoverEnd(const Path &path, const LineAttr &line) const
         { _ASSERT(!dirty); return endArrowHead.TargetCover(path, false, line, line); }
     /** Returns what area we cover from the arrow line (i.e., 'path').
      * @param [in] path The path with the arrowhead at its end.
      * @param [in] line The line style of the path.
      * @param [in] ignore_opaque If true, the caller will draw the arrowhead *after* the
-     *                           arrow line and thus opaque, filled arrowheads does not 
+     *                           arrow line and thus opaque, filled arrowheads does not
      *                           have to return clip - a performance optimization.
      * @returns the area, where background should show instead of the arrow line.*/
     Contour ClipForLine(const Path &path, const LineAttr &line, bool ignore_opaque=false) const
@@ -628,9 +630,9 @@ public:
 
 
 
-/** A class containing full arrowhead info for an arrow at a specific path with 
+/** A class containing full arrowhead info for an arrow at a specific path with
  * a single (tail) arrowhead. */
-class OneArrowHeadInstance 
+class OneArrowHeadInstance
 {
 protected:
     mutable ArrowHeadCoverCache endArrowHeadCache; ///<The cache storing the computed covers;
@@ -645,8 +647,8 @@ public:
     * @param [in] path The path with the arrowhead at its end.
     * @param [in] line The line style of the path.
     * @returns the area, where background should show instead of the target.*/
-    Contour TargetCoverEnd(const OneArrowHead &ah, const Path &path, const LineAttr &line) const { 
-        ComputeCache(ah, path, line); 
+    Contour TargetCoverEnd(const OneArrowHead &ah, const Path &path, const LineAttr &line) const {
+        ComputeCache(ah, path, line);
         return ah.endArrowHead.TargetCover(path, false, line, line, true, false, &endArrowHeadCache);
     }
     /** Returns what area we cover from the arrow line (i.e., 'path').
@@ -687,17 +689,17 @@ public:
     mutable ArrowHead endArrowHead;   ///<The actual ending arrowhead (calculated from size, xmul, ymul and endType)
     mutable ArrowHead startArrowHead; ///<The actual starting arrowhead (calculated from size, xmul, ymul and startType)
     using TwoArrowAttr::TwoArrowAttr;
-    void CreateArrowHeads(double scale = 1) const { 
-        _ASSERT(TwoArrowAttr::IsComplete()); 
-        if (dirty) { 
-            CreateAnArrowHead(endArrowHead, scale, endType, gvEndType); 
-            CreateAnArrowHead(startArrowHead, scale, startType, gvStartType); 
-            dirty = false; 
-        } 
+    void CreateArrowHeads(double scale = 1) const {
+        _ASSERT(TwoArrowAttr::IsComplete());
+        if (dirty) {
+            CreateAnArrowHead(endArrowHead, scale, endType, gvEndType);
+            CreateAnArrowHead(startArrowHead, scale, startType, gvStartType);
+            dirty = false;
+        }
     }
 
     TwoArrowHeads() = default;
-
+    const ArrowHead& GetArrowHead(bool bidir, EArrowEnd which) const noexcept { _ASSERT(!dirty); return which==EArrowEnd::END ? endArrowHead : which != EArrowEnd::END ? ArrowHead::noneArrowHead : bidir ? endArrowHead : startArrowHead; }
     /** Returns what area we cover from the target at the start (e.g., an entity line).
     * (Used only for symmetric empty arrowheads, others return empty.)
     * @param [in] path The path with the arrowhead at its start.
@@ -758,7 +760,7 @@ public:
      * @param [in] line The line style of the path.
      * @returns the area, where background should show instead of the target.*/
     Contour TargetCoverEnd(const TwoArrowHeads &ah, const Path &path, const LineAttr &line) const {
-        ComputeCache(ah, path, line); 
+        ComputeCache(ah, path, line);
         return ah.endArrowHead.TargetCover(path, false, line, line, true, false, &endArrowHeadCache);
     }
     /** Returns what area the arrowheads cover from the arrow line (i.e., 'path').
@@ -770,7 +772,7 @@ public:
      *                           have to return clip - a performance optimization.
      * @returns the area, where background should show instead of the arrow line.*/
     Contour ClipForLine(const TwoArrowHeads &ah, const Path &path, const LineAttr &line, bool ignore_opaque = false) const
-    { 
+    {
         ComputeCache(ah, path, line);
         return ah.endArrowHead.ClipForLine(path, false, ignore_opaque, line, line, true, false, &endArrowHeadCache) +
                ah.startArrowHead.ClipForLine(path, true, ignore_opaque, line, line, false, false, &startArrowHeadCache);
@@ -825,14 +827,13 @@ Contour CoverPathWithArrows(const ArrowHeadsInstance &ahi, const ArrowHeads &ah,
     return ret;
 }
 
-                                                                                       
+
 /** A class containing full arrowhead info for an arrow including start and end arrowhead (head and tail),
  * but also arrowheads for mid-arrow stops and for locations where the arrow crosses, but does not stop
  * at another line (it skips that stop or 'jumps over' the other line).
  * For more examples of how this can be used, see libmscgen.*/
 class FullArrowHeads : public FullArrowAttr
 {
-    static const ArrowHead noneArrowHead; ///<A pre-established 'NONE' arrowhead, that can be returned, when needed.
 public:
     mutable ArrowHead endArrowHead;   ///<The actual end arrowhead (calculated from size, xmul, ymul and midType)
     mutable ArrowHead startArrowHead; ///<The actual start arrowhead (calculated from size, xmul, ymul and skipType)
@@ -840,24 +841,24 @@ public:
     mutable ArrowHead skipArrowHead;  ///<The actual skipping arrowhead (calculated from size, xmul, ymul and skipType)
 
     using FullArrowAttr::FullArrowAttr;
-    void CreateArrowHeads(double scale = 1) const { 
-        _ASSERT(FullArrowAttr::IsComplete()); 
-        if (dirty) { 
-            if (type!=EArcArrowType::BIGARROW) 
-                CreateAnArrowHead(skipArrowHead, scale, skipType, gvSkipType);  
-            CreateAnArrowHead(endArrowHead, scale, endType, gvEndType); 
-            CreateAnArrowHead(startArrowHead, scale, startType, gvStartType); 
-            CreateAnArrowHead(midArrowHead, scale, midType, gvMidType); 
-            dirty = false; 
+    void CreateArrowHeads(double scale = 1) const {
+        _ASSERT(FullArrowAttr::IsComplete());
+        if (dirty) {
+            if (type!=EArcArrowType::BIGARROW)
+                CreateAnArrowHead(skipArrowHead, scale, skipType, gvSkipType);
+            CreateAnArrowHead(endArrowHead, scale, endType, gvEndType);
+            CreateAnArrowHead(startArrowHead, scale, startType, gvStartType);
+            CreateAnArrowHead(midArrowHead, scale, midType, gvMidType);
+            dirty = false;
         }
     }
 
     FullArrowHeads() = default;
     /** Return a reference to the arrowhead specified in 'which'. If bidir is true then we return 'endArrowHead' even for which==START. */
-    ArrowHead &GetArrowHead(bool bidir, EArrowEnd which) { _ASSERT(!dirty); return which==EArrowEnd::END ? endArrowHead : which==EArrowEnd::MIDDLE ? midArrowHead : which==EArrowEnd::SKIP ? skipArrowHead : bidir ? endArrowHead : startArrowHead; }
+    ArrowHead &GetArrowHead(bool bidir, EArrowEnd which) noexcept { _ASSERT(!dirty); return which==EArrowEnd::END ? endArrowHead : which==EArrowEnd::MIDDLE ? midArrowHead : which==EArrowEnd::SKIP ? skipArrowHead : bidir ? endArrowHead : startArrowHead; }
     /** Return a reference to the arrowhead specified in 'which'. If bidir is true then we return 'endArrowHead' even for which==START. */
-    const ArrowHead &GetArrowHead(bool bidir, EArrowEnd which) const { _ASSERT(!dirty);  return which==EArrowEnd::END ? endArrowHead : which==EArrowEnd::MIDDLE ? midArrowHead : which==EArrowEnd::SKIP ? skipArrowHead : bidir ? endArrowHead : startArrowHead; }
-    const ArrowHead &GetArrowHead(bool forward, bool bidir, EArrowEnd which, bool left) const;
+    const ArrowHead &GetArrowHead(bool bidir, EArrowEnd which) const noexcept { _ASSERT(!dirty);  return which==EArrowEnd::END ? endArrowHead : which==EArrowEnd::MIDDLE ? midArrowHead : which==EArrowEnd::SKIP ? skipArrowHead : bidir ? endArrowHead : startArrowHead; }
+    const ArrowHead &GetArrowHead(bool forward, bool bidir, EArrowEnd which, bool left) const noexcept;
 };
 
 #endif //ARROWHEAD_H
