@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -348,7 +348,11 @@ public:
             const clock::time_point now = clock::now();
             if (has_current_section) {
                 _ASSERT(!current_section_is_bulk);
-                _ASSERT(unsigned(section)== current_section);
+                //When processing a section, we sometimes generate a new Element
+                //and need to mark prior sections for that Element as Done.
+                //So it is not the case that when starting a section we only report
+                //work from that section, hence this check sometimes fail.
+                //_ASSERT(unsigned(section)== current_section);
             };
             //We need to be very flexible in bookkeeping items done:
             //We may do any items multiple times (e.g., draw)
@@ -361,7 +365,7 @@ public:
                 //We have already reported the processing of all registered items.
                 //Do nothing now, the time spent processing this extra item will go
                 //unaccounted for.
-                _ASSERT(0);
+//                _ASSERT(0);
                 started = now;
                 return;
             } else if (done_items[unsigned(section)][unsigned(category)]+num <= registered_items[unsigned(category)]) {
@@ -371,7 +375,7 @@ public:
                 counted_item_ticks[unsigned(section)][unsigned(category)] += now - started;
             } else {
                 done_items[unsigned(section)][unsigned(category)] = registered_items[unsigned(category)];
-                _ASSERT(0);
+//                _ASSERT(0);
             }
             started = now;
             Callback();
@@ -468,15 +472,11 @@ public:
     *          is substituted with the defaults.*/
     bool ReadLoadData(std::string_view inp) override
     {
-        if (inp.length()==0 || inp.front()==0) {
-        error:
-            Reset();
-            return false;
-        }
+        if (inp.length()==0 || inp.front()==0) return false;
         auto ver = DeserializeInt(inp);
-        if (!ver.first || ver.second != 1) goto error; //too advanced or too old version
+        if (!ver.first || ver.second != 1) return false; //too advanced or too old version
         auto v = DeserializeInts(inp, (size_t)num_bulk_sections);
-        if (v.size()!=(size_t)num_bulk_sections) goto error;
+        if (v.size()!=(size_t)num_bulk_sections) return false;
         if constexpr ((bool)(size_t)num_bulk_sections)
             for (size_t i = 0; i<(size_t)num_bulk_sections; i++)
                 loaded_bulk_ns[i] = std::chrono::nanoseconds(v[i]);
@@ -484,7 +484,7 @@ public:
         if constexpr ((bool)(size_t)num_itemized_sections)
             for (unsigned s = 0; s<(size_t)num_itemized_sections; s++) {
                 v = DeserializeInts(inp, (size_t)num_item_categories);
-                if (v.size()!=(size_t)num_item_categories) goto error;
+                if (v.size()!=(size_t)num_item_categories) return false;
                 if constexpr ((bool)(size_t)num_item_categories)
                     for (size_t i = 0; i<(size_t)num_item_categories; i++)
                         loaded_item_ns[s][i] = std::chrono::nanoseconds(v[i]);
@@ -526,4 +526,4 @@ public:
 };
 
 
-#endif //CGEN_PROGRESS_H
\ No newline at end of file
+#endif //CGEN_PROGRESS_H
