@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -245,33 +245,27 @@ void Shape::CairoPath(cairo_t *cr, unsigned section, const Block &o) const
 void Shape::Draw(Canvas & canvas, const Block & o, const LineAttr & line, const 
                  FillAttr & fill, const ShadowAttr &shadow) const
 {
-    cairo_t *cr = canvas.GetContext();
-    if (GetSection(0).size() && shadow.color.is_set && shadow.offset.is_set &&
-        shadow.offset.value) {
+    cairo_t * const cr = canvas.GetContext();
+    if (!cr) return;
+    if (GetSection(0).size() && shadow.color && shadow.offset.value_or(0))
         canvas.Shadow(Contour(GetSection(0, o), ECloseType::CLOSE_ALL_CONNECTED), shadow);
-    }
-    if (GetSection(0).size() && 
-        fill.color.is_set && fill.color.value.type!=ColorType::INVALID &&
-        fill.color.value.a!=0) {
+    if (GetSection(0).size() && fill.color && !fill.color->IsFullyTransparent()) {
         CairoPath(cr, 0, o);
         cairo_save(cr);
         cairo_clip(cr);
         canvas.Fill(o, fill);
         cairo_restore(cr);
     }
-    if (line.type.is_set && line.type.value!=ELineType::NONE &&
-        line.color.is_set && line.color.value.type!=ColorType::INVALID &&
-        line.color.value.a!=0) {
+    if (line.type && *line.type!=ELineType::NONE && line.color && !line.color->IsFullyTransparent()) {
         if (GetSection(1).size()) {
-            FillAttr linefill(line.color.value, EGradientType::NONE);
+            FillAttr linefill = FillAttr::Solid(*line.color);
             CairoPath(cr, 1, o);
             cairo_save(cr);
             cairo_clip(cr);
             canvas.Fill(o, linefill);
             cairo_restore(cr);
         }
-        if (GetSection(2).size() &&
-            line.width.is_set && line.width.value>0) {
+        if (GetSection(2).size() && line.width && *line.width>0) {
             CairoPath(cr, 2, o);
             canvas.SetLineAttr(line);
             cairo_stroke(cr);
@@ -281,6 +275,7 @@ void Shape::Draw(Canvas & canvas, const Block & o, const LineAttr & line, const
 
 ShapeCollection & ShapeCollection::operator = (const ShapeCollection &o)
 {
+    if (&o==this) return *this;
     shapes.clear();
     shapes.reserve(o.shapes.size());
     for (auto s : o.shapes)
@@ -565,8 +560,7 @@ bool CshHintGraphicCallbackForPorts(Canvas *canvas, CshHintGraphicParam p, CshHi
         hint.y.from + (hint.y.Spans() - HINT_GRAPHIC_SIZE_Y/r)/2);
     Block max = (*csh.pShapes)[shape].GetMax();
     max.Shift(-orig).Scale(r);
-    csh.pShapes->Draw(*canvas, shape, max,
-                      LineAttr(), FillAttr(ColorType(), EGradientType::NONE));
+    csh.pShapes->Draw(*canvas, shape, max, LineAttr(), FillAttr::None());
     auto pp = csh.pShapes->GetShape(shape)->GetPort(port, max);
     if (pp) {
         canvas->Fill(Contour(pp.value().xy, 2), FillAttr(ColorType::red(), EGradientType::OUTWARD));
