@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -26,6 +26,7 @@
 
 #include "chartbase.h"
 #include "cgen_progress.h"
+#include "canvas.h"
 
 /** Jumps over tabs and spaces in 's'. If 's' starts with none of these, nothing is done.
  * Any zero character stops processing.
@@ -281,7 +282,7 @@ Chart::ParseCompassPoint(std::string_view n, const FileLineCol &l,
 
 /** Draws the chart into one of more files.
 * If the chart contains only one page, we create a single file. If there
-* are multiple pages, we create one for each. If case of PDF output format
+* are multiple pages, we create one for each. If case of PDF/PPT output format
 * if pageSize is nonzero, we create a single file with multiple pages.
 * 'pageSize' can only be valid (positive `x` and `y`) if file format is PDF (which supports
 * multiple pages). In that
@@ -331,7 +332,7 @@ bool Chart::DrawToFile(Canvas::EOutputType ot, const XY &scale,
                     Error.FatalError(FileLineCol(0, 0), "Could not open output file '" + fileName + "'.");
                     return false;
                 }
-                Canvas canvas(ot);
+                Canvas canvas(Canvas::Empty::Query);
                 CollectIsMapElements(canvas);
                 for (auto &e : ismapData) {
                     if (page>0 && pageBreakData[page-1]->xy.y <= e.rect.x.from &&
@@ -343,17 +344,18 @@ bool Chart::DrawToFile(Canvas::EOutputType ot, const XY &scale,
                 fclose(fout);
             } else {
                 Canvas canvas(ot, total, copyrightTextHeight, fn, scale, &pageBreakData, page);
-                if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData, true)) return false;
-                if (include_chart_text)
-                    canvas.AddiTXtChunk(GetLanguageExtensions().front(), include_chart_text);
+                if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData)) return false;
+                if (include_chart_text) 
+                    canvas.AddiTXtChunk(EmbedChartData{ .chart_type = GetLanguageExtensions().front(), .chart_text = include_chart_text,
+                                                        .chart_size = GetCanvasSize(), .gui_state=SerializeGUIState(), .page=page});
                 DrawComplete(canvas, bPageBreak, page);
             }
         } 
     else {
         _ASSERT(ot!=Canvas::ISMAP);
-        _ASSERT(ot==Canvas::PDF); //only PDF supports multiple pages.
-        Canvas canvas(ot, total, fn, scale, pageSize, margins, ha, va, copyrightTextHeight, &pageBreakData);
-        if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData, true)) return false;
+        _ASSERT(ot==Canvas::PDF || ot==Canvas::PPT); //only PDF/PPT supports multiple pages.
+        Canvas canvas(ot, total, fn, scale, pageSize, margins, ha, va, copyrightTextHeight, from || till ? &pageBreakData : nullptr);
+        if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData)) return false;
         for (unsigned page = from; page<=till; page++) {
             DrawComplete(canvas, bPageBreak, page);
             if (page<till)
@@ -387,7 +389,7 @@ HENHMETAFILE Chart::DrawToMetaFile(Canvas::EOutputType ot,
         return 0;
     if (page>0) bPageBreaks = false;
     Canvas canvas(ot, HDC(nullptr), total, copyrightTextHeight, XY(1, 1), &pageBreakData, page);
-    if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData, true)) return 0;
+    if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData)) return 0;
     if (fallback_image_resolution>0)
         canvas.SetFallbackImageResolution(fallback_image_resolution);
     DrawComplete(canvas, bPageBreaks, page);
@@ -420,7 +422,7 @@ size_t Chart::DrawToDC(Canvas::EOutputType ot, HDC hdc, const XY &scale,
     if (page>0) bPageBreaks = false;
     const PBDataVector pageBreakData = GetPageVector();
     Canvas canvas(ot, hdc, total, copyrightTextHeight, scale, &pageBreakData, page);
-    if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData, true)) return 0;
+    if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData)) return 0;
     if (fallback_image_resolution>0)
         canvas.SetFallbackImageResolution(fallback_image_resolution);
     DrawComplete(canvas, bPageBreaks, page);
@@ -450,7 +452,7 @@ cairo_surface_t *Chart::DrawToRecordingSurface(Canvas::EOutputType ot, bool bPag
     }
     const PBDataVector pageBreakData = GetPageVector();
     Canvas canvas(ot, ret, total);
-    if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData, true)) {
+    if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData)) {
         cairo_surface_destroy(ret);
         return nullptr;
     }
@@ -479,10 +481,11 @@ cairo_surface_t *Chart::ReDrawOnePage(cairo_surface_t *full, unsigned page,
     }
     const PBDataVector pageBreakData = GetPageVector();
     Canvas canvas(Canvas::SVG, ret, total, copyrightTextHeight, XY(1., 1.), &pageBreakData, page);
-    if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData, true)) {
+    if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData)) {
         cairo_surface_destroy(ret);
         return nullptr;
     }
+    //GetContext() returns non-null after the check above
     cairo_set_source_surface(canvas.GetContext(), full, 0, 0);
     cairo_paint(canvas.GetContext());
 
@@ -530,6 +533,16 @@ Chart::Include(std::string_view filename, const FileLineCol &linenum_command)
     return ret;
 }
 
+XY Chart::GetCanvasSize(unsigned page) const noexcept {
+    XY origSize, origOffset;
+    PBDataVector pages = GetPageVector();
+    const PageBreakData* pbData;
+    Canvas::GetPagePosition(total, &pages, page, origOffset, origSize, &pbData);
+    XY size = origSize + (pbData ? pbData->headingLeftingSize : XY(0, 0));
+    size.y += copyrightTextHeight;
+    return size;
+}
+
 
 /** Ensure that the attribute value specifies a color.
  *
