@@ -1,6 +1,6 @@
 /*
   This file is part of Msc-generator.
-Copyright (C) 2008-2021 Zoltan Turanyi
+Copyright (C) 2008-2022 Zoltan Turanyi
 Distributed under GNU Affero General Public License.
 
 Msc-generator is free software: you can redistribute it and/or modify
@@ -75,7 +75,7 @@ struct LabelInfo
     {
         INVALID = 0, ///<Not valid
         ENTITY,    ///<The label belongs to an entity header
-        ARROW,     ///<The label belongs to an arrow 
+        ARROW,     ///<The label belongs to an arrow
         DIVIDER,   ///<The label belongs to a divider (title, etc.)
         BOX,       ///<The label belongs to a box with content
         EMPTYBOX,  ///<The label belongs to an empty box
@@ -104,7 +104,7 @@ struct RefNameData
  * (as opposed to partial).*/
 using DesignNameCollection = std::map<std::string, bool>;
 
-/** Struct to store the background fill for calculating the 
+/** Struct to store the background fill for calculating the
  * background fill for headers and footers. */
 struct BackgroundFill {
     FillAttr fill;  ///<The fill applied to the chart background
@@ -116,11 +116,11 @@ struct BackgroundFill {
  * and their cover (for tracking), designs, supports pagination, copyright text,
  * references inside the chart and more.
  * It defines a three step processing model: ParseText() takes the input text and parses
- * it creating some (language dependent) internal data structure. ParseText() shall be 
+ * it creating some (language dependent) internal data structure. ParseText() shall be
  * possible to be called several times (on several chunks of files). Design and shape
  * definitions must be fully collected during this phase, so ExportDesigns() shall be
  * possible to be called after ParseText() runs. Also, ImportDesigns() shall be possible
- * to be called before ParseText()s to emulate as if those designlib files were actually 
+ * to be called before ParseText()s to emulate as if those designlib files were actually
  * parsed again to get the designs.
  * Second step is a call to CompleteParse(). This happens only once and shall prepare
  * everything that is required for parsing. This is the last place where you can emit
@@ -137,7 +137,7 @@ class Chart
 public:
     /** @name Language attributes
     * @{ */
-    virtual std::string GetLanguageDescription() const = 0; ///<Returns the human-readable (UTF-8) short description 
+    virtual std::string GetLanguageDescription() const = 0; ///<Returns the human-readable (UTF-8) short description
     virtual std::vector<std::string> GetLanguageExtensions() const = 0; ///<Returns a set of extensions in order of preference
     virtual std::string GetLanguageEntityName() const = 0; ///<How is the 'entity' called in this language
     virtual std::string GetLanguageDefaultText() const = 0; ///<Returns the default text of the chart in UTF-8
@@ -173,7 +173,7 @@ protected:
     * @{ */
     Block  total;               ///<Total bounding box of the chart (minus copyright), in chart space, not considering pagination.
     double copyrightTextHeight; ///<Height of the copyright text
-    std::vector<BackgroundFill> bkFill; ///<The list of background fills applied in the chart. Used to calculate the bkg of the header, footer. When empty, we default to white. 
+    std::vector<BackgroundFill> bkFill; ///<The list of background fills applied in the chart. Used to calculate the bkg of the header, footer. When empty, we default to white.
     unsigned noLabels;          ///<The number of labels (except entities)
     unsigned noOverflownLabels; ///<The number of labels overflown
     /** @} */
@@ -182,7 +182,7 @@ public:
     /** A pointer to a procedure that reads this file and returns its content.
      * The second parameter is passed transparently.
      * Returns the error message ("" if OK) and the read text.
-     * first returned element is error message ("" if OK), second is full UTF-8 path 
+     * first returned element is error message ("" if OK), second is full UTF-8 path
      * of the file we have read, third is the file content.*/
     typedef std::tuple<std::string, std::string, std::string> FileReadProcedure(std::string_view filename, void *param, std::string_view included_from);
 
@@ -192,12 +192,12 @@ public:
     FileReadProcedure * const file_read_proc;
     void * const file_read_proc_param;
 
-    /** Consumes a command-line argument specific to this chart type. 
+    /** Consumes a command-line argument specific to this chart type.
      * If the command-line argument is a general one, this function will not be called.
      * Return true if the argument was recognized for this language/chart type.
-     * If command line arguments need a parameter, use the equal sign-format, so that 
+     * If command line arguments need a parameter, use the equal sign-format, so that
      * "-arg=param" and not "-arg param". All strings may be UTF-8.
-     * If the parameter is not recognized and you return false here, but it starts 
+     * If the parameter is not recognized and you return false here, but it starts
      * with a double hyphen, it is tested for a design (like "--omegapple") and applied,
      * if so. If no such design is available, or the parameter has an equal sign
      * AddCommandLineOption will be called.*/
@@ -205,6 +205,14 @@ public:
     /** Adds a command-line chart option. Shall emit all errors, as the callee
      * will not know if succeeded.*/
     virtual void AddCommandLineOption(const Attribute &a) = 0;
+    /** Sets the default font and font language. Leave empty if none.
+     * You does not have to call it, but if you do, do it right after the
+     * creation of the chart object, before parsing.
+     * For signalling charts and block diagrams, this is synonimous with
+     * setting the text.font.{face,lang} chart options, for graphs it is
+     * synonimous with issuing the {graph,node,edge} [ fontname = <face> ];
+     * commands (and lang is ignored).*/
+    virtual void SetDefaultFont(const std::string& face, const std::string& lang, const FileLineColRange &l) = 0;
 
     /** Opens and loads a file for inclusion. Filename is UTF-8.
      * Returns the text to include and the poistion of its first character.*/
@@ -216,15 +224,18 @@ public:
     /** Return the current state of the pedantic chart option at the outermost scope.
      * Shall be used after compilation.*/
     virtual bool GetPedantic() const noexcept = 0;
-    /** Set the initial value of the pedantic chart option. 
+    /** Set the initial value of the pedantic chart option.
      * Shall be used before calling ParseText().*/
     virtual void SetPedantic(bool pedantic) noexcept = 0;
     /** Returns the total bounding box of the chart (minus copyright), in chart space, not considering pagination.*/
-    const Block &GetTotal() const { return total; }
+    const Block &GetTotal() const noexcept { return total; }
     /** Returns the height of the copyright at the bottom.*/
-    double GetCopyrightTextHeight() const { return copyrightTextHeight; }
+    double GetCopyrightTextHeight() const noexcept { return copyrightTextHeight; }
+    /** Get the required canvas size to draw a page (of the whole chart if page==0)
+     * The returned size is in chart units.*/
+    XY GetCanvasSize(unsigned page = 0) const noexcept;
 
-    /** @name Get page, context and design data 
+    /** @name Get page, context and design data
      * @{ */
     /** Get the pointers to the pagination information of us. Valid after CompleteParse().*/
     virtual PBDataVector GetPageVector() const = 0;
@@ -268,11 +279,11 @@ public:
     /** @name Parse Options
     * @{ */
     bool ignore_designs;         ///<Ignore design changes
-    bool prepare_for_tracking;   ///<If true, all elements shall compute an 'area' member, we fill 'AllCovers' and 'AllArcs'. 
-    bool prepare_element_controls;///<If true, We collect all signatures to force_collapse_box. Used if we need to do box collapses from a GUI. 
+    bool prepare_for_tracking;   ///<If true, all elements shall compute an 'area' member, we fill 'AllCovers' and 'AllArcs'.
+    bool prepare_element_controls;///<If true, We collect all signatures to force_collapse_box. Used if we need to do box collapses from a GUI.
     std::string copyrightText;   ///<The copyright text we display at the bottom of each page.
-    double trackFrameWidth = 4;  ///<Width of the frames used for tracking boxes and pipes on screen 
-    double trackExpandBy = 2;    ///<How much do we expand tracking covers 
+    double trackFrameWidth = 4;  ///<Width of the frames used for tracking boxes and pipes on screen
+    double trackExpandBy = 2;    ///<How much do we expand tracking covers
     /** @} */
 
     /** Adds the list of chart options to a Csh object. */
@@ -286,12 +297,12 @@ public:
                                             bool report_error);
 
     /** Parse a piece of input text as a new input file.
-     * This must be possible to be called several times, and must keep in-parse state 
-     * ready to accept new blocks of top-level syntax. (We will not break 
+     * This must be possible to be called several times, and must keep in-parse state
+     * ready to accept new blocks of top-level syntax. (We will not break
      * top level blocks or instructions into two files, but may parse
      * several design libs in one go, and then the user input after; or this may be
-     * used later to implement \#inlcude-like behaviour. 
-     * It must maintain a ready copy of 'Shapes' and 'ChartBase::Designs'. 
+     * used later to implement \#inlcude-like behaviour.
+     * It must maintain a ready copy of 'Shapes' and 'ChartBase::Designs'.
      * @param [in] input string of UTF8 encoded chart text
      * @param [in] filename The UTF8 name of the file to use in error messages.
      * @returns the number of the file as will appear in FileLineCol errors.*/
@@ -307,10 +318,9 @@ public:
      * ismapData - if requestes by the caller (else we will not generate ismaps even if the user wanted them) (leaving it to empty is OK)
      * AllElements, AllCovers - if one wishes tracking (leaving it to empty is OK)
      * file_url, file_info - if one has these chart options (leaving it to empty is OK)*/
-    virtual void CompleteParse(Canvas::EOutputType, bool avoidEmpty,
-                               bool autoPaginate = false, bool addHeading = true,
-                               XY pageSize = XY(0, 0), bool fitWidth = true,
-                               bool collectLinkInfo = false) = 0;
+    virtual void CompleteParse(bool autoPaginate = false,
+                               bool addHeading = true, XY pageSize = XY(0, 0),
+                               bool fitWidth = true, bool collectLinkInfo = false) = 0;
 
     /** Used to count labels. Call once for every label and indicate if it has been overflowing.*/
     void CountLabel(bool overflown) { noLabels++; if (overflown) noOverflownLabels++; }
@@ -350,7 +360,7 @@ public:
     void RegisterLabel(const Label &l, LabelInfo::LabelType type,
                        double s, double d, double t, ESide side, double c = -CONTOUR_INFINITY)
     { if (l.size()) labelData.emplace_back(type, std::string(l), l.Cover(Shapes, s, d, t, side, c).GetBoundingBox());}
-    
+
     /** Register a label (with arbitrary cover).
     * This is used to generate lmaps
     * @param [in] l The label to register.
@@ -377,7 +387,7 @@ public:
     * @param [in] page The page to draw. Zero means the whole chart. */
     virtual void DrawComplete(Canvas &canvas, bool pageBreaks, unsigned page) = 0;
 
-    virtual bool DrawToFile(Canvas::EOutputType, const XY &scale, 
+    virtual bool DrawToFile(Canvas::EOutputType, const XY &scale,
                             std::string_view fn, bool bPageBreak, bool ignore_pagebreaks,
                             const char *include_chart_text = nullptr,
                             const XY &pageSize=XY(0,0), const double margins[4]=nullptr,
@@ -391,7 +401,7 @@ public:
                             double fallback_image_resolution=-1,
                             bool generateErrors=false);
 #endif
-    virtual cairo_surface_t *DrawToRecordingSurface(Canvas::EOutputType, 
+    virtual cairo_surface_t *DrawToRecordingSurface(Canvas::EOutputType,
         bool bPageBreaks, bool generateErrors = false);
     virtual cairo_surface_t *ReDrawOnePage(cairo_surface_t *full,
         unsigned page, bool generateErrors = false);
@@ -403,7 +413,7 @@ public:
  * ChartBase<> can operate on styles, progress tracker, page and reference data specific to the
  * given chart type.
  * 'Ctx' must be a descendant of ContextBase<>, 'Prog' must be a descendant of ProgressBase,
- * 'RefData' must be a descendant of RefNameData, 
+ * 'RefData' must be a descendant of RefNameData,
  * and 'Pagedata' must be a descendant of PageBreakData. By supplying your own classes
  * for these you can customize the pagination, referencing and/or style behaviour
  * (Context is really a set of active styles).
@@ -420,7 +430,7 @@ public:
     std::list<Ctx>                Contexts;        ///<A stack of ContextBase objects used during parsing
     std::map<string, Ctx>         Designs;         ///<A set of named designs. Each one was created via a `defdesign` command, except `plain`.
     std::map<string, RefData>     ReferenceNames;  ///<A set of named arcs. Each one was named via the 'refname' attribute.
-    std::vector<PageData>         pageBreakData;   ///<Starting xy pos and (later) other info for each page. pageBreakData[0].xy is always 0,0. 
+    std::vector<PageData>         pageBreakData;   ///<Starting xy pos and (later) other info for each page. pageBreakData[0].xy is always 0,0.
     /** @} */
 
     virtual ProgressBase *GetProgress() override { return &Progress; }
@@ -441,17 +451,17 @@ public:
     /** Push a new, fresh context to the context stack.
      * @param [in] l The line of definition for the new context. The location of the first '{', e.g.;
      * @param [in] p The parse mode of the new context.
-     * @param [in] t CLEAR will result in a totally empty context. 
-     *               EMPTY will have all default styles (but those empty). 
+     * @param [in] t CLEAR will result in a totally empty context.
+     *               EMPTY will have all default styles (but those empty).
      *               PLAIN will have the plain default design.*/
-    void PushContext(const FileLineCol &l, EContextParse p, EContextCreate t) 
+    void PushContext(const FileLineCol &l, EContextParse p, EContextCreate t)
         { Contexts.emplace_back(t!=EContextCreate::CLEAR, p, t, l); }
     /** Duplicates the last context on the stack, potentially changing the parse mode.
      * Used when switching to the not selected branch of an ifthenelse or replaying a procedure.*/
-    void PushContext(const FileLineCol &l, EContextParse p) 
+    void PushContext(const FileLineCol &l, EContextParse p)
         { auto &last = Contexts.back(); Contexts.emplace_back(last, p, l); }
     /** Duplicates the last context on the stack. Used when a user opens a new context via a '{'.*/
-    void PushContext(const FileLineCol &l) 
+    void PushContext(const FileLineCol &l)
         { auto &last = Contexts.back(); Contexts.emplace_back(last, last.GetParseMode(), l); }
     const Procedure *GetProcedure(const std::string &name) const override;
     ProcParamResolved *GetParameter(const std::string &name) override;
@@ -461,6 +471,8 @@ public:
     void PopContext();
     /** @} */
 
+    /** Sets the font in all contexts and all full designs. */
+    void SetDefaultFont(const std::string& face, const std::string& lang, const FileLineColRange& l) override;
     /** Adds an attribute list to each of a list of styles.*/
     void AddAttributeListToStyleList(const AttributeList &al, const std::list<std::string> &styles);
     /** Adds an attribute list to each of a list of styles.*/
@@ -524,7 +536,7 @@ void ChartBase<Ctx, Prog, RefData, PageData>::ImportDesigns(const ContextPtrCata
             Designs.emplace(d.first, *p);
     }
 }
-                                                                        
+
 template <class Ctx, class Prog, class RefData, class PageData>
 const RefNameData *ChartBase<Ctx, Prog, RefData, PageData>::GetReference(const std::string &refname) const
 {
@@ -548,11 +560,11 @@ const Procedure *ChartBase<Ctx, Prog, RefData, PageData>::GetProcedure(const std
 }
 
 template <class Ctx, class Prog, class RefData, class PageData>
-ProcParamResolved* ChartBase<Ctx, Prog, RefData, PageData>::GetParameter(const std::string & name) 
+ProcParamResolved* ChartBase<Ctx, Prog, RefData, PageData>::GetParameter(const std::string & name)
 {
-    //We accept a parameter with the searched name only if we are 
+    //We accept a parameter with the searched name only if we are
     //parsing or replaying a procedure. Else only variables play.
-    bool accept_params = SkipContent() || Reparsing(); 
+    bool accept_params = SkipContent() || Reparsing();
     for (auto i = Contexts.rbegin(); !(i==Contexts.rend()); i++) {
         auto p = i->parameters.find(name);
         if (p!=i->parameters.end()) {
@@ -606,6 +618,27 @@ inline void ChartBase<Ctx, Prog, RefData, PageData>::PopContext()
     Contexts.pop_back(); //auto-deletes the context via shared_ptr
 }
 
+template<class Ctx, class Prog, class RefData, class PageData>
+inline void ChartBase<Ctx, Prog, RefData, PageData>::SetDefaultFont(const std::string& face, const std::string& lang,
+                                                                    const FileLineColRange& l) {
+    //We need to set the font in the current context, but also in all full designs
+    //(even if setting it) - so that if the design is used later, we have this font
+    if (face.length()) {
+        for (Ctx& context : Contexts)
+            context.text.face = face;
+        for (auto& [name, design] : Designs)
+            if (design.IsFull())
+                design.text.face = face;
+    }
+    if (lang.length()) {
+        for (Ctx& context : Contexts)
+            context.text.lang = face;
+        for (auto& [name, design] : Designs)
+            if (design.IsFull())
+                design.text.lang = face;
+    }
+}
+
 template <class Ctx, class Prog, class RefData, class PageData>
 void ChartBase<Ctx, Prog, RefData, PageData>::AddAttributeListToStyleList(
         const AttributeList &al, const std::list<std::string> &sl) {
@@ -659,7 +692,7 @@ void ChartBase<Ctx, Prog, RefData, PageData>::DrawPageBreaks(Canvas &canvas)
     //last page is degenerate - otherwise single page
     _ASSERT(pageBreakData.back().wh.x && pageBreakData.back().wh.y);
     if (pageBreakData.back().wh.x == 0 || pageBreakData.back().wh.y == 0)
-        if (pageBreakData.size()==2)  
+        if (pageBreakData.size()==2)
             return;
     LineAttr line;
     StringFormat format;
@@ -669,18 +702,21 @@ void ChartBase<Ctx, Prog, RefData, PageData>::DrawPageBreaks(Canvas &canvas)
     double last_y = DBL_MIN;
     for (unsigned page = 1; page<pageBreakData.size(); page++) {
         //if this is the last page and it is of zero size, we skip drawing a page end for it.
-        if (page == pageBreakData.size()-1 && 
+        if (page == pageBreakData.size()-1 &&
               (pageBreakData[page].wh.x==0 || pageBreakData[page].wh.y==0))
             break;
         char text[20];
         const double y = pageBreakData[page].xy.y;
-        line.type.value = pageBreakData[page].manual ? ELineType::DASHED : ELineType::DOTTED;
-        canvas.Line(XY(total.x.from, y), XY(total.x.till, y), line);
+        line.type = pageBreakData[page].manual ? ELineType::DASHED : ELineType::DOTTED;
+        if (canvas.does_graphics())
+            canvas.Line(XY(total.x.from, y), XY(total.x.till, y), line);
+        else
+            canvas.Add(GSPath({ XY(total.x.from, y), XY(total.x.till, y) }, line));
         snprintf(text, sizeof(text), "page %u", page);
         label.Set(text, canvas, Shapes, format);
         const double ylabelpos = y-label.getTextWidthHeight().y;
         if (last_y<ylabelpos)
-            //dont draw page number for very short (likely zero-height) pages 
+            //dont draw page number for very short (likely zero-height) pages
             label.Draw(canvas, Shapes, total.x.from, total.x.till, ylabelpos);
         last_y = y;
     }
@@ -688,7 +724,7 @@ void ChartBase<Ctx, Prog, RefData, PageData>::DrawPageBreaks(Canvas &canvas)
 
 
 
-/** Override this to draw the page header and footer (and lefting and righting). 
+/** Override this to draw the page header and footer (and lefting and righting).
  * The default implementation of ChartBase:: just draws the copyright text.*/
 template <class Ctx, class Prog, class RefData, class PageData>
 void ChartBase<Ctx, Prog, RefData, PageData>::DrawHeaderFooter(Canvas &canvas, unsigned page)
@@ -706,7 +742,12 @@ void ChartBase<Ctx, Prog, RefData, PageData>::DrawHeaderFooter(Canvas &canvas, u
 
     //Draw background for copyright text. Calculate best fill.
     //-1 here makes the fill connect to the chart background without any gap. (Hack I know.)
-    canvas.Fill(Block(xr, {y-1, y+copyrightTextHeight}), GetBkFill({y, y+copyrightTextHeight}, false));
+    const Block B(xr, { y-1, y+copyrightTextHeight });
+    const FillAttr f = GetBkFill({ y, y+copyrightTextHeight }, false);
+    if (canvas.does_graphics())
+        canvas.Fill(B, f);
+    else
+        canvas.Add(GSBox(B, LineAttr::None(), f));
 
     //Draw Copyright Text
     StringFormat sf; sf.Default();
@@ -717,7 +758,7 @@ void ChartBase<Ctx, Prog, RefData, PageData>::DrawHeaderFooter(Canvas &canvas, u
 template<class Ctx, class Prog, class RefData, class PageData>
 inline void ChartBase<Ctx, Prog, RefData, PageData>::SetToEmpty()
 {
-    Progress.Reset(); 
+    Progress.Reset();
     Contexts.clear();
     Designs.clear();
     ReferenceNames.clear();
