@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -71,20 +71,35 @@ struct RayAngle {
      * 0 is used for straight edges, positive values if the edge curves toward
      * clockwise, larger positive values the smaller the turn radius is.
      */
-    double curve;
-    RayAngle() = default;
-    constexpr explicit RayAngle(double a, double b=0) : angle(a), curve(b) {}
-    constexpr bool ExactEqual(const RayAngle&o) const { return angle==o.angle && curve == o.curve; }  ///<Compares two RayAngles
-    constexpr bool IsSimilar(const RayAngle&o) const {return test_equal(angle, o.angle) && test_equal(curve, o.curve);}  ///<Compares two RayAngles
-    constexpr bool Smaller(const RayAngle&o) const {return test_equal(angle, o.angle) ? test_smaller(curve, o.curve) : angle < o.angle;} ///<True we are smaller than `o`, that is, `o` is clockwise after us. We do not define operator < since this is not strict, but approximate.
-    constexpr RayAngle Opposite() const { return RayAngle(fmod_negative_safe(angle+4, 4.), -curve); }
+    double curve = 0;
+    constexpr bool ExactEqual(const RayAngle&o) const noexcept { return angle==o.angle && curve == o.curve; }  ///<Compares two RayAngles
+    constexpr bool IsSimilar(const RayAngle& o) const noexcept { return test_equal(angle, o.angle) && test_equal(curve, o.curve); }  ///<Compares two RayAngles
+    constexpr bool IsABitSimilar(const RayAngle& o) const noexcept { return -0.1<angle-o.angle && angle-o.angle<0.1; }  ///<Compares two RayAngles a bit looser
+    constexpr bool Smaller(const RayAngle&o) const noexcept {return test_equal(angle, o.angle) ? test_smaller(curve, o.curve) : angle < o.angle;} ///<True we are smaller than `o`, that is, `o` is clockwise after us. We do not define operator < since this is not strict, but approximate.
+    constexpr RayAngle Opposite() const noexcept { return RayAngle(fmod_negative_safe(angle+2, 4.), -curve); }
+    /** Is a1 closer to a2 than b1 to b2?
+     * We can go clockwise from a1->a2 / b1->b2 or counterclockwise,
+     * we return true if the angle to cover is smaller for a1->a2.*/
+    constexpr static bool IsCloserAbs(RayAngle a1, RayAngle a2, RayAngle b1, RayAngle b2) noexcept {
+        //note: we do this manual logic as fmod_negative_safe makes tiny mistakes that
+        //may be relevant around angle==3.9999999999999996
+        _ASSERT(0<=a1.angle && a1.angle<4);
+        _ASSERT(0<=a2.angle && a2.angle<4);
+        _ASSERT(0<=b1.angle && b1.angle<4);
+        _ASSERT(0<=b2.angle && b2.angle<4);
+        if (std::pair(a1.angle, a1.curve)>std::pair(a2.angle, a2.curve)) std::swap(a1, a2);
+        if (std::pair(b1.angle, b1.curve)>std::pair(b2.angle, b2.curve)) std::swap(b1, b2);
+        return std::pair(a2.angle-a1.angle, a2.curve-a1.curve)
+             < std::pair(b2.angle-b1.angle, b2.curve-b1.curve);
+    }
 };
-RayAngle Avg(const RayAngle &a1, const RayAngle &a2);
+RayAngle Avg(const RayAngle &a1, const RayAngle &a2) noexcept;
 
 /** Test if q is on the a->b circular section.
 * If the section is warped, that is b < a, we test if q is outside (b,a).
-* If q is almost equal to a or b, we return false */
-inline bool really_between_warp(const RayAngle &q, const RayAngle &a, const RayAngle &b) {
+* If q is almost equal to a or b, we return false
+* If a is almost equal to b the return value is quite undecided.*/
+constexpr bool really_between_warp(const RayAngle &q, const RayAngle &a, const RayAngle &b) noexcept {
     if (a.Smaller(b)) return q.Smaller(b) && !q.Smaller(a);
     return a.Smaller(q) || q.Smaller(b);
 }
@@ -251,7 +266,7 @@ constexpr double get_pos_on_segment(const XY &A, const XY &B, const XY &r)
 
 /** Returns the point on position 't' of the section A->B.
  * 't' must be [0..1], 0 returns 'A', 1 returns 'B', 0.5 returns their midpoint.*/
-constexpr XY Mid(const XY &A, const XY &B, double t) noexcept { return A+t*(B-A); }
+constexpr XY Mid(const XY &A, const XY &B, double t = 0.5) noexcept { return A+t*(B-A); }
 
 
 /** Calculates the distance of point 'M' from the infinite line of 'A'->'B'.
@@ -285,6 +300,16 @@ constexpr bool ApplyValueUpdate(bool b, EBinaryValueUpdate v) noexcept { return
  * if v==SET, we return 'value', if INVERT and b==0, we return 1, not 'value'*/
 template <typename uint> requires(std::is_integral_v<uint> && !std::is_signed_v<uint>)
 constexpr uint ApplyValueUpdate(uint b, EBinaryValueUpdate v, uint value) noexcept { return v==EBinaryValueUpdate::DONT_CHANGE ? b : v==EBinaryValueUpdate::INVERT ? b==0 : v==EBinaryValueUpdate::SET ? value : 0; }
+/** Checks if the value update matches the current value. 
+ * If DONT_CHANGE or INVERT, we return true. */
+constexpr bool MatchValueUpdate(bool b, EBinaryValueUpdate v) noexcept { return v==EBinaryValueUpdate::DONT_CHANGE || v==EBinaryValueUpdate::INVERT || (v==EBinaryValueUpdate::SET && b) || (v==EBinaryValueUpdate::CLEAR && !b); } 
+/** Checks if the value update matches the current value. 
+ * If DONT_CHANGE or INVERT, we return true. */
+template <typename uint> requires(std::is_integral_v<uint> && !std::is_signed_v<uint>)
+constexpr bool MatchValueUpdate(uint b, EBinaryValueUpdate v, uint value) noexcept { return v==EBinaryValueUpdate::DONT_CHANGE || v==EBinaryValueUpdate::INVERT || (v==EBinaryValueUpdate::SET && b==value) || (v==EBinaryValueUpdate::CLEAR && b==0); } 
+/** DONT_CHANGE and INVERT remains, but SET and CLEAR are flipped.*/
+constexpr EBinaryValueUpdate Invert(EBinaryValueUpdate v) noexcept { return v == EBinaryValueUpdate::DONT_CHANGE || v == EBinaryValueUpdate::INVERT ? v : v == EBinaryValueUpdate::SET ? EBinaryValueUpdate::CLEAR : EBinaryValueUpdate::SET; }
+
 
 template <unsigned num_cp>
 struct EdgeCPsData
@@ -364,7 +389,7 @@ public:
          * Note the edge direction!
          * for clarification you can see the whole shape to the right in small. The edges in question
          * are in dots there.
-         * 
+         *
           @verbatim
             ..<.....+
                 |   :
@@ -373,7 +398,7 @@ public:
                 |   :         +...+   |
                 |   :             :   |
                 |   :             +---+
-          
+
           @endverbatim */
         CP_REAL,
         /** They do not have a crosspoint, but if we extend them, they have
@@ -383,7 +408,7 @@ public:
          *
          * Below see two edges with dots, the expanded edges with solid lines and their extension
          * also using dots. The `o` marks the crosspoint of the extensions.
-         * 
+         *
           @verbatim
              -->--...o
                      :
@@ -392,7 +417,7 @@ public:
                  :   |
                  :   |
                  :   |
-          
+
           @endverbatim */
         CP_EXTENDED,
         /** They are parallel and hence have no crosspoints.
@@ -406,16 +431,16 @@ public:
          * point in between the two edges, currently 5 times further than the distance
          * between the two `x`s. See `o` below. * `x` shows the end of the expanded edges
          * and solid lines show linear extensions.
-         * 
+         *
           @verbatim
              ----->-x------------------
-         
+
               ..>...+                 o
                    :
                   :      -x----------------
                  :      /
                  :      |
-          
+
           @endverbatim */
         NO_CP_PARALLEL,
         /** The two edges do not meet, but their extension do - but at the opposite end
@@ -447,30 +472,43 @@ public:
     /** True if two expanded edges or their extrension has crosspoints.*/
     static constexpr bool HasCP(EExpandCPType t) { return t==CP_REAL || t==CP_EXTENDED || t==CP_TRIVIAL; }
     static constexpr unsigned MAX_CP = 9; ///<The maximum number of crosspoints two Edges may have.
-    static constexpr unsigned MARK_BIT_WIDTH = 1; ///<How many bits the mark field of an edge holds
+    static constexpr unsigned MARK_BIT_WIDTH = 24; ///<How many bits the mark field of an edge holds
 
     /** A structure containing instructions to change the mutable status of an edge*/
-    struct Update
+    class Update
     {
-        EBinaryValueUpdate visible : 8;      ///<How to change the visibility of the edge
-        EBinaryValueUpdate mark : 8;         ///<How to change the marking of the edge. If SET we use mark_value, if INVERT we set to 0 or 1.
-        unsigned mark_value : MARK_BIT_WIDTH;///<This value is used when mark==SET
-        constexpr explicit Update(EBinaryValueUpdate v=EBinaryValueUpdate::DONT_CHANGE, EBinaryValueUpdate m=EBinaryValueUpdate::DONT_CHANGE, unsigned mv=1) :
-            visible(v), mark(m), mark_value(mv) {}
-        constexpr bool DoesSomething() const { return visible!=EBinaryValueUpdate::DONT_CHANGE || mark !=EBinaryValueUpdate::DONT_CHANGE; }
+        unsigned visible_ : 2;  ///<How to change the visibility of the edge
+        unsigned imark_ : 2;    ///<The mark bit we use internally in contour
+        unsigned mark_ : 2;     ///<How to change the marking of the edge. If SET we use mark_value, if INVERT we set to 0 or 1.
+        unsigned mark_value_ : MARK_BIT_WIDTH;///<This value is used when mark==SET
+    public:
+        constexpr Update(EBinaryValueUpdate v = EBinaryValueUpdate::DONT_CHANGE, EBinaryValueUpdate im = EBinaryValueUpdate::DONT_CHANGE,
+                         EBinaryValueUpdate m = EBinaryValueUpdate::DONT_CHANGE, unsigned mv=1) :
+            visible_(unsigned(v)), imark_(unsigned(im)), mark_(unsigned(m)), mark_value_(mv) {}
+        constexpr EBinaryValueUpdate visible() const noexcept { return EBinaryValueUpdate(visible_); }
+        constexpr EBinaryValueUpdate internal_mark() const noexcept { return EBinaryValueUpdate(imark_); }
+        constexpr EBinaryValueUpdate mark() const noexcept { return EBinaryValueUpdate(mark_); }
+        constexpr unsigned mark_value() const noexcept { return mark_value_; }
+        constexpr bool DoesSomething() const { return visible()!=EBinaryValueUpdate::DONT_CHANGE || internal_mark() != EBinaryValueUpdate::DONT_CHANGE || mark()!=EBinaryValueUpdate::DONT_CHANGE; }
         constexpr static Update set_visible() { return Update(EBinaryValueUpdate::SET); }
         constexpr static Update clear_visible() { return Update(EBinaryValueUpdate::CLEAR); }
-        constexpr static Update set_mark(unsigned v=1) { return Update(EBinaryValueUpdate::DONT_CHANGE, EBinaryValueUpdate::SET, v); }
-        constexpr static Update clear_mark() { return Update(EBinaryValueUpdate::DONT_CHANGE, EBinaryValueUpdate::CLEAR); }
-        constexpr Update operator +(Update u) const { return Update(visible+u.visible, mark+u.mark, u.mark_value); }
+        constexpr static Update set_mark(unsigned v) { return Update(EBinaryValueUpdate::DONT_CHANGE, EBinaryValueUpdate::DONT_CHANGE, EBinaryValueUpdate::SET, v); }
+        constexpr static Update clear_mark() { return Update(EBinaryValueUpdate::DONT_CHANGE, EBinaryValueUpdate::DONT_CHANGE, EBinaryValueUpdate::CLEAR); }
+        constexpr static Update set_internal_mark() { return Update(EBinaryValueUpdate::DONT_CHANGE, EBinaryValueUpdate::SET); }
+        constexpr static Update clear_internal_mark() { return Update(EBinaryValueUpdate::DONT_CHANGE, EBinaryValueUpdate::CLEAR); }
+        constexpr Update operator +(Update u) const { return Update(visible()+u.visible(), internal_mark()+u.internal_mark(), mark()+u.mark(), u.mark_value()); }
+        constexpr bool operator ==(const Update& o) const noexcept {
+            return visible_==o.visible_ && imark_==o.imark_ && mark_==o.mark_ && (mark()==EBinaryValueUpdate::DONT_CHANGE || mark_value_==o.mark_value_);
+        }
+        constexpr Update Invert() const noexcept { return Update(contour::Invert(visible()), contour::Invert(internal_mark()), contour::Invert(mark()), mark_value()); }
     };
 
     /** Contsruct a straight edge. */
-    constexpr Edge(const XY &A, const XY &B, bool v = true, bool m = false) noexcept
-        : mark(m), visible(v), straight(true), start(A), end(B) {}
+    constexpr Edge(const XY &A, const XY &B, bool v = true, bool im = false) noexcept
+        : imark_(im), visible_(v), straight(true), start(A), end(B) {}
     /** Construct a cubic bezier. Will be converted to straight line if too flat. */
-    Edge(const XY &A, const XY &B, const XY &C, const XY &D, bool v = true, bool m = false) noexcept
-        : mark(m), visible(v), straight(false), start(A), end(B), c1(C), c2(D) { MakeStraightIfNeeded(); }
+    Edge(const XY &A, const XY &B, const XY &C, const XY &D, bool v = true, unsigned m = 0) noexcept
+        : imark_(m), visible_(v), straight(false), start(A), end(B), c1(C), c2(D) { MakeStraightIfNeeded(); }
     constexpr Edge() noexcept = default;
     constexpr Edge(const Edge &) noexcept = default;
     /** Construct the Edge as a section of another one.
@@ -499,7 +537,17 @@ public:
     constexpr bool IsVertical() const noexcept { return straight && start.x == end.x; }
     /** Returns the point at position 'pos' on the Edge. 'pos' must be in [0..1]*/
     constexpr XY Pos2Point(double pos) const noexcept { return pos==0 ? start : pos==1 ? end : straight ? Mid(start, end, pos) : Split(pos); }
-    constexpr void Apply(Update u) const noexcept { visible = ApplyValueUpdate(visible, u.visible); mark = ApplyValueUpdate(mark, u.mark, u.mark_value); }
+    constexpr void Apply(Update u) const noexcept {
+        visible_ = ApplyValueUpdate(visible_, u.visible());
+        imark_ = ApplyValueUpdate(imark_, u.internal_mark());
+        mark_ = ApplyValueUpdate(mark_, u.mark(), u.mark_value());
+    }
+    void SetVisible(bool visible = true) const noexcept { visible_ = visible; }
+    void SetInternalMark(bool imark = true) const noexcept { imark_ = imark; }
+    void SetMark(unsigned mark) const noexcept { mark_ = mark; }
+    bool IsVisible() const noexcept { return visible_; }
+    bool IsInternalMarked() const noexcept { return imark_; }
+    unsigned GetMark() const noexcept { return mark_; }
 
     /** Returns the point at position 'pos' of a bezier curve. 'pos' must be in [0..1]*/
     constexpr XY Split(double t) const noexcept {
@@ -518,6 +566,8 @@ public:
     int WhichSide(const XY &A, const XY &B) const noexcept;
     bool MakeStraightIfNeeded(double maximum_distance_sqr = 0.001, bool keep_cp = false) noexcept;
     void Still() noexcept;
+    bool Match(const XY& p, double pos = -1, double tolerance = 0.1) const noexcept;
+    constexpr bool Match(Update u) const noexcept { return MatchValueUpdate(visible_, u.visible()) && MatchValueUpdate(imark_, u.internal_mark()) && MatchValueUpdate(mark_, u.mark(), u.mark_value()); }
 
     Edge& SetStart(double pos) noexcept { return SetStart(Pos2Point(pos), pos); }
     Edge& SetStart(const XY &p, double pos) noexcept;
@@ -533,20 +583,20 @@ public:
     Edge& SetC1(const XY &C1) noexcept;
     Edge& SetC2(const XY &C2) noexcept;
 
-    Edge &Shift(const XY&p) noexcept { start += p; end += p; if (!straight) { c1 += p; c2 += p; } return *this; }  ///<Shift the egde
+    Edge &Shift(const XY&p) noexcept { start += p; end += p; if (!ignore_cp()) { c1 += p; c2 += p; } return *this; }  ///<Shift the egde
     Edge CreateShifted(const XY&p) const noexcept { return Edge(*this).Shift(p); } ///<Create a shifted version of the edge
-    Edge &Scale(double sc) noexcept { start *= sc; end *= sc; if (!straight) { c1 *= sc; c2 *= sc; } return *this; }    ///<Scale the edge.
+    Edge &Scale(double sc) noexcept { start *= sc; end *= sc; if (!ignore_cp()) { c1 *= sc; c2 *= sc; } return *this; }    ///<Scale the edge.
     Edge CreateScaled(double sc) const noexcept { return Edge(*this).Scale(sc); }    ///<Scale the edge.
-    Edge &Scale(const XY &sc) noexcept { start.Scale(sc); end.Scale(sc); if (!straight) { c1.Scale(sc); c2.Scale(sc); } return *this; }    ///<Scale the edge.
+    Edge &Scale(const XY &sc) noexcept { start.Scale(sc); end.Scale(sc); if (!ignore_cp()) { c1.Scale(sc); c2.Scale(sc); } return *this; }    ///<Scale the edge.
     Edge CreateScaled(const XY &sc) const noexcept { return Edge(*this).Scale(sc); }    ///<Scale the edge.
-    Edge &Rotate(double cos, double sin) noexcept { start.Rotate(cos, sin); end.Rotate(cos, sin); if (!straight) { c1.Rotate(cos, sin); c2.Rotate(cos, sin); } return *this; } ///<Rotates the edge around the origin
+    Edge &Rotate(double cos, double sin) noexcept { start.Rotate(cos, sin); end.Rotate(cos, sin); if (!ignore_cp()) { c1.Rotate(cos, sin); c2.Rotate(cos, sin); } return *this; } ///<Rotates the edge around the origin
     Edge CreateRotated(double cos, double sin) const noexcept { return Edge(*this).Rotate(cos, sin); } ///<Creates a rotated version of the edge
-    Edge &RotateAround(const XY&c, double cos, double sin) noexcept { start.RotateAround(c, cos, sin); end.RotateAround(c, cos, sin); if (!straight) { c1.RotateAround(c, cos, sin); c2.RotateAround(c, cos, sin); } return *this; } ///<Rotates the edge around a point
+    Edge &RotateAround(const XY&c, double cos, double sin) noexcept { start.RotateAround(c, cos, sin); end.RotateAround(c, cos, sin); if (!ignore_cp()) { c1.RotateAround(c, cos, sin); c2.RotateAround(c, cos, sin); } return *this; } ///<Rotates the edge around a point
     Edge CreateRotatedAround(const XY&c, double cos, double sin) const noexcept { return Edge(*this).RotateAround(c, cos, sin); } ///<Creates a version of the edge rotated around a point
-    Edge &SwapXY() noexcept { start.SwapXY(); end.SwapXY(); if (!straight) { c1.SwapXY(); c2.SwapXY(); } return *this; } ///<Swaps XY coordinates
+    Edge &SwapXY() noexcept { start.SwapXY(); end.SwapXY(); if (!ignore_cp()) { c1.SwapXY(); c2.SwapXY(); } return *this; } ///<Swaps XY coordinates
     Edge CreateSwapXYd() const noexcept { return Edge(*this).SwapXY(); } ///<Created a version of the edge with swapped x and y coordinates.
-    Edge &Invert() noexcept { std::swap(start, end); if (!straight) std::swap(c1, c2); return *this; } ///<Reverses the direction of the edge effectively swapping start and end.
-    Edge &Apply(const TRMatrix &M) noexcept { start.Apply(M); end.Apply(M); if (!straight) { c1.Apply(M); c2.Apply(M); } return *this; } ///Apply a transformation
+    Edge &Invert() noexcept { std::swap(start, end); if (!ignore_cp()) std::swap(c1, c2); return *this; } ///<Reverses the direction of the edge effectively swapping start and end.
+    Edge &Apply(const TRMatrix &M) noexcept { start.Apply(M); end.Apply(M); if (!ignore_cp()) { c1.Apply(M); c2.Apply(M); } return *this; } ///Apply a transformation
 
     /** Returns a point on the line of a tangent at "pos", the point being towards the start of curve/edge.*/
     XY PrevTangentPoint(double pos) const noexcept { return straight ? 2*start-end : pos ? Mid(Mid(start, c1, pos), Mid(c1, c2, pos), pos) : start.test_equal(c1) ? 2*start-c2 : 2*start-c1; }
@@ -629,7 +679,7 @@ public:
     { return atXorY<true>(y, roots, precision); }
 
     //helpers
-    std::pair<double, double> UnboundedBezierDistance(const XY & M) const;    
+    std::pair<double, double> UnboundedBezierDistance(const XY & M) const;
     bool HullOverlap(const Edge &, bool is_next) const;
     double HullDistance(const XY &A, const XY &B) const;
     double HullDistance(const XY &A, const XY &B, const XY &C, const XY &D) const;
@@ -670,20 +720,22 @@ public:
     static bool RemoveLoops(std::vector<E> &edges, bool self=true);
     bool CreateExpandOneSegment(double gap, std::list<Edge> &expanded, std::vector<Edge> *original) const;
     bool CreateLinearExpandOneSegment(double gap1, double gap2, double cos, double sin, std::list<Edge> &expanded, std::vector<Edge> *original) const;
+    /** Prints the content of the edge into a string as a
+     * valid C++ ctor call.
+     * @param [in] precise When set, we emit the hex representation of doubles, else a human readable one.*/
+    std::string Dump(bool precise) const;
 
-    std::string Dump() const;
-
-    //members
-    mutable unsigned mark : MARK_BIT_WIDTH; ///<Marker used by Path::DoubleWiden() and others. It is generally preserved and carried over during path or contour operations to the resulting edge
-    mutable bool     visible : 1;           ///<True if the edge is visible.
 private:
     constexpr bool ignore_cp() const noexcept { return straight && !has_cp; }
-    bool straight : 1;  ///<True if the Edge is a straight line (members c1 and c2 are still valid, but lie very close to start->end), false if a cubic bezier.
+    mutable unsigned mark_ : MARK_BIT_WIDTH = 0; ///<Mark value to be used by the user. It is generally preserved and carried over during path or contour operations to the resulting edge
+    mutable unsigned imark_ : 1;                 ///<Marker used by Path::DoubleWiden() and others. It is generally preserved and carried over during path or contour operations to the resulting edge
+    mutable unsigned visible_ : 1;               ///<True if the edge is visible.
+    unsigned straight : 1;                       ///<True if the Edge is a straight line (members c1 and c2 are still valid, but lie very close to start->end), false if a cubic bezier.
     /** Relevant only if straight.Used when we have just flattened an edge during MovePos andPos2Point.
      * In this case the control points are relevant so that a parameter<->length conversions are more accurate.
-     * This property is used only during such conversion and is cleared every time we change any of start/end, 
+     * This property is used only during such conversion and is cleared every time we change any of start/end,
      * since we can no longer cater for the control points being valid. */
-    bool has_cp : 1 = false;    
+    unsigned has_cp : 1 = false;
     XY start; ///<The startpoint of the Edge (first control point for cubic beziers)
     XY end;   ///<The endpoint of the Edge (fourth control point for cubic beziers)
     XY c1;    ///<If a cubic bezier, then the second control point - the one associated with 'start'. If a straight line, this is equal to start or on the start->end line.
@@ -695,14 +747,15 @@ private:
  *                                  If the actual distance is smaller than this, we switch to straight.
  * @param [in] keep_cp If set and we make the edge straight, we keep the control points (and set has_cp),
  *                     so that we can more accurately compute length->position.
- * @returns true if we have switched from bezier to straight line.*/
+ * @returns true if we have switched from bezier to straight line. False if we started with a straight line.*/
 inline bool Edge::MakeStraightIfNeeded(double maximum_distance_sqr, bool keep_cp) noexcept
 {
     _ASSERT(!std::isnan(start.x) && !std::isnan(start.y));
     _ASSERT(!std::isnan(end.x) && !std::isnan(end.y));
-    _ASSERT(!std::isnan(c1.x) && !std::isnan(c1.y));
-    _ASSERT(!std::isnan(c2.x) && !std::isnan(c2.y));
-
+    if (has_cp) {
+        _ASSERT(!std::isnan(c1.x) && !std::isnan(c1.y));
+        _ASSERT(!std::isnan(c2.x) && !std::isnan(c2.y));
+    }
     //see http://antigrain.com/research/adaptive_bezier/
     if (straight) return false;
     const double dx = end.x-start.x;
@@ -734,6 +787,23 @@ inline void Edge::Still() noexcept
     }
 }
 
+/** Checks whether 'p' is on the edge and matches 'pos'
+ * well enough.
+ * @param [in] p The point supposedly on the edge
+ * @param [in] pos The position parameter to check. Ignored if outside [0..1]
+ *             In that case only the distance of 'p' from the edge is checked.
+ * @param [in] Tolerance in terms of pixel distnce. We fail if 'p' is further
+ *             away from the edge than this or if the point of the edge
+ *             corresponding to 'pos' is further from 'p' than this. */
+inline bool Edge::Match(const XY& p, double pos, double tolerance) const noexcept {
+    XY closest_on_edge;
+    double pos_on_edge;
+    //test that p and pos correspond - compiles to NOP in release mode
+    return fabs(Distance(p, closest_on_edge, pos_on_edge))<tolerance    //'p' is close to the edge; and
+        && (!between01(pos)                                             //..either 'pos' is outside [0..1]; or
+            || Pos2Point(pos).DistanceSqr(p)<tolerance*tolerance);      //'p' is close to the requested position
+}
+
 
 /** Set the start of the Edge thereby effectively chopping its head away.
  * We assume 'p' and 'pos' correspond to the same point and use the one resulting
@@ -742,25 +812,9 @@ inline void Edge::Still() noexcept
  * caller context (as it is not used) due to inlining.*/
 inline Edge& Edge::SetStart(const XY &p, double pos) noexcept
 {
-    if (!straight) {
-#ifdef _DEBUG
-        XY dummy;
-        double t;
-        //test that p and pos correspond - compiles to NOP in release mode
-        _ASSERT(fabs(Distance(p, dummy, t))<0.1);
-        //_ASSERT(fabs(t-pos)<0.001);
-#endif
+    _ASSERT(Match(p, pos));
+    if (!straight)
         Chop(pos, 1);
-    } else {
-        //test that p and pos correspond - compiles to NOP in release mode
-        if (std::max(fabs(start.x-end.x), fabs(start.y-end.y))>0.02) {//does not make sense with very short edges
-            if (fabs(start.x-end.x)<fabs(start.y-end.y))
-                _ASSERT(fabs((p.y-start.y)/(end.y-start.y) - pos)<0.001); 
-            else {
-                _ASSERT(fabs((p.x-start.x)/(end.x-start.x) - pos)<0.001);
-            }
-        }
-    }
     start = p;
     has_cp = false;
     return *this;
@@ -773,25 +827,9 @@ inline Edge& Edge::SetStart(const XY &p, double pos) noexcept
  * caller context (as it is not used) due to inlining.*/
 inline Edge& Edge::SetEnd(const XY &p, double pos) noexcept
 {
-    if (!straight) {
-#ifdef _DEBUG
-        XY dummy;
-        double t;
-        //test that p and pos correspond - compiles to NOP in release mode
-        _ASSERT(fabs(Distance(p, dummy, t))<0.1);
-        //_ASSERT(fabs(t-pos)<0.001);
-#endif
+    _ASSERT(Match(p, pos));
+    if (!straight)
         Chop(0, pos);
-    } else {
-        //test that p and pos correspond - compiles to NOP in release mode
-        if (std::max(fabs(start.x-end.x),fabs(start.y-end.y))>0.02) {//does not make sense with very short edges
-            if (fabs(start.x-end.x)<fabs(start.y-end.y))
-                _ASSERT(fabs((p.y-start.y)/(end.y-start.y) - pos)<0.001);
-            else {
-                _ASSERT(fabs((p.x-start.x)/(end.x-start.x) - pos)<0.001);
-            }
-        }
-    }
     end = p;
     has_cp = false;
     return *this;
@@ -806,13 +844,7 @@ inline Edge& Edge::SetEnd(const XY &p, double pos) noexcept
 inline Edge& Edge::SetStartIgn(const XY &p, double pos) noexcept
 {
     if (!straight) {
-#ifdef _DEBUG
-        XY dummy;
-        double t;
-        //test that p and pos correspond - compiles to NOP in release mode
-        _ASSERT(fabs(Distance(p, dummy, t))<0.1);
-        _ASSERT(fabs(t-pos)<0.01);
-#endif
+        _ASSERT(Match(p, pos));
         Chop(pos, 1);
     }
     start = p;
@@ -829,13 +861,7 @@ inline Edge& Edge::SetStartIgn(const XY &p, double pos) noexcept
 inline Edge& Edge::SetEndIgn(const XY &p, double pos) noexcept
 {
     if (!straight) {
-#ifdef _DEBUG
-        XY dummy;
-        double t;
-        //test that p and pos correspond - compiles to NOP in release mode
-        _ASSERT(fabs(Distance(p, dummy, t))<0.1);
-        _ASSERT(fabs(t-pos)<0.01);
-#endif
+        _ASSERT(Match(p, pos));
         Chop(0, pos);
     }
     end = p;
@@ -852,15 +878,8 @@ inline Edge& Edge::SetEndIgn(const XY &p, double pos) noexcept
 inline Edge& Edge::SetStartEndIgn(const XY &s, const XY &d, double spos, double dpos) noexcept
 {
     if (!straight) {
-#ifdef _DEBUG
-        XY dummy;
-        double tt, ss;
-        //test that p and pos correspond - compiles to NOP in release mode
-        _ASSERT(fabs(Distance(s, dummy, tt))<0.1);
-        _ASSERT(fabs(Distance(d, dummy, ss))<0.1);
-        _ASSERT(fabs(tt-spos)<0.01);
-        _ASSERT(fabs(ss-dpos)<0.01);
-#endif
+        _ASSERT(Match(s, spos));
+        _ASSERT(Match(d, dpos));
         Chop(spos, dpos);
     }
     start = s;
@@ -875,7 +894,7 @@ inline Edge& Edge::SetStartEndIgn(const XY &s, const XY &d, double spos, double
  * If you want to set only one of the points use SetC1() or SetC2(). */
 inline Edge & Edge::SetC1C2(const XY & C1, const XY & C2) noexcept
 {
-    c1 = C1; 
+    c1 = C1;
     c2 = C2;
     straight = false;
     MakeStraightIfNeeded();
@@ -936,8 +955,8 @@ inline Range Edge::GetHullYRange() const
 template<bool y>
 std::array<std::pair<XY, double>,2> Edge::Extremes() const
 {
-    if (straight) 
-        return (start[y]<end[y]) ? 
+    if (straight)
+        return (start[y]<end[y]) ?
         std::array<std::pair<XY, double>, 2>{std::pair<XY, double>{start, 0.}, std::pair<XY, double>{end, 1.}} :
         std::array<std::pair<XY, double>, 2>{std::pair<XY, double>{ end, 1. }, std::pair<XY, double>{start, 0.}};
 
@@ -974,7 +993,7 @@ std::array<std::pair<XY, double>,2> Edge::Extremes() const
             }
         }
     }
-    //If we have equal x or y values, take the one with the smaller index 
+    //If we have equal x or y values, take the one with the smaller index
     //(more likely to be an endpoint and hence precise)
     std::pair<unsigned, unsigned> X = { 0,0 };
     for (unsigned u = 0; u<xy.size(); u++) {
@@ -983,7 +1002,7 @@ std::array<std::pair<XY, double>,2> Edge::Extremes() const
         if (xy[u][y]>xy[X.second][y])
             X.second = u;
     }
-    return std::array<std::pair<XY, double>, 2>{ std::pair<XY, double>{ xy[X.first],  pos[X.first] }, 
+    return std::array<std::pair<XY, double>, 2>{ std::pair<XY, double>{ xy[X.first],  pos[X.first] },
                                                  std::pair<XY, double>{ xy[X.second], pos[X.second] }};
 }
 
@@ -1180,7 +1199,7 @@ inline EdgeCPsDataWithXY<Edge::MAX_CP> Edge::Crossing(const Edge &A, bool is_nex
  * @returns A number of crosspoints and two static arrays, the first for the positions
  * of the crosspoints on 'this' the second for the positions on 'A'->'B'.
  * We do not filter the result by position values falling outside [0,1] for pos_other.
- * In contrast all values of pos_my will be inside [0,1] pos_my. 
+ * In contrast all values of pos_my will be inside [0,1] pos_my.
  * If the edge is straight and it coincides with the infinit line, we return two crosspoints,
  * with pos_my =0 and =1.*/
 inline EdgeCPsData<Edge::MAX_CP> Edge::Crossing(const XY &A, const XY &B) const
@@ -1249,8 +1268,8 @@ std::pair<bool, double> Edge::TangentFromNoInflection(const XY &xy) const
  *     `from` towards the arc touches the ellipse in the clockwise direction.
  * For straight edges:
  *     We return the `start` in both clockwise and counterclockwise.
- *     This is because unless 'include_end' is true we ignore the endpoint - 
- *     then it will be a helper for connected paths and the endpoint will be the 
+ *     This is because unless 'include_end' is true we ignore the endpoint -
+ *     then it will be a helper for connected paths and the endpoint will be the
  *     startpoint of the next edge and will be considered there.
  *     If the 'include_end' template parameter is true, we also include the endpoint.
  * @param [in] from The point from which the tangents are drawn.
@@ -1258,7 +1277,7 @@ std::pair<bool, double> Edge::TangentFromNoInflection(const XY &xy) const
  *               tangent touches the curve.
  */
 template <bool include_end>
-std::pair<std::pair<double, XY>, std::pair<double, XY>> 
+std::pair<std::pair<double, XY>, std::pair<double, XY>>
 Edge::TangentFrom(const XY &from) const
 {
     std::pair<std::pair<double, XY>, std::pair<double, XY>> ret;
@@ -1320,11 +1339,11 @@ Edge::TangentFrom(const XY &from) const
  * Given the two edges, four such tangents can be drawn, here we focus on the two
  * outer ones, the ones that touch either both edges from above or both from below,
  * but not mixed.
- * Note that unless 'include_end' is true, we do not consider the endpoint - 
- * in that case this is a helper for closed contours - where the start next edge 
+ * Note that unless 'include_end' is true, we do not consider the endpoint -
+ * in that case this is a helper for closed contours - where the start next edge
  * will be the same as our endpoint.
  * The clockwise and cclockwise members are considered - we only change them if
- * we find a more clockwise or cclockwise tangent. At start they shall be initialized 
+ * we find a more clockwise or cclockwise tangent. At start they shall be initialized
  * to a point on the curves.
  * @param [in] o The other edge.
  * @param [out] clockwise The points where the clockwise tangent touches us
@@ -1358,9 +1377,9 @@ void Edge::TangentFrom(const Edge &o, XY clockwise[2], XY cclockwise[2]) const
         e1[0] = *this;
     } else switch (num1 = 1+InflectionPoints(infl)) {
     default: _ASSERT(0); FALLTHROUGH;
-    case 1: 
+    case 1:
         e1[0] = *this; break;
-    case 2: 
+    case 2:
         e1[1] = e1[0] = *this;
         e1[0].SetEnd(infl[0]);
         e1[1].SetStart(infl[1]);
