@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -34,16 +34,15 @@
 
 namespace contour {
 
- 
+
 
 
 //Do not create degenerate triangles.
 //Always create clockwise.
 SimpleContour::SimpleContour(XY a, XY b, XY c) :
     clockwise_fresh(true), boundingBox_fresh(true), area_fresh(false),
-    clockwise(true), boundingBox(a, b)
-{
-    switch (triangle_dir(a,b,c)) {
+    clockwise(true), boundingBox(a, b) {
+    switch (triangle_dir(a, b, c)) {
     default: //create empty if degenerate triangle
         boundingBox.MakeInvalid();
         return;
@@ -72,11 +71,10 @@ SimpleContour::SimpleContour(XY a, XY b, XY c) :
  * @param [in] d_deg The endpoint of the arc. If equal to `s_deg` a full ellipse results, else a straight line is added to close the arc.
  * If `radius_x` is zero, we return an empty shape. If `radius_y` is zero, we assume it to be the same as `radius_x` (circle).
 */
-SimpleContour::SimpleContour(const XY &c, double radius_x, double radius_y, double tilt_deg, double s_deg, double d_deg) :
+SimpleContour::SimpleContour(const XY& c, double radius_x, double radius_y, double tilt_deg, double s_deg, double d_deg) :
     edges(c, radius_x, radius_y, tilt_deg, s_deg, d_deg, true, true),
     clockwise_fresh(true), boundingBox_fresh(false), area_fresh(false),
-    clockwise(true)
-{
+    clockwise(true) {
     if (size()==0) {
         area_fresh = true;
         boundingBox_fresh = true;
@@ -87,8 +85,7 @@ SimpleContour::SimpleContour(const XY &c, double radius_x, double radius_y, doub
 }
 
 /** Clear the shape and assign a rectangle shape. */
-SimpleContour &SimpleContour::operator =(const Block &b)
-{
+SimpleContour& SimpleContour::operator =(const Block& b) {
     clear(); //clockwise is set to fresh and true here, bounding box & area is set to fresh
     if (b.IsInvalid() || !test_smaller(b.x.from, b.x.till) || !test_smaller(b.y.from, b.y.till)) return *this;
     edges.push_back(Edge(b.UpperLeft(), XY(b.x.till, b.y.from)));
@@ -101,11 +98,10 @@ SimpleContour &SimpleContour::operator =(const Block &b)
 }
 
 /** Revert the clockwisedness of the shape by reverting, all edges and their order. */
-SimpleContour &SimpleContour::Invert() noexcept 
-{
-    for (size_t i=0; i<size(); i++)
+SimpleContour& SimpleContour::Invert() noexcept {
+    for (size_t i = 0; i<size(); i++)
         at(i).Invert();
-    for (size_t i=0; i<size()/2; i++)
+    for (size_t i = 0; i<size()/2; i++)
         std::swap(at(i), at(size()-1-i));
     area *= -1;
     clockwise = !clockwise;
@@ -121,8 +117,7 @@ SimpleContour &SimpleContour::Invert() noexcept
  * @returns The relation. Note that for counerclockwise 'inside' is the same as if the
             contour were clockwise, that is in the limited space enclosed.
  */
-EPointRelationType SimpleContour::IsWithin(XY p, size_t *edge, double *pos, bool strict) const noexcept
-{
+EPointRelationType SimpleContour::IsWithin(XY p, size_t* edge, double* pos, bool strict) const noexcept {
     if (size()==0 || !GetBoundingBox().IsWithinBool(p)) return WI_OUTSIDE;
 
     //Follow the contour and see how much it crosses the vertical line going through p
@@ -185,98 +180,160 @@ EPointRelationType SimpleContour::IsWithin(XY p, size_t *edge, double *pos, bool
 /** Test if q is on the a->b section.
  * If the section is warped, that is b < a, we test if q is outside (b,a).
  * If q is almost equal to a or b, we return false */
-constexpr bool really_between_warp (double q, double a, double b) noexcept
-{
-    if (a<b) return test_smaller(q,b) && !test_smaller(q,a);
-    return test_smaller(a,q) || test_smaller(q,b);
+constexpr bool really_between_warp(double q, double a, double b) noexcept {
+    if (a<b) return test_smaller(q, b) && !test_smaller(q, a);
+    return test_smaller(a, q) || test_smaller(q, b);
 }
 
 /** A helper for CheckContainment().
- * Can result in SAME, APRT, A_INSIDE_B and these results represent the actual
- * relation between 'this' and 'b'.
- * It can also return OVERLAP, which only means one of our point is outside 'b',
- * either because we are APART or because B_INSIDE_A.
+ * We assume 2 non-empty simple contours that have no crosspoints,
+ * only touchpoints. So they can be exact equals, one can be fully inside the other
+ * or they can be apart.
  * clockwiseness is ignored: the inside of a counterclockwise contour is the limited space
- * it encircles.*/
-EContourRelationType SimpleContour::CheckContainmentHelper(const SimpleContour &b) const
-{
+ * it encircles. We assume none of the contours is empty.
+ * @param [in] b The other contour (the 'B' contour, we are the 'A' contour)
+ * @param [in] threshold How many vertices must be similar (outside or in) before making a verdict.
+ *             If 'this' is smaller than threshold, we lower threshold to the size of 'this'
+ * @param [in] allow How many vertices may be different from the majority to still not return inconclusive.
+ *             Must be lower than threshold. If not, we reduce it to threshold-1.
+ * @returns one of the following values
+ * - REL_SAME: The two contours are (almost) the same
+ * - REL_OVERLAP: One of our vertices is outside 'b', we are either APART of B_INSIDE_A
+ * - REL_A_INSIDE_B: One of our vertices is inside 'b', we are A_INSIDE_B.
+ * - REL_IN_HOLE_APART: The relation could not have been decided (may be actual overlap)
+ *
+ * No other value will be returned.
+ */
+EContourRelationType SimpleContour::CheckContainmentHelper(const SimpleContour& b, size_t threshold, size_t allow) const {
     size_t edge;
     _ASSERT(!b.IsEmpty() && !IsEmpty());
-    const size_t max = size();
-    for (size_t i=0; i<max; i++) {
-        double pos;
-        const XY p = at(i).GetStart();
-        //Optimize: check BB here and not in SimpleContour::IsWithin below
-        if (!b.GetBoundingBox().IsWithinBool(p)) return REL_OVERLAP;
-        //check bb here
-        switch (b.IsWithin(p, &edge, &pos, /*strict=*/false)) {
-        default:
-            _ASSERT(0);
-            FALLTHROUGH;
-        case WI_INSIDE:  return REL_A_INSIDE_B;
-        case WI_OUTSIDE: return REL_OVERLAP;
-        case WI_ON_VERTEX:
-            pos = 0; //if we are on b's vertex, this is the right pos for that edge
-            FALLTHROUGH;
-        case WI_ON_EDGE:
-            RayAngle prev1 = at_prev(i).Angle(true, 1);
-            RayAngle next1 = at(i).Angle(false, 0);
-            RayAngle prev2 = pos == 0 ? b.at_prev(edge).Angle(true, 1) : b.at(edge).Angle(true, pos);
-            RayAngle next2 = pos == 1 ? b.at_next(edge).Angle(false, 0) : b.at(edge).Angle(false, pos);
-            if (!GetClockWise()) std::swap(prev1, next1); //make angles as if clockwise
-            if (!b.GetClockWise()) std::swap(prev2, next2); //make angles as if clockwise
+    if (threshold>size()) threshold = size();
+    if (allow>=threshold) allow = threshold-1;
+    size_t inside = 0, outside = 0;
+    for (bool try_edges : {false, true}) {
+        inside = 0, outside = 0;
+        for (size_t i = 0; i<size(); i++) {
+            double pos;
+            const XY p = at(i).GetStart();
+            //Optimize: check BB here and not in SimpleContour::IsWithin below
+            const EPointRelationType bb_rel = b.GetBoundingBox().IsWithin(p);
+            if (bb_rel==WI_OUTSIDE)
+                outside++;
+            else switch (b.IsWithin(p, &edge, &pos, /*strict=*/false)) {
+            default:
+                _ASSERT(0);
+                FALLTHROUGH;
+            case WI_OUTSIDE:
+                outside++;
+                goto check_vertical;
+            case WI_INSIDE:
+                if (bb_rel==WI_ON_EDGE || bb_rel==WI_ON_VERTEX)
+                    //This is clearly a bad result as the point cannot be inside,
+                    //if it is on the preimiter of the bouinding box. Try another.
+                    continue;
+                inside++;
+            check_vertical:
+                //If the previous edge was vertical, then here we were checking
+                //being in/out using the same X coordinate. If there were some imprecision
+                //before, we may have that, too, so inrease the threshold, to force us to check one more.
+                if (i>0 && threshold<size() && test_equal(at(i-1).GetStart().x, p.x))
+                    threshold++;
+                break;
+            case WI_ON_VERTEX:
+                pos = 0; //if we are on b's vertex, this is the right pos for that edge
+                FALLTHROUGH;
+            case WI_ON_EDGE:
+                if (!try_edges) continue;
+                RayAngle prev2 = pos == 0 ? b.at_prev(edge).Angle(true, 1) : b.at(edge).Angle(true, pos);
+                RayAngle next2 = pos == 1 ? b.at_next(edge).Angle(false, 0) : b.at(edge).Angle(false, pos);
+                //If 'b' has a too sharp vertex (too small angle), we cannot conclude that 'this'
+                //is inside (as really_between_warp() is undecided then), go to the next vertex.
+                if (prev2.IsSimilar(next2)) continue;
+                RayAngle prev1 = at_prev(i).Angle(true, 1);
+                RayAngle next1 = at(i).Angle(false, 0);
+                //Normalize so that the clockwise next->prev angle transition is the inside of the
+                //space enclosed by the paths, irrespective of whether the direction of the
+                //simplecontours is clockwise or clounterclockwise.
+                if (!GetClockWise()) std::swap(prev1, next1); //make angles as if clockwise
+                if (!b.GetClockWise()) std::swap(prev2, next2); //make angles as if clockwise
 
-            //if both the same, we continue: this vertex is non-decisive
-            if (prev1.IsSimilar(prev2) && next1.IsSimilar(next2)) break; //SAME - do another edge
-
-            //if values are too close it is dangerous, we look for other vertices
-            if (prev1.IsSimilar(next1)) break;
-            if (prev2.IsSimilar(next2)) break;
-
-            if (really_between_warp(prev1, next2, prev2) &&
-                really_between_warp(next1, next2, prev2)) return REL_A_INSIDE_B;
-            if (really_between_warp(prev2, next1, prev1) &&
-                really_between_warp(next2, next1, prev1)) return REL_B_INSIDE_A;
-            return REL_APART;
+                if (prev1.IsSimilar(prev2)) {
+                    //The two contours arrive via overlapping edges to 'p'
+                    if (next1.IsSimilar(next2))
+                        continue; //...and they also leave via overlapping edges - go check the next vertex of us
+                    if (next1.IsSimilar(prev2))
+                        continue; // Here prev1~~prev2~~next1, which means a sharp prev1~~next1 vertex. Too risky to form a verdict, continue
+                    //The above 2 checks are needed, as the below one signals false if next1~~next2 or next1~~prev2
+                    if (really_between_warp(next1, next2, prev2))
+                        inside++; //The next1 leaves into the inside of 'b'
+                    else
+                        outside++; //The next1 leaves away from the circumference of 'b'
+                } else if (next1.IsSimilar(next2)) {
+                    if (prev1.IsSimilar(next2))
+                        continue; // Here prev1~~prev2~~next1, which means a sharp prev1~~next1 vertex. Too risky to form a verdict, continue
+                    if (really_between_warp(prev1, next2, prev2))
+                        inside++;
+                    else
+                        outside++;
+                } else if (next1.IsSimilar(prev2))
+                    outside++;
+                else if (prev1.IsSimilar(next2))
+                    outside++;
+                else if (really_between_warp(prev1, next2, prev2) || really_between_warp(next1, next2, prev2))
+                    inside++;
+                else
+                    outside++;
+            }
+            if (inside>threshold || outside>threshold) goto finish;
         }
+        if (inside>allow && outside>allow) return {};
+        if (inside>allow) return REL_A_INSIDE_B;
+        if (outside>allow) return REL_OVERLAP;
     }
-    //All points were on a vertex equal in one of the directions
-    return REL_SAME;
+    if (inside==0 && outside==0) //All our vertices were on an edge of 'b' with all the angles matching
+        return REL_SAME;
+    if (inside==outside)
+        return REL_IN_HOLE_APART; //inconclusive
+finish:
+    return inside>outside ? REL_A_INSIDE_B : REL_OVERLAP;
 }
 
-
 /** Determines the relation of two simplecontours assuming none of their edges cross.
  *
- * Gives valid result only if the two contours have no crosspoints.
+ * Gives valid result only if the two contours have no crosspoints - but may touch.
+ * Can only return REL_SAME, REL_APART, REL_A_INSIDE_B or REL_B_INSIDE_A,
+ * REL_A_IS_EMPTY, REL_B_IS_EMPTY, REL_BOTH_EMPTY.
  * Clockwiseness fully ignored: the inside of counterclockwise contours is the
  * limited space they encloses.
+ *
+ * This one may be called by RelationTo().
  */
 EContourRelationType SimpleContour::CheckContainment(const SimpleContour &other) const
 {
     //fast path
-    if (!GetBoundingBox().Overlaps(other.GetBoundingBox()))
-        return REL_APART;
     if (other.IsEmpty())
         return IsEmpty() ? REL_BOTH_EMPTY : REL_B_IS_EMPTY;
     if (IsEmpty())
         return REL_A_IS_EMPTY;
-    EContourRelationType this_in_other = CheckContainmentHelper(other);
-    if (this_in_other != REL_OVERLAP) return this_in_other;
+    if (GetBoundingBox().HasZeroOverlap(other.GetBoundingBox()))
+        return REL_APART;
+    //If 'this' is smaller or same area than 'other', check if they are the same or if
+    //'this' is inside 'other'.
+    if (fabs(GetArea())<fabs(other.GetArea())*1.01)
+        switch (CheckContainmentHelper(other)) {
+        case REL_SAME:       _ASSERT_PRINT(fabs(GetArea())*0.99 < fabs(other.GetArea()) && fabs(other.GetArea())<fabs(GetArea())*1.01, *this, other, "REL_SAME diff area"); return REL_SAME;
+        case REL_A_INSIDE_B: _ASSERT_PRINT(fabs(GetArea())<fabs(other.GetArea())*1.01, *this, other, "REL_A_INSIDE_B bad area"); return REL_A_INSIDE_B;
+        default: _ASSERT(0); FALLTHROUGH;
+        case REL_OVERLAP: break;
+        }
     switch (other.CheckContainmentHelper(*this)) {
-    default:
-    case REL_A_IN_HOLE_OF_B:
-    case REL_B_IN_HOLE_OF_A:
-    case REL_IN_HOLE_APART:
-    case REL_SAME: _ASSERT(0); return REL_SAME;
-    case REL_OVERLAP:
-    case REL_APART:       return REL_APART;
-    case REL_A_INSIDE_B:  return REL_B_INSIDE_A; //other is in us
-    case REL_B_INSIDE_A:  return REL_A_INSIDE_B; //we are in other
+    default: _ASSERT(0); return REL_APART;
+    case REL_SAME:       _ASSERT_PRINT(fabs(GetArea())*0.99 < fabs(other.GetArea()) && fabs(other.GetArea())<fabs(GetArea()*1.01), *this, other, "REL_SAME diff area"); return REL_SAME; //If one dir measured it very similar, lets go with it.
+    case REL_OVERLAP:    return REL_APART;
+    case REL_A_INSIDE_B: _ASSERT_PRINT(fabs(other.GetArea())<fabs(GetArea())*1.01, *this, other, "REL_A_INSIDE_B bad area"); return REL_B_INSIDE_A; //other is in us
     }
 }
 
-
-
 /** Swap data with `b`. */
 void SimpleContour::swap(SimpleContour& b) noexcept
 {
@@ -320,27 +377,6 @@ bool SimpleContour::IsRectangle() const noexcept {
 }
 
 
-/** Checks if all edges connect, the shape is closed, no edges
- * are degenerated to a dot, the shape has an actual area, etc.*/
-bool SimpleContour::IsSane() const
-{
-    if (size()==0) return true;
-    if (size()==1) {
-        if (at(0).IsStraight()) return false;
-        if (at(0).IsDot()) return false;
-        if (at(0).GetStart().test_equal(at(0).GetEnd())) return true;
-        return false;
-    }
-    if (size()==2 && at(0).IsStraight() && at(1).IsStraight())
-        return false;
-    for (size_t u = 0; u<size(); u++)
-        if (at(u).IsDot())
-            return false;
-        else if (at(u).GetEnd() != at_next(u).GetStart())
-            return false;
-    return true;
-}
-
 /** Fix problems in a shape (edges not joint, etc).
  *
  * @returns true if we were sane, false if changes were needed.
@@ -474,7 +510,7 @@ double SimpleContour::GetCircumference(bool include_hidden) const noexcept
 {
     double ret = 0;
     for (const auto &e : edges)
-        if (include_hidden || e.visible)
+        if (include_hidden || e.IsVisible())
             ret += e.GetLength();
     return ret;
 }
@@ -593,8 +629,8 @@ void SimpleContour::CreateRoundForExpand(XY center, const XY &start, const XY &e
     round.edges.back().SetEndOnly(end);
     for (auto &e: round.edges) {
         if (!visible)
-            e.visible = false;
-        e.mark = true;
+            e.SetVisible(false);
+        e.SetInternalMark(true);
         insert_to.emplace(here, EXPAND_ROUND, e); //all inserted edges marked as trivially connecting with ROUND type
     }
 }
@@ -621,9 +657,12 @@ void SimpleContour::CreateRoundForExpand(XY center, const XY &start, const XY &e
  *                    E.g., miter_limit=2 says they can be as long as `2*gap`.
  *                    The rest is cut away with a bevel-like edge.
  * @param [in] tolerance Edges shorter than this are ignored.
+ * @param [in] always_simplify_result If set to yes, we merge resulting edges no matter what.
+ *             If false, we only merge (and remove) edges that have no corresponding edge in
+ *             the original contour.
  * @returns A list of expanded edges, which form a (perhaps tangled, but) continuous simplecontour.*/
 SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double gap, double miter_limit,
-                                                        double tolerance) const
+                                                        double tolerance, bool always_simplify_result) const
 {
     _ASSERT(IsSane());
     //Return us if nothing to do.
@@ -641,7 +680,7 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
     EdgeList<Edge> tmp;
     std::optional<XY> prev_non_dot_end; //end of the last non-dot edge, if there was a dot before us.
     //Find the last non-dot edge backwards
-    for (size_t u = size(); u; u--) 
+    for (size_t u = size(); u; u--)
         if (!at(u-1).IsDot(tolerance)) {
             if (u!=size())
                 prev_non_dot_end = at(u-1).GetEnd();
@@ -649,7 +688,7 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
         }
     for (size_t u = 0; u<size(); u++) {
         if (at(u).IsDot(tolerance)) {
-            if (!prev_non_dot_end) 
+            if (!prev_non_dot_end)
                 prev_non_dot_end = at(u).GetStart(); //we have removed a previous edge, as well
             continue;
         }
@@ -668,10 +707,10 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
         if (!success) {
             //false is returned if a bezier degenerates - replace with a line
             if (prev_non_dot_end) {
-                Edge(prev_non_dot_end.value(), at(u).GetEnd(), at(u).visible).CreateExpand(gap, tmp, prev_tangent, next_tangent);
+                Edge(prev_non_dot_end.value(), at(u).GetEnd(), at(u).IsVisible()).CreateExpand(gap, tmp, prev_tangent, next_tangent);
                 prev_non_dot_end.reset();
             } else {
-                Edge(at(u).GetStart(), at(u).GetEnd(), at(u).visible).CreateExpand(gap, tmp, prev_tangent, next_tangent);
+                Edge(at(u).GetStart(), at(u).GetEnd(), at(u).IsVisible()).CreateExpand(gap, tmp, prev_tangent, next_tangent);
             }
         }
         if (tmp.size()==0) {
@@ -726,7 +765,7 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
                     i->next_tangent, r.next(i)->prev_tangent,
                     !at(i->original_edge).IsStraight(), !at(r.next(i)->original_edge).IsStraight(),
                     i->us_end_pos, i->next_start_pos);
-                _ASSERT(!isnan(i->cross_point.x) && !isnan(i->cross_point.y));
+                _ASSERT(!std::isnan(i->cross_point.x) && !std::isnan(i->cross_point.y));
                 //note if we have changed the status of at least one such CP
                 //this means we may need to do another pass
                 if (i->cross_type!=Edge::CP_INVERSE)
@@ -789,7 +828,7 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
                         new_end = i->GetEnd() + (new_end-i->GetEnd()).Normalize()*gap_limit_to_use;
                     //The value of 'need_miter_limit_bevel' will also be re-used next
                     _ASSERT(fabs(new_start.length()) < 100000);
-                    _ASSERT(!isnan(new_end.x) && !isnan(new_end.y));
+                    _ASSERT(!std::isnan(new_end.x) && !std::isnan(new_end.y));
                     _ASSERT(fabs(new_end.length()) < 100000);
                 }
 
@@ -811,7 +850,7 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
                 } else {
                     //add straight edge before the inserted bezier for miters
                     if (IsMiter(prev_i->join_type) && prev_i->cross_type==Edge::CP_EXTENDED)
-                        r.emplace(i, prev_i->join_type, new_start, i->GetStart(), bool(i->visible)); //mark=true
+                        r.emplace(i, prev_i->join_type, new_start, i->GetStart(), i->IsVisible()); //mark=true
                     if (prev_i->cross_type == Edge::CP_REAL && i->cross_type == Edge::CP_REAL)
                         i->SetStartEndIgn(new_start, new_end, prev_i->next_start_pos, i->us_end_pos);
                     else if (prev_i->cross_type == Edge::CP_REAL)
@@ -824,7 +863,7 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
                     if (IsMiter(i->join_type) && i->cross_type ==Edge::CP_EXTENDED)
                         //make sure we copy the metadata, as well ('cross_type' and afterwards)
                         //beacuse the next iteration will rely on these, when adding
-                        r.emplace(next_i, i->join_type, i->GetEnd(), new_end, bool(!!i->visible),
+                        r.emplace(next_i, i->join_type, i->GetEnd(), new_end, i->IsVisible(),
                                           i->cross_type, i->cross_point, double(), i->next_start_pos); //mark=true
                 }
 
@@ -841,7 +880,7 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
                     if (bevel_end != new_end)
                         //make sure we copy the metadata, as well ('cross_type' and afterwards)
                         //beacuse the next iteration will rely on these, when adding
-                        r.emplace(next_i, i->join_type, new_end, bevel_end, i->visible && next_i->visible,
+                        r.emplace(next_i, i->join_type, new_end, bevel_end, i->IsVisible()&& next_i->IsVisible(),
                                           i->cross_type, i->cross_point, double(), i->next_start_pos); //mark=true
                     prev_miter_limit_bevel_point = bevel_end;
                 }
@@ -861,11 +900,11 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
                 if (i->cross_type == Edge::CP_EXTENDED) {
                     if (i->join_type == EXPAND_BEVEL) {
                         //Add a bevel from new_end and the next start
-                        r.emplace(next_i, i->join_type, new_end, next_i->GetStart(), i->visible && next_i->visible); //mark=true
+                        r.emplace(next_i, i->join_type, new_end, next_i->GetStart(), i->IsVisible()&& next_i->IsVisible()); //mark=true
                     } else if (i->join_type == EXPAND_ROUND) {
                         //Add a circle, from new_end and the next start
                         CreateRoundForExpand(at(i->original_edge).GetEnd(), new_end, next_i->GetStart(), gap>0,
-                            r, next_i, i->visible && next_i->visible);
+                            r, next_i, i->IsVisible()&& next_i->IsVisible());
                     }
                 }
                 //We have done everything for non-parallel joins
@@ -928,14 +967,14 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
                     //We add two lines (two miters)
                     //Remember, i->cross_point contains the point in between the two lines.
                     //We do not limit these miters, even if miter_limit is not infinite.
-                    r.emplace(next_i, i->join_type, new_end, i->cross_point, bool(i->visible)); //mark=true
-                    auto ii = r.emplace(next_i, i->join_type, i->cross_point, next_i->GetStart(), bool(next_i->visible)); //mark=true
+                    r.emplace(next_i, i->join_type, new_end, i->cross_point, i->IsVisible()); //mark=true
+                    auto ii = r.emplace(next_i, i->join_type, i->cross_point, next_i->GetStart(), next_i->IsVisible());//mark=true
                     Edge::RemoveLoop(r, start_orig_us, ii);
                     Edge::RemoveLoop(r, ii, end_orig_next);
                 } else if (IsRound(i->join_type)) {
                     //Add a circle, from new_end and the next start
                     CreateRoundForExpand(at(i->original_edge).GetEnd(), new_end, next_i->GetStart(), gap>0,
-                        r, next_i, i->visible && next_i->visible);
+                        r, next_i, i->IsVisible() && next_i->IsVisible());
                     Edge::RemoveLoop(r, start_orig_us, next_i);
                     //i and next_i may be destroyed here. Search loops from start_orig_us again
                     Edge::RemoveLoop(r, start_orig_us, end_orig_next);
@@ -946,8 +985,8 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
                                         at(i->original_edge).GetEnd()).Normalize();
                     const XY cp = (new_end + next_i->GetStart())/2 +
                                     tangent * (new_end - next_i->GetStart()).length()/2;
-                    r.emplace(next_i, i->join_type, new_end, cp, i->visible && next_i->visible); //mark=true
-                    auto ii = r.emplace(next_i, i->join_type, cp, next_i->GetStart(), i->visible && next_i->visible); //mark=true
+                    r.emplace(next_i, i->join_type, new_end, cp, i->IsVisible() && next_i->IsVisible()); //mark=true
+                    auto ii = r.emplace(next_i, i->join_type, cp, next_i->GetStart(), i->IsVisible() && next_i->IsVisible()); //mark=true
                     Edge::RemoveLoop(r, start_orig_us, ii);
                     Edge::RemoveLoop(r, ii, end_orig_next);
                 } else if (i->join_type == EXPAND_MITER_SQUARE) {
@@ -956,9 +995,9 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
                     const double dist = new_end.Distance(next_start)/2;
                     const XY tangent = (at(i->original_edge).NextTangentPoint(1.0) -
                                         at(i->original_edge).GetEnd()).Normalize()*dist;
-                    r.emplace(next_i, i->join_type, new_end, new_end+tangent, i->visible && next_i->visible); //mark=true
-                    r.emplace(next_i, i->join_type, new_end+tangent, next_start+tangent, i->visible && next_i->visible);  //mark=true
-                    r.emplace(next_i, i->join_type, next_start+tangent, next_start, i->visible && next_i->visible); //mark=true
+                    r.emplace(next_i, i->join_type, new_end, new_end+tangent, i->IsVisible() && next_i->IsVisible()); //mark=true
+                    r.emplace(next_i, i->join_type, new_end+tangent, next_start+tangent, i->IsVisible() && next_i->IsVisible());  //mark=true
+                    r.emplace(next_i, i->join_type, next_start+tangent, next_start, i->IsVisible() && next_i->IsVisible()); //mark=true
                     Edge::RemoveLoop(r, start_orig_us, next_i);
                     //i and next_i may be destroyed here. Search loops from start_orig_us again
                     Edge::RemoveLoop(r, start_orig_us, end_orig_next);
@@ -997,15 +1036,16 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
 #endif
 
     //Insert straight segments for CP_INVERSE joins (not visible) & try to combine edges
-    for (auto i = r.begin(); i!=r.end(); i++) 
+    for (auto i = r.begin(); i!=r.end(); i++)
         if (i->cross_type == Edge::CP_INVERSE) {
             const auto next_i = r.next(i);
             r.emplace(next_i, i->join_type, i->GetEnd(), next_i->GetStart(), false); //mark=true
         }
     for (auto i = r.begin(); i!=r.end(); i++) {
         const auto next_i = r.next(i);
-        if (i->CheckAndCombine(*next_i))
-            r.erase(next_i);
+        if (always_simplify_result || next_i->original_edge<0)
+            if (i->CheckAndCombine(*next_i))
+                r.erase(next_i);
     }
     //Ok, now we have the expanded contour in 'r', hopefully all its edges connected to the
     //next one.
@@ -1123,9 +1163,9 @@ Contour SimpleContour::CreateExpand(EExpandType type, double gap, double miter_l
     if (0) {
         contour::debug::Snapshot("test.pdf", EC::STRINGFORMAT, "\\mn(1)", EC::START, "%n", EC::LINEWIDTH, 0.1, EC::MARKERWIDTH, 0.5,
                                  ColorType(255, 0, 0), *this, ColorType(0, 0, 255), sp_r2, ColorType(0, 127, 0), ret[0].Outline());
-        std::string r = edges.Dump();
-        std::string a = sp_r2.edges.Dump();
-        std::string b = ret.Dump();
+        std::string r = edges.Dump(true);
+        std::string a = sp_r2.edges.Dump(true);
+        std::string b = ret.Dump(true);
     }
 #endif
     return ret;
@@ -1169,13 +1209,13 @@ void SimpleContour::Expand2DHelper(const XY &gap, Path &a,
         case -4: cp.y += 2*gap.y; break;
         default: _ASSERT(0); break;
         }
-        const Edge A(a[original_last].GetEnd(), cp, !!a[original_last].visible);
-        const Edge B(cp, a[next].GetStart(), !!a[next].visible);
+        const Edge A(a[original_last].GetEnd(), cp, !!a[original_last].IsVisible());
+        const Edge B(cp, a[next].GetStart(), !!a[next].IsVisible());
         a.insert(a.begin()+original_last+1, A);
         a.insert(a.begin()+original_last+2, B);
         return;
     }
-    const Edge C(a[original_last].GetEnd(), a[next].GetStart(), a[original_last].visible && a[next].visible);
+    const Edge C(a[original_last].GetEnd(), a[next].GetStart(), a[original_last].IsVisible() && a[next].IsVisible());
     a.insert(a.begin()+original_last+1, C);
 }
 
@@ -1233,19 +1273,24 @@ Contour SimpleContour::CreateExpand2D(const XY &gap) const
     return res;
 }
 
- /** Determines the relation of two shapes with no prior assumptions. */
+/** Determines the relation of two shapes with no prior assumptions. */
 EContourRelationType SimpleContour::RelationTo(const SimpleContour &c) const
 {
+    if (c.IsEmpty())
+        return IsEmpty() ? REL_BOTH_EMPTY : REL_B_IS_EMPTY;
+    if (IsEmpty())
+        return REL_A_IS_EMPTY;
     if (!GetBoundingBox().Overlaps(c.GetBoundingBox())) return REL_APART;
-    //Look for crosspoints
-    XY r[4];
-    double one_pos[4], two_pos[4];
-    for (size_t u1 = 0; u1<size(); u1++)
-        for (size_t u2 = 0; u2<c.size(); u2++)
-            if (at(u1).Crossing(c.at(u2), false, r, one_pos, two_pos))
-                if (one_pos[0]>0 || two_pos[0]>0) //Crosspoint found, we overlap
-                    return REL_OVERLAP;
-    //No real crosspoint (only perhaps vertices are equal)
+    std::vector<CPData> cps;
+    //Crosspoints returns no cps, but true for exactly the same contours
+    if (CrossPoints(cps, c.edges.begin(), c.edges.end(), true, &c.GetBoundingBox(), CalcRelation))
+        return REL_SAME;
+    //If there is any crossing CP, we must overlap
+    if (std::ranges::any_of(cps, [](const CPData& d) { return d.type.rel==CPRel::Cross; }))
+        return REL_OVERLAP;
+    //Perhaps here we could evaluate the touching crosspoints to see relation for touching
+    //contours as an optimization. Left as excercise to the reader for now.
+    //The two shapes are apart or one inside the other, safe to call this
     return CheckContainment(c);
 }
 
@@ -1283,9 +1328,9 @@ void SimpleContour::VerticalCrossSection(double x, DoubleMap<bool> &section) con
 
 /** Creates a 'gap' wide mesh gradient at our perimiter, but completely within us.
   * 'gap' is the width of the graient, {r,g,b,a}1 is the inner color, {r,g,b,a}2 is the outer.
-  * If shrinking us makes us fall into pieces, we return nullptr as sign of failure. 
+  * If shrinking us makes us fall into pieces, we return nullptr as sign of failure.
   * We also return the inner edge of the mesh, so that it can be filled solid.*/
-std::pair<cairo_pattern_t *, Contour> 
+std::pair<cairo_pattern_t *, Contour>
 SimpleContour::CairoMeshGradient(double gap, double r1, double g1, double b1, double a1,
                                  double r2, double g2, double b2, double a2,
                                  EExpandType type, double miter_limit) const
@@ -1298,7 +1343,7 @@ SimpleContour::CairoMeshGradient(double gap, double r1, double g1, double b1, do
         return ret;
     cairo_pattern_t *pattern = cairo_pattern_create_mesh();
     const SimpleContour &sh = ret.second[0].outline;
-    const MetaDataList r = sh.ExpandHelper(type, gap, miter_limit, 0.1);
+    const MetaDataList r = sh.ExpandHelper(type, gap, miter_limit, 0.1, false);
     //search the last edge corresponding to an original edge
     auto i = std::find_if(r.rbegin(), r.rend(), [](auto &e) { return e.original_edge>=0; });
     if (i==r.rend()) //no edge with any correspondence to original edges. Also handles the case of r.size()==0
@@ -1431,7 +1476,7 @@ std::array<CPData, 2> SimpleContour::CutEx(const Edge &s) const
         ret[0] = *min;
         ret[1] = *max;
     } else {
-        //invalidate 
+        //invalidate
         ret[0].other.pos = 1;
         ret[1].other.pos = 0;
     }
@@ -1442,7 +1487,7 @@ std::array<CPData, 2> SimpleContour::CutEx(const Edge &s) const
 bool SimpleContour::CrossPoints(std::vector<CPData> &ret, const Edge &e, CPTasks to_do, bool swap) const
 {
     const size_t orig_ret_size = ret.size();
-    bool r = edges.CrossPoints(ret, true, boundingBox_fresh ? &boundingBox : nullptr, 
+    bool r = edges.CrossPoints(ret, true, boundingBox_fresh ? &boundingBox : nullptr,
                                &e, &e+1, false, boundingBox_fresh ? &static_cast<const Block &>(e.CreateBoundingBox()) : nullptr, to_do, swap);
     for (size_t i = orig_ret_size; i<ret.size(); i++)
         (swap ? ret[i].contour_other : ret[i].contour_me) = this;
@@ -1450,7 +1495,7 @@ bool SimpleContour::CrossPoints(std::vector<CPData> &ret, const Edge &e, CPTasks
 }
 
 
-/** Chop the Contour into paths at points. We return an empty vector if 
+/** Chop the Contour into paths at points. We return an empty vector if
  * a pathpos is invalid or you give us only one.
  * We take pos by value because we sort it.
  * If two pos in the vector are equal, we return an empty Path there, unless 'remove_empty'*/
@@ -1562,4 +1607,21 @@ void SimpleContour::Cut(const XY &A, const XY &B, DoubleMap<bool> &map) const
             map.Set(Range(j->first, i->first), clockwise);
 }
 
-} //namespace contour bezier
\ No newline at end of file
+/** Creates a C++ representation of this simple contour as a Contour.
+ * Calls Edge->Dump(precise).
+ * @param [in] precise When set, we emit the hex representation of doubles, else a human readable one.
+ * @returns the full string.*/
+std::string SimpleContour::Dump(bool precise) const {
+    Path p;
+    AppendToPath(p);
+    return "SimpleContour::UnsafeMake("+p.Dump(precise, "Path", "    ")+")";
+}
+
+SimpleContour SimpleContour::UnsafeMake(Path&& v) {
+    SimpleContour ret;
+    ret.edges.swap(v);
+    ret.clockwise_fresh = ret.boundingBox_fresh = ret.area_fresh = false;
+    return ret;
+}
+
+} //namespace contour bezier
