@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -36,18 +36,16 @@
 #include "cgen_context.h"
 
 /** A range inside an input file. */
-struct CshPos 
+struct CshPos
 {
     int first_pos; ///<The index of the first character of the range.
     int last_pos;  ///<The index of the last character of the range.
-    CshPos() = default;
-    CshPos(int a, int b) : first_pos(a), last_pos(b) {}
-    CshPos operator + (const CshPos &a) const { return CshPos(std::min(first_pos, a.first_pos), std::max(last_pos, a.last_pos)); }
-    CshPos &operator += (const CshPos &a) { first_pos = std::min(first_pos, a.first_pos); last_pos = std::max(last_pos, a.last_pos); return *this; }
-    bool operator ==(const CshPos &a) const { return first_pos==a.first_pos && last_pos==a.last_pos; }
-    bool IsWithin(int p) const { return first_pos<=p && last_pos>=p; } ///<Returns true if positin p is inside the range.
-    bool IsWithin(const CshPos &p) const {return IsWithin(p.first_pos) && IsWithin(p.last_pos);} ///<Returns true if another range is completely within the range.
-    bool Overlaps(const CshPos &p) const { return p.first_pos<=last_pos && first_pos<=p.last_pos; }
+    CshPos operator + (const CshPos &a) const noexcept { return CshPos(std::min(first_pos, a.first_pos), std::max(last_pos, a.last_pos)); }
+    CshPos &operator += (const CshPos &a) noexcept { first_pos = std::min(first_pos, a.first_pos); last_pos = std::max(last_pos, a.last_pos); return *this; }
+    bool operator ==(const CshPos &a) const noexcept { return first_pos==a.first_pos && last_pos==a.last_pos; }
+    bool IsWithin(int p) const noexcept { return first_pos<=p && last_pos>=p; } ///<Returns true if positin p is inside the range.
+    bool IsWithin(const CshPos &p) const noexcept {return IsWithin(p.first_pos) && IsWithin(p.last_pos);} ///<Returns true if another range is completely within the range.
+    bool Overlaps(const CshPos &p) const noexcept { return p.first_pos<=last_pos && first_pos<=p.last_pos; }
     template<class PosList> static void AdjustList(PosList &list, int start, int offset);
 };
 
@@ -79,10 +77,10 @@ enum EColorSyntaxType : uint8_t {
     COLOR_PARAMNAME,       ///<The name of a procedure parameter ($xxx)
     COLOR_OPTIONNAME,      ///<The name of a chart option
     COLOR_OPTIONNAME_PARTIAL,///<The name of a chart option partially written
-    COLOR_OPTIONNAME_MSCGEN,///<The name of an mscgen-only chart option 
+    COLOR_OPTIONNAME_MSCGEN,///<The name of an mscgen-only chart option
     COLOR_ATTRNAME,        ///<The name of an attribute
     COLOR_ATTRNAME_PARTIAL,///<The name of a attribute partially written
-    COLOR_ATTRNAME_MSCGEN, ///<The name of an mscgen-only attribute 
+    COLOR_ATTRNAME_MSCGEN, ///<The name of an mscgen-only attribute
     COLOR_ATTRVALUE,       ///<The value of an attribute
     COLOR_ATTRVALUE_EMPH,  ///<Emphasized part of the attribute value
     COLOR_LABEL_TEXT,      ///<The text of a label, either after a colon or between quotation marks
@@ -117,7 +115,7 @@ public:
     template <typename PosList1, typename PosList2>
     void DiffInto(const PosList1 &old_list, const PosList2 &new_list,
                   EColorSyntaxType neutral);
-    void SortByPos() { 
+    void SortByPos() {
         std::sort(begin(), end(),
             [](CshEntry const & a, CshEntry const &b) {
             return a.first_pos==b.first_pos ? a.last_pos < b.last_pos : a.first_pos < b.first_pos;
@@ -168,13 +166,13 @@ enum EColorSyntaxFlag {
 
 /** Describes the appearance of colored text*/
 struct ColorSyntaxAppearance {
-    unsigned effects;    ///<Value of flags 
+    unsigned effects;    ///<Value of flags
     unsigned mask;       ///<Mask for the flags. Needed to be able to combine.
     unsigned char r;     ///<Red value
     unsigned char g;     ///<Green value
     unsigned char b;     ///<Blue value
     void SetColor(unsigned char rr, unsigned char gg, unsigned char bb)
-    { r = rr; g = gg; b = bb; effects |= COLOR_FLAG_COLOR; } 
+    { r = rr; g = gg; b = bb; effects |= COLOR_FLAG_COLOR; }
     bool operator==(const struct ColorSyntaxAppearance &p) const;
     std::string Print() const;
 };
@@ -214,7 +212,7 @@ public:
     StyleNameSet StyleNames;  ///<The names of styles defined so far
     CshContext(bool f, EContextParse p) : ContextParams(f, p), if_condition(2) {}
     /** Set the context to a given design (styles and colors)*/
-    void SetToDesign(const Context&);                   
+    void SetToDesign(const Context&);
     /** Combine two contexts */
     CshContext &operator+=(const CshContext &o) {
         Colors.insert(o.Colors.begin(), o.Colors.end());
@@ -244,9 +242,9 @@ enum class EHintType {
     FILES            ///<A list of files (param to the include command)
 };
 
-/** True for those hint types that compress along the dot in the filename. 
+/** True for those hint types that compress along the dot in the filename.
  * Well, not useful for graphviz, where attribute names include no dots.*/
-constexpr bool WantsDotCompress(EHintType t) 
+constexpr bool WantsDotCompress(EHintType t)
 { return t == EHintType::ENTITY || t==EHintType::ATTR_NAME || t==EHintType::ATTR_VALUE || t==EHintType::MARKER; }
 
 /** The situation the cursor is in when we provide hints. These can be set from preferences.*/
@@ -266,14 +264,14 @@ enum EEscapeHintType
 {
     HINTE_NONE = 0,      ///<The cursor is not at a hint position.
     HINTE_ESCAPE,        ///<The cursor is around an escape
-    HINTE_PARAM_COLOR,   ///<The cursor is inside the parenthesis of a \\c() escape
+    HINTE_PARAM_COLOR,   ///<The cursor is inside the parenthesis of a \\c() or \\C() escape
     HINTE_PARAM_STYLE,   ///<The cursor is inside the parenthesis of a \\s() escape
     HINTE_PARAM_FONT,    ///<The cursor is inside the parenthesis of a \\f() escape
     HINTE_PARAM_REF,     ///<The cursor is inside the parenthesis of a \\r() escape
     HINTE_PARAM_NUMBER,  ///<The cursor is inside the parenthesis of a \\mX() escape
     HINTE_PARAM_LINK,    ///<The cursor is inside the parenthesis of a \\L() escape
     HINTE_PARAM_SHAPE,   ///<The cursor is inside the parenthesis of a \\S() escape in the shape name section
-    HINTE_PARAM_PARAM,   ///<The cursor is inside the parenthesis of a \\Q() escape 
+    HINTE_PARAM_PARAM,   ///<The cursor is inside the parenthesis of a \\Q() escape
 };
 
 /** Merge two values. Error if neither are NONE */
@@ -303,7 +301,7 @@ enum EHintItemSelectionState {
 /** One possibility for autocompletion valid at the cursor */
 struct CshHint {
 public:
-    size_t sort;                    ///<Governs ordering of entries. Used for HINT_ESCAPEs, which do not sort alphabetically
+    std::tuple<int, int, int> sort; ///<Governs ordering of entries. Used for HINT_ESCAPEs, which do not sort alphabetically or at FUZZY matching
     std::string decorated;          ///<Full text of the hint with formatting escapes
     EHintType  type;                ///<The type of the hint.
     bool selectable;                ///<True if this hint can be inserted to the chart text, false if it is just an explanation
@@ -314,19 +312,25 @@ public:
      * @{  */
     mutable std::string plain;  ///<Plain text of the hint for sorting
     mutable std::string replaceto;///<When selected, insert this to the chart text. When empty, use 'plain'.
-    mutable bool keep;          ///<wether after insertion we shall keep the hint box up
-    mutable int x_size;         ///<calculated size of hint text in the hint listbox. NOT scaled for high DPI screens.
-    mutable int y_size;         ///<calculated size of hint text in the hint listbox. NOT scaled for high DPI screens.
-    mutable int ul_x;           ///<Size of the rectange shown in list box
-    mutable int ul_y;           ///<Size of the rectange shown in list box
-    mutable int br_x;           ///<Size of the rectange shown in list box
-    mutable int br_y;           ///<Size of the rectange shown in list box
-    mutable EHintItemSelectionState state; ///<Will show if this hint is selected or not
+    mutable bool keep = false;  ///<wether after insertion we shall keep the hint box up
+    mutable int x_size = 0;     ///<calculated size of hint text in the hint listbox. NOT scaled for high DPI screens.
+    mutable int y_size = 0;     ///<calculated size of hint text in the hint listbox. NOT scaled for high DPI screens.
+    mutable std::pair<uint16_t, uint16_t> highlight = {0,0}; ///<The character indexes (begin,end) in 'plain' to highlight. Used with fuzzy or starts_with search.
+    mutable bool can_autoselect = true; ///<When we do not erase bad hints, we set this to false. As the user types, we will avoid selecting any having this set to false
     /** @} */
-    CshHint(std::string &&d, const char *desc, EHintType t, bool s = true, CshHintGraphicCallback c = nullptr, CshHintGraphicParam p = 0, size_t so = 0) :
-        sort(so), decorated(std::move(d)), type(t), selectable(s), description(desc ? desc: ""), callback(c), param(p), keep(false) {}
-    CshHint(std::string &&d, std::string &&desc, EHintType t, bool s = true, CshHintGraphicCallback c = nullptr, CshHintGraphicParam p = 0, size_t so = 0) :
-        sort(so), decorated(std::move(d)), type(t), selectable(s), description(desc), callback(c), param(p), keep(false) {}
+    /** @name Derived values used by the Windows MFC GUI. They should not have been here
+     *  but easier this way. TODO: refactor these to an optional.
+     * @{  */
+    mutable int ul_x = 0;       ///<Size of the rectange shown in list box
+    mutable int ul_y = 0;       ///<Size of the rectange shown in list box
+    mutable int br_x = 0;       ///<Size of the rectange shown in list box
+    mutable int br_y = 0;       ///<Size of the rectange shown in list box
+    mutable EHintItemSelectionState state = HINT_ITEM_NOT_SELECTED; ///<Will show if this hint is selected or not
+    /** @} */
+    CshHint(std::string &&d, const char *desc, EHintType t, bool s = true, CshHintGraphicCallback c = nullptr, CshHintGraphicParam p = 0, int so = 0) :
+        sort(0,0,so), decorated(std::move(d)), type(t), selectable(s), description(desc ? desc: ""), callback(c), param(p), keep(false) {}
+    CshHint(std::string &&d, std::string &&desc, EHintType t, bool s = true, CshHintGraphicCallback c = nullptr, CshHintGraphicParam p = 0, int so = 0) :
+        sort(0,0,so), decorated(std::move(d)), type(t), selectable(s), description(desc), callback(c), param(p), keep(false) {}
     bool operator < (const CshHint &o) const;
     bool operator ==(const CshHint &o) const {return type == o.type && sort == o.sort && decorated == o.decorated;}
     std::string &GetReplacementString() { return replaceto.size() ? replaceto : plain; }
@@ -334,7 +338,7 @@ public:
 };
 
 /** The max length of the keywords, attribute and option names.*/
-#define ENUM_STRING_LEN 30
+#define ENUM_STRING_LEN 35
 
 bool CshHintGraphicCallbackForAttributeNames(Canvas *canvas, CshHintGraphicParam /*p*/, CshHintStore &);
 bool CshHintGraphicCallbackForColors(Canvas *canvas, CshHintGraphicParam p, CshHintStore &);
@@ -346,17 +350,26 @@ bool CshHintGraphicCallbackForKeywords(Canvas *canvas, CshHintGraphicParam, CshH
 class ShapeCollection;
 class StyleSetBase;
 
+/** How shall we filter the list of hints (by the word under the cursor).*/
+enum class EHintFilter {
+    None = 0,   ///<No filtering: all hints are displayed
+    Fuzzy,      ///<Drop hints not (fuzzy) matching the word under the cursor, sort by how fuzzy the matches are
+    Substr,     ///<Sort hints by how long common substring the hint has with the cursor
+    StartsWith, ///<Keep only hints that start with the word under the cursor
+    Max = StartsWith
+};
+
 /** A class storing hints during and after a csh parse and everything needed to draw the
  * hint popup dialog.
  * Descendants actually contain functions to collect this info.
  *
  * A few words on how hints are collected.
  * Often we realize what is the type of hint in a low-level yacc rule, but know no specifics.
- * For example, we have yacc rues describing how an attribute looks like, that is "name = value". 
+ * For example, we have yacc rues describing how an attribute looks like, that is "name = value".
  * In this rule we can realize that the cursor is inside value (or we match to a rule saying
  * "name =" and realize that the cursor is just after). Here we can conclude that the hint
  * will be a sort of attribute value. (So we pick EHintSourceType::ATTR_VALUE.) But we do not know
- * what values the attribute can take, because that depens on what element we are parsing. 
+ * what values the attribute can take, because that depens on what element we are parsing.
  * So after we macth
  * the low level rule, we set the hint type only and when processing the higher level rule for the
  * full element, do we fill in the actual hints. In the meantime we set the hintStatus member to
@@ -371,12 +384,12 @@ public:
     const ShapeCollection* pShapes = nullptr;
     /** @name The collected Hint info
     * @{ */
-    std::set<CshHint> Hints;          ///<The collected hints
+    std::vector<CshHint> Hints;       ///<The collected hints
     bool              was_partial;    ///<Indicates if the cursor is at the end of a partial keyword. On Sel Change we need to re-csh
     bool              allow_anything; ///<If true the user can define anything here besides the hints. Do not autocomplete a hint as she types.
     EHintSourceType   hintSource;     ///<The type of situation we found the cursor is at and for which the hint is provided. Used also in status HINT_LOCATED.
     bool              hadEscapeHint;  ///<True, if we had escape hints - we select character under the cursor and next char differently in MiniEditor.
-    bool              hadFileHint;     ///<True, if we had file hints - we will not trim hinted sting to the cursor only in MiniEditor.
+    bool              hadFileHint;    ///<True, if we had file hints - we will not trim hinted sting to the cursor only in MiniEditor.
     EHintStatus       hintStatus;     ///<Shows if we have located the hint type and if we have filled in the hints or not
     CshPos            hintedStringPos;///<The actual text location the hints refer to (can be the cursor only, which is a zero length range)
     std::string       hintAttrName;   ///<In case of an ATTR_VALUE contains the name of the attribute
@@ -393,10 +406,10 @@ struct CshParams
     int m_II = 4;            ///<instruction indent: how much an instruction is indented inside a {} block. The default tab size, too.
     int m_IB = 0;            ///<block indent: how much a block of a block series is indented compared to the first one.
     int m_IM = 2;            ///<middle indent: how much non-attribute elements inside an instruction are indented compared to the first line of the instruction
-    int m_IA = 2;            ///<attribute indent: how much attributes are indented compared to the [ 
+    int m_IA = 2;            ///<attribute indent: how much attributes are indented compared to the [
     bool m_bSIText = true;   ///<True, if we indent labels specially
     bool m_bSIAttr = true;   ///<True, if we indent attributes specially
-    int color_scheme=2;      ///<A pointer to the color scheme to use or nullptr if unknown. Used to ignore CSH entries same as COLOR_NORMAL.                   
+    int color_scheme=2;      ///<A pointer to the color scheme to use or nullptr if unknown. Used to ignore CSH entries same as COLOR_NORMAL.
     std::string ForcedDesign;///<What design is forced on us (so its colors and styles can be offered)
     std::set<std::string> fontnames;///<A list of font names for hinting
 };
@@ -416,15 +429,15 @@ protected:
     std::set<std::string> option_names;
     std::set<std::string> attribute_names;
     std::set<std::string> keyword_names;
-    static unsigned FindPrefix(const std::set<std::string> &set, std::string_view text);
-    static unsigned FindPrefix(const StyleNameSet &set, std::string_view text);
+    static unsigned FindPrefix(const std::set<std::string> &set, std::string_view text) noexcept;
+    static unsigned FindPrefix(const StyleNameSet &set, std::string_view text) noexcept;
     template <unsigned STRING_LEN>
-    static unsigned FindPrefix(const char set[][STRING_LEN], std::string_view text);
+    static unsigned FindPrefix(const char set[][STRING_LEN], std::string_view text) noexcept;
     void MoveHintsToNameCollection(std::set<std::string> &c);
     virtual void FillNamesHints() = 0;
     /** @} */
     virtual bool CalculateIsCursorAtLineBegin() const;
-    virtual void BeforeYaccParse(std::string_view input, int cursor_p);
+    virtual void BeforeYaccParse(const std::string& input, int cursor_p);
     virtual void AfterYaccParse();
     virtual bool DoHintLocated(EHintSourceType hsource, std::string_view a_name);
 public:
@@ -441,15 +454,16 @@ public:
     /** @name Input parameters to the color parsing process
     * @{  */
     /**A callback procedure to provide us with the list of files in a directory.
-    * prefix=the part of the filename already typed. May include a path component.
-    *        in that case, we return files from that dir.
-    * included_from=the path of the current file (for partial paths in prefix).
-    * param=arbitrary parameter passed.*/
-    typedef std::list<std::string>(*FileListProc)(std::string_view prefix, std::string_view included_from, void*param);
+    * @param prefix the part of the filename already typed, we return all filenames not
+    *        only the ones prefixed by this.
+    *        May include a path component, in that case, we return files from that dir.
+    * @param included_from the path of the current file (for relative paths in prefix).
+    * @param param arbitrary parameter passed.*/
+    typedef std::vector<std::string>(*FileListProc)(std::string_view prefix, std::string_view included_from, void*param);
 
     StyleNameSet          ForbiddenStyles; ///<Styles we never offer as hints (e.g., ->)
     int                   cursor_pos;      ///<The location of the cursor now (used to identify partial keyword names & hint list). The zero-based index of the *byte* (and not UTF-8 character) the cursor is before. If ==input_text.length(), we are after the last character. Larger or negative values are invalid.
-    std::shared_ptr<CshParams> params;     ///<Parameters to use 
+    std::shared_ptr<CshParams> params;     ///<Parameters to use
     std::string           FileName;        ///<The disk path of the file we parse - to provide correct include filename hints. May be empty if we are sure to ignore hints or if no disk file.
     FileListProc const    file_list_proc;  ///<The callback to call for a list of files to "include"
     void* const           file_list_proc_param; ///<The parameter to pass to the callback fn.
@@ -469,11 +483,11 @@ public:
     /** @}*/
     /** Type to store designs in. (These designs are just to maintain info needed for coloring and hints, not full attribute sets.)*/
     using CshContextMap = std::map<std::string, CshContext>;
-    /** @name The collected CSH info  
+    /** @name The collected CSH info
      * @{ */
     CshListType       CshList;        ///<The collected color syntax info
-    CshPosList        ColonLabels;    ///<The position of colon labels 
-    CshPosList        QuotedStrings;  ///<The position of quoted strings 
+    CshPosList        ColonLabels;    ///<The position of colon labels
+    CshPosList        QuotedStrings;  ///<The position of quoted strings
     CshPosList        Instructions;   ///<The position of all things that end in a semicolon (arcs, commands, chart options)
     CshPosList        BracePairs;     ///<The position of all {} pairs (the trailing '}' may be in fact missing).
     CshPosList        SqBracketPairs; ///<The position of all [] pairs (the trailing ']' may be in fact missing).
@@ -484,14 +498,14 @@ public:
     /** @} */
 
     /** Initializes the Csh Object.
-     * @param [in] defaultDesign Specifies a ContextBase to collect 
-     *   forbidden and default style names, color names and definitons to learn 
-     *   from. 
+     * @param [in] defaultDesign Specifies a ContextBase to collect
+     *   forbidden and default style names, color names and definitons to learn
+     *   from.
      * @param [in] proc A procedure to call when we need to list file candidates to the 'include' command.
      * @param [in] param An opaque value to pass to the 'proc'.*/
     Csh(const Context &defaultDesign, FileListProc proc, void*param);
     /** Create a copy of the current Csh object (full copy of a descendant).
-     * It is used in GUIs to clone a Csh containing the parsed designs and 
+     * It is used in GUIs to clone a Csh containing the parsed designs and
      * keywords, options, etc. before each parsing of the chart text.
      * TODO: This copy could be elided if we could restore an existing Csh
      * object to its state before the last parse of the chart text.*/
@@ -503,16 +517,16 @@ public:
     * @param [in] input The chart text
     * @param [in] cursor_p The current position of the cursor.
     * @param [in] pedantic The starting value of 'pedantic' chart option.*/
-    virtual void ParseText(std::string_view input, int cursor_p, bool pedantic) = 0;
+    virtual void ParseText(const std::string& input, int cursor_p, bool pedantic) = 0;
     /** Parse design library text to extract designs and shapes
      * By default it is equivalent to ParseText(), but may tweak settings
      * (restoring them afterwards).
      * @param [in] input The chart text
      * @param [in] cursor_p The current position of the cursor.*/
-    virtual void ParseDesignText(std::string_view input, int cursor_p) { ParseText(input, cursor_p, true); }
+    virtual void ParseDesignText(const std::string& input, int cursor_p) { ParseText(input, cursor_p, true); }
 
     //fill in size, plain and filter/compact if needed
-    virtual void ProcessHints(Canvas &canvas, StringFormat *format, std::string_view uc, bool filter_by_uc, bool compact_same);
+    virtual void ProcessHints(Canvas &canvas, StringFormat *format, std::string_view uc, EHintFilter filter_by_uc, bool compact_same);
 
 
     /**Add a name of a shape and its port names (whithout the actual shape)
@@ -531,7 +545,7 @@ public:
     virtual void AddCSH_ColonString_CheckAndAddEscapeHint(const CshPos& pos, std::string_view value, bool unquoted, bool two_colons=false);
     virtual void AddCSH_AttrName(const CshPos&, std::string_view name, EColorSyntaxType); ///<At pos there is either an option or attribute name (specified by the type). Search and color.
     virtual void AddCSH_AttrValue_CheckAndAddEscapeHint(const CshPos& pos, std::string_view value, std::string_view name);
-    virtual void AddCSH_AttrColorValue(const CshPos& pos); ///<At pos there is an attribute value that looks like a color definition (with commas and all). 
+    virtual void AddCSH_AttrColorValue(const CshPos& pos); ///<At pos there is an attribute value that looks like a color definition (with commas and all).
     virtual void AddCSH_StyleOrAttrName(const CshPos&pos, std::string_view name); ///<At pos there is either an attribute name or a style. Decide and color.
     virtual void AddCSH_EntityName(const CshPos&pos, std::string_view name); ///<At pos there is an entity name. Search and color.
     virtual void AddCSH_AllCommentBeyond(const CshPos&pos); ///<Mark anything beyond the end of 'pos' as comment
@@ -542,12 +556,12 @@ public:
      * Done in a virtual function so that descendants can override and add more info.*/
     virtual void AddEntityNamesAtTheEnd(std::string_view msg) { addEntityNamesAtEnd = msg; }
     /** @}*/
-   
+
     /** @name Functions to register info (mostly for precise indenting)
     * @{  */
     /** Adds the range to the list of instructions. Used for auto-indenting to the head of it.*/
     void AddInstruction(const CshPos &pos) { Instructions.emplace_back(std::max(1, pos.first_pos), std::max(1, pos.last_pos)); } //to avoid positions of zero (can be if an instruction starts with a potentially empty rule as with DOT
-    /** Adds the range to the list of instructions, if this range does not represent a curly brace pair. 
+    /** Adds the range to the list of instructions, if this range does not represent a curly brace pair.
      * Used in ifthenelse. There we do not want braces to become their own instruction.*/
     void AddInstructionIfNotBrace(const CshPos &pos);
     /** Adds a brace pair that is missing its closing element.
@@ -579,7 +593,7 @@ public:
     /** Returns if we shall ignore content being parsed. If so, we shall ignore anything defined (colors, entities, nodes, etc.). */
     bool SkipContent() const { return Contexts.back().SkipContent(); }
     /** Push the context stack copying what was on top.*/
-    void PushContext() { Contexts.push_back(Contexts.back());}                         
+    void PushContext() { Contexts.push_back(Contexts.back());}
     /** Push the context stack  with an empty context.*/
     void PushContext(bool f, EContextParse p) { Contexts.emplace_back(f, p); }
     void PopContext() { Contexts.pop_back(); }  ///<Pop the context stack.
@@ -600,7 +614,7 @@ public:
     * @{ */
     virtual bool StartsInAlpha(const CshPos &one) const;
     virtual bool EndsInAlpha(const CshPos &one) const;
-    
+
     bool CheckHintBefore(const CshPos &one, EHintSourceType hsource, std::string_view a_name = {}, bool alpha=true);
     bool CheckHintAt(const CshPos &one, EHintSourceType hsource, std::string_view a_name = {}, bool alpha = true);
     bool CheckHintAfter(const CshPos &one, EHintSourceType hsource, std::string_view a_name = {}, bool alpha = true);
@@ -673,7 +687,7 @@ public:
 
     /** A function to clean up elements after parsing a design library.
      * Only elements are kept that need to be carried over from a design library to the actual charts.
-     * These include the designs parsed, the shape library (set after parsing the design library 
+     * These include the designs parsed, the shape library (set after parsing the design library
      * language-wise, not just for coloring as we do in this class), the font name and cheme to use
      * pointer and the forbidden styles. */
     virtual void CleanupAfterDesignlib();
@@ -683,7 +697,7 @@ public:
     * @{ */
     /** Extracts the entity name under the cursor.
      * We also return the byte (not character) index of 'pos'
-     * within the name returned. The view returned point to 
+     * within the name returned. The view returned point to
      * Csh::input_text or to some other Csh member.
      * 'pos' shall be in CshPos units: first char is index of 1.*/
     virtual std::pair<std::string_view, size_t> EntityNameUnder(long pos) const;
@@ -692,26 +706,24 @@ public:
      * entity names are not structured, so the whole of it will be replaced. For Block
      * Diagrams, we just modify the segment under the cursor.*/
     virtual std::string_view AskReplace(std::string_view full_entity, int /*pos*/) const { return full_entity; }
-    /** Returns the chart text with a certain entity name replaced. 
-     * Takes what is returned by EntityNameUnder and what to replace the 
+    /** Returns the chart text with a certain entity name replaced.
+     * Takes what is returned by EntityNameUnder and what to replace the
      * clicked segment (for block diagrams) or the full entity name (signalling & dot) to.
      * Also takes the selection (in windows character style: pos zero is before the
      * first char. If lStart==lEnd we replace in the entire chart text.*/
-    virtual std::string ReplaceEntityName(std::string_view full_entity, int pos, 
+    virtual std::string ReplaceEntityName(std::string_view full_entity, int pos,
                                           std::string_view replace_to,
                                           long lStart, long lEnd) const;
     /** @}*/
 };
 
-
-int CaseInsensitiveBeginsWith(std::string_view a, std::string_view b); //defined in attribute.h for real;
-
+int CaseInsensitiveBeginsWith(std::string_view a, std::string_view b) noexcept; //defined in attribute.h for real;
 
 /** Finds a text in a collection - or its prefix if no exact match.
 @returns 0 if no match, 1 if text is a prefix of one of the entries,
 2 if text is one of the entries.*/
 template <unsigned STRING_LEN>
-unsigned Csh::FindPrefix(const char set[][STRING_LEN], std::string_view text)
+unsigned Csh::FindPrefix(const char set[][STRING_LEN], std::string_view text) noexcept
 {
     unsigned ret = 0;
     for (unsigned i = 0; set[i][0]; i++) {
@@ -751,7 +763,7 @@ void CshPos::AdjustList(PosList &list, int start, int offset)
             continue;
         }
         if (offset>0) {
-            //here 'upper==start' thus it must be that 
+            //here 'upper==start' thus it must be that
             // 'first pos < start' and 'last pos >= start'
             csh.last_pos += offset;
             continue;
@@ -795,7 +807,7 @@ inline void CshListType::DiffInto(const PosList1 &old_list, const PosList2 &new_
                 o++; //equal entries - sort of fast path
             else if (o->last_pos<n->last_pos)
                 //old entry is shorter, add that part of 'n' that
-                //is beyond of 'o' 
+                //is beyond of 'o'
                 AddToBack(CshEntry(o->last_pos+1, n->last_pos, n->color));
             done_till = n->last_pos+1;
             n++;
@@ -837,7 +849,7 @@ public:
     ~DummyCsh() override = default;
     std::unique_ptr<Csh> Clone() const override { return std::make_unique<DummyCsh>(*this); }
     void FillNamesHints() override {};
-    void ParseText(std::string_view input, int /*cursor_p*/, bool /*pedantic*/) override {}
+    void ParseText(const std::string& input, int /*cursor_p*/, bool /*pedantic*/) override {}
 
 };
 
