@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -23,17 +23,20 @@
 #if !defined(ERROR_H)
 #define ERROR_H
 
-#include<string>
-#include<string_view>
-#include<vector>
-#include<cassert>
+#include <string>
+#include <string_view>
+#include <vector>
+#include <cassert>
 #include <tuple>
 #include <climits>
 #include <algorithm>
 #include <memory>
+#include <compare>
 
 #ifndef _ASSERT
-#define  _ASSERT(A) 
+#define  _ASSERT(A) assert(A)
+#elif !defined(_DEBUG) && !defined(NDEBUG)
+#define NDEBUG
 #endif
 
 class FileLineCol;
@@ -188,7 +191,7 @@ struct ErrorElement
     std::string message;       ///<The original text of the error/warning.
     bool operator < (const ErrorElement &other) const noexcept {return ordering_line < other.ordering_line;}
     bool AppearsSame(const ErrorElement &o) const noexcept { return text==o.text; }  ///<Return true if the text of the two error is identical including line numbers (so we remove the redundant ones.)
-    bool Fits(bool oWarnings, bool oTechnicalInfo) const noexcept { return type==ErrorElement::EType::Error || (type==ErrorElement::EType::Warning && oWarnings) || (type==ErrorElement::EType::TechnicalInfo && oTechnicalInfo); }    
+    bool Fits(bool oWarnings, bool oTechnicalInfo) const noexcept { return type==ErrorElement::EType::Error || (type==ErrorElement::EType::Warning && oWarnings) || (type==ErrorElement::EType::TechnicalInfo && oTechnicalInfo); }
 };
 
 class Attribute;
@@ -287,19 +290,19 @@ public:
     const ErrorElement *GetError(unsigned num, bool oWarnings, bool oTechnicalInfo) const;
 
     /** Delete the Nth message, if we consider only error or warnings and/or tech info, as well.*/
-    void DeleteError(unsigned num, bool oWarnings, bool oTechnicalInfo) 
+    void DeleteError(unsigned num, bool oWarnings, bool oTechnicalInfo)
     {
         auto err = GetError(num, oWarnings, oTechnicalInfo);
         if (err) Messages.erase(Messages.begin() + (err-&Messages.front()));
     }
     /** Get the location of an error */
     FileLineCol GetErrorLoc(unsigned num, bool oWarnings, bool oTechnicalInfo) const {
-        auto p = GetError(num, oWarnings, oTechnicalInfo); 
+        auto p = GetError(num, oWarnings, oTechnicalInfo);
         return p ? p->relevant_line : FileLineCol();
     }
     /** Get the text of an error, including line numbers and all. */
     std::string_view GetErrorText(unsigned num, bool oWarnings, bool oTechnicalInfo) const {
-        auto p = GetError(num, oWarnings, oTechnicalInfo); 
+        auto p = GetError(num, oWarnings, oTechnicalInfo);
         return p ? p->text : std::string_view{};
     }
     /** Get the message of an error, excluding line numbers and 'error' or 'warning'. */
@@ -336,5 +339,7 @@ inline FileLineColDiff::FileLineColDiff(const FileLineCol &t, const FileLineCol
         col(t.line==o.line ? abs(int(t.col)-int(o.col)) :
             t.line < o.line ? int(o.col)-int(t.col) : int(t.col)-int(o.col)) {}
 
+/** A function that opens a text file of utf8 name. Returns nullptr if file not found. */
+typedef FILE* (OpenNamedFileFunction)(const char*, bool, bool); //write, binary
 
 #endif
