@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -159,24 +159,25 @@ public:
         ColorType fillcolor;    ///<The fill color, we have extracted, fully transparent if none
     };
 
-    OptAttr<ColorType>   color;             ///<The color of the font. Not set if `is_set` is false.
-    OptAttr<ColorType>   bgcolor;           ///<The color behind the text. Not set if `is_set` is false.
-    OptAttr<EFontType>   fontType;          ///<The size/index of the font. Not set if `is_set` is false.
-    OptAttr<double>      spacingBelow;      ///<The extra spacing to add below this line. Not set if `is_set` is false.
-    OptAttr<ETriState>   bold;              ///<Indicates if the font is thick. Not set if `is_set` is false.
-    OptAttr<ETriState>   italics;           ///<Indicates if the font is italic. Not set if `is_set` is false.
-    OptAttr<ETriState>   underline;         ///<Indicates if the font is underlined. Not set if `is_set` is false.
-    OptAttr<std::string> face;              ///<The face name of the font. Not set if `is_set` is false.
+    OptAttr<ColorType>   color;        ///<The color of the font.
+    OptAttr<ColorType>   bgcolor;      ///<The color behind the text.
+    OptAttr<EFontType>   fontType;     ///<The size/index of the font.
+    OptAttr<double>      spacingBelow; ///<The extra spacing to add below this line.
+    OptAttr<ETriState>   bold;         ///<Indicates if the font is thick.
+    OptAttr<ETriState>   italics;      ///<Indicates if the font is italic.
+    OptAttr<ETriState>   underline;    ///<Indicates if the font is underlined.
+    OptAttr<std::string> face;         ///<The face name of the font.
+    OptAttr<std::string> lang;         ///<The language to use when selecting the font.
 
-    OptAttr<double>     textHGapPre;        ///<The margin left of the text. Not set if `is_set` is false.
-    OptAttr<double>     textHGapPost;       ///<The margin right of the text. Not set if `is_set` is false.
-    OptAttr<double>     textVGapAbove;      ///<The margin above the text. Not set if `is_set` is false.
-    OptAttr<double>     textVGapBelow;      ///<The margin below the text. Not set if `is_set` is false.
-    OptAttr<double>     textVGapLineSpacing;///<The spacing between the lines of the text. Not set if `is_set` is false.
-    OptAttr<EIdentType> ident;              ///<The identation/alignment of the text. Not set if `is_set` is false.
+    OptAttr<double>     textHGapPre;        ///<The margin left of the text.
+    OptAttr<double>     textHGapPost;       ///<The margin right of the text.
+    OptAttr<double>     textVGapAbove;      ///<The margin above the text.
+    OptAttr<double>     textVGapBelow;      ///<The margin below the text.
+    OptAttr<double>     textVGapLineSpacing;///<The spacing between the lines of the text.
+    OptAttr<EIdentType> ident;              ///<The identation/alignment of the text.
 
-    OptAttr<double>      normalFontSize;    ///<The height of normal-sized font. Not set if `is_set` is false.
-    OptAttr<double>      smallFontSize;     ///<The height of small, superscript and subscript font. Not set if `is_set` is false.
+    OptAttr<double>      normalFontSize;    ///<The height of normal-sized font.
+    OptAttr<double>      smallFontSize;     ///<The height of small, superscript and subscript font.
 
     OptAttr<bool>        word_wrap;         ///<If true, this label shall be word wrapped
     OptAttr<std::string> link_format;       ///<What formatting to apply to Links
@@ -221,7 +222,7 @@ public:
           TEXT_FORMAT,   ///<The text is a value assigned to "text.format" attribute. Cannot contain numbering format token escapes ("\0x2{1aAiI}").
           NUMBER_FORMAT  ///<The text is a value assigned to "numbering.*" attribute. Cannot contain numbering escapes ("\N").
       };
-    /** Generate an empty formatting, no `is_set` is set to true.
+    /** Generate an empty formatting, no attribute is set.
      * This is contrary to all other attributes, for which the default
      * constructor generates a fully specified object with the default values.*/
     StringFormat() {Empty();}
@@ -234,15 +235,15 @@ public:
     /** Parse `s` up until the first non-escape or non-formatting escape and assignes values.*/
     explicit StringFormat(std::string_view s) {Empty(); Apply(s);}
 
-    /** Deletes all values, setting `is_set` values to false.*/
+    /** Deletes all values, clearing all attributes. */
     void Empty();
     /** True if all attributes of text formatting is specified.*/
     bool IsComplete() const;
     /** True if none of the attributes of text formatting are specified.*/
     bool IsEmpty() const;
     void Default();
-    void UnsetWordWrap() {word_wrap.is_set = true; word_wrap.value = false;} ///<Unset the wordwrap attribute
-    bool IsWordWrap() const {_ASSERT(word_wrap.is_set); return word_wrap.value;} ///<True if wordwrap is on.
+    void UnsetWordWrap() {word_wrap = false;} ///<Unset the wordwrap attribute
+    bool IsWordWrap() const {_ASSERT(word_wrap); return word_wrap.value_or(false); } ///<True if wordwrap is on.
 
     /** Parse a sequence of escapes and apply the formatting to us. 
      * If you hit something like a non-formatting escapea or a bad 
@@ -261,7 +262,7 @@ public:
      * If you hit something like a non-formatting escapea or a bad formatting one or 
      * one that includes style/color name, stop and return.*/
     StringFormat &operator +=(const char*s) {Apply(s); return *this;}
-    /** Merge another StringFormat to us by copying all attributes where `is_set` is set there.*/
+    /** Merge another StringFormat to us by copying all attributes that are set. */
     StringFormat &operator +=(const StringFormat& toadd);
     /** Determines what do we need to add to 'base' in order to get to us. (We must be complete.)*/
     StringFormat &operator -=(const StringFormat& base);
@@ -276,10 +277,10 @@ public:
 
     /** Returns the ident value, a MSC_IDENT_CENTER if not set.*/
     EIdentType GetIdent() const
-        {return ident.is_set?ident.value:MSC_IDENT_CENTER;}
+        {return ident.value_or(MSC_IDENT_CENTER); }
     /** Returns the extra spacing below this line, 0 if not set.*/
     double getSpacingBelow(void) const
-        {return spacingBelow.is_set?spacingBelow.value:0;}
+        {return spacingBelow.value_or(0);}
     /** Prints all attributes set as escape sequences */
     string Print() const;
 
@@ -304,6 +305,7 @@ public:
     static bool ReplaceHiddenEscapes(string& text, bool to_its_param = false, std::string_view to = {});
     static bool ReplaceAsteriskEscapes(string &text, std::string_view to);
     static size_t FindVerbatim(std::string_view decorated, std::string_view find, std::string_view asterisk_replacement = "*");
+    static size_t FindPosInDecorated(std::string_view decorated, size_t plain_pos);
     static StringWithPosList Split(std::string_view n, FileLineCol start = FileLineCol(),
                                    std::string_view separator = ",", std::string_view action = "");
     /** Splits a list by a verbatim character. The all the other escapes kept intact. */
@@ -389,6 +391,8 @@ public:
         { return heightAboveBaseLine; }
     const std::string &GetLine() const { return line; }
     const StringFormat &GetStartFormat() const { return startFormat; }
+    const XY GetScale() const noexcept { return scale; }
+    bool EndsInHardNewLine() const noexcept { return hard_new_line; }
 };
 
 /** A class holding a list of parsed lines*/
@@ -425,6 +429,8 @@ public:
      * @returns the number of lines in the label.*/
     size_t Set(std::string_view s, Canvas &c, const ShapeCollection &shapes, StringFormat f, const XY &scale = {1,1});
     /** Set the scaling of the label. Results in re-calculation of the dimensions.*/
+    void SetScale(double s) { SetScale(s,s); }
+    /** Set the scaling of the label. Results in re-calculation of the dimensions.*/
     void SetScale(double x, double y);
     /** Set the scaling of the label. Results in re-calculation of the dimensions.*/
     void SetScale(const XY &xy) { SetScale(xy.x, xy.y); }
@@ -457,7 +463,7 @@ public:
      * If it is on, it is the first parameter */
     double getSpaceRequired(double def = 0, int line = -1) const;
     /** Returns true if word wrapping is enabled for this label */
-    bool IsWordWrap() const {if (size()==0) return false; return at(0).startFormat.word_wrap.GetValue(false);} 
+    bool IsWordWrap() const {if (size()==0) return false; return at(0).startFormat.word_wrap.value_or(false);} 
     /** Creates one ISMapElement for each link.
      * @param [out] ismap Place generated elements into this collection.
      * @param canvas The canvas to calculate geometry on.
