@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -86,12 +86,9 @@ void SimpleStyle::MakeCompleteButText()
     if (f_shadow) shadow.MakeComplete();
     else shadow.Empty();
     //text untouched
-    numbering.is_set = f_numbering;
-    numbering.value = false;
-    shape.is_set = true;
-    shape.value = -1; //no shape
-    shape_size.is_set = true;
-    shape_size.value = EArrowSize::SMALL;
+    if (f_numbering) numbering = false;
+    shape = -1; //no shape
+    shape_size = EArrowSize::SMALL;
 }
 
 /** Make the style empty be unsetting all attributes it contains.*/
@@ -102,9 +99,9 @@ void SimpleStyle::Empty()
     fill.Empty();
     text.Empty();
     shadow.Empty();
-    numbering.is_set = false;
-    shape.is_set = false;
-    shape_size.is_set = false;
+    numbering.reset();
+    shape.reset();
+    shape_size.reset();
 }
 
 /** Merge another style to us by copying the value of those attributes which are contained by both and are set in `toadd`.*/
@@ -118,10 +115,10 @@ Style & SimpleStyle::operator +=(const Style &toadd)
     if (p->f_fill && f_fill) fill += p->fill;
     if (p->f_shadow &&f_shadow) shadow += p->shadow;
     if (p->f_text && f_text) text += p->text;
-    if (p->f_numbering && f_numbering && p->numbering.is_set) numbering = p->numbering;
+    if (p->f_numbering && f_numbering && p->numbering) numbering = p->numbering;
     if (p->f_shape && f_shape) {
-        if (p->shape.is_set) shape = p->shape;
-        if (p->shape_size.is_set) shape_size = p->shape_size;
+        if (p->shape) shape = p->shape;
+        if (p->shape_size) shape_size = p->shape_size;
     }
     return *this;
 }
@@ -177,15 +174,14 @@ bool SimpleStyle::AddAttribute(const Attribute &a, Chart*chart)
     if (a.Is("number") && f_numbering) {
         if (a.type == EAttrType::CLEAR) {
             if (a.EnsureNotClear(chart->Error, type))
-                numbering.is_set = false;
+                numbering.reset();
             return true;
         }
         if (a.type != EAttrType::BOOL) {
             chart->Error.Error(a, true, "The 'number' attribute must be 'yes' or 'no' for styles. Ignoring it.");
             return true;
         }
-        numbering.is_set = true;
-        numbering.value = a.yes;
+        numbering = a.yes;
         return true;
     }
     if (f_shape) {
@@ -193,7 +189,7 @@ bool SimpleStyle::AddAttribute(const Attribute &a, Chart*chart)
             if (chart->SkipContent())
                 return true;
             if (a.type == EAttrType::CLEAR) {
-                shape.is_set = false;
+                shape.reset();
                 return true;
             }
             if (!a.CheckType(EAttrType::STRING, chart->Error))
@@ -202,9 +198,8 @@ bool SimpleStyle::AddAttribute(const Attribute &a, Chart*chart)
             if (msg.first.length()) {
                 chart->Error.Error(a, true, msg.first+" Ignoring attribute.", msg.second);
             } else {
-                shape.is_set = true;
-                shape.value = chart->Shapes.GetShapeNo(a.value);
-                _ASSERT(shape.value>=0);
+                shape = chart->Shapes.GetShapeNo(a.value);
+                _ASSERT(*shape>=0);
             }
             return true;
         }
@@ -215,15 +210,14 @@ bool SimpleStyle::AddAttribute(const Attribute &a, Chart*chart)
                 return true;
             }
             if (a.type == EAttrType::CLEAR) {
-                shape_size.is_set = false; 
+                shape_size.reset(); 
                 return true;
             }
-            if (a.type == EAttrType::STRING &&
-                Convert(a.value, shape_size.value)) {
-                shape_size.is_set = true;
+            if (EArrowSize s; a.type == EAttrType::STRING && Convert(a.value, s)) {
+                shape_size = s;
                 return true;
-            }
-            a.InvalidValueError(CandidatesFor(shape_size.value), chart->Error);
+            } else 
+                a.InvalidValueError(CandidatesFor<EArrowSize>(), chart->Error);
             return true;
         }
     }
