@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -131,7 +131,7 @@ public:
     FillAttr            fill;      ///<The fill attributes
     ShadowAttr          shadow;    ///<The shadow attributes
     StringFormat        text;      ///<The text attributes
-    OptAttr<bool>       numbering; ///<The value of the 'number' attribute. Not set if `is_set` is false.
+    OptAttr<bool>       numbering; ///<The value of the 'number' attribute.
     OptAttr<int>        shape;     ///<The shape we shall use when drawing the entity. -1 if no shape to use.
     OptAttr<EArrowSize> shape_size;///<Its size
 
@@ -194,7 +194,7 @@ public:
     ~AttrCoW() { Dereference(); }
     /** Applies another style to us */
     AttrCoW &operator +=(const AttrCoW &o) { AttrCoW<Attr> tmp(*this); tmp.write() += o.read(); if (tmp.read()!=read()) operator=(tmp); return *this; }
-    AttrCoW &operator = (const AttrCoW &o) { Dereference(); p = o.p; p->ref_count++; return *this; }
+    AttrCoW& operator = (const AttrCoW& o) { if (&o!=this) { Dereference(); p = o.p; p->ref_count++; }return *this; }
     AttrCoW &operator = (const Attr &o) { Dereference(); p = new ReferencedAttr<Attr>(o); p->ref_count = 1;  return *this; }
     AttrCoW &operator = (Attr &&o) { Dereference(); p = new ReferencedAttr<Attr>(std::move(o)); p->ref_count = 1;  return *this; }
     /** Take a reference to the MscStyle object for reading */
@@ -365,14 +365,15 @@ public:
     StyleCoW(const StyleCoW &o) : p(o.p) {p->ref_count++;}
     ~StyleCoW() {Dereference();}
     /** Applies another style to us */
-    StyleCoW &operator +=(const StyleCoW &o) {write()+=o.read(); return *this;}
-    StyleCoW &operator = (const StyleCoW &o) {Dereference(); p=o.p; p->ref_count++; return *this;}
+    StyleCoW &operator +=(const StyleCoW &o) { if (&o!=this) write()+=o.read(); return *this;}
+    StyleCoW& operator = (const StyleCoW& o) { if (&o!=this) { Dereference(); p = o.p; p->ref_count++; } return *this; }
     StyleCoW &operator = (const St &o) { Dereference(); p = new St(o); p->ref_count = 1;  return *this; }
     StyleCoW &operator = (St &&o) { Dereference(); p = new St(std::move(o)); p->ref_count = 1;  return *this; }
     /** Take a reference to the MscStyle object for reading */
     const St &read() const {return *p;}
     /** Take a reference to the Style object for writing */
     St &write() { if (p->ref_count>1) { p->ref_count--; p = new St(*p); p->ref_count = 1; } return *p; }
+    bool is_same_as(const StyleCoW<St>& o) const noexcept { return p==o.p; }
 };
 
 /** Empty, abstract base class for style sets. */
@@ -654,10 +655,8 @@ void ContextBase<St>::Plain()
     Empty();
     Context::Plain(); //colors and is_full is set here
     numberingStyle.Reset();
-    numberingStyle.pre.is_set = true;
-    numberingStyle.pre.value.clear();
-    numberingStyle.post.is_set = true;
-    numberingStyle.post.value = ": ";
+    numberingStyle.pre = "";
+    numberingStyle.post = ": ";
 
     //Ok, now "weak" and "strong" - if we are descendants of SimpleStyle
     St style = St(EStyleType::STYLE); //has everything, but is empty
