@@ -1,13 +1,9 @@
 /** @file version.h Containing version macros.
  * @ingroup libmscgen_files */
-
-#ifndef VERSION_H
-#define VERSION_H
+#pragma once
 /** The major version of libmscgen.*/
-#define LIBMSCGEN_MAJOR 7
+#define LIBMSCGEN_MAJOR 8
 /** The minor version of libmscgen.*/
 #define LIBMSCGEN_MINOR 1
 /** The micro version of libmscgen.*/
 #define LIBMSCGEN_SUPERMINOR 0
-
-#endif //VERSION_H
