@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2021 Zoltan Turanyi
+	Copyright (C) 2008-2022 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -59,12 +59,12 @@ FlowChart::FlowChart(FileReadProcedure *p, void *param) : ChartBase(p, param)
     Designs.emplace("plain", Contexts.back());
 }
 
-FlowElement *FlowChart::PopContext()
+std::unique_ptr<FlowElement> FlowChart::PopContext()
 {
     Contexts.pop_back();
     //you may return an instruction if needed. It will be the last instruction
     //in the list of instructions for this scope
-    return nullptr;
+    return {};
 }
 
 
@@ -141,10 +141,10 @@ unsigned FlowChart::ParseText(std::string_view input, std::string_view filename)
 }
 
 
-void FlowChart::CompleteParse(Canvas::EOutputType ot, bool avoidEmpty, bool autoPaginate, bool addHeading,
-    XY pageSize, bool fitWidth, bool collectLinkInfo)
+void FlowChart::CompleteParse(bool autoPaginate, bool addHeading,
+                              XY pageSize, bool fitWidth, bool collectLinkInfo)
 {
-    Canvas canvas(ot);
+    Canvas canvas(Canvas::Empty::Query);
     pageBreakData.clear();
     //Consider the copyright text
     StringFormat sf;
@@ -156,17 +156,18 @@ void FlowChart::CompleteParse(Canvas::EOutputType ot, bool avoidEmpty, bool auto
     ConvertToSeries();
 
     //Arrange blocks
-    Layout(ot);
+    Layout();
 
     //Reorder columns to minimize arrows crossing columns
     BestOrdering();
 
     //Layout again
-    Layout(ot);
+    Layout();
 
     //PostPosProcess
-    PostPosProcess(ot);
+    PostPosProcess();
 
+    total += Block(0, 1, 0, 1); //have at least this size
     if (total.x.till < crTexSize.x) total.x.till = crTexSize.x;
     if (pageBreakData.size()==0)
         pageBreakData.emplace_back(XY(0, 0), XY(0, 0)); //we *must* have one page
@@ -343,9 +344,9 @@ NodeRef FlowChart::FindRefNameInSeries(const std::string & refname) const
 void FlowChart::DeleteItemFromSeries(NodeRef t, NodeRef replaceto)
 {
     _ASSERT(t.col < (int)series.size());
-    _ASSERT(t.element < (int)series[t.col].nodes.size());
+    _ASSERT(t.element < series[t.col].nodes.size());
     _ASSERT(replaceto.col < (int)series.size());
-    _ASSERT(replaceto.element < (int)series[replaceto.col].nodes.size());
+    _ASSERT(replaceto.element < series[replaceto.col].nodes.size());
 
     auto adjust = [t, replaceto](Arrow& a) {
         if (a.target.col != t.col) return;
@@ -384,13 +385,13 @@ void FlowChart::DeleteColumnFromSeries(int col)
 }
 
 
-void FlowChart::Layout(Canvas::EOutputType ot) {
+void FlowChart::Layout() {
     if (series.size()==0) {
         total = Block(0, 0, 0, 0);
         return;
     }
     _ASSERT(IsHorizontal(sdir)!=IsHorizontal(ndir)); //ndir and sdir must be perpendicular 
-    Canvas canvas(ot);
+    Canvas canvas(Canvas::Empty::Query);
     //Calculate node sizes
     for (auto& sl : series)
         for (auto& fn : sl.nodes) {
@@ -461,9 +462,9 @@ XY flow::FlowChart::GetKneePoint(const KneePos & k) const
     return ret;
 }
 
-void FlowChart::PostPosProcess(Canvas::EOutputType ot)
+void FlowChart::PostPosProcess()
 {
-    Canvas canvas(ot);
+    Canvas canvas(Canvas::Empty::Query);
     for (auto &sl : series)
         for (auto &n : sl.nodes)
             n.FlowPostPosProcess(canvas);
