@@ -62,7 +62,7 @@ static inline int ImTextCharToUtf8(char *buf, int buf_size, unsigned int c) {
 
 unsigned TextEditor::Line::num_chars() const noexcept {
 	if (num_char>=0) {
-		_ASSERT(num_char == calculate_num_chars());
+		_ASSERT(num_char == (int)calculate_num_chars());
 	} else {
 		num_char = calculate_num_chars();
 	}
@@ -97,8 +97,9 @@ TextEditor::TextEditor()
 	, mHandleMouseInputs(true)
 	, mIgnoreImGuiChild(false)
 	, mShowWhitespaces(true)
+	, mAlwaysRenderCursor(false)
 	, mStartTime(clock::now())
-	, mLastClick(-1.0f) 
+	, mLastClick(-1.0f)
 	, mFonts{}
 	, mFontSize(13.0f)
 {
@@ -186,18 +187,21 @@ std::string TextEditor::GetText(const Coordinates & aStart, const Coordinates &
 		}
 	}
 
+	if (result.size() && result.back()=='\n' && lend>=(int)mLines.size()-1)
+		result.pop_back();
+
 	return result;
 }
 
 /** Search from the current cursor position. Set the selection to the text found & return true;
  * or do nothing & return false.
- * @param [in] what The UTF-8 text to look for. 
+ * @param [in] what The UTF-8 text to look for.
  * @param [in] case_sensitive If false, we find text where only the letter case is different.
- * @param [in] whole_words_only If set, we only find text which as no alphanum chars before & 
+ * @param [in] whole_words_only If set, we only find text which as no alphanum chars before &
  *             after - but only if the search string starts or ends with an alphanum char, resp.
  * @param [in] fw When true we search for items after the current cursor.
  * @param [in] selection When set the search is limited to this selection.
- * @param [in] last_found_selected When set we are continuing the search and the current 
+ * @param [in] last_found_selected When set we are continuing the search and the current
  *             selection still corresponds to the previously found match of the search str.*/
 bool TextEditor::Find(std::string_view what, bool case_sensitive, bool whole_words_only, bool fw,
 					  const std::optional<std::pair<Coordinates, Coordinates>>& selection,
@@ -428,7 +432,7 @@ void TextEditor::AddUndo(UndoRecord& aValue)
 	//	aValue.mRemoved.c_str(), aValue.mRemovedStart.mLine, aValue.mRemovedStart.mColumn, aValue.mRemovedEnd.mLine, aValue.mRemovedEnd.mColumn,
 	//	aValue.mAfter.mCursorPosition.mLine, aValue.mAfter.mCursorPosition.mColumn
 	//	);
-	if (mUndoIndex>0 && (int)mUndoBuffer.size()>=mUndoIndex 
+	if (mUndoIndex>0 && (int)mUndoBuffer.size()>=mUndoIndex
 		    &&  mUndoBuffer[mUndoIndex-1].Combine(aValue))
 		return;
 	mUndoBuffer.resize((size_t)(mUndoIndex + 1));
@@ -612,7 +616,7 @@ int TextEditor::GetCharacterIndex(const Coordinates& aCoordinates) const
 		return -1;
 	auto& line = mLines[aCoordinates.mLine];
 	int i = 0;
-	for (int c=0; i < (int)line.size() && c < aCoordinates.mColumn; c++) 
+	for (int c=0; i < (int)line.size() && c < aCoordinates.mColumn; c++)
 		i += UTF8CharLength(line[i].mChar);
 	return i;
 }
@@ -671,7 +675,7 @@ void TextEditor::RemoveLine(int aStart, int aEnd)
 
 	ErrorMarkers etmp;
 	for (auto &i : mErrorMarkers)
-		if (i.first.mLine<aStart) 
+		if (i.first.mLine<aStart)
 			etmp.insert(etmp.end(), std::move(i));
 		else if (i.first.mLine>=aEnd)
 			etmp.insert(etmp.end(), {Coordinates(i.first.mLine - (aEnd-aStart), i.first.mColumn), std::move(i.second)});
@@ -729,14 +733,14 @@ TextEditor::GlyphStyle TextEditor::GetGlyphStyle(const Glyph & aGlyph) const
 	return mPalette[aGlyph.mColorIndex];
 }
 
-void TextEditor::HandleKeyboardInputs()
+void TextEditor::HandleKeyboardInputs(bool only_when_focused)
 {
 	ImGuiIO& io = ImGui::GetIO();
 	auto shift = io.KeyShift;
 	auto ctrl = io.ConfigMacOSXBehaviors ? io.KeySuper : io.KeyCtrl;
 	auto alt = io.ConfigMacOSXBehaviors ? io.KeyCtrl : io.KeyAlt;
 
-	if (ImGui::IsWindowFocused())
+	if (ImGui::IsWindowFocused() || !only_when_focused)
 	{
 		if (ImGui::IsWindowHovered())
 			ImGui::SetMouseCursor(ImGuiMouseCursor_TextInput);
@@ -745,55 +749,55 @@ void TextEditor::HandleKeyboardInputs()
 		io.WantCaptureKeyboard = true;
 		io.WantTextInput = true;
 
-		if (!IsReadOnly() && ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Z)))
+		if (!IsReadOnly() && ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGuiKey_Z))
 			Undo();
-		else if (!IsReadOnly() && !ctrl && !shift && alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Backspace)))
+		else if (!IsReadOnly() && !ctrl && !shift && alt && ImGui::IsKeyPressed(ImGuiKey_Backspace))
 			Undo();
-		else if (!IsReadOnly() && ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Y)))
+		else if (!IsReadOnly() && ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGuiKey_Y))
 			Redo();
-		else if (!ctrl && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_UpArrow)))
+		else if (!ctrl && !alt && ImGui::IsKeyPressed(ImGuiKey_UpArrow))
 			MoveUp(1, shift);
-		else if (!ctrl && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_DownArrow)))
+		else if (!ctrl && !alt && ImGui::IsKeyPressed(ImGuiKey_DownArrow))
 			MoveDown(1, shift);
-		else if (!alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_LeftArrow)))
+		else if (!alt && ImGui::IsKeyPressed(ImGuiKey_LeftArrow))
 			MoveLeft(1, shift, ctrl);
-		else if (!alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_RightArrow)))
+		else if (!alt && ImGui::IsKeyPressed(ImGuiKey_RightArrow))
 			MoveRight(1, shift, ctrl);
-		else if (!alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_PageUp)))
+		else if (!alt && ImGui::IsKeyPressed(ImGuiKey_PageUp))
 			MoveUp(GetPageSize() - 4, shift);
-		else if (!alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_PageDown)))
+		else if (!alt && ImGui::IsKeyPressed(ImGuiKey_PageDown))
 			MoveDown(GetPageSize() - 4, shift);
-		else if (!alt && ctrl && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Home)))
+		else if (!alt && ctrl && ImGui::IsKeyPressed(ImGuiKey_Home))
 			MoveTop(shift);
-		else if (ctrl && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_End)))
+		else if (ctrl && !alt && ImGui::IsKeyPressed(ImGuiKey_End))
 			MoveBottom(shift);
-		else if (!ctrl && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Home)))
+		else if (!ctrl && !alt && ImGui::IsKeyPressed(ImGuiKey_Home))
 			MoveHome(shift);
-		else if (!ctrl && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_End)))
+		else if (!ctrl && !alt && ImGui::IsKeyPressed(ImGuiKey_End))
 			MoveEnd(shift);
-		else if (!IsReadOnly() && !io.KeySuper && !shift && !io.KeyAlt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Delete)))
+		else if (!IsReadOnly() && !io.KeySuper && !shift && !io.KeyAlt && ImGui::IsKeyPressed(ImGuiKey_Delete))
 			Delete(io.KeyCtrl);
-		else if (!IsReadOnly() && !io.KeySuper && !shift && !io.KeyAlt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Backspace)))
+		else if (!IsReadOnly() && !io.KeySuper && !shift && !io.KeyAlt && ImGui::IsKeyPressed(ImGuiKey_Backspace))
 			Backspace(io.KeyCtrl);
-		else if (!ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Insert)))
+		else if (!ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGuiKey_Insert))
 			mOverwrite ^= true;
-		else if (ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Insert)))
+		else if (ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGuiKey_Insert))
 			Copy();
-		else if (ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_C)))
+		else if (ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGuiKey_C))
 			Copy();
-		else if (!IsReadOnly() && !ctrl && shift && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Insert)))
+		else if (!IsReadOnly() && !ctrl && shift && !alt && ImGui::IsKeyPressed(ImGuiKey_Insert))
 			Paste();
-		else if (!IsReadOnly() && ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_V)))
+		else if (!IsReadOnly() && ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGuiKey_V))
 			Paste();
-		else if (ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_X)))
+		else if (ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGuiKey_X))
 			Cut();
-		else if (!ctrl && shift && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Delete)))
+		else if (!ctrl && shift && !alt && ImGui::IsKeyPressed(ImGuiKey_Delete))
 			Cut();
-		else if (ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_A)))
+		else if (ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGuiKey_A))
 			SelectAll();
-		else if (!IsReadOnly() && !ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Enter)))
+		else if (!IsReadOnly() && !ctrl && !shift && !alt && ImGui::IsKeyPressed(ImGuiKey_Enter))
 			EnterCharacter('\n', false);
-		else if (!IsReadOnly() && !ctrl && !alt && ImGui::IsKeyPressed(ImGui::GetKeyIndex(ImGuiKey_Tab)))
+		else if (!IsReadOnly() && !ctrl && !alt && ImGui::IsKeyPressed(ImGuiKey_Tab))
 			EnterCharacter('\t', shift);
 
 		if (!IsReadOnly() && !io.InputQueueCharacters.empty())
@@ -890,7 +894,7 @@ void TextEditor::Render()
 	// Ensure fonts (if [0] is empty all is empty)
 	if (!mFonts[0])
 		mFonts.fill(ImGui::GetIO().FontDefault);
-	
+
 	/* Compute mCharAdvance regarding to scaled font size (Ctrl + mouse wheel)*/
 	const float fontSize = mFonts[0]->CalcTextSizeA(mFontSize, FLT_MAX, -1.0f, "#", nullptr, nullptr).x;
 	mCharAdvance = ImVec2(fontSize,  (mFontSize + ImGui::GetStyle().ItemSpacing.y) * mLineSpacing);
@@ -970,7 +974,7 @@ void TextEditor::Render()
 
 			auto lineNoWidth = mFonts[1]->CalcTextSizeA(mFontSize, FLT_MAX, -1.0f, buf, nullptr, nullptr).x;
 			drawList->AddText(mFonts[1], mFontSize, //bold
-							  ImVec2(lineStartScreenPos.x + mTextStart - lineNoWidth, lineStartScreenPos.y), 
+							  ImVec2(lineStartScreenPos.x + mTextStart - lineNoWidth, lineStartScreenPos.y),
 							  mColorLineNumber, buf);
 
 			if (mState.mCursorPosition.mLine == lineNo)
@@ -986,7 +990,7 @@ void TextEditor::Render()
 				}
 
 				// Render the cursor
-				if (focused)
+				if (focused || mAlwaysRenderCursor)
 				{
 					auto timeEnd = clock::now();
 					auto elapsed = timeEnd - mStartTime;
@@ -1858,7 +1862,7 @@ void TextEditor::Backspace(bool aWordMode)
 		} else {
 			const int del = mIndent && mIndentChars.find('\x08')==mIndentChars.npos
 				? mIndent(mIndentData, mLines, '\x08', false, pos, {})
-				: 1; 
+				: 1;
 			mInteractiveEnd = mInteractiveStart = pos;
 			MoveLeft(del, true, false); //select the word left
 			u.mRemoved = GetSelectedText();
@@ -1946,7 +1950,7 @@ void TextEditor::Insert(const char *txt) {
 	if (IsReadOnly())
 		return;
 
-	if (!txt || !*txt)
+	if (!txt)
 		return;
 
 	UndoRecord u;
@@ -1970,6 +1974,7 @@ void TextEditor::Insert(const char *txt) {
 }
 
 bool TextEditor::CanUndo() const
+
 {
 	return !mReadOnly && mUndoIndex > 0;
 }
@@ -2180,6 +2185,7 @@ void TextEditor::UndoRecord::Redo(TextEditor * aEditor)
 }
 
 bool TextEditor::UndoRecord::Combine(const UndoRecord &o) {
+	if (prevent_combine) return false;
 	if (mAfter!=o.mBefore) return false;
 	if (mAdded.empty()!=o.mAdded.empty()) return false;
 	if (mRemoved.empty()!=o.mRemoved.empty()) return false;
