@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2021 Zoltan Turanyi
+	Copyright (C) 2008-2022 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 
  This library offers a wrapper around graphviz.
  It has its own parser and drawing code, but uses graphviz for layout.
- This allows us to add extensions to graphviz, such as subrgaph collapse, 
+ This allows us to add extensions to graphviz, such as subrgaph collapse,
  styles and designs, richer edge symbols, such as => for double line edges
  and coloring and hinting (so that you do not have to remember all attributes).
 
@@ -98,7 +98,7 @@ bool ClusterCollapseMap::Deserialize(std::string_view &s)
 /** Serialize the signature to a string*/
 std::string ClusterCollapseMap::Serialize() const
 {
-    std::string s("0\n"); //version                                           
+    std::string s("0\n"); //version
     s << size() << "\n"; //number of entries
     for (auto &i : *this) {
         s << i.first << "\n";
@@ -159,9 +159,9 @@ void GraphChart::BeforePushScope()
     }
 }
 
-GraphNodePortList * GraphChart::PopContext()
+std::unique_ptr<GraphNodePortList> GraphChart::PopContext()
 {
-    GraphNodePortList *ret = nullptr;
+    std::unique_ptr<GraphNodePortList> ret;
     if (parse_stack.size()) {
         ret = parse_stack.back()->CopyOfNodes();
         parse_stack.pop_back();
@@ -178,13 +178,13 @@ ClusterCollapseMap GraphChart::GetAllCollapsedGUIState() const
     return ret;
 }
 
-void GraphChart::NewGraph(bool d, bool s, const char *name) 
+void GraphChart::NewGraph(bool d, bool s, const char *name)
 {
     Graphs.emplace_back(d, s);
     Graph &GG = Graphs.back();
-    if (name) GG.name = name;  
+    if (name) GG.name = name;
     else GG.name << "@" << Graphs.size();
-    lst_scope_is_graph = true; 
+    lst_scope_is_graph = true;
     cluser_counter = 0;
     GG.style = MyCurrentContext().styles["graph"].read();
     //Add all attributes in the style to the list of attributes to bind
@@ -194,12 +194,12 @@ void GraphChart::NewGraph(bool d, bool s, const char *name)
 
 ///<The current graph we put stuff into.
 
-void GraphChart::FinalizeGraph(bool keep) 
+void GraphChart::FinalizeGraph(bool keep)
 {
     if (keep) {
         Graphs.back().collapsed_edge_style = MyCurrentContext().styles["edge_collapsed"];
-    } else 
-        Graphs.pop_back(); 
+    } else
+        Graphs.pop_back();
 }
 
 GraphNode *GraphChart::CreateNode(Graph &g, const char *name)
@@ -250,10 +250,20 @@ bool GraphChart::AddCommandLineArg(const std::string & /*arg*/)
     return false;
 }
 
-void GraphChart::AddCommandLineOption(const Attribute & /*a*/)
+void GraphChart::AddCommandLineOption(const Attribute& /*a*/)
 {
 }
 
+void GraphChart::SetDefaultFont(const std::string &face, const std::string& /*lang*/,
+                                const FileLineColRange &l) {
+    if (face.length()) {
+        for (GraphContext& context : Contexts)
+            context.SetDefaultFont(face, this, l);
+        for (auto& [name, design] : Designs)
+            design.SetDefaultFont(face, this, l);
+    }
+}
+
 bool GraphChart::DeserializeGUIState(std::string_view s)
 {
     return force_collapse.Deserialize(s);
@@ -290,7 +300,7 @@ bool GraphChart::ControlClicked(Element *arc, EGUIControlType t)
         i.first->second.erase(ii.first);
         if (i.first->second.size()==0)
             force_collapse.erase(i.first);
-        return !ii.second; //if we *did* insert, we actually made no change (as we have removed) 
+        return !ii.second; //if we *did* insert, we actually made no change (as we have removed)
     }
     return ii.second;
 }
@@ -316,7 +326,7 @@ void GraphChart::ApplyForcedLayout(const string & name)
     }
 }
 
-void GraphChart::AddAttributeListToStyleList(const GraphStyle * s, 
+void GraphChart::AddAttributeListToStyleList(const GraphStyle * s,
     const std::list<std::string>* styles)
 {
     if (s==nullptr) return;
@@ -333,8 +343,8 @@ void GraphChart::AddAttributeListToStyleList(const GraphStyle * s,
  bool GraphChart::AddChartOption(const Attribute &a)
 {
     if (a.Is("pedantic")) {
-        if (GraphStyle::CheckBoolValue(a.value.c_str(), MyCurrentContext().pedantic.value))
-            MyCurrentContext().pedantic.is_set = true;
+        if (bool p; GraphStyle::CheckBoolValue(a.value.c_str(), p))
+            MyCurrentContext().pedantic = p;
         else
             GraphStyle::BooleanValueError(a, Error);
         return true;
@@ -345,12 +355,12 @@ void GraphChart::AddAttributeListToStyleList(const GraphStyle * s,
         parse_stack.back()->AddAttribute(a, this);
         GetCurrentGraph().AddAttrName(GraphStyle::GRAPH, a.name);
         return true;
-    } 
+    }
     if (Graphs.size()) {
         GetCurrentGraph().AddAttribute(a, this);
         GetCurrentGraph().AddAttrName(GraphStyle::GRAPH, a.name);
         return true;
-    } 
+    }
     //no graph, it must be a chart option, but we did not recognize it.
     Error.Error(a, false, "Unrecognized chart option. Ignoring it.");
     return false;
@@ -368,7 +378,7 @@ void GraphChart::AddAttributeListToStyleList(const GraphStyle * s,
          MyCurrentContext().styles["node"].write() += *al;
          break;
      case GraphStyle::EDGE:
-         MyCurrentContext().styles["edge"].write() += *al; 
+         MyCurrentContext().styles["edge"].write() += *al;
          //Also add it to all edge refinement styles to mimic graphviz behaviour
          for (auto &s : MyCurrentContext().styles)
              if (s.second.read().type==EStyleType::DEF_ADD &&
@@ -409,7 +419,7 @@ void CollapsibleGraph::Collapse(std::set<std::string>& clusters)
     const bool do_strict = strict || (style.read().attributes.end()!=i && i->second.value=="1");
     //first sanitize our list
     //1. return only those clusters that actually exist
-    for (auto i = clusters.begin(); i!=clusters.end(); /*nope*/) 
+    for (auto i = clusters.begin(); i!=clusters.end(); /*nope*/)
         if (GetSubGraph(*i)) i++;
         else clusters.erase(i++);
     //2. if of two clusters on the list one contains the other, collapse only the outer one
@@ -490,10 +500,9 @@ void CollapsibleGraph::Collapse(std::set<std::string>& clusters)
     }
 }
 
-void GraphChart::CompleteParse(Canvas::EOutputType ot, bool avoidEmpty, bool autoPaginate, bool addHeading,
-    XY pageSize, bool fitWidth, bool collectLinkInfo)
-{
-    Canvas canvas(ot);
+void GraphChart::CompleteParse(bool autoPaginate, bool addHeading,
+                               XY pageSize, bool fitWidth, bool collectLinkInfo) {
+    Canvas canvas(Canvas::Empty::Query);
     pageBreakData.clear();
     //Consider the copyright text
     StringFormat sf;
@@ -501,9 +510,9 @@ void GraphChart::CompleteParse(Canvas::EOutputType ot, bool avoidEmpty, bool aut
     XY crTexSize = Label(copyrightText, canvas, Shapes, sf).getTextWidthHeight().RoundUp();
     copyrightTextHeight = crTexSize.y;
     if (do_orig_lang) {
-        orig_lang_CompleteParse(ot, avoidEmpty, autoPaginate, addHeading, pageSize, fitWidth, collectLinkInfo);
+        orig_lang_CompleteParse(autoPaginate, addHeading, pageSize, fitWidth, collectLinkInfo);
     } else {
-        Canvas canvas(ot);
+        Canvas canvas(Canvas::Empty::Query);
         for (auto g = Graphs.begin(); g!=Graphs.end(); /*nope*/) {
             auto i = force_collapse.find(g->name);
             if (i!=force_collapse.end()) {
@@ -533,9 +542,9 @@ void GraphChart::CompleteParse(Canvas::EOutputType ot, bool avoidEmpty, bool aut
             pageBreakData.emplace_back(g->boundingBox.UpperLeft(), g->boundingBox.Spans());
             if (pageBreakData.back().wh.x < crTexSize.x) pageBreakData.back().wh.x = crTexSize.x;
             g++;
-        }                               
+        }
     }
-    total += Block(0, 10, 0, 10); //have at least this size
+    total += Block(0, 1, 0, 1); //have at least this size
     if (total.x.till < crTexSize.x) total.x.till = crTexSize.x;
     if (pageBreakData.size()==0)
         pageBreakData.emplace_back(XY(0, 0), XY(0, 0)); //we *must* have one page
@@ -635,7 +644,7 @@ unsigned GraphChart::orig_lang_ParseText(std::string_view input, std::string_vie
     return ret;
 }
 
-void GraphChart::orig_lang_CompleteParse(Canvas::EOutputType, bool /*avoidEmpty*/, bool /*autoPaginate*/,
+void GraphChart::orig_lang_CompleteParse(bool /*autoPaginate*/,
     bool /*addHeading*/, XY /*pageSize*/, bool /*fitWidth*/, bool /*collectLinkInfo*/)
 {
     if (had_layout) return;
@@ -697,8 +706,8 @@ void GraphChart::orig_lang_DrawComplete(Canvas & c, bool /*pageBreaks*/, unsigne
 
 ///<Returns a set of extensions in order of preference
 
-std::string GraphChart::GetLanguageDefaultText() const 
-{ 
+std::string GraphChart::GetLanguageDefaultText() const
+{
     return
         "#This is the default graph text\n"
         "graph {\n"
