@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -102,17 +102,31 @@ Contour GraphDrawingPrimitive::Cover(const ShapeCollection &shapes) const
 
 void GraphDrawingPrimitive::Draw(Canvas &canvas, const ShapeCollection &shapes) const
 {
-    if (!path.IsEmpty()) {
-        canvas.Line(path, line);
-    } else if (!contour.IsEmpty()) {
-        canvas.Shadow(contour, shadow);
-        canvas.Fill(contour, fill);
-        //Dont stroke if of same color as a solid fill
-        if (fill.gradient.value!=EIntGradType::NONE ||
-            fill.color.value!=line.color.value)
-            canvas.Line(contour, line);
-    } else {
+    if (canvas.does_graphics()) {
+        if (!path.IsEmpty()) {
+            canvas.Line(path, line);
+        } else if (!contour.IsEmpty()) {
+            canvas.Shadow(contour, shadow);
+            canvas.Fill(contour, fill);
+            //Dont stroke if of same color as a solid fill
+            if (fill.gradient!=EIntGradType::NONE || fill.color!=line.color)
+                canvas.Line(contour, line);
+        }
         label.Draw(canvas, shapes, sx, dx, y);
+    } else {
+        if (!path.IsEmpty()) {
+            canvas.Add(GSPath(path, line));
+            label.Draw(canvas, shapes, sx, dx, y);
+        } else if (contour.IsEmpty()) {
+            label.Draw(canvas, shapes, sx, dx, y);
+        } else {
+            //Dont stroke if of same color as a solid fill
+            const LineAttr l = fill.gradient!=EIntGradType::NONE || fill.color!=line.color
+                ? line : LineAttr::None();
+            const Block bb = label.IsEmpty() 
+                ? Block(false) : Block(sx, dx, y, y+label.getTextWidthHeight().y);
+            canvas.Add(GSShape(contour, l, fill, shadow, bb, label));
+        }
     }
 }
 
@@ -214,30 +228,46 @@ void GraphElement::ShiftBy(const XY &xy)
 
 void GraphElement::PostPosProcess(Canvas &canvas, Chart * ch)
 {
+    //add a shadow draw element if needed    
+    if (ShadowAttr shadow = GetShadow(ch); !shadow.IsEmpty()) {
+        //find the largest fill contour
+        const auto max = std::ranges::max_element(drawingList, {}, [](const GraphDrawingPrimitive& d) 
+                                                  { return d.contour.IsEmpty() ? 0 : d.contour.GetArea(); });
+        if (max!=drawingList.end()) {
+            max->shadow = shadow;
+        } else {
+            GraphDrawingPrimitive d(Contour(area), LineAttr::None());
+            d.shadow = std::move(shadow);
+            drawingList.insert(drawingList.begin(), std::move(d));
+        }
+    }
+    //Special case: we have one shape and one label element (in this order)
+    if (drawingList.size()==2)
+        if (auto &c = drawingList[0]
+            ; c.path.IsEmpty() && c.label.IsEmpty() && !c.contour.IsEmpty())
+            if (auto& l = drawingList[1]
+                ; l.path.IsEmpty() && l.contour.IsEmpty() && !l.label.IsEmpty()) {
+                //Merge them
+                std::swap(c.label, l.label);
+                c.sx = l.sx; 
+                c.dx = l.dx;
+                c.y = l.y;
+                drawingList.pop_back();
+            }
     //if style is "double" or "triple" adjust the line attributes
-    //these are ChartGen extensions
+    //these are Msc-generator extensions
     auto i = style.read().attributes.find("style");
     if (i!=style.read().attributes.end() &&
         (i->second.value=="double" || i->second.value=="triple"))
-        for (auto &e : drawingList)
-            if (e.line.type.value!=ELineType::NONE &&
-                    Convert(i->second.value, e.line.type.value)) {
-                e.line.width.value = 1;
+        for (auto& e : drawingList)
+            if (ELineType t; !e.line.IsEmpty() && Convert(i->second.value, t)) {
+                e.line.type = t;
+                e.line.width = 1;
                 if (!e.path.IsEmpty())
                     area += e.path.SimpleWiden(e.line.LineWidth());
                 if (!e.contour.IsEmpty())
                     area += e.contour.CreateExpand(e.line.LineWidth()/2);
             }
-    //add a shadow draw element if needed
-    ShadowAttr shadow = GetShadow(ch);
-    if (shadow.offset.value) {
-        //Add our whole area with no line and fill...
-        drawingList.emplace_front(Contour(area), 
-                                  LineAttr(ELineType::NONE, ColorType(), 0, ECornerType::NONE, 0), 
-                                  FillAttr(ColorType(), EGradientType::NONE));
-        //...but with shadow
-        drawingList.front().shadow = shadow;
-    }
     Element::PostPosProcess(canvas, ch); //this expands 'area' (among others)
     if (ch->prepare_for_tracking)
         ch->AllCovers += GetAreaToSearch();
@@ -509,7 +539,7 @@ void Graph::PseudoRenderFrom(Chart*chart)
     boundingBox.MakeInvalid();
     if (in_gv) {
         _ASSERT(_current==nullptr);
-        Canvas c(Canvas::PNG);
+        Canvas c(Canvas::Empty::Query);
         _canvas = &c;
         _chart = chart;
         _err_msg_accum.clear();
@@ -572,7 +602,7 @@ void Graph::AddToGvGraph()
         strict ?
         (directed ? Agstrictdirected : Agstrictundirected) :
         (directed ? Agdirected : Agundirected),
-        NIL(Agdisc_t *));
+        nullptr);
 
     //Bind all attributes first
     BindAttrs(in_gv, AGRAPH, attr_graph);
@@ -631,10 +661,7 @@ void Graph::AdjustBkArea(const Range &total_x) noexcept {
         Block b = drawingList.front().contour.GetBoundingBox();
         b.x += total_x;
         drawingList.front().contour = b;
-    } else {
-        _ASSERT(drawingList.empty()); //We should have a single drawing element for background and nothing else
     }
-
 }
 
 std::optional<BackgroundFill> Graph::GetBkFill() const noexcept {
@@ -812,9 +839,9 @@ void SubGraph::ShiftBy(const XY &xy)
 /** Returns a list of nodes (no ports). 
  * Used during parsing to generate edges for constructs, like
  * "{ a b c } -> { d e f }"*/
-GraphNodePortList *SubGraph::CopyOfNodes() const
+std::unique_ptr<GraphNodePortList> SubGraph::CopyOfNodes() const
 {
-    GraphNodePortList *ret = new GraphNodePortList;
+    std::unique_ptr<GraphNodePortList> ret = std::make_unique<GraphNodePortList>();
     for (auto n : nodes) {
         ret->emplace_back();
         ret->back().node = n;
@@ -913,9 +940,9 @@ inline LineAttr GetLineAttr(const obj_state_t *obj)
     LineAttr line(ELineType::SOLID, ConvertColor(obj->pencolor),
                   obj->penwidth, ECornerType::NONE, 0);
     switch (obj->pen) {
-    case PEN_NONE:   line.type.value = ELineType::NONE; break;
-    case PEN_DASHED: line.type.value = ELineType::DASHED; break;
-    case PEN_DOTTED: line.type.value = ELineType::DOTTED; break;
+    case PEN_NONE:   line.type = ELineType::NONE; break;
+    case PEN_DASHED: line.type = ELineType::DASHED; break;
+    case PEN_DOTTED: line.type = ELineType::DOTTED; break;
     default: break;
     }
     return line;
@@ -925,18 +952,18 @@ inline LineAttr GetLineAttr(const obj_state_t *obj)
 inline FillAttr GetFillAttr(const obj_state_t *obj)
 {
     FillAttr fill; //a completely specified attr with defaults (no gradient, 0.5 frac, zero angle)
-    fill.color.value = ConvertColor(obj->fillcolor);
-    fill.color2.is_set = obj->stopcolor.type == RGBA_BYTE; //else this color is not set: we requested colors in this format
-    if (fill.color2.is_set) {
-        fill.color2.value = ConvertColor(obj->stopcolor);
-        fill.gradient_angle.value = obj->gradient_angle; //linear gradient by def if we have a second color
-        fill.gradient.value = EIntGradType::LINEAR;
-    }
+    fill.color = ConvertColor(obj->fillcolor);
+    if (obj->stopcolor.type == RGBA_BYTE) { //else this color is not set: we requested colors in this format
+        fill.color2 = ConvertColor(obj->stopcolor);
+        fill.gradient_angle = obj->gradient_angle; //linear gradient by def if we have a second color
+        fill.gradient = EIntGradType::LINEAR;
+    } else
+        fill.color2.reset();
     if (obj->fill == FILL_RADIAL)
-        fill.gradient.value = EIntGradType::RADIAL;
-    else if (obj->gradient_frac && fill.color2.is_set) {
-        fill.frac.value = std::max((float)0, std::min((float)1, obj->gradient_frac));
-        fill.gradient.value = EIntGradType::DEGENERATE;
+        fill.gradient = EIntGradType::RADIAL;
+    else if (obj->gradient_frac && fill.color2) {
+        fill.frac = std::max((float)0, std::min((float)1, obj->gradient_frac));
+        fill.gradient = EIntGradType::DEGENERATE;
     }
     return fill;
 }
@@ -1058,10 +1085,9 @@ void canvas_cover_polygon(bool draw, GVJ_t * job, pointf * A, int n, int filled)
             GraphElement *e = gc->GetElement(job->obj->u.n);
             if (e) {
                 gc->boundingBox += c.GetBoundingBox();
-                if (filled)// && job->obj->fill != FILL_NONE)
-                    e->AddDrawingPrimitive(std::move(c), line, fill, Graph::_current->_chart->Shapes);
-                else
-                    e->AddDrawingPrimitive(std::move(c), line);
+                if (!filled)
+                    fill = FillAttr::None();
+                e->AddDrawingPrimitive(std::move(c), line, fill, Graph::_current->_chart->Shapes);
             }
         }
     }
@@ -1142,10 +1168,9 @@ void canvas_cover_ellipse(bool draw, GVJ_t * job, pointf * A, int filled)
             GraphElement *e = gc->GetElement(job->obj->u.n);
             if (e) {
                 gc->boundingBox += c.GetBoundingBox();
-                if (filled)// && job->obj->fill != FILL_NONE)
-                    e->AddDrawingPrimitive(std::move(c), line, fill, Graph::_current->_chart->Shapes);
-                else
-                    e->AddDrawingPrimitive(std::move(c), line);
+                if (!filled)
+                    fill = FillAttr::None();
+                e->AddDrawingPrimitive(std::move(c), line, fill, Graph::_current->_chart->Shapes);
             }
         }
     }
