@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -67,9 +67,9 @@ inline Agedge_t *agedge(Agraph_t * g, Agnode_t * t, Agnode_t * h, const char *na
 
 
 /** Global variable needed by DOT.
- * On windows it is initialized statically in gvraphs.cpp, on Linux in main.cpp 
+ * On windows it is initialized statically in gvraphs.cpp, on Linux in main.cpp
  * (On windows we only use statically linked (well, DLLs) DOT components, on Linux we auto-load them via gvc. */
-extern GVC_t *Gvc;  
+extern GVC_t *Gvc;
 
 /** Read a list of layout plugins from gvc. */
 std::set<std::string> GetLayoutMethods();
@@ -101,13 +101,13 @@ struct GraphDrawingPrimitive
 {
     Contour contour; ///<If the drawing primitive is closed area, we hold it here
     Path path;       ///<If the drawing primitive is an open path, we hold it here
-    LineAttr line;   ///<The attributes of the line to draw, if any
-    FillAttr fill;   ///<The attributes of the fill to use on 'contour' if any
+    LineAttr line = LineAttr::None();   ///<The attributes of the line to draw, if any
+    FillAttr fill = FillAttr::None();   ///<The attributes of the fill to use on 'contour' if any
     ShadowAttr shadow;///<The shadow to draw to 'contour' if any
     Label label;     ///<The text to emit if the drawing primitive is a label
-    double sx;       ///<The left side of the label if the drawing primitive is a label
-    double dx;       ///<The right side of the label if the drawing primitive is a label
-    double y;        ///<The top side of the label if the drawing primitive is a label
+    double sx = 0;    ///<The left side of the label if the drawing primitive is a label
+    double dx = 0;    ///<The right side of the label if the drawing primitive is a label
+    double y = 0;     ///<The top side of the label if the drawing primitive is a label
     GraphDrawingPrimitive(Path &&p, const LineAttr &l) noexcept : path(std::move(p)), line(l) {}
     GraphDrawingPrimitive(Contour &&c, const LineAttr &l) noexcept : path(std::move(c)), line(l) {}
     GraphDrawingPrimitive(Contour &&c, const LineAttr &l, const FillAttr &f) noexcept :
@@ -120,9 +120,9 @@ struct GraphDrawingPrimitive
 };
 
 /** A list of drawing primitives */
-typedef std::list<GraphDrawingPrimitive> DrawingList;
+typedef std::vector<GraphDrawingPrimitive> DrawingList;
 
-/** A base class for all graph elements: graphs, cluster subgraphs, nodes and edges. 
+/** A base class for all graph elements: graphs, cluster subgraphs, nodes and edges.
  * As a descendant to 'Element' it can be tracked and may have element controls.*/
 class GraphElement : public Element
 {
@@ -152,22 +152,22 @@ public:
 };
 
 /** Placeholder for a graph. All nodes and edges refer back to this.
- * It is able to manage descendants of SubGraph, GraphNode and GraphEdge, if they have 
+ * It is able to manage descendants of SubGraph, GraphNode and GraphEdge, if they have
  * constructor of the following syntax (one for defining at top level and one
  * for adding it to a subgraph).
- * - SubGraph(Graph &graph, const char *name, bool is_cluster, ...);      
- * - SubGraph(SubGraph *parent, const char *name, bool is_cluster, ...);  
- * - GraphNode(Graph &graph, const char *name, ...);                                   
+ * - SubGraph(Graph &graph, const char *name, bool is_cluster, ...);
+ * - SubGraph(SubGraph *parent, const char *name, bool is_cluster, ...);
+ * - GraphNode(Graph &graph, const char *name, ...);
  * - GraphNode(SubGraph &parent, const char *name, ...);
  * Or two versions for GraphEdge based on how to handle port string memory ownership
  * - GraphEdge(GraphNode *_1, GraphNode *_2, const std::string &p1, const std::string &p2, const FileLineColRange &pos1, const FileLineColRange &pos2, ...);
  * - GraphEdge(GraphNode *_1, GraphNode *_2, std::string &&p1, std::string &&p2, const FileLineColRange &pos1, const FileLineColRange &pos2, ...);
  * Descendants are free to add any more parameters.
- * These elements will be created by CreateSubGraph(), CreateNode() and CreateEdge() functions, 
+ * These elements will be created by CreateSubGraph(), CreateNode() and CreateEdge() functions,
  * which ensure proper linking inside class Graph.
  *
  * One node can only be in one visible subgraph (cluster==true), but in any number of non-visible ones.
- * So when we insert a node to a visible subgraph, we do the following: 
+ * So when we insert a node to a visible subgraph, we do the following:
  * - If it is not present in any other visible subgraphs, we add it.
  * - If it is present only in one of our parent visible subgraphs, we move it to us.
  * - If it is present in a visible subgraph, that is not our parent, we do not add it.
@@ -185,7 +185,7 @@ protected:
     std::set<std::string> attr_graph; ///<All graph attributes mentioned (so that we can bind them)
 public:
     NPtrList<SubGraph> subgraphs; ///<Only the subgraphs directly in us.
-    NPtrList<GraphNode> nodes;    ///<Nodes appearing directly in us. 
+    NPtrList<GraphNode> nodes;    ///<Nodes appearing directly in us.
     UPtrList<GraphEdge> edges;    ///<All edges of the graph
     const bool directed;          ///<True, if a directed graph.
     const bool strict;            ///<True, if a strict graph.
@@ -199,7 +199,7 @@ public:
     Canvas *_canvas = nullptr;           ///<The canvas during pseudo-rendering.
     std::string _err_msg_accum;       ///<We collect multi-segment DOT error messages in this.
     Chart *_chart = nullptr;             ///<The chart to add errors to during layout or parsing.
-    static int DotError(char *msg);   
+    static int DotError(char *msg);
     /** @} */
 
     Graph(bool d = true, bool s = false) : GraphElement(GraphStyle::GRAPH), directed(d), strict(s) {}
@@ -233,7 +233,7 @@ public:
      * Returns false in *created if non-null and if we have found this node
      * already added (by name). In this case we return the one found.*/
     template <class Node, typename ...Args>
-    GraphNode *CreateNode(const char *name, bool *created, Args &&... args);        
+    GraphNode *CreateNode(const char *name, bool *created, Args &&... args);
     /** Create a node of your choice (see notes for class Graph) and add it
      * at the top level to the graph.
      * Returns false in *created if non-null and if we have found this node
@@ -243,10 +243,10 @@ public:
      * except from 'g' or subgraphs of 'g'. Hopefully this mimics what
      * graphviz is doing. (One node can be in one visible subgraph, at most.*/
     template <class Node, typename ...Args>
-    GraphNode *CreateNode(SubGraph *g, const char *name, bool *created, Args &&... args); 
+    GraphNode *CreateNode(SubGraph *g, const char *name, bool *created, Args &&... args);
     /** Create an edge of your choice (see notes for class Graph) and add it
      * to the graph.
-     * If an edge already exists between these nodes, we return it if the graph is 
+     * If an edge already exists between these nodes, we return it if the graph is
      * strict. If the graph is not directed, then we consider an edge existing if
      * any heads and tails equal or vice versa. If the edge does not yet exist
      * or the graph is not strict, we allocate a new edge (and return true
@@ -255,7 +255,7 @@ public:
     GraphEdge *CreateEdge(GraphNode *_1, GraphNode *_2, const std::string &p1, const std::string &p2, const FileLineColRange &pos1, const FileLineColRange &pos2, bool *created, Args &&... args);
     /** Create an edge of your choice (see notes for class Graph) and add it
      * to the graph.
-     * If an edge already exists between these nodes, we return it if the graph is 
+     * If an edge already exists between these nodes, we return it if the graph is
      * strict. If the graph is not directed, then we consider an edge existing if
      * any heads and tails equal or vice versa. If the edge does not yet exist
      * or the graph is not strict, we allocate a new edge (and return true
@@ -302,7 +302,7 @@ public:
     Graph &graph;                     ///<The graph we belong to.
     SubGraph *const parent;       ///<Our parent subgraph, nullptr if we are directly in a graph.
     const bool cluster;               ///<True, if we are a visible subgraph
-    NPtrList<GraphNode> nodes;    ///<The nodes mentioned directly in us. 
+    NPtrList<GraphNode> nodes;    ///<The nodes mentioned directly in us.
     NPtrList<SubGraph> subgraphs; ///<We own only the directly included subgraphs
 
     virtual ~SubGraph() = default;
@@ -320,7 +320,7 @@ public:
     GraphElement *GetElement(void *in_gv); ///<Return the node, edge or subgraph from their pointer in cgraph (if found)
     NPtrList<GraphNode> GetAllNodes() const;
     virtual void ShiftBy(const XY &) override;
-    GraphNodePortList *CopyOfNodes() const;
+    std::unique_ptr<GraphNodePortList> CopyOfNodes() const;
     virtual void PostPosProcess(Canvas &, Chart *ch) override;
     virtual void RegisterCover(Chart *);
     /** Draw all contained subgraphs (and us too), but no nodes or edges*/
@@ -382,7 +382,7 @@ public:
 
 
 /** Adds the attribute to the list of attribute names to bind.
- * graphviz requires the registration (binding) of all attribute 
+ * graphviz requires the registration (binding) of all attribute
  * names (globally) before used for a given kind of object
  * (graph, edge or node).
  * @param t [in] Specifies what kind of object the attribute is for.
@@ -391,9 +391,9 @@ inline void Graph::AddAttrName(GraphStyle::EGraphElementType t, const std::strin
 {
     if (t==GraphStyle::EGraphElementType::ANY) {
         attr_node.insert(attr_name);
-        attr_edge.insert(attr_name); 
+        attr_edge.insert(attr_name);
         //fallthrough: graph goes below
-    } 
+    }
     std::set<std::string> &attrs = t==GraphStyle::EGraphElementType::NODE ? attr_node :
         t==GraphStyle::EGraphElementType::EDGE ? attr_edge : attr_graph;
     attrs.insert(attr_name);
@@ -403,9 +403,9 @@ inline void Graph::AddAttrName(GraphStyle::EGraphElementType t, const std::strin
 template <class SG, typename ...Args>
 SubGraph *Graph::CreateSubGraph(const char *name, bool cluster, bool *created, Args &&... args)
 {
-    //Do nothing when a user-re-defines a subgraph - we re-use the same one if it exists among our sisters    
+    //Do nothing when a user-re-defines a subgraph - we re-use the same one if it exists among our sisters
     //For unnamed subgraphs: always create a new one
-    auto i = name && name[0] ? 
+    auto i = name && name[0] ?
                  std::find_if(this->subgraphs.begin(), this->subgraphs.end(), [name](auto p) {return p->name==name; }) :
                  this->subgraphs.end();
     SubGraph *p = i==this->subgraphs.end() ? nullptr : *i;
@@ -415,7 +415,7 @@ SubGraph *Graph::CreateSubGraph(const char *name, bool cluster, bool *created, A
         p = up.get();
         this->storage_subgraphs.push_back(std::move(up));
         this->subgraphs.Append(p);
-    } 
+    }
     return p;
 }
 
@@ -424,7 +424,7 @@ SubGraph *Graph::CreateSubGraph(SubGraph *g, const char *name, bool cluster, boo
 {
     _ASSERT(g);
     if (g==nullptr) return this->CreateSubGraph<SubGraph>(name, cluster, created, std::forward<Args>(args)...);
-    //Do nothing when a user-re-defines a subgraph - we re-use the same one if it exists among our sisters    
+    //Do nothing when a user-re-defines a subgraph - we re-use the same one if it exists among our sisters
     //For unnamed subgraphs: always create a new one
     auto i = name && name[0] ?
         std::find_if(g->subgraphs.begin(), g->subgraphs.end(), [name](auto p) {return p->name==name; }) :
@@ -436,7 +436,7 @@ SubGraph *Graph::CreateSubGraph(SubGraph *g, const char *name, bool cluster, boo
         p = up.get();
         this->storage_subgraphs.push_back(std::move(up));
         g->subgraphs.Append(p);
-    } 
+    }
     return p;
 }
 
@@ -472,7 +472,7 @@ GraphNode *Graph::CreateNode(SubGraph *g, const char *name, bool *created, Args
     } else {
         if (g->cluster) {
             SubGraph *host = this->GetNodeFromVisibleSubgraph(p);
-            if (host==nullptr) 
+            if (host==nullptr)
                 nodes.remove(p); //we may have it at top level
             else if (g->IsMyParent(host))
                 host->Remove(p); //If the node is in one of g's ancestors, we move the node to 'g'
@@ -488,8 +488,8 @@ leave:
 
 template <class Edge, typename ...Args>
 GraphEdge *Graph::CreateEdge(GraphNode *_1, GraphNode *_2,
-                             const std::string &p1, const std::string &p2, 
-                             const FileLineColRange &pos1, const FileLineColRange &pos2, 
+                             const std::string &p1, const std::string &p2,
+                             const FileLineColRange &pos1, const FileLineColRange &pos2,
                              bool *created, Args &&... args)
 {
     _ASSERT(_1 && _2);
@@ -509,7 +509,7 @@ GraphEdge *Graph::CreateEdge(GraphNode *_1, GraphNode *_2,
 
 template <class Edge, typename ...Args>
 GraphEdge *Graph::CreateEdge(GraphNode *_1, GraphNode *_2,
-                             std::string && p1, std::string && p2, 
+                             std::string && p1, std::string && p2,
                              const FileLineColRange &pos1, const FileLineColRange &pos2,
                              bool *created, Args &&... args)
 {
