@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -124,18 +124,18 @@ EDirType dir;         ///<Are we forward, backward or bidir or without arrowhead
 * at the location pointed by this structure.*/
 struct ArrowEnding
 {
-    EntityRef   entity;  ///<The iterator of the entity we refer to either from AllEntities or ActiveEntities
-    FileLineCol linenum; ///<The position of the 'start before' or 'end after' clause, if generated from source. In calls to AddStartPos() and AddEndPos() this is just the value specified by the user and not the absolute offset.
-    double      offset;  ///<The offset from the middle of the entity line. Up to the end of AddAttributeList() larger value is more towards the end of the arrow; after that positive is towards right, negative is towards left.
-    ArrowEnding() : entity(nullptr), offset(0) {}
+    EntityRef   entity = nullptr; ///<The iterator of the entity we refer to either from AllEntities or ActiveEntities
+    FileLineCol linenum;          ///<The position of the 'start before' or 'end after' clause, if generated from source. In calls to AddStartPos() and AddEndPos() this is just the value specified by the user and not the absolute offset.
+    double      offset = 0;       ///<The offset from the middle of the entity line. Up to the end of AddAttributeList() larger value is more towards the end of the arrow; after that positive is towards right, negative is towards left.
+    ArrowEnding() noexcept = default;
     ArrowEnding(MscChart &chart, const char *e, const FileLineColRange &l, double o = 0);
     explicit ArrowEnding(EntityRef e, double o = 0, const FileLineCol &l = FileLineCol()) : entity(e), linenum(l), offset(o) {}
     ArrowEnding(const ArrowEnding&) = default;
     /** Compares position of two valid endings. */
-    bool operator < (const ArrowEnding &o) const { return std::tie(entity->pos_exp, offset)<std::tie(o.entity->pos_exp, o.offset); }
-    bool operator > (const ArrowEnding &o) const { return std::tie(entity->pos_exp, offset)>std::tie(o.entity->pos_exp, o.offset); }
+    bool operator < (const ArrowEnding& o) const noexcept { _ASSERT(entity && o.entity); return std::tie(entity->pos_exp, offset)<std::tie(o.entity->pos_exp, o.offset); }
+    bool operator > (const ArrowEnding &o) const noexcept { _ASSERT(entity && o.entity); return std::tie(entity->pos_exp, offset)>std::tie(o.entity->pos_exp, o.offset); }
     /** Compares position of two valid endings. */
-    bool operator ==(const ArrowEnding &o) const { return entity == o.entity && offset == o.offset; }
+    bool operator ==(const ArrowEnding &o) const noexcept { return entity == o.entity && offset == o.offset; }
 };
 
 
@@ -300,6 +300,16 @@ public:
     virtual bool CanBeAlignedTo() const { return false; }
     /** True if implicitly generated entities (MscChart::AutoGenEntities) shall be placed *after* this element (like titles, etc) */
     virtual bool BeforeAutoGenEntities() const { return false; }
+    /** If there is an indicator both before and after this element, we
+     * are NOT allowed to merge them. */
+    virtual bool SeparatesIndicators() const { return true; }
+    /** If this element is followed by a Note, it can store the note for later re-insertion.
+     * This feature is useful for entity commands and arrow joins: when a series of 
+     * instrcutions are merged and we may want to note only one of them.
+     * @returns true if we have stored the Note internally (and took ownership)*/
+    virtual bool StoreNoteTemporarily(Note*) { return false; }
+    /** After PostParseProcess, we can re-insert the notes stored above.*/
+    virtual void ReinsertTmpStoredNotes(ArcList& list, ArcList::iterator after) { };
     /** @} */
 
     /** @name Geometry & other getters
@@ -463,8 +473,10 @@ public:
     /** Set which entities the iterator is between. */
     void SetEntities(EntityRef s, EntityRef d) {src=s; dst=d;}
     /** Expand the `file_pos` member with that of another interator if at same location. 
-     * @returns false, if `src` or `dst` differ (and then no change to `file_pos`.*/
+     * @returns false, if `src` or `dst` differ (and then no change to `file_pos`).*/
     bool Combine(const Indicator *o);
+    /** Expand the `file_pos` member with that of another hidden element. */
+    void ExtendFilePos(const ArcBase* o);
     /** Returns the x coordinate in chart space of the middle of the indicator */
     double GetXCoord() const;
     EDirType GetToucedEntities(class NEntityList &el) const override;
@@ -582,20 +594,20 @@ struct VertXPos
     static bool IsOneEntityPos(EPosType t) { return t==POS_AT || t==POS_LEFT_BY || t==POS_RIGHT_BY || t==POS_LEFT_SIDE || t==POS_RIGHT_SIDE || t==POS_THIRD_LEFT || t==POS_THIRD_RIGHT;  }
     static bool IsTwoEntityPos(EPosType t) { return t==POS_CENTER; }
 
-    bool valid;               ///<True if the position is valid
-    EPosType pos;             ///<The type of the position
-    EntityRef entity1;        ///<The first (or only one) entity
-    EntityRef entity2;        ///<The second entity, used only for POS_CENTER
-    FileLineColRange e1line;  ///<Location of the mention of the first entity in the input line (inside this vertpos fragment)
-    FileLineColRange e2line;  ///<Location of the mention of the second entity in the input line (inside this vertpos fragment)
-    double offset;            ///<An arbitrary user specified horizontal offset on top of the poition defined by `pos`
+    bool valid = true;           ///<True if the position is valid
+    EPosType pos = POS_INVALID;  ///<The type of the position
+    EntityRef entity1 = nullptr; ///<The first (or only one) entity
+    EntityRef entity2 = nullptr; ///<The second entity, used only for POS_CENTER
+    FileLineColRange e1line;     ///<Location of the mention of the first entity in the input line (inside this vertpos fragment)
+    FileLineColRange e2line;     ///<Location of the mention of the second entity in the input line (inside this vertpos fragment)
+    double offset = 0;           ///<An arbitrary user specified horizontal offset on top of the poition defined by `pos`
 
     /** Create (yet) invalid object */
-    VertXPos() : valid(true), pos(POS_INVALID), entity1(nullptr), entity2(nullptr), offset(0) {}
+    VertXPos() noexcept = default;
     /** Create a position with 2 entities (always POS_CENTER*) */
-    VertXPos(MscChart&m, const char *e1, const FileLineColRange &e1l, const char *e2, const FileLineColRange &e2l, double off = 0);
+    VertXPos(MscChart& m, const char* e1, const FileLineColRange& e1l, const char* e2, const FileLineColRange& e2l, double off = 0);
     /** Create a position with one entity (must not be POS_CENTER) */
-    VertXPos(MscChart&m, const char *e1, const FileLineColRange &e1l, EPosType p = POS_AT, double off = 0);
+    VertXPos(MscChart& m, const char* e1, const FileLineColRange& e1l, EPosType p = POS_AT, double off = 0);
     /** Calculate the x coordinate of the middle of the element we specify the location for (only after entities are placed)
     * @param m The chart
     * @param width The width of the object
@@ -612,13 +624,16 @@ struct VertXPos
 class Arrow : public LabelledArc
 {
 protected: 
-    bool headless_mscgen_arrow; ///<True if this is an mscgen syntax arc defined without arrowhead via one of (--, ==, .., ::)
+    bool headless_mscgen_arrow;   ///<True if this is an mscgen syntax arc defined without arrowhead via one of (--, ==, .., ::)
+    StoredNotes tmp_stored_notes; ///<A set of notes attached to us.
 public:
     const bool bidir;           ///<True if the arrow is bidirectional.
     /** This is the typical constructor */
     Arrow(bool b, EArcCategory c, MscChart *msc) : LabelledArc(c, msc), headless_mscgen_arrow(false), bidir(b){}
     /** Constructor used only when converting an Box to and BlockArrow*/
     Arrow(bool b, EArcCategory c, const LabelledArc &al) : LabelledArc(c, al), headless_mscgen_arrow(false), bidir(b) {}
+    Arrow(const Arrow&) = delete;
+    Arrow(Arrow&&) = default;
     /** Get the refinement style for a given arrow symbol 
      * The Arrow:: version returns the style for self and dir arrows, BlockArrow overrides. */
     virtual const MscStyleCoW *GetRefinementStyle4ArrowSymbol(EArcSymbol t) const;
@@ -631,7 +646,7 @@ public:
     virtual Arrow *AddSegment(ArrowSegmentData data, const char *m, const FileLineColRange &ml,
                                  const FileLineColRange &l) = 0;
     /** Add a loss position. */
-    virtual Arrow *AddLostPos(VertXPos *pos, const FileLineColRange &l) = 0;
+    virtual Arrow* AddLostPos(gsl::owner<VertXPos*> pos, const FileLineColRange& l) = 0;
     /** Add the starting position. If ending is null, we assume src is 
      * LSIDE or RSIDE and make it a special ending (used when user specifies pipe symbol)*/
     virtual Arrow *AddStartPos(ArrowEnding *ending=nullptr) = 0;
@@ -640,6 +655,8 @@ public:
     virtual Arrow *AddEndPos(ArrowEnding *ending=nullptr) = 0;
     /** Indicate that this arrow was defined via an mscgen headless arrow symbol (--, ==, ., ::)*/
     void Indicate_Mscgen_Headless() { headless_mscgen_arrow = true; }
+    bool StoreNoteTemporarily(Note* cn) override;
+    void ReinsertTmpStoredNotes(ArcList& list, ArcList::iterator after) override;
     bool AddAttribute(const Attribute &) override;
     static void AttributeNames(Csh &csh);
     static bool AttributeValues(const std::string &attr, Csh &csh);
@@ -686,7 +703,7 @@ public:
                  MscChart *msc, double ys);
     Arrow *AddSegment(ArrowSegmentData data, const char *m, const FileLineColRange &ml, 
                          const FileLineColRange &l) override;
-    Arrow *AddLostPos(VertXPos *pos, const FileLineColRange &l) override;
+    Arrow *AddLostPos(gsl::owner<VertXPos*> pos, const FileLineColRange &l) override;
     Arrow *AddStartPos(ArrowEnding *ending = nullptr) override;
     Arrow *AddEndPos(ArrowEnding *ending = nullptr) override;
     bool AddAttribute(const Attribute &) override;
@@ -725,39 +742,39 @@ protected:
     std::vector<FileLineCol> linenum_middle;       ///<Location of mid-stop entity names in the input file (within this arrow definition)
     std::vector<EArcSymbol>  segment_types;        ///<Types of segments: one for each segment ([0] is the one from src). Set during AddSegment() calls.
     std::vector<LineAttr>    segment_lines;        ///<Line types of segments. Set during AddAttributeList() from `segment_types`.
-    const bool               specified_as_forward; ///<True if user specified "a->b", false if "b<-a"
-    double                   slant_angle;          ///<The angle of the arrow. Taken from the context, may be modified by style and/or attribute.
-    double                   slant_depth;          ///<The value of the slant depth, an alternative to the slant arrow. Used only if the user specified the "slant_depth" attr.
-    int                      lost_at;              ///<Index of where the message was lost if expressed via an asterisk: -2=none, -1=src, 0..n = at a midpoint, n=dst; where n is the size of 'middle'.
-    bool                     lost_is_forward;      ///<True if the loss was at the fw side of 'lost_at' entity, false if at the backwards size
+    const bool               specified_as_forward = false; ///<True if user specified "a->b", false if "b<-a"
+    double                   slant_angle = 0;      ///<The angle of the arrow. Taken from the context, may be modified by style and/or attribute.
+    double                   slant_depth = 0;      ///<The value of the slant depth, an alternative to the slant arrow. Used only if the user specified the "slant_depth" attr.
+    int                      lost_at = -2;         ///<Index of where the message was lost if expressed via an asterisk: -2=none, -1=src, 0..n = at a midpoint, n=dst; where n is the size of 'middle'.
+    bool                     lost_is_forward = true;///<True if the loss was at the fw side of 'lost_at' entity, false if at the backwards size
     FileLineCol              linenum_asterisk;     ///<The line and column number of the asterisk marking the loss. Invalid if none.
     VertXPos                 lost_pos;             ///<The position of the loss symbol
     FileLineCol              linenum_lost_at;      ///<The line and column number of the 'lost at' clause marking the loss. Invalid if none.
     constexpr static double  lsym_side_by_offset= 0.5;  ///<if 'lost at a++' or 'lost at a--' is specified, we add the size of the loss symbol times this much extra offset compared to 'lost at a+/-'
-    ArcBase                 *curly_join_to;        ///<Set to the arrow above us, we curly join to. nullptr if none.
-    bool                     curly_join_to_src;    ///<True if we curly join to the 'src' end of 'curly_join_to'
-    std::pair<bool, bool>    user_unspecified_ends;///<True if the user originally left src and/or dst unspecified. We may set src/dst in SetEndingDueToJoin(), but these show if originally we had them unspecified - so that we can update them. Valid after AddAttributeList.
+    ArcBase                 *curly_join_to = nullptr;///<Set to the arrow above us, we curly join to. nullptr if none.
+    bool                     curly_join_to_src = false;    ///<True if we curly join to the 'src' end of 'curly_join_to'
+    std::pair<bool, bool>    user_unspecified_ends = {false, false};///<True if the user originally left src and/or dst unspecified. We may set src/dst in SetEndingDueToJoin(), but these show if originally we had them unspecified - so that we can update them. Valid after AddAttributeList.
     
-    mutable bool is_forward;  ///<True if the src of the arrow is left of the dst (considering *_ending and *_to_box). Valid only after parsing (specifically after AddAttributeList()). (Before that we have no entity ordering yet.)
-    mutable double sin_slant; ///<Pre-calculated value of `slant_angle`
-    mutable double cos_slant; ///<Pre-calculated value of `slant_angle`
-    mutable double sx;        ///<Calculated value of the start arrowhead (sx can be > dx)
-    mutable double dx;        ///<Calculated value of the end arrowhead (sx can be > dx)
-    mutable double sx_text;   ///<Calculated value of the left of the label (usually sx adjusted by arrowhead)
-    mutable double dx_text;   ///<Calculated value of the right of the label (usually dx adjusted by arrowhead)
-    mutable double cx_text;   ///<Calculated value of the middle of the label (usually middle of sx and dx)
-    mutable double cx_lsym;   ///<Calculated value of the middle of the loss symbol (if any)
-    mutable XY lsym_size;     ///<Calculates size of the lost symbol (if any loss)
-    mutable bool has_skip;    ///<True if any in skip is true (is so, if the arrowhead has a skipType != ARROW_NONE & we actually skip over entities
+    mutable bool is_forward = false;///<True if the src of the arrow is left of the dst (considering *_ending and *_to_box). Valid only after parsing (specifically after AddAttributeList()). (Before that we have no entity ordering yet.)
+    mutable double sin_slant = 0; ///<Pre-calculated value of `slant_angle`
+    mutable double cos_slant = 0; ///<Pre-calculated value of `slant_angle`
+    mutable double sx = 0;        ///<Calculated value of the start arrowhead (sx can be > dx)
+    mutable double dx = 0;        ///<Calculated value of the end arrowhead (sx can be > dx)
+    mutable double sx_text = 0;   ///<Calculated value of the left of the label (usually sx adjusted by arrowhead)
+    mutable double dx_text = 0;   ///<Calculated value of the right of the label (usually dx adjusted by arrowhead)
+    mutable double cx_text = 0;   ///<Calculated value of the middle of the label (usually middle of sx and dx)
+    mutable double cx_lsym = 0;   ///<Calculated value of the middle of the loss symbol (if any)
+    mutable XY lsym_size = {0,0}; ///<Calculates size of the lost symbol (if any loss)
+    mutable bool has_skip = false;///<True if any in skip is true (is so, if the arrowhead has a skipType != ARROW_NONE & we actually skip over entities
     mutable std::vector<bool> skip;          ///<True, if this entity is inserted as a skip entity. src and dst are never skip.
     mutable std::vector<double> xPos;        ///<X coordinates of arrowheads (sorted to increase, [0] is for left end of the arrow). Always the middle of the entity line.
     mutable std::vector<double> act_size;    ///<Activation size of the entities (sorted from left to right). ==0 if not active
     mutable std::vector<DoublePair> margins; ///<How much of the arrow line is covered by the arrowhead (sorted left to right), see MscArrowAttr::getWidths()
     mutable Contour text_cover; ///<The cover of the label, so that we can hide entity lines in PostPosProcess() - not used by BlockArrow
-    mutable double centerline;  ///<y offset of line compared to yPos
+    mutable double centerline = 0;///<y offset of line compared to yPos
     mutable Contour clip_area;  ///<Used to mask out the line at arrowheads - not used by BlockArrow.
-    mutable double curly_margin;///<The size of the arrowhead at the top of the curly join (if any)
-    mutable double curly_radius;///<The radius of a curly join (if any)
+    mutable double curly_margin = 0;///<The size of the arrowhead at the top of the curly join (if any)
+    mutable double curly_radius = 0;///<The radius of a curly join (if any)
     mutable Path src_line;      ///<In case of an arrow with curly join to a previous one this is the curvy join plus the first line segment. In FinalizeLayout().
 
     void InsertSkipEntity(unsigned u, Entity *pEntity);
@@ -771,7 +788,7 @@ public:
     DirArrow(const NEntityList &el, bool bidir, EArcSymbol boxsymbol, const LabelledArc &al);
     Arrow *AddSegment(ArrowSegmentData data, const char *m, const FileLineColRange &ml,
                          const FileLineColRange &l) override;
-    Arrow *AddLostPos(VertXPos *pos, const FileLineColRange &l) override;
+    Arrow *AddLostPos(gsl::owner<VertXPos*> pos, const FileLineColRange &l) override;
     Arrow *AddStartPos(ArrowEnding *ending = nullptr) override;
     Arrow *AddEndPos(ArrowEnding *ending = nullptr) override;
     bool IsMscgenBroadcastArrow() const;
@@ -814,9 +831,11 @@ public:
     virtual void PostPosProcess(Canvas &, Chart *ch) override;
     void RegisterLabels() override;
     void CollectIsMapElements(Canvas &canvas) override;
+    void AddClippedLine(Canvas& canvas, Path path, const LineAttr& line, 
+                        const MscArrowHeads& loc_arrow, unsigned i, const Block* clip_by) const;
     void DrawArrow(Canvas &canvas, const Label &loc_parsed_label,
                    const std::vector<LineAttr>& loc_segment_lines,
-                   const MscArrowHeads &loc_arrow) const;
+                   const MscArrowHeads &loc_arrow, const Block* clip_by) const;
     void Draw(Canvas &canvas, EMscDrawPass pass) const override;
 };
 
@@ -837,10 +856,12 @@ protected:
     mutable std::vector<Contour> outer_contours; ///<The outline for each segment
 public:
     /** Upgrade an DirArrow to a block arrow: the regular way of creating one. */
-    BlockArrow(const DirArrow &);
+    BlockArrow(DirArrow&&);
     /** Used when converting a collapsed Box to an BlockArrow */
     BlockArrow(const NEntityList &, bool bidir, EArcSymbol boxsymbol, const LabelledArc &al, const BoxSignature *s);
     ~BlockArrow() override;
+    bool StoreNoteTemporarily(Note* cn) override { return false; }
+    void SetTemporarilyStoredNotes(StoredNotes&&); ///<Used by Box::PostParseProcess(), when collapsing to block arrow.
     void SetStyleBeforeAttributes(AttributeList *l) override;
     const MscStyleCoW *GetRefinementStyle4ArrowSymbol(EArcSymbol t) const override;
     const BoxSignature* GetSignature() const override { return sig; }
@@ -869,11 +890,12 @@ class Vertical : public Arrow
 public:
     /** Lists Vertical shapes */
     enum EVerticalShape {
-        ARROW_OR_BOX, ///<A block arrow or a box
-        BRACE,        ///<A curly brace
-        BRACKET,      ///<A square bracket (potentially with custom corners)
-        RANGE,        ///<A range with potential arrow
-        POINTER,      ///<An arrow that looks like a pointer to self
+        ARROW,    ///<A block arrow
+        BOX,      ///<A box
+        BRACE,    ///<A curly brace
+        BRACKET,  ///<A square bracket (potentially with custom corners)
+        RANGE,    ///<A range with potential arrow
+        POINTER,  ///<An arrow that looks like a pointer to self
     };
 protected:
     const EArcSymbol type;   ///<Tells us what kind of symbol was used to define us (-> or -- etc)
@@ -897,14 +919,15 @@ protected:
     mutable double t_text;   ///<Top/Right/Left edge of text for side==END/LEFT/RIGHT. An x coordinate for LEFT/RIGHT, an y for END
     mutable double xpos;     ///<x coordinate of middle (ARROW_OR_BOX) or the key line (BRACE, BRACKET, RANGE)
     mutable double vline_off;///<The distance of the edge of the construct facing the entity line from the main vertical line (signed value), only used for brace/pointer/range/bracket
-    mutable Contour clip_line;  ///<The clip of the line for ranges, brackets and pointers
+    mutable Block clip_block;///<The clip of the line for brackets and pointers, ignoring arrowheads (for PPT)
+    mutable Contour clip_line;  ///<The clip of the line for brackets and pointers
     mutable std::vector<Contour> outer_contours; ///<Calculated contour (only one element)
 public:
     /** Constructor for arrow or box symbols with two marker names (one can be nullptr)*/
     Vertical(const ArcTypePlusDir *t, const char *s, const char *d, MscChart *msc);
-    Arrow *AddSegment(ArrowSegmentData data, const char *m, const FileLineColRange &ml, 
+    Arrow *AddSegment(ArrowSegmentData data, const char *m, const FileLineColRange &ml,
                          const FileLineColRange &l) override;
-    Arrow *AddLostPos(VertXPos *pos, const FileLineColRange &l) override;
+    Arrow *AddLostPos(gsl::owner<VertXPos*> pos, const FileLineColRange &l) override;
     Arrow *AddStartPos(ArrowEnding *ending = nullptr) override;
     Arrow *AddEndPos(ArrowEnding *ending = nullptr) override;
     /** Add the parsed horizontal position (starting with the AT keyword) */
@@ -913,11 +936,12 @@ public:
     void SetVerticalShape(EVerticalShape sh);
     bool CanBeNoted() const override { return false; }
     bool CanBeAlignedTo() const override { return false; }
+    bool StoreNoteTemporarily(Note* cn) override { return false; }
     void SetStyleBeforeAttributes(AttributeList *l) override;
     const MscStyleCoW *GetMyRefinementStyle() const;
     bool AddAttribute(const Attribute &) override;
     static void AttributeNames(Csh &csh);
-    static bool AttributeValues(const std::string &attr, Csh &csh);
+    static bool AttributeValues(const std::string &attr, Csh &csh, EVerticalShape shape);
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
                               Numbering &number, MscElement **note_target, ArcBase *vertical_target) override;
     void Width(Canvas &canvas, DistanceRequirements &vdist) override;
@@ -932,7 +956,8 @@ public:
     virtual void PostPosProcess(Canvas &, Chart *ch) override;
     void RegisterLabels() override;
     void CollectIsMapElements(Canvas &canvas) override;
-    void DrawBraceLostPointer(Canvas &canvas, const LineAttr &line, const MscArrowHeads &arrow) const;
+    void DrawBraceLostPointer(Canvas &canvas, const LineAttr &line, const MscArrowHeads &arrow,
+                              const Contour *clip_by = nullptr, bool start = true) const;
     void Draw(Canvas &canvas, EMscDrawPass pass) const override;
 };
 
