@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -128,6 +128,7 @@ protected:
     UPtrList<Box> series;         ///<The boxes in the series - may contain only a single one
     int           drawing_variant;///<One of 3 variants on how to draw double or triple lines
     AttributeList collected_attrs;///<The collected attributes of all boxes in the series - applied to each new box.
+    StoredNotes tmp_stored_notes; ///<A set of notes attached to us.
 
     mutable double total_height;  ///<Total height of the series
     mutable double left_space;    ///<How much do we expand left of the entity line of src. Includes linewidth.
@@ -136,10 +137,12 @@ public:
     /** Create a series using a box as a first element (and taking ownership!)*/
     BoxSeries(gsl::owner<Box*> first, gsl::owner<AttributeList*> l);
     bool CanBeAlignedTo() const override { return true; }
+    bool StoreNoteTemporarily(Note* cn) override;
+    void ReinsertTmpStoredNotes(ArcList& list, ArcList::iterator after) override;
     /** Expands the file_pos of the first element in a series (used when adding the 'box' keyword)*/
     void ExpandFirstLineEnd(const FileLineColRange &l) { _ASSERT(series.size()); if (series.size()) series.front()->ExpandLineEnd(l); }
     /** Add the first and append subsequent boxes to the series */
-    BoxSeries* AddBox(Box *f, AttributeList *l);
+    BoxSeries* AddBox(gsl::owner<Box*> f, gsl::owner<AttributeList*> l);
     EDirType GetToucedEntities(NEntityList &el) const override;
 
     bool CanJoin() const override { return true; }
@@ -215,7 +218,7 @@ public:
     void RegisterCover(EMscDrawPass /*pass*/) const override { _ASSERT(0); } //will never be called
     /** Helper to draw various parts of the pipe */
     void DrawPipe(Canvas &canvas, EMscDrawPass pass, bool topSideFill, bool topSideLine,
-                  bool backSide, bool shadow, bool text, double next_lw, int drawing_variant);
+                  bool backSide, bool text, double next_lw, int drawing_variant);
     void Draw(Canvas &/*canvas*/, EMscDrawPass /*pass*/) const override { _ASSERT(0); } //will never be called
 };
 
@@ -376,6 +379,7 @@ public:
     JoinSeries(MscChart *msc, ArcBase *first);
     bool CanBeAlignedTo() const override { return true; }
     void Append(ArcBase *);
+    void ReinsertTmpStoredNotes(ArcList& list, ArcList::iterator after) override;
     EDirType GetToucedEntities(NEntityList &el) const override;
     static JoinableResult CanJoinTo(ArcBase *a, ArcBase *b, ESide side, MscError *Error = nullptr);
 
