@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -562,8 +562,8 @@ public:
     EntityRef EntityMaxByPos(EntityRef i, EntityRef j) const { return EntityMinMaxByPos(i, j, false); }
     /** Return the entity with the larger `pos` value*/
     CEntityRef EntityMaxByPos(CEntityRef i, CEntityRef j) const { return EntityMinMaxByPos(i, j, false); }
-    /** Finds entity named `e` (mentioned at `l` inthe source file in AllEntities.
-     * If not found, it creates one. */
+    /** Finds entity named `e` (mentioned at position `l` in the source file).
+     * If not found, it creates one (and uses 'l' only then). */
     EntityRef FindAllocEntity(const char *e, const FileLineColRange &l);
     EntityRef FindLeftRightDescendant(EntityRef, bool left, bool stop_at_collapsed);
     EntityRef FindActiveParentEntity(EntityRef);
@@ -587,10 +587,11 @@ public:
     void AddArcs(ArcList *a) {if (!a) return; Arcs.splice(Arcs.end(), *a); delete a;}
     Arrow *CreateArrow(ArrowSegmentData data, const char*s, const FileLineColRange &sl,
                        const char*d, bool fw, const FileLineColRange &dl, bool had_param);
-    /** Create a block arrow from an arrow. Returns null, emits an error if not possible*/
-    BlockArrow *CreateBlockArrow(const ArcBase *);
+    /** Create a block arrow from an arrow. Returns null, emits an error if not possible 
+     * On success, it brings the given arc to a "valid, but unspecified state" .*/
+    BlockArrow *CreateBlockArrow(ArcBase *);
     /** Pop the top context. May return arcs to be appended as a result (SetNumbering)*/
-    ArcBase *PopContext();
+    std::unique_ptr<ArcBase> PopContext();
 
     /** Parse a piece of input text as a new input file.*/
     unsigned ParseText(std::string_view input, std::string_view filename) override;
@@ -607,7 +608,7 @@ public:
      * still open. This is because if the user opens a "defdesign {" at the very
      * end of the file, without closing it, a totally empty (no hscale) context 
      * will be the current at the end of the parsing. */
-    double GetHScale() const {_ASSERT(Contexts.size() && Contexts.front().hscale.is_set); return Contexts.front().hscale.value;}
+    double GetHScale() const {_ASSERT(Contexts.size() && Contexts.front().hscale); return *Contexts.front().hscale;}
     /** Converts an x coordinate from pos coordinat space to pixel space. Always returns an integer */
     double XCoord(double pos) const {return floor(pos*130*(GetHScale()>0?GetHScale():1)+0.5);} //rounded
     /** Returns the x coordinate of the middle of an entity in pixel space, rounded*/
@@ -648,10 +649,9 @@ public:
     void RegisterAllLabels() override { RegisterLabelArcList(Arcs); } ///<Register all labels in Arcs
     void CollectIsMapElementsArcList(ArcList &arcs, Canvas &canvas) { for (auto &pArc : arcs) pArc->CollectIsMapElements(canvas); }  ///<Collect all ISMAPs from an arc list.
     void CollectIsMapElements(Canvas &canvas) override { CollectIsMapElementsArcList(Arcs, canvas); } ///<Collect all ISMAPs from an Arcs
-    void CompleteParse(Canvas::EOutputType, bool avoidEmpty,
-                       bool autoPaginate=false, bool addHeading=true, 
-                       XY pageSize=XY(0,0), bool fitWidth=true, 
-                       bool collectLinkInfo=false) override;
+    void CompleteParse(bool autoPaginate=false, 
+                       bool addHeading=true, XY pageSize=XY(0,0), 
+                       bool fitWidth=true, bool collectLinkInfo=false) override;
 
     void DrawEntityLines(Canvas &canvas, double y, double height, NEntityList::const_iterator from, NEntityList::const_iterator to);
     void DrawEntityLinesIncludeLast(Canvas &canvas, double y, double height, CEntityRef from, CEntityRef to_included)
