@@ -1,6 +1,6 @@
   /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -24,42 +24,6 @@
 
 using namespace msc;
 
-    
-//Transformation functions for angled arrows
-//In the transformed space (sx,y) will be the same, the space will be rotated by "radian"
-//and the arrow will be vertical
-
-//Transformation functions for angled arrows
-//"radian" is assumed to be between -45..+45 degrees (but in radian)
-
-/** Transform the space for angled arrows. 
- * Useful, so that we can draw angled arrows as if they were horizontal.
- * Transform space such that if we draw a horizontal arrow, it will be of 
- * angle="radian" on the canvas. 
- * In the transformed space (sx,y) will be the same, the space will be rotated by "radian".
- * @param [in] degree The angle to rotate clockwise (with y growing downwards. 
- *                    Assumed to be between -45..+45 degrees.
- * @param [in] sx The x coordinate of the center of the rotation.
- * @param [in] y The y coordinate of the center of the rotation.
- * @param canvas The canvas to rotate.
- *
- * Saves cairo context, always call MscArrowHeads::UntransformCanvas() to kill it, 
- * when done.*/
-void MscArrowHeads::TransformCanvasForAngle(double degree, Canvas &canvas, double sx, double y) 
-{
-    cairo_t *cr = canvas.GetContext();
-    cairo_save(cr);
-    cairo_translate(cr, sx, y);
-    cairo_rotate(cr, degree*M_PI/180.);
-    cairo_translate(cr, -sx, -y);
-}
-
-/** Rotates back the canvas rotated by MscArrowHeads::TransformCanvasForAngle()*/
-void MscArrowHeads::UnTransformCanvas(Canvas &canvas) 
-{
-    canvas.UnClip();
-}
-
 /** Tells how wide and high a specific arrowhead. The y coordinate is just half the height.*/
 double MscArrowHeads::getHeight(bool bidir, EArrowEnd which, const LineAttr &mainline_before, const LineAttr &mainline_after) const
 {
@@ -69,8 +33,8 @@ double MscArrowHeads::getHeight(bool bidir, EArrowEnd which, const LineAttr &mai
 }
 
 
-/** Tells how much of the arrow line is covered by the arrowhead (on both sides of the entity line). 
- * Used for text margin calculation. 
+/** Tells how much of the arrow line is covered by the arrowhead (on both sides of the entity line).
+ * Used for text margin calculation.
  * @param [in] forward True if arrow points from left to right, that is a->b and not a<-b.
  * @param [in] bidir True if the arrow is bi-directional, that is like a<->b and not a->b.
  * @param [in] which Tells us which end are we interested in (or middle).
@@ -83,7 +47,7 @@ DoublePair MscArrowHeads::getWidths(bool forward, bool bidir, EArrowEnd which,
     DoublePair ret(0,0);
     auto ah = GetArrowHead(bidir, which);
     if (ah.IsNone()) return ret;
-    auto wh = ah.WidthHeight(forward ? mainline_left : mainline_right, 
+    auto wh = ah.WidthHeight(forward ? mainline_left : mainline_right,
                              forward ? mainline_right: mainline_left);
     if (bidir && IsMiddle(which) && !ah.IsSymmetricOrNone()) {
         //duplicate for bidir in the middle
@@ -122,14 +86,14 @@ Contour MscArrowHeads::EntityLineCover(XY xy, bool forward, bool bidir, EArrowEn
  * @param [in] bidir If the arrowhead is bidirectional
  * @param [in] which Which end of the arrow (or middle).
  * @param [in] forward True if arrow points from left to right, that is a->b and not a<-b.
- * @param [in] act_size Zero if the entity is not active at this point. Else it contains the 
+ * @param [in] act_size Zero if the entity is not active at this point. Else it contains the
  *                      half-width of the activated entity line.
  * @param [in] mainline_left The line style of the arrow line on the left side.
  * @param [in] mainline_right The line style of the arrow line on the right side.
- * @returns An area covering the area where the arrow line should NOT be drawn. 
- *          If no clipping is needed (all of the line can be shown at this arrowhead), 
+ * @returns An area covering the area where the arrow line should NOT be drawn.
+ *          If no clipping is needed (all of the line can be shown at this arrowhead),
  *          we return an empty contour.*/
-Contour MscArrowHeads::ClipForLine(XY xy, double act_size, bool forward, bool bidir, EArrowEnd which, 
+Contour MscArrowHeads::ClipForLine(XY xy, double act_size, bool forward, bool bidir, EArrowEnd which,
                                const LineAttr &mainline_left, const LineAttr &mainline_right) const
 {
     auto ah = GetArrowHead(bidir, which);
@@ -167,7 +131,7 @@ Contour MscArrowHeads::ClipForLine(XY xy, double act_size, bool forward, bool bi
  * @param [in] bidir If the arrowhead is bidirectional
  * @param [in] which Which end of the arrow (or middle).
  * @param [in] forward True if arrow points from left to right, that is a->b and not a<-b.
- * @param [in] act_size Zero if the entity is not active at this point. Else it contains the 
+ * @param [in] act_size Zero if the entity is not active at this point. Else it contains the
  *                      half-width of the activated entity line.
  * @param [in] mainline_left The line style left of the arrowhead
  * @param [in] mainline_right The line style right of the arrowhead
@@ -184,7 +148,7 @@ Contour MscArrowHeads::Cover(XY xy, double act_size, bool forward, bool bidir, E
 
     const bool both_sides = bidir && IsMiddle(which);
     Contour area;
-    if (forward || both_sides) 
+    if (forward || both_sides)
         area += ah.Cover(xy - XY(act_size, 0), false,
                             forward ? mainline_left : mainline_right,
                             forward ? mainline_right : mainline_left);
@@ -206,12 +170,12 @@ Contour MscArrowHeads::Cover(XY xy, double act_size, bool forward, bool bidir, E
  * @param [in] bidir If the arrowhead is bidirectional
  * @param [in] which Which end of the arrow (or middle).
  * @param [in] forward True if arrow points from left to right, that is a->b and not a<-b.
- * @param [in] act_size Zero if the entity is not active at this point. Else it contains the 
+ * @param [in] act_size Zero if the entity is not active at this point. Else it contains the
  *                      half-width of the activated entity line.
  * @param [in] mainline_left The line style of the arrow line on the left side.
  * @param [in] mainline_right The line style of the arrow line on the right side.
 */
-void MscArrowHeads::Draw(Canvas &canvas, XY xy, double act_size, 
+void MscArrowHeads::Draw(Canvas &canvas, XY xy, double act_size,
                      bool forward, bool bidir, EArrowEnd which,
                      const LineAttr &mainline_left, const LineAttr &mainline_right) const
 {
@@ -245,17 +209,17 @@ void MscArrowHeads::Draw(Canvas &canvas, XY xy, double act_size,
 }
 
 
-/** The full width of the arrowhead (on both sides of the entity line). 
+/** The full width of the arrowhead (on both sides of the entity line).
  * The values returned here are used to determine spacing between entities
  * in for BlockArrow::Width().
  * @param [in] forward True if arrow points from left to right, that is a->b and not a<-b.
  * @param [in] bidir True if the arrow is bi-directional, that is like a<->b and not a->b.
  * @param [in] which Tells us which end are we interested in (or middle).
- * @param [in] body_height The height of the arrow body 
+ * @param [in] body_height The height of the arrow body
  * @param [in] act_size The (half) width of the entity line due to entity activation.
  * @param [in] ltype The line style that will be used to draw the block arrow.
- * @returns The size of the arrowhead from the middle of the entity line towards the 
- *          middle on the arrow. 'first' returns the size on the left side, 
+ * @returns The size of the arrowhead from the middle of the entity line towards the
+ *          middle on the arrow. 'first' returns the size on the left side,
  *          'second' returns the space on the right side.*/
 DoublePair MscArrowHeads::getBigWidthsForSpace(bool forward, bool bidir, EArrowEnd which,
                                                double body_height, double act_size, const LineAttr &ltype) const
@@ -269,14 +233,14 @@ DoublePair MscArrowHeads::getBigWidthsForSpace(bool forward, bool bidir, EArrowE
     else {
         ret = {who.before_margin, who.after_margin};
         if ((which==EArrowEnd::START && forward) || //only on the left side if (-> and END) or (<- and START)
-            (which==EArrowEnd::END && !forward))            
+            (which==EArrowEnd::END && !forward))
             ret.swap();
     }
     return ret;
 }
 
 /** Determines how much margin is needed for a text with a given cover.
- * Determines the margin of the text (from the entity line). Can be negative, 
+ * Determines the margin of the text (from the entity line). Can be negative,
  * if the block arrowhead spans over the entity line (e.g., dot does) and
  * hence the text can be wider than the space between the entity lines.
  * (sx, sy) should be equal to the y aspect of the bounding box of text_cover
@@ -290,7 +254,7 @@ DoublePair MscArrowHeads::getBigWidthsForSpace(bool forward, bool bidir, EArrowE
  * @param [in] bidir True if the arrow is bidirectional.
  * @param [in] which tells us which kind of arrow we talk about start, middle or end
  * @param [in] ltype The line style that will be used to draw the block arrow. */
-double MscArrowHeads::getBigMargin(Contour text_cover, double sy, double dy, bool margin_side_is_left, 
+double MscArrowHeads::getBigMargin(Contour text_cover, double sy, double dy, bool margin_side_is_left,
                                   bool forward, bool bidir, EArrowEnd which, const LineAttr &ltype) const
 {
     return GetArrowHead(forward, bidir, which, !margin_side_is_left).
@@ -303,13 +267,13 @@ double MscArrowHeads::getBigMargin(Contour text_cover, double sy, double dy, boo
  * @param [in] bidir True if the arrow is bi-directional, that is like a<->b and not a->b.
  * @param [in] line Specifies the line style to use. It can be null if lines!=nullptr.
  * @param [in] lines Specifies the line style for each segment.
- *                   If nullptr, we assume there are no middle arrowheads (arrow is of a 
+ *                   If nullptr, we assume there are no middle arrowheads (arrow is of a
  *                   single segment) and `line` is the line type.
- *                   Else we use the linewidths in 'lines' to estimate top height, 
+ *                   Else we use the linewidths in 'lines' to estimate top height,
  *                   except if mid arrowhead is not segmenting.
- *                   We assume `lines` is ordered in increasing x coordinate. 
+ *                   We assume `lines` is ordered in increasing x coordinate.
  *                   If both lines and line are specified, lines takes precedence.*/
-double MscArrowHeads::bigYExtent(double height, bool forward, bool bidir, 
+double MscArrowHeads::bigYExtent(double height, bool forward, bool bidir,
                                  const LineAttr *line, const std::vector<LineAttr> *lines) const
 {
     _ASSERT(line || (lines && lines->size()));   //only one of them can be null
@@ -320,17 +284,17 @@ double MscArrowHeads::bigYExtent(double height, bool forward, bool bidir,
             startArrowHead.BlockWidthHeight(height, *line, false).y_extent_above_body,
             endArrowHead.  BlockWidthHeight(height, *line, false).y_extent_above_body);
         if (lines==nullptr || lines->size()==0) return h;
-        return std::max(h, 
+        return std::max(h,
             midArrowHead.BlockWidthHeight(height, *line, true).y_extent_above_body);
     }
     double ret = 0;
     for (unsigned i=0; i<lines->size(); i++) {
         EArrowEnd t_left = EArrowEnd::MIDDLE, t_right = EArrowEnd::MIDDLE;
-        if (i==0) 
+        if (i==0)
             t_left = forward ? EArrowEnd::START : EArrowEnd::END;
-        if (i==lines->size()) 
+        if (i==lines->size())
             t_right = forward ? EArrowEnd::END : EArrowEnd::START;
-        
+
         const double h_left =  GetArrowHead(forward, bidir, t_left, false).
             BlockWidthHeight( height, lines->at(i), t_left ==EArrowEnd::MIDDLE).y_extent_above_body;
         const double h_right = GetArrowHead(forward, bidir, t_right, true).
@@ -342,10 +306,10 @@ double MscArrowHeads::bigYExtent(double height, bool forward, bool bidir,
 }
 
 
-/** Returns the contours of one arrowhead. 
+/** Returns the contours of one arrowhead.
  * Either on the left or the right side of the entity line.
  * @param [in] x The x coordinate of the tip of the arrowhead.
- * @param [in] act_size Zero if the entity is not active at this point. Else it contains the 
+ * @param [in] act_size Zero if the entity is not active at this point. Else it contains the
  *                      half-width of the activated entity line.
  * @param [in] sy The top of the arrow body (outer edge of the line).
  * @param [in] dy The bottom of the arrow body (outer edge of the line).
@@ -358,15 +322,15 @@ double MscArrowHeads::bigYExtent(double height, bool forward, bool bidir,
  * @param [out] body_margin The space from the entity line the body shoud end at.
  *                          Essentially the width of the arrowhead.
  * @returns The contour of the arrowhead.*/
-Contour MscArrowHeads::BigContourOneEntity(double x, double act_size, double sy, double dy, 
-                                           bool forward, bool bidir, EArrowEnd which, bool left, 
+Contour MscArrowHeads::BigContourOneEntity(double x, double act_size, double sy, double dy,
+                                           bool forward, bool bidir, EArrowEnd which, bool left,
                                            const LineAttr &ltype, double *body_margin) const
 {
     auto ah = GetArrowHead(forward, bidir, which, left);
     const bool middle = IsMiddle(which);
     const auto who = ah.BlockWidthHeight(dy-sy, ltype, middle);
     const double act_off = act_size * who.add_activation;
-    if (body_margin) 
+    if (body_margin)
         *body_margin = who.body_connect_before-act_off;  //margin will be substracted on right, added on left
     Contour area = ah.BlockContour(x+act_off, sy, dy, ltype, middle);
     if (!left)
@@ -379,8 +343,8 @@ Contour MscArrowHeads::BigContourOneEntity(double x, double act_size, double sy,
  * @param [in] xPos The x coordinate of entities the arrow visits. If the arrow is
  *                  slanted, these shall be spaced somewhat wider apart than in the
  *                  horizontal direction. This must be done by the caller.
- * @param [in] act_size The activation status for each visited entity. 
- *                      Zero if the entity is not active at this point. Else it contains the 
+ * @param [in] act_size The activation status for each visited entity.
+ *                      Zero if the entity is not active at this point. Else it contains the
  *                      half-width of the activated entity line.
  * @param [in] sy The top of the arrow body (outer edge of the line).
  * @param [in] dy The bottom of the arrow body (outer edge of the line).
@@ -396,9 +360,9 @@ Contour MscArrowHeads::BigContourOneEntity(double x, double act_size, double sy,
  *                   If both lines and line are specified, lines takes precedence.
  * @param [out] result The contours for individual segments. Only one if there is
  *                     only one segment.
- * @returns The union of segment contours.*/ 
-Contour MscArrowHeads::BigContour(const std::vector<double> &xPos, const std::vector<double> &act_size, 
-                              double sy, double dy, bool forward, bool bidir, 
+ * @returns The union of segment contours.*/
+Contour MscArrowHeads::BigContour(const std::vector<double> &xPos, const std::vector<double> &act_size,
+                              double sy, double dy, bool forward, bool bidir,
                               const LineAttr *line, const std::vector<LineAttr> *lines,
                               std::vector<Contour> &result) const
 {
@@ -406,12 +370,12 @@ Contour MscArrowHeads::BigContour(const std::vector<double> &xPos, const std::ve
     Contour area_overall, area_segment;
     const bool segment = xPos.size()>2 && BreaksBlockArrow(GetArrowHead(bidir,EArrowEnd::MIDDLE).front().type);
     const EArrowEnd e_left  = forward ? EArrowEnd::START : EArrowEnd::END;
-    const LineAttr *ltype_current = (lines && lines->size()) ? &*lines->begin() : line;
+    const LineAttr *ltype_current = (lines && lines->size()) ? lines->data() : line;
 
     //draw leftmost arrowhead
     double from_x;
-    area_segment = BigContourOneEntity(xPos.front(), act_size.front(), sy, dy, 
-                                       forward,bidir, e_left, false, 
+    area_segment = BigContourOneEntity(xPos.front(), act_size.front(), sy, dy,
+                                       forward,bidir, e_left, false,
                                        *ltype_current, &from_x);
     from_x = xPos[0] + from_x;
 
@@ -419,8 +383,8 @@ Contour MscArrowHeads::BigContour(const std::vector<double> &xPos, const std::ve
     if (!midArrowHead.IsNone())
         for (unsigned i=1; i<xPos.size()-1; i++) {
             //draw left side of the entity line
-            double margin; 
-            area_segment += BigContourOneEntity(xPos[i], act_size[i], sy, dy, 
+            double margin;
+            area_segment += BigContourOneEntity(xPos[i], act_size[i], sy, dy,
                                                 forward,bidir, EArrowEnd::MIDDLE, true,
                                                 *ltype_current, &margin);
             //draw body that connects them
@@ -432,13 +396,13 @@ Contour MscArrowHeads::BigContour(const std::vector<double> &xPos, const std::ve
 
             area_overall += area_segment;
             //if we are segmented, add us to result
-            if (segment) 
+            if (segment)
                 result.push_back(std::move(area_segment));
             area_segment.clear();
 
             //if segmented, draw arrowhead at right side of entity line
-            if (segment) 
-                area_segment = BigContourOneEntity(xPos[i], act_size[i], sy, dy, 
+            if (segment)
+                area_segment = BigContourOneEntity(xPos[i], act_size[i], sy, dy,
                                                    forward,bidir, EArrowEnd::MIDDLE, false,
                                                    *ltype_current, &margin);
             from_x = xPos[i] + margin;
@@ -446,12 +410,12 @@ Contour MscArrowHeads::BigContour(const std::vector<double> &xPos, const std::ve
     //draw rightmost arrowhead
     const EArrowEnd e_right = forward ? EArrowEnd::END : EArrowEnd::START;
     double right_margin;
-    area_segment += BigContourOneEntity(xPos.back(), act_size.back(), sy, dy, 
+    area_segment += BigContourOneEntity(xPos.back(), act_size.back(), sy, dy,
                                         forward,bidir, e_right, true,
                                         *ltype_current, &right_margin);
     area_segment += Block(from_x, xPos.back() - right_margin, sy, dy);
     area_overall += area_segment;
-    if (segment) 
+    if (segment)
         result.push_back(std::move(area_segment));
     else
         result.push_back(area_overall);
@@ -462,8 +426,8 @@ Contour MscArrowHeads::BigContour(const std::vector<double> &xPos, const std::ve
  * If the two ends are ARROW_NONE, a small block is added.
  * This is used to mark places where floating notes shall not trudge (ArcBase::area_important).
  * @param [in] xPos The x coordinate of entities the arrow visits.
- * @param [in] act_size The activation status for each visited entity. 
- *                      Zero if the entity is not active at this point. Else it contains the 
+ * @param [in] act_size The activation status for each visited entity.
+ *                      Zero if the entity is not active at this point. Else it contains the
  *                      half-width of the activated entity line.
  * @param [in] sy The top of the arrow body (outer edge of the line).
  * @param [in] dy The bottom of the arrow body (outer edge of the line).
@@ -471,31 +435,31 @@ Contour MscArrowHeads::BigContour(const std::vector<double> &xPos, const std::ve
  * @param [in] bidir If the arrowhead is bidirectional
  * @param [in] line Specifies the line style to use. It can be null if lines!=nullptr.
  * @param [in] lines Specifies the line style for each segment.
- *                   If nullptr, we assume there are no middle arrowheads (arrow is of a 
+ *                   If nullptr, we assume there are no middle arrowheads (arrow is of a
  *                   single segment) and `line` is the line type.
- *                   Else we use the linewidths in 'lines' to estimate top height, 
+ *                   Else we use the linewidths in 'lines' to estimate top height,
  *                   except if mid arrowhead is not segmenting.
- *                   We assume `lines` is ordered in increasing x coordinate. 
+ *                   We assume `lines` is ordered in increasing x coordinate.
  *                   If both lines and line are specified, lines takes precedence.
  * @param [in] compressGap How far we shall be.
  * @returns The contours of the heads, but no body.
  */
-Contour MscArrowHeads::BigHeadContour(const std::vector<double> &xPos, const std::vector<double> &act_size, 
-                                  double sy, double dy, bool forward, bool bidir, 
-                                  const LineAttr *line, const std::vector<LineAttr> *lines, 
+Contour MscArrowHeads::BigHeadContour(const std::vector<double> &xPos, const std::vector<double> &act_size,
+                                  double sy, double dy, bool forward, bool bidir,
+                                  const LineAttr *line, const std::vector<LineAttr> *lines,
                                   double compressGap) const
 {
     Contour ret;
     const bool segment = !GetArrowHead(bidir,EArrowEnd::MIDDLE).IsSymmetricOrNone() && xPos.size()>2;
     const EArrowEnd e_left  = forward ? EArrowEnd::START : EArrowEnd::END;
-    const LineAttr *ltype_current = (lines && lines->size()) ? &*lines->begin() : line;
+    const LineAttr *ltype_current = (lines && lines->size()) ? lines->data() : line;
 
     //add leftmost arrowhead
     double dummy_margin;
     if (GetArrowHead(bidir, e_left).IsNone())
         ret = Block(xPos[0]+act_size[0]-compressGap/2, xPos[0]+act_size[0]+compressGap/2, sy, dy);
-    else 
-        ret = BigContourOneEntity(xPos[0], act_size[0], sy, dy, 
+    else
+        ret = BigContourOneEntity(xPos[0], act_size[0], sy, dy,
                                   forward,bidir, e_left, false,
                                   *ltype_current, &dummy_margin);
 
@@ -503,7 +467,7 @@ Contour MscArrowHeads::BigHeadContour(const std::vector<double> &xPos, const std
     if (!midArrowHead.IsNone())
         for (unsigned i=1; i<xPos.size()-1; i++) {
             //draw left side of the entity line
-            ret += BigContourOneEntity(xPos[i], act_size[i], sy, dy, 
+            ret += BigContourOneEntity(xPos[i], act_size[i], sy, dy,
                                        forward,bidir, EArrowEnd::MIDDLE, true,
                                        *ltype_current, &dummy_margin);
             //calculate next linewidth
@@ -511,8 +475,8 @@ Contour MscArrowHeads::BigHeadContour(const std::vector<double> &xPos, const std
                 ltype_current  = &lines->at(i);
 
             //if segmented, draw arrowhead at right side of entity line
-            if (segment) 
-                ret += BigContourOneEntity(xPos[i], act_size[i], sy, dy, 
+            if (segment)
+                ret += BigContourOneEntity(xPos[i], act_size[i], sy, dy,
                                            forward,bidir, EArrowEnd::MIDDLE, false,
                                            *ltype_current, &dummy_margin);
     }
@@ -520,15 +484,15 @@ Contour MscArrowHeads::BigHeadContour(const std::vector<double> &xPos, const std
     const EArrowEnd e_right = forward ? EArrowEnd::END : EArrowEnd::START;
     if (GetArrowHead(bidir, e_right).IsNone())
         ret += Block(*xPos.rbegin()+ *act_size.rbegin()-compressGap/2, *xPos.rbegin()+*act_size.rbegin()+compressGap/2, sy, dy);
-    else 
-        ret += BigContourOneEntity(*xPos.rbegin(), *act_size.rbegin(), sy, dy, 
+    else
+        ret += BigContourOneEntity(*xPos.rbegin(), *act_size.rbegin(), sy, dy,
                                    forward,bidir, e_right, true,
                                    *ltype_current, &dummy_margin);
     return ret;
 }
 
 /** Draw the block arrow, assuming we already calculated its contour.
- * 
+ *
  * @param [out] result The contours for individual segments. Only one if there is
  *                     only one segment.
  * @param [in] line Specifies the line style to use. It can be null if lines!=nullptr.
@@ -542,11 +506,11 @@ Contour MscArrowHeads::BigHeadContour(const std::vector<double> &xPos, const std
  * @param [in] fill The fill we shall use inside the block arrow.
  * @param [in] shadow The shadow we shall use below the block arrow.
  * @param canvas The canvas to draw on.
- * @param [in] angle_radian Our slant. We assume the space around us is rotated 
-                            this much when we are called. We also assume 
+ * @param [in] angle_radian Our slant. We assume the space around us is rotated
+                            this much when we are called. We also assume
                             the contours are enlengthtened already. This value
                             is used to adjust the shadow to be in the right dir. */
-void MscArrowHeads::BigDrawFromContour(std::vector<Contour> &result, 
+void MscArrowHeads::BigDrawFromContour(std::vector<Contour> &result,
                                        const LineAttr *line, const std::vector<LineAttr> *lines,
                                        const FillAttr &fill, const ShadowAttr &shadow, Canvas &canvas,
                                        double angle_radian) const
@@ -557,16 +521,19 @@ void MscArrowHeads::BigDrawFromContour(std::vector<Contour> &result,
         if (lines && lines->size()>i)
             l = &lines->at(i);
         const Contour midline = result[i].CreateExpand(-l->LineWidth()/2);
-        canvas.Shadow(midline, shadow, angle_radian);
-        canvas.Fill  (midline.CreateExpand(-l->Spacing()), fill); 
-        canvas.Line  (midline, *l);
+        if (canvas.does_graphics()) {
+            canvas.Shadow(midline, shadow, angle_radian);
+            canvas.Fill(midline.CreateExpand(-l->Spacing()), fill);
+            canvas.Line(midline, *l);
+        } else
+            canvas.Add(GSShape(midline, *l, fill, shadow));
     }
 }
 
 /** Draw the circle or diamond if a DOT_EMPTY or DIAMOND_EMPTY style is in the
  * middle of an arrow.
  * But clip the drawing area, so that text is not disturbed*/
-void MscArrowHeads::BigDrawEmptyMid(const std::vector<double> &xPos, double sy, double dy, 
+void MscArrowHeads::BigDrawEmptyMid(const std::vector<double> &xPos, double sy, double dy,
                                     Canvas &canvas, const LineAttr &line, const Contour *clip) const
 {
     const Path path = midArrowHead.BlockMidPath(0, sy, dy, line);
@@ -582,8 +549,8 @@ void MscArrowHeads::BigDrawEmptyMid(const std::vector<double> &xPos, double sy,
  * @param [in] xPos The x coordinate of entities the arrow visits. If the arrow is
  *                  slanted, these shall be spaced somewhat wider apart than in the
  *                  horizontal direction. This must be done by the caller.
- * @param [in] act_size The activation status for each visited entity. 
- *                      Zero if the entity is not active at this point. Else it contains the 
+ * @param [in] act_size The activation status for each visited entity.
+ *                      Zero if the entity is not active at this point. Else it contains the
  *                      half-width of the activated entity line.
  * @param [in] sy The top of the arrow body (outer edge of the line).
  * @param [in] dy The bottom of the arrow body (outer edge of the line).
@@ -601,13 +568,13 @@ void MscArrowHeads::BigDrawEmptyMid(const std::vector<double> &xPos, double sy,
  * @param [in] shadow The shadow we shall use below the block arrow.
  * @param canvas The canvas to draw on.
  * @param [in] clip Some clip value, not sure what.
- * @param [in] angle_radian Our slant. We assume the space around us is rotated 
-                            this much when we are called. We also assume 
+ * @param [in] angle_radian Our slant. We assume the space around us is rotated
+                            this much when we are called. We also assume
                             the contours are enlengthtened already. This value
                             is used to adjust the shadow to be in the right dir. */
-void MscArrowHeads::BigCalculateAndDraw(const std::vector<double> &xPos, const std::vector<double> &act_size, 
+void MscArrowHeads::BigCalculateAndDraw(const std::vector<double> &xPos, const std::vector<double> &act_size,
                                     double sy, double dy, bool forward, bool bidir,
-                                    const LineAttr *line, const std::vector<LineAttr> *lines, 
+                                    const LineAttr *line, const std::vector<LineAttr> *lines,
                                     const FillAttr &fill, const ShadowAttr &shadow, Canvas &canvas,
                                     const Contour *clip, double angle_radian) const
 {
