@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -215,10 +215,8 @@ bool msc::CshHintGraphicCallbackForMarkers(Canvas *canvas, CshHintGraphicParam /
     LineAttr line(ELineType::SOLID, ColorType(64,0,255), 1, ECornerType::NONE, 0);
     FillAttr fill(ColorType(64,0,255).Lighter(0.2), EGradientType::UP);
     ShadowAttr shadow(ColorType(0,0,0));
-    shadow.offset.is_set=true;
-    shadow.offset.value=3;
-    shadow.blur.is_set=true;
-    shadow.blur.value=3;
+    shadow.offset = 3;
+    shadow.blur = 3;
 
     Contour c(XY(int(HINT_GRAPHIC_SIZE_X*0.3), int(HINT_GRAPHIC_SIZE_Y*0.2)), 
               XY(int(HINT_GRAPHIC_SIZE_X*0.3), int(HINT_GRAPHIC_SIZE_Y*0.8)),
@@ -229,7 +227,7 @@ bool msc::CshHintGraphicCallbackForMarkers(Canvas *canvas, CshHintGraphicParam /
     return true;
 }
 
-void MscCsh::BeforeYaccParse(std::string_view input, int cursor_p)
+void MscCsh::BeforeYaccParse(const std::string& input, int cursor_p)
 {
     Csh::BeforeYaccParse(input, cursor_p);
     ResetMarkers();
@@ -255,18 +253,18 @@ void MscCsh::AfterYaccParse()
  * @param [in] cursor_p The current position of the cursor.
  * @param [in] pedantic The initial value of the pedantic chart option.
    (We ignore it in CSH parsing.)*/
-void MscCsh::ParseText(std::string_view input, int cursor_p, bool pedantic)
+void MscCsh::ParseText(const std::string& input, int cursor_p, bool pedantic)
 {
     //initialize data struct
     BeforeYaccParse(input, cursor_p);
     (void)pedantic;
     //call parsing
-    MscCshParse(*this, input_text.c_str(), input_text.length());
+    MscCshParse(*this, input_text.c_str(), unsigned(input_text.length()+1)); //include terminating zero
     //Tidy up afterwards
     AfterYaccParse();
 }
 
-void MscCsh::ParseDesignText(std::string_view input, int cursor_p) {
+void MscCsh::ParseDesignText(const std::string& input, int cursor_p) {
     //Design libs shall be parsed with EMscgenCompat::AUTODETECT.
     //Save & restore the value
     const EMscgenCompat saved_mscgen_compat = std::exchange(mscgen_compat, EMscgenCompat::AUTODETECT);
@@ -434,6 +432,7 @@ void MscCsh::AddKeywordsToHints(bool includeParallel)
         "join", "Using this keyword between arrows, you can connect them. Also works for connecting arrows and boxes.",
         "parallel", "If you prepend 'parallel' in front of an element, subsequent elements will be laid besides it (and not strictly below), if possible without overlap.",
         "overlap", "If you prepend 'overlap' in front of an element, subsequent elements will be laid besides it even if overlap occurs. Useful for slanted arrows crossing each other.",
+        "box", "Use before a box definition as syntactic sugar.",
         "block", "Use before an an arrow definition to create a block arrow, like 'block a->b;'.",
         "pipe", "Create a pipe between two entitie.",
         "nudge", "Add a bit of a vertical space to pust subsequent elements lower.",
@@ -456,7 +455,7 @@ void MscCsh::AddKeywordsToHints(bool includeParallel)
         "note", "Add a note (floating in a callout) to the previous chart element.",
         "comment", "Add a comment (visible on the side) to the previous chart element.",
         "endnote", "Add an endnote (visible at the bottom of the chart) referring to the previous chart element.",
-        /*"footnote", "Add footnote (visible at the bottom of the page) referring to the previous chart element.", */
+        "footnote", "Add footnote (visible at the bottom of the page) referring to the previous chart element.",
         "title", "Add a title to the chart.",
         "subtitle", "Add a subtitle to a section of the chart.",
         "defshape", "Define a new shape.",
@@ -481,10 +480,8 @@ bool msc::CshHintGraphicCallbackForMscEntities(Canvas *canvas, CshHintGraphicPar
     LineAttr vline(ELineType::SOLID, ColorType(0,0,0), 2, ECornerType::NONE, 0);
     FillAttr fill(ColorType(192,192,192), EGradientType::UP);
     ShadowAttr shadow(ColorType(0,0,0));
-    shadow.offset.is_set=true;
-    shadow.offset.value=2;
-    shadow.blur.is_set=true;
-    shadow.blur.value=0;
+    shadow.offset = 2;
+    shadow.blur = 0;
 
     Block b(HINT_GRAPHIC_SIZE_X*0.25, HINT_GRAPHIC_SIZE_X*0.75, HINT_GRAPHIC_SIZE_Y*0.1, HINT_GRAPHIC_SIZE_Y*0.5);
     canvas->Line(XY(HINT_GRAPHIC_SIZE_X/2, HINT_GRAPHIC_SIZE_Y*0.5), XY(HINT_GRAPHIC_SIZE_X/2, HINT_GRAPHIC_SIZE_Y*0.9), vline);
@@ -501,22 +498,22 @@ void MscCsh::AddVertXPosSyntaxNonSelectableToHints(bool include_at)
                            HintPrefixNonSelectable();
     AddToHints(CshHint(prefix + "<entity> [offset]",
         "Position exactly on the entity line (optionally offset by the specified number of pixels).",
-        EHintType::KEYWORD));
+        EHintType::KEYWORD, false));
     AddToHints(CshHint(prefix + "<entity>+ [offset]",
         "Position a bit right of the entity line (optionally offset by the specified number of pixels).",
-        EHintType::KEYWORD));
+        EHintType::KEYWORD, false));
     AddToHints(CshHint(prefix + "<entity>- [offset]",
         "Position a bit left of the entity line (optionally offset by the specified number of pixels).",
-        EHintType::KEYWORD));
+        EHintType::KEYWORD, false));
     AddToHints(CshHint(prefix + "<entity>++ [offset]",
         "Position a somewhat right of the entity line (optionally offset by the specified number of pixels).",
-        EHintType::KEYWORD));
+        EHintType::KEYWORD, false));
     AddToHints(CshHint(prefix + "<entity>-- [offset]",
         "Position a somewhat left of the entity line (optionally offset by the specified number of pixels).",
-        EHintType::KEYWORD));
+        EHintType::KEYWORD, false));
     AddToHints(CshHint(prefix + "<entity>-<entity> [offset]",
         "Position to the midpoint between the two entities (optionally offset by the specified number of pixels).",
-        EHintType::KEYWORD));
+        EHintType::KEYWORD, false));
     if (include_at)
         AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "at", nullptr, EHintType::KEYWORD, true));
 }
@@ -540,7 +537,7 @@ void MscCsh::AddStartBeforeEndAfterContentToHints()
     AddEntitiesToHints();
     AddToHints(CshHint(HintPrefixNonSelectable() + "<entity> [+-distance]",
         "Use a number after the entity to adjust the distance of the arrow ending from the entity line.",
-        EHintType::KEYWORD, true));
+        EHintType::KEYWORD, false));
 
 }
 
