@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -112,8 +112,8 @@ Entity::Entity(const string &n, const string &l, const string &ol,
     double p, double pe, const MscStyleCoW &entity_style, const FileLineCol &fp,
     bool coll, bool larg) :
     name(n), label(l), orig_label(ol), file_pos(fp),
-    shape(entity_style.read().shape.is_set ? entity_style.read().shape.value : -1),
-    shape_size(entity_style.read().shape_size.is_set ? entity_style.read().shape_size.value : EArrowSize::SMALL),
+    shape(entity_style.read().shape.value_or(-1)),
+    shape_size(entity_style.read().shape_size.value_or(EArrowSize::SMALL)),
     pos(p), pos_exp(pe), index(0), status(entity_style),
     parse_running_style(entity_style), running_style(entity_style),
     running_shown(EEntityStatus::SHOW_OFF),
@@ -185,26 +185,34 @@ string Entity::Print(int indent) const
     return ss;
 }
 
+
+EntityAppHelper* EntityAppHelper::Prepend(EntityAppHelper* edh) { 
+    if (edh) { 
+        entities.Prepend(&edh->entities); 
+        std::ranges::move(notes, std::back_inserter(edh->notes));
+        std::swap(notes, edh->notes);
+        edh->notes.clear();
+        edh->entities.clear();
+        //leave "target" as that of the latter edh
+    } 
+    return this; 
+} 
+
+
 /** Create an EntityApp for entity named `s` onto the chart `chart`
  * Make all attributes unset, empty the style (but activate those style
  * elements that are set in the default entity style in chart->Contexts*/
 EntityApp::EntityApp(const char *s, MscChart* msc) : MscElement(msc),
     name(s),
     only_shown_if_multi(),
-    label(false, "", FileLineCol()),
-    pos(false, 0, FileLineCol()),                    //field 'pos.value' is used even if no such attribute
-    rel(false, "", FileLineCol()),
-    large(false,false,FileLineCol()),
-    collapsed(false, false, FileLineCol()),
-    show(false, true),
-    active(false, false, FileLineCol()),
     show_is_explicit(false),
     active_is_explicit(false),
     centerlined(false),
     centerline_target(nullptr),
     style(msc->MyCurrentContext().styles["entity"]),  //we will Empty it but use it for f_* values
     defining(false),
-    draw_heading(false)
+    draw_heading(false),
+    large_appears(false)
 {
     style.write().Empty();
 }
@@ -217,27 +225,21 @@ EntityApp::EntityApp(MscChart* chart, bool shown_only, const char *prefix, const
     : MscElement(chart), 
     name(),
     only_shown_if_multi(shown_only),
-    label(false, "", FileLineCol()),
-    pos(false, 0, FileLineCol()),                    //field 'pos.value' is used even if no such attribute
-    rel(false, "", FileLineCol()),
-    large(false, false, FileLineCol()),
-    collapsed(false, false, FileLineCol()),
-    show(false, true),
-    active(false, false, l),
     show_is_explicit(true),
     active_is_explicit(true),
     centerlined(false),
     centerline_target(nullptr),
     style(chart->MyCurrentContext().styles["entity"]),  //we will Empty it but use it for f_* values
     defining(false),
-    draw_heading(false)
+    draw_heading(false),
+    large_appears(false)
 {
     const bool _show = CaseInsensitiveEqual(prefix, "show");
     const bool _show_or_hide = _show || CaseInsensitiveEqual(prefix, "hide");
     const bool _activate = CaseInsensitiveEqual(prefix, "activate");
     const bool _activate_or_deactivate = _activate || CaseInsensitiveEqual(prefix, "deactivate");
-    show = {_show_or_hide, _show};
-    active = {_activate_or_deactivate, _activate, l};
+    if (_show_or_hide) show = _show;
+    if (_activate_or_deactivate) active = {_activate, l};
     style.write().Empty();
 }
 
@@ -264,9 +266,7 @@ bool EntityApp::AddAttribute(const Attribute& a)
         if (!defining)
             chart->Error.Error(a, false, "This attribute can only be set when you define an entity. Ignoring it.");
         else if (a.CheckType(EAttrType::BOOL, chart->Error)) {
-            large.is_set = true;
-            large.value = a.yes;
-            large.file_pos = a.linenum_attr.start;
+            large = {a.yes, a.linenum_attr.start};
         }
         return true;
     }
@@ -284,54 +284,41 @@ bool EntityApp::AddAttribute(const Attribute& a)
     if (a.Is("label")) {
         if (!a.CheckType(EAttrType::STRING, chart->Error)) return true;
         //if EAttrType::CLEAR, we are OK above and a.value is ""
-        label.is_set = true;
-        label.value = a.value;
-        label.file_pos = a.linenum_attr.start;
+        label = {a.value, a.linenum_attr.start};
         linenum_label_value = a.linenum_value.start;
         return true;
     }
     if (a.Is("pos")) {
         if (a.type == EAttrType::CLEAR) {
-            pos.is_set = true;
-            pos.value = 0;
-            pos.file_pos = a.linenum_attr.start;
+            pos = {0, a.linenum_attr.start};
             return true;
         }
         if (!a.CheckType(EAttrType::NUMBER, chart->Error)) return true;
-        pos.is_set = true;
-        pos.value = a.number;
-        pos.file_pos = a.linenum_attr.start;
+        pos = {a.number, a.linenum_attr.start};
         return true;
     }
     if (a.Is("relative")) {
         if (!a.CheckType(EAttrType::STRING, chart->Error)) return true;
         //if EAttrType::CLEAR, we are OK above and a.value is ""
-        rel.value = a.value;
-        rel.is_set = true;
-        rel.file_pos = a.linenum_attr.start;
+        rel = {a.value, a.linenum_attr.start};
         return true;
     }
     if (a.Is("collapsed")) {
         if (!a.CheckType(EAttrType::BOOL, chart->Error)) return true;
-        collapsed.value = a.yes;
-        collapsed.is_set = true;
-        collapsed.file_pos = a.linenum_attr.start;
+        collapsed = {a.yes, a.linenum_attr.start};
         return true;
     }
     if (a.Is("show")) {
         if (!a.CheckType(EAttrType::BOOL, chart->Error)) return true;
         // EAttrType::CLEAR is handled above
-        show.value = a.yes;
-        show.is_set = true;
+        show = a.yes;
         show_is_explicit = true;
         return true;
     }
     if (a.Is("active")) {
         if (!a.CheckType(EAttrType::BOOL, chart->Error)) return true;
         // EAttrType::CLEAR is handled above
-        active.value = a.yes;
-        active.is_set = true;
-        active.file_pos = a.linenum_attr.start;
+        active = {a.yes, a.linenum_attr.start};
         active_is_explicit = true;
         return true;
     }
@@ -374,14 +361,16 @@ bool EntityApp::AddAttribute(const Attribute& a)
  *           other than EntityCommand and Note, in that case we need to give an error.
  * @param [in] l The position of the EntityApp in the file.
  * @return An EntityAppHelper which contains us and our children (if any), plus all notes.
- *         We will be the first EntityApp in the returned list, children will come after.*/
-EntityAppHelper* EntityApp::AddAttributeList(AttributeList *al, ArcList *ch, FileLineCol l)
+ *         We will be the first EntityApp in the returned list, children will come after. 
+ * NOTE: ownership of us (this object) is transferred to the returned object by creating a 
+ *   unique ptr from 'this'!*/
+std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al, ArcList *ch, FileLineCol l)
 {
     EntityRef our_entity = chart->AllEntities.Find_by_Name(name);
     if (our_entity) {
         // Existing entity: kill auto-assumed "show=on" and "active=no"
-        show.is_set = false;
-        active.is_set = false;
+        show.reset();
+        active.reset();
         // Leave show_explicit as false
         //take the entity's draw_pass as default
         draw_pass = our_entity->running_draw_pass;
@@ -397,22 +386,22 @@ EntityAppHelper* EntityApp::AddAttributeList(AttributeList *al, ArcList *ch, Fil
             AddAttribute(*pAttr);
         //Check that if we have a shape and shape.size=auto then the shape has a label box
         if (auto &s = style.read(); 
-                  s.shape.GetValue(-1)>=0 
-               && s.shape_size.GetValue(EArrowSize::BIG)==EArrowSize::INVALID
-               && chart->Shapes[s.shape.value].GetLabelPos().IsInvalid()) {
+                  s.shape.value_or(-1)>=0 
+               && s.shape_size.value_or(EArrowSize::BIG)==EArrowSize::INVALID
+               && chart->Shapes[*s.shape].GetLabelPos().IsInvalid()) {
             //Find the (last) shape.size attr
             auto i = std::find_if(al->rbegin(), al->rend(), [](const std::unique_ptr<Attribute> &pA) {return pA && pA->type==EAttrType::STRING && CaseInsensitiveEqual(pA->name, "shape.size"); });
             if (i!=al->rend()) {
-                chart->Error.Error(**i, true, "Shape '" + chart->Shapes[s.shape.value].name +"' has no label box defined.Nothing to size the shape for.Ignoring this attribute.");
+                chart->Error.Error(**i, true, "Shape '" + chart->Shapes[*s.shape].name +"' has no label box defined.Nothing to size the shape for.Ignoring this attribute.");
             } else {
                 _ASSERT(0);
             }
-            style.write().shape_size.is_set = false;
+            style.write().shape_size.reset();
         }
         delete al;
     }
 
-    EntityAppHelper *ret = new EntityAppHelper;
+    std::unique_ptr<EntityAppHelper> ret = std::make_unique<EntityAppHelper>();
     //If we have children, add them to "ret->entities"
     ret->target = name; //this will be the target for any note after us.
     if (ch) {
@@ -426,8 +415,7 @@ EntityAppHelper* EntityApp::AddAttributeList(AttributeList *al, ArcList *ch, Fil
                 j++;
             } else if (cn!=nullptr) {
                 //Ok, a note, assume it wants to point to us or our last child so far
-                ret->notes.Append(cn);
-                ret->note_targets.push_back(ret->target);
+                ret->notes.emplace_back(std::unique_ptr<Note>(cn), ret->target);
                 j->release();
                 ch->erase(j++);
             } else {
@@ -440,50 +428,49 @@ EntityAppHelper* EntityApp::AddAttributeList(AttributeList *al, ArcList *ch, Fil
 
     //Check that we apply certain attributes the right way for grouped entities
     if (ret->entities.size()) {
-        if (pos.is_set || rel.is_set)
-            chart->Error.Error(pos.is_set ? pos.file_pos : rel.file_pos,
+        if (pos || rel)
+            chart->Error.Error(pos ? pos.file_pos : rel.file_pos,
                                "The position of grouped entities is derived from its member entities.",
                                " Ignoring attribute.");
-        pos.is_set = rel.is_set = false;
-        if (active.is_set && active_is_explicit)
+        pos.reset(); rel.reset();
+        if (active && active_is_explicit)
             chart->Error.Error(active.file_pos, "You cannot directly activate or deactivate a grouped entity.",
                                "The active/inactive state of grouped entities is derived from its member entities. Ignoring attribute.");
-        active.is_set = active_is_explicit = false;
+        active.reset(); 
+        active_is_explicit = false;
         //consider forced collapse
         auto force_itr = chart->force_entity_collapse.find(name);
         if (force_itr != chart->force_entity_collapse.end()) {
             //If force_entity_collapse is same as chart value, remove
-            if ((collapsed.is_set && collapsed.value) == force_itr->second)
+            if (collapsed.value_or(false) == force_itr->second)
                 chart->force_entity_collapse.erase(force_itr);
-            else {
-                collapsed.is_set = true;
-                collapsed.value = force_itr->second;
-            }
+            else 
+                collapsed = force_itr->second;
         }
     } else {
-        if (collapsed.is_set && collapsed.value)
+        if (collapsed.value_or(false))
             chart->Error.Error(collapsed.file_pos, "Only grouped entities can be collapsed.",
                 " Ignoring attribute.");
-        collapsed.is_set = false;
-        if (large.is_set && large.value)
+        collapsed.reset();
+        if (large.value_or(false))
             chart->Error.Error(large.file_pos, "Only grouped entities can have large headings.",
                 " Ignoring attribute.");
-        large.is_set = false;
+        large.reset();
     }
 
     if (our_entity == nullptr) {
         double position = chart->GetEntityMaxPos()+1;
         double position_exp = chart->GetEntityMaxPosExp()+1;
-        if (rel.is_set && !chart->SkipContent()) {
+        if (rel && !chart->SkipContent()) {
             //Look up the entity in a potential 'relative' attribute
-            EntityRef rel_entity = chart->AllEntities.Find_by_Name(rel.value);
+            EntityRef rel_entity = chart->AllEntities.Find_by_Name(*rel);
             if (rel_entity == nullptr) {
                 if (chart->pedantic) {
-                    string s = "Cound not find entity '" + rel.value;
+                    string s = "Cound not find entity '" + *rel;
                     s += "' in attribute 'relative'. Ignoring attriute.";
                     chart->Error.Error(rel.file_pos, s);
                 } else {
-                    std::unique_ptr<EntityApp> ed = std::make_unique<EntityApp>(rel.value.c_str(), chart);
+                    std::unique_ptr<EntityApp> ed = std::make_unique<EntityApp>(rel->c_str(), chart);
                     ed->AddAttributeList(nullptr, nullptr, rel.file_pos);
                     chart->AutoGenEntities.Append(std::move(ed));
                     //position remains at the old, not incremented Entity_max_pos
@@ -491,14 +478,14 @@ EntityAppHelper* EntityApp::AddAttributeList(AttributeList *al, ArcList *ch, Fil
                 }
             } else if (rel_entity->children_names.size()) {
                 //the entity in the "relative" attribute is a grouped one
-                if (!pos.is_set)
+                if (!pos)
                     chart->Error.Error(rel.file_pos, "Cannot put an entity exactly onto a grouped entity. Specify the 'pos' attribute.",
                                        " Ignoring attribute.");
-                else if (pos.value==0)
+                else if (pos==0)
                     chart->Error.Error(pos.file_pos, "Cannot put an entity exactly onto a grouped entity. Specify the 'pos' attribute.",
                                        " Ignoring positioning attributes.");
                 else {
-                    const EntityRef right = chart->FindLeftRightDescendant(rel_entity, pos.value<0, false);
+                    const EntityRef right = chart->FindLeftRightDescendant(rel_entity, *pos<0, false);
                     position     = right->pos;
                     position_exp = right->pos_exp;
                 }
@@ -509,79 +496,78 @@ EntityAppHelper* EntityApp::AddAttributeList(AttributeList *al, ArcList *ch, Fil
         }
 
         //Add the effect of pos attribute to position
-        if (pos.is_set) {
-            if (pos.value<-10 || pos.value>10) {
+        if (pos) {
+            if (*pos<-10 || *pos>10) {
                 string msg = "Exterme value for 'pos' attribute: '";
-                msg << pos.value << "'. Ignoring it.";
+                msg << *pos << "'. Ignoring it.";
                 chart->Error.Error(pos.file_pos, msg, "Use a value between [-10..10].");
             } else {
-                position     += pos.value;
-                position_exp += pos.value;
+                position     += *pos;
+                position_exp += *pos;
             }
         }
 
-        const char *style_name;
-        if (ret->entities.size()) //we are group entity
-            style_name = collapsed.is_set && collapsed.value ? "entitygroup_collapsed" :
-                            large.is_set && large.value ? "entitygroup_large" : "entitygroup";
-        else
-            style_name = "entity";
-
-        //Detemine a proper starting style, which is the default style and text
-        //Do not add the contents of "style" (from attributes), those will be applied
-        //at EntityCommand->PostParseProcess() to the running_style
-        //(which is initialized with what we specify here)
-        //(parse_running_style OTOH will be added "style" right now, to have the current
-        //style during parsing)
-        MscStyleCoW style_to_use = chart->MyCurrentContext().styles[style_name];
-        style_to_use.write().text = chart->MyCurrentContext().text;                     //default text
-        style_to_use.write().text +=chart->MyCurrentContext().styles[style_name].read().text;  //entity style text
-
-        //Unset word wrapping as something not available for entity headings
-        //(not implemented by EntityApp::Width() and EntityApp::Height() anyway)
-        style_to_use.write().text.UnsetWordWrap();
-
-        //If "entity" style contains no "indicator" value (the default in plain)
-        //we use the value from the context (true by default)
-        if (!style_to_use.read().indicator.is_set) {
-            style_to_use.write().indicator.is_set = true;
-            style_to_use.write().indicator.value = chart->MyCurrentContext().indicator.value;
-        }
-        //Copy shapeand shape size from 'style' (where set by attributes) to 'style_to_use'
-        //which contains the default shape.
-        //Note that all other attributes will be reflected via the 'running_style' mechanism,
-        //so we need not copy them, but this is a const member of Entity and we need it in
-        //Entity::Entity in the style we pass.
-        if (style.read().shape.is_set)
-            style_to_use.write().shape = style.read().shape;
-        if (style.read().shape_size.is_set)
-            style_to_use.write().shape_size = style.read().shape_size;
-
-        //Create a fully specified string format for potential
-        //\s() \f() \c() and \mX() in label that are resolved here
-        StringFormat label_format = style_to_use.read().text;
-        label_format += style.read().text;
-        //Create parsed label
-        string orig_label = label.is_set?label.value:name;
-        string proc_label = orig_label;
-        StringFormat::RemovePosEscapes(orig_label);
-        //Make the label a URL, if user has specified a url attribute.
-        if (url.length()) {
-            if (StringFormat::HasLinkEscapes(proc_label.c_str()))
-                chart->Error.Error(linenum_url_attr, "The label of the entity contains '\\L' escapes, ignoring 'url' attribute.",
-                    "Use only one of the 'url' attribute and '\\L' escapes.");
-            else
-                proc_label.insert(0, "\\L("+url+")").append("\\L()");
-        }
-        StringFormat::ExpandReferences(proc_label, chart, linenum_label_value,
-                                       &label_format, false, true, StringFormat::LABEL, true);
-        if (proc_label.length()==0)
-            proc_label = "\\c(0,0,0,0)I"; //instead of an empty string we use an invisible 'I'.
         if (!chart->SkipContent()) {
+            const char *style_name;
+            if (ret->entities.size()) //we are group entity
+                style_name = collapsed.value_or(false) ? "entitygroup_collapsed" :
+                                large.value_or(false) ? "entitygroup_large" : "entitygroup";
+            else
+                style_name = "entity";
+
+            //Detemine a proper starting style, which is the default style and text
+            //Do not add the contents of "style" (from attributes), those will be applied
+            //at EntityCommand->PostParseProcess() to the running_style
+            //(which is initialized with what we specify here)
+            //(parse_running_style OTOH will be added "style" right now, to have the current
+            //style during parsing)
+            MscStyleCoW style_to_use = chart->MyCurrentContext().styles[style_name];
+            style_to_use.write().text = chart->MyCurrentContext().text;                     //default text
+            style_to_use.write().text +=chart->MyCurrentContext().styles[style_name].read().text;  //entity style text
+
+            //Unset word wrapping as something not available for entity headings
+            //(not implemented by EntityApp::Width() and EntityApp::Height() anyway)
+            style_to_use.write().text.UnsetWordWrap();
+
+            //If "entity" style contains no "indicator" value (the default in plain)
+            //we use the value from the context (true by default)
+            if (!style_to_use.read().indicator) 
+                style_to_use.write().indicator = chart->MyCurrentContext().indicator.value();
+            //Copy shapeand shape size from 'style' (where set by attributes) to 'style_to_use'
+            //which contains the default shape.
+            //Note that all other attributes will be reflected via the 'running_style' mechanism,
+            //so we need not copy them, but this is a const member of Entity and we need it in
+            //Entity::Entity in the style we pass.
+            if (style.read().shape)
+                style_to_use.write().shape = style.read().shape;
+            if (style.read().shape_size)
+                style_to_use.write().shape_size = style.read().shape_size;
+
+            //Create a fully specified string format for potential
+            //\s() \f() \c() and \mX() in label that are resolved here
+            StringFormat label_format = style_to_use.read().text;
+            label_format += style.read().text;
+            //Create parsed label
+            string orig_label = label.value_or(name);
+            string proc_label = orig_label;
+            StringFormat::RemovePosEscapes(orig_label);
+            //Make the label a URL, if user has specified a url attribute.
+            if (url.length()) {
+                if (StringFormat::HasLinkEscapes(proc_label.c_str()))
+                    chart->Error.Error(linenum_url_attr, "The label of the entity contains '\\L' escapes, ignoring 'url' attribute.",
+                        "Use only one of the 'url' attribute and '\\L' escapes.");
+                else
+                    proc_label.insert(0, "\\L("+url+")").append("\\L()");
+            }
+            StringFormat::ExpandReferences(proc_label, chart, linenum_label_value,
+                                           &label_format, false, true, StringFormat::LABEL, true);
+            if (proc_label.length()==0)
+                proc_label = "\\c(0,0,0,0)I"; //instead of an empty string we use an invisible 'I'.
+
             //Allocate new entity with correct label and children and style
             chart->AllEntities.emplace_back(name, proc_label, orig_label, position, position_exp,
                                             style_to_use, file_pos.start,
-                                            collapsed.is_set && collapsed.value, large.is_set && large.value);
+                                            collapsed.value_or(false), large.value_or(false));
             Entity &e = chart->AllEntities.back();
             e.AddChildrenList(&ret->entities, chart);  //also fixes positions & updates running_style
             e.running_draw_pass = draw_pass;
@@ -589,18 +575,23 @@ EntityAppHelper* EntityApp::AddAttributeList(AttributeList *al, ArcList *ch, Fil
             if (!chart->IsVirtualEntity(e))
                 ret->target = name;  //if we were a group entity, use us as target for a subsequent note
             //If we used a Shape register it (so that we can save it in OLE objects)
-            if (style.read().shape.is_set && style.read().shape.value>=0)
-                chart->used_shapes.insert(style.read().shape.value);
+            if (style.read().shape && style.read().shape.value()>=0) {
+                chart->used_shapes.insert(style.read().shape.value());
+                if (large.value_or(false)) {
+                    chart->Error.Error(large.file_pos, "Cannot make large an entity with shape. Ignoring this.");
+                    large = false;
+                }
+            }
         }
     } else {
         FileLineCol p;
         // An existing entity. Disallow attributes that change labels or drawing positions
-        std::string ll = label.value;
+        std::string ll = label.value_or(std::string{});
         StringFormat::RemovePosEscapes(ll);
-        if (label.is_set && ll != our_entity->label) {
+        if (label && ll != our_entity->label) {
             p = label.file_pos;
             string mscgen_msg;
-            string plain_label = label.value;
+            string plain_label = label.value_or(std::string{});
             StringFormat::ConvertToPlainText(plain_label);
             if (chart->mscgen_compat != EMscgenCompat::FORCE_MSCGEN && plain_label.length() &&
                 (plain_label[0]=='>' || plain_label[0]==':')) {
@@ -616,22 +607,26 @@ EntityAppHelper* EntityApp::AddAttributeList(AttributeList *al, ArcList *ch, Fil
                              + our_entity->orig_label + "'.", mscgen_msg);
         }
 
-        if (pos.is_set) {
+        if (pos) {
             p = pos.file_pos;
             chart->Error.Error(p, "Cannot change the position of an entity after declaration. Ignoring attribute 'pos'.");
         }
-        if (rel.is_set) {
+        if (rel) {
             p = rel.file_pos;
             chart->Error.Error(p, "Cannot change the position of an entity after declaration. Ignoring attribute 'relative'.");
         }
-        if (collapsed.is_set) {
+        if (collapsed) {
             p = collapsed.file_pos;
             chart->Error.Error(p, "You can only declare an entity collapsed at its definition. Ignoring attribute 'collapsed'.");
         }
+        if (large) {
+            p = large.file_pos;
+            chart->Error.Error(p, "You can only declare an entity large at its definition. Ignoring attribute 'large'.");
+        }
         //if any of the above errors, add extra info
         if (p.file!=-1)
             chart->Error.Error(our_entity->file_pos, p, "Entity '" + name + "' was defined here.");
-        label.is_set = pos.is_set = rel.is_set = collapsed.is_set = false;
+        label.reset(); pos.reset(); rel.reset(); collapsed.reset();
 
         //Also indicate an error at any attempt to add children (even if "ch" did not
         //contain actual entities, it signals we had a "{ ... }" block
@@ -735,10 +730,8 @@ bool EntityApp::AttributeValues(const std::string attr, Csh &csh)
  * combines show, active & style members.*/
 void EntityApp::Combine(EntityApp *ed)
 {
-    if (ed->show.is_set)
-        show = ed->show;
-    if (ed->active.is_set)
-        active = ed->active;
+    if (ed->show) show = ed->show;
+    if (ed->active) active = ed->active;
     style += ed->style;
     CombineComments(ed);
  }
@@ -773,7 +766,7 @@ double EntityApp::Width() const
     }
     //Entity labels cannot be word wrapped
     double inner = parsed_label.getTextWidthHeight().x;
-    if (entity->children_names.size() && style.read().indicator.value && entity->collapsed)
+    if (entity->children_names.size() && style.read().indicator.value() && entity->collapsed)
         inner = std::max(inner, GetIndiactorSize().x + 2*chart->boxVGapInside);
     const double width = ceil(style.read().line.LineWidth()*2 + inner);
     return width + fmod_negative_safe(width, 2.); //always return an even number
@@ -813,7 +806,7 @@ Range EntityApp::Height(Area &cover, const NPtrList<EntityApp> &children)
     const XY wh = draw_heading ? parsed_label.getTextWidthHeight() : XY(0,0);
     const double lw = style.read().line.LineWidth();
     if (children.size()==0 && !entity->large) {
-        if (entity->children_names.size() && style.read().indicator.value)
+        if (entity->children_names.size() && style.read().indicator.value())
             indicator_ypos_offset = wh.y + lw + chart->boxVGapInside;
         const double width = Width();
         const double indicator_height = (indicator_ypos_offset > 0) ? GetIndiactorSize().y + 2*chart->boxVGapInside : 0;
@@ -843,7 +836,7 @@ Range EntityApp::Height(Area &cover, const NPtrList<EntityApp> &children)
         double top = 0, bottom = 0;
         for (auto i = children.begin(); i!=children.end(); i++) {
             top =    std::min(top,    (*i)->outer_edge.y.from);
-            bottom = std::max(bottom, (*i)->outer_edge.y.till + (*i)->style.read().shadow.offset.value);
+            bottom = std::max(bottom, (*i)->outer_edge.y.till + (*i)->style.read().shadow.offset.value());
         }
         outer_edge.y.from = top - chart->headingVGapAbove - ceil(wh.y + lw);
         outer_edge.y.till = bottom + chart->headingVGapBelow + lw;
@@ -855,7 +848,7 @@ Range EntityApp::Height(Area &cover, const NPtrList<EntityApp> &children)
     }
     area.arc = this;
     //Add shadow to outer_edge and place that to cover
-    Area my_cover(Block(outer_edge).Shift(XY(style.read().shadow.offset.value,style.read().shadow.offset.value)) += outer_edge, this);
+    Area my_cover(Block(outer_edge).Shift(XY(style.read().shadow.offset.value(), style.read().shadow.offset.value())) += outer_edge, this);
     my_cover.mainline = Block(chart->GetDrawing().x, outer_edge.y);
     cover = std::move(my_cover);
     const Block b = outer_edge.CreateExpand(-lw/2);
@@ -863,7 +856,7 @@ Range EntityApp::Height(Area &cover, const NPtrList<EntityApp> &children)
     if (children.size())
         area_draw += area_important;
     chart->NoteBlockers.Append(this);
-    return Range(outer_edge.y.from, outer_edge.y.till + style.read().shadow.offset.value);
+    return Range(outer_edge.y.from, outer_edge.y.till + style.read().shadow.offset.value());
 }
 
 /** Add a small block blocking notes for EntityApp objects displaying no heading.
@@ -904,10 +897,8 @@ void EntityApp::PostPosProcess(Canvas &canvas, Chart *ch)
             yPos;
     const EEntityStatus old_status = entity->status.GetStatus(yUse);
     EEntityStatus new_status = old_status;
-    if (show.is_set)
-        new_status.Show(show.value);
-    if (active.is_set)
-        new_status.Activate(active.value);
+    if (show) new_status.Show(*show);
+    if (active) new_status.Activate(*active);
     if (!(new_status == old_status))
         entity->status.SetStatus(yUse, new_status);
     entity->status.ApplyStyle(yUse, style);
@@ -981,15 +972,43 @@ void EntityApp::CollectIsMapElements(Canvas &canvas)
 void EntityApp::Draw(Canvas &canvas)
 {
     Block b(outer_edge);
-    if (entity->shape<0 && entity->large) {
+    const bool large = entity->large && !entity->collapsed;
+    const auto compatible = [](const MscStyleCoW& a, const MscStyleCoW& b) {
+        if (a.is_same_as(b)) return true;
+        return a.read().f_line && b.read().f_line
+            && a.read().f_fill && b.read().f_fill
+            && a.read().f_shadow && b.read().f_shadow
+            && a.read().line==b.read().line
+            && a.read().fill==b.read().fill
+            && a.read().shadow==b.read().shadow;
+    };
+    if (large) {
         //expand to the bottom of our current status
-        b.y.till = std::min({entity->status.ShowTill(b.y.till), entity->status.StyleTill(b.y.till), chart->GetDrawing().y.till});
+        do {
+            b.y.till = std::min({ entity->status.ShowTill(b.y.till), entity->status.StyleTill(b.y.till), chart->GetTotal().y.till });
+        } while (b.y.till < chart->GetTotal().y.till
+                 && entity->status.GetStatus(b.y.till).IsOn() 
+                 && compatible(style, entity->status.GetStyle(b.y.till)));
     }
-    canvas.Entity(b, parsed_label, style.read(), chart->Shapes, entity->shape);
+    canvas.Entity(b, parsed_label, style.read(), chart->Shapes, entity->shape, !entity->children_names.empty());
 
     //Draw indicator
     if (indicator_ypos_offset > 0)
         DrawIndicator(XY(outer_edge.Centroid().x, outer_edge.y.from + indicator_ypos_offset),
         &canvas);
+    
+    //For large keep drawing all the span of 'on'-ness
+    if (!large) return;
+    while (b.y.till < chart->GetTotal().y.till
+           && entity->status.GetStatus(b.y.till).IsOn()) {
+        b.y.from = b.y.till;
+        MscStyleCoW local_style = entity->status.GetStyle(b.y.till);
+        do {
+            b.y.till = std::min({ entity->status.ShowTill(b.y.till), entity->status.StyleTill(b.y.till), chart->GetTotal().y.till });
+        } while (b.y.till < chart->GetTotal().y.till
+                 && entity->status.GetStatus(b.y.till).IsOn()
+                 && compatible(local_style, entity->status.GetStyle(b.y.till)));
+        canvas.Entity(b, {}, local_style.read(), chart->Shapes);
+    }
 }
 
