@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -211,16 +211,22 @@ class ArcBase;
 /** A list of arcs (ArcBase descendants) by pointer*/
 typedef UPtrList<ArcBase> ArcList;
 
+/** Store a note temporarily */
+struct StoredNote {
+    std::unique_ptr<Note> note;  //The note we have stored
+    std::string target;          //The entity it targets (empty if none)
+};
+using StoredNotes = std::vector<StoredNote>;
+
 /** A list of EntityApp objects with potential associated CommandNotes.
  * This is used during parsing to build up EntityCommand objects. */
 struct EntityAppHelper
 {
-    EntityAppList          entities;     ///<A list of entity definitions. 
-    UPtrList<Note>         notes;        ///<A set of notes associated with one of the entities in `entities` member.
-    std::list<std::string> note_targets; ///<The name of entities the notes in `notes` correspond to. Has as many members as `notes`.
-    std::string            target;       ///<A name of an entity in `entity` to be the target for a subsequent note, if that has no 'at' clause.
+    EntityAppList entities; ///<A list of entity definitions. 
+    StoredNotes   notes;    ///<A set of notes associated with one of the entities in `entities` member.
+    std::string   target;   ///<A name of an entity in `entity` to be the target for a subsequent note, if that has no 'at' clause.
     /** Prepend a set of entities, and merge the list of notes.*/
-    EntityAppHelper *Prepend(EntityAppHelper*edh) {if (edh) {entities.Prepend(&edh->entities); notes.Prepend(&edh->notes); note_targets.splice(note_targets.begin(), edh->note_targets);} return this;} //leave "target" as that of the latter edh
+    EntityAppHelper* Prepend(EntityAppHelper* edh);
 };
 
 class DirArrow;
@@ -277,6 +283,7 @@ public:
     Label               parsed_label;///<The complete label with the right formatting. Finalized during PostParse.
     bool                defining;    ///<True if this is the first EntityApp object for this entity = this EntityApp created the entity (set in AddAttrList).
     bool                draw_heading;///<True if we have to draw heading of this entity at this point. Set in PostParse.
+    bool                large_appears;///<True if we are for a non-collapsed, large group entirty, which was not viaible, but appears here.
 
     mutable EntityRef left_ent;          ///<For a grouped entity the leftmost of its active child entities, else same as `entity` member.
     mutable EntityRef right_ent;         ///<For a grouped entity the rightmost of its active child entities, else same as `entity` member.
@@ -289,7 +296,7 @@ public:
     EntityApp(MscChart* chart, bool shown_only, const char *prefix, const  FileLineCol &l);
 
     bool AddAttribute(const Attribute&) override;
-    EntityAppHelper* AddAttributeList(AttributeList *, ArcList *children, FileLineCol l);
+    std::unique_ptr<EntityAppHelper> AddAttributeList(AttributeList *, ArcList *children, FileLineCol l);
     static void AttributeNames(Csh &csh);
     static bool AttributeValues(const std::string attr, Csh &csh);
     void Combine(EntityApp *ed); 
