@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2021 Zoltan Turanyi
+    Copyright (C) 2008-2022 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -97,24 +97,22 @@ void MscStyle::MakeCompleteButText()
     if (f_vfill) vfill.MakeComplete();
     else vfill.Empty();
     //no change to aline and atext - those may remain incomplete
-    solid.is_set=f_solid;
-    solid.value = 128;
-    side.is_set = f_side != ESideType::NO;  
-    side.value = ESide::RIGHT;
-    vspacing.is_set = f_vspacing;
-    vspacing.value = 0;
-    indicator.is_set = f_indicator;
-    indicator.value = true;
-    makeroom.is_set = f_makeroom;
-    makeroom.value = true;
+    solid = 128;
+    if (f_side != ESideType::NO) side = ESide::RIGHT;
+    else side.reset();
+    if (f_vspacing) vspacing = 0;
+    else vspacing.reset();
+    if (f_indicator) indicator = true;
+    else indicator.reset();
+    if (f_makeroom) makeroom = true;
+    else makeroom.reset();
     if (f_note) note.MakeComplete();
     else note.Empty();
     //no change to lost fields - those may remain incomplete, 
     lsym_line.MakeComplete(); 
-    lsym_line.color.value = ColorType(255, 0, 0);
-    lsym_line.width.value = 2;
-    lsym_size.is_set = true;
-    lsym_size.value = EArrowSize::NORMAL;
+    lsym_line.color = ColorType(255, 0, 0);
+    lsym_line.width = 2;
+    lsym_size = EArrowSize::NORMAL;
     tag_line.MakeComplete();
     tag_fill.MakeComplete();
     //tag_text unchanged
@@ -128,17 +126,17 @@ void MscStyle::Empty()
     vfill.Empty();
     aline.Empty();
     atext.Empty();
-    solid.is_set = false;
-    side.is_set = false;
-    vspacing.is_set = false;
-    indicator.is_set = false;
-    makeroom.is_set = false;
+    solid.reset();
+    side.reset();
+    vspacing.reset();
+    indicator.reset();
+    makeroom.reset();
     note.Empty();
     lost_line.Empty();
     lost_arrow.Empty();
     lost_text.Empty();
     lsym_line.Empty();
-    lsym_size.is_set = false;
+    lsym_size.reset();
     tag_line.Empty();
     tag_fill.Empty();
     tag_text.Empty();
@@ -157,11 +155,11 @@ Style & MscStyle::operator +=(const Style &t)
         aline += p->aline; 
         atext += p->atext; 
     }
-    if (p->f_solid && f_solid && p->solid.is_set) solid = p->solid;
-    if (p->f_side != ESideType::NO && p->side.is_set && IsValidSideValue(f_side, p->side.value)) side = p->side;
-    if (p->f_vspacing && f_vspacing && p->vspacing.is_set) vspacing = p->vspacing;
-    if (p->f_indicator && f_indicator && p->indicator.is_set) indicator = p->indicator;
-    if (p->f_makeroom && f_makeroom && p->makeroom.is_set) makeroom = p->makeroom;
+    if (p->f_solid && f_solid && p->solid) solid = p->solid;
+    if (p->f_side != ESideType::NO && p->side && IsValidSideValue(f_side, *p->side)) side = p->side;
+    if (p->f_vspacing && f_vspacing && p->vspacing) vspacing = p->vspacing;
+    if (p->f_indicator && f_indicator && p->indicator) indicator = p->indicator;
+    if (p->f_makeroom && f_makeroom && p->makeroom) makeroom = p->makeroom;
     if (p->f_note && f_note) note += p->note;
     if (p->f_lost && f_lost) {
         lost_line += p->lost_line;
@@ -170,7 +168,7 @@ Style & MscStyle::operator +=(const Style &t)
     }
     if (p->f_lsym && f_lsym) {
         lsym_line += p->lsym_line;
-        if (p->lsym_size.is_set) lsym_size = p->lsym_size;
+        if (p->lsym_size) lsym_size = p->lsym_size;
     }
     if (p->f_tag && f_tag) {
         tag_line += p->tag_line;
@@ -268,12 +266,11 @@ bool MscStyle::AddAttribute(const Attribute &a, Chart *c)
     if (a.Is("solid") && f_solid) {
         if (a.type == EAttrType::CLEAR) {
             if (a.EnsureNotClear(chart->Error, type))
-                solid.is_set = false;
+                solid.reset();
             return true;
         }
         if (a.type == EAttrType::NUMBER && a.number>=0 && a.number <= 255) {
-            solid.is_set = true;
-            solid.value = a.number<=1 ? (unsigned char)(a.number*255) : (unsigned char)a.number;
+            solid = a.number<=1 ? (unsigned char)(a.number*255) : (unsigned char)a.number;
             return true;
         }
         a.InvalidValueError("0..1' or '0..255", chart->Error);
@@ -281,14 +278,13 @@ bool MscStyle::AddAttribute(const Attribute &a, Chart *c)
     if (a.Is("side") && f_side != ESideType::NO) {
         if (a.type == EAttrType::CLEAR) {
             if (a.EnsureNotClear(chart->Error, type))
-                side.is_set = false;
+                side.reset();
             return true;
         }
         ESide tmp_side;
         if (a.type == EAttrType::STRING && Convert(a.value, tmp_side)) 
             if (IsValidSideValue(f_side, tmp_side)) {
-                side.is_set = true;
-                side.value = tmp_side;
+                side = tmp_side;
                 return true;
             }
         a.InvalidValueError(f_side == ESideType::LEFT_RIGHT ? "left/right" : "left/right/end", chart->Error);
@@ -297,47 +293,42 @@ bool MscStyle::AddAttribute(const Attribute &a, Chart *c)
     if (a.Is("compress") && f_vspacing) {
         if (a.type == EAttrType::CLEAR) {
             if (a.EnsureNotClear(chart->Error, type))
-                vspacing.is_set = false;
+                vspacing.reset();
             return true;
         }
         if (!a.CheckType(EAttrType::BOOL, chart->Error)) return true;
-        vspacing.is_set = true;
-        vspacing.value = a.yes ? DBL_MIN : 0;
+        vspacing = a.yes ? DBL_MIN : 0;
         return true;
     }
     if (a.Is("vspacing") && f_vspacing) {
         if (a.type == EAttrType::CLEAR) {
             if (a.EnsureNotClear(chart->Error, type))
-                vspacing.is_set = false;
+                vspacing.reset();
         } else if (a.type == EAttrType::STRING && a.value == "compress") {
-            vspacing.is_set = true;
-            vspacing.value = DBL_MIN;
+            vspacing = DBL_MIN;
         } else if (a.CheckType(EAttrType::NUMBER, chart->Error)) {
-            vspacing.is_set = true;
-            vspacing.value = a.number;
+            vspacing = a.number;
         }
         return true;
     }
     if (a.Is("indicator") && f_indicator) {
         if (a.type == EAttrType::CLEAR) {
             if (a.EnsureNotClear(chart->Error, type))
-                indicator.is_set = false;
+                indicator.reset();
             return true;
         }
         if (!a.CheckType(EAttrType::BOOL, chart->Error)) return true;
-        indicator.is_set = true;
-        indicator.value = a.yes;
+        indicator = a.yes;
         return true;
     }
     if (a.Is("makeroom") && f_makeroom) {
         if (a.type == EAttrType::CLEAR) {
             if (a.EnsureNotClear(chart->Error, type))
-                makeroom.is_set = false;
+                makeroom.reset();
             return true;
         }
         if (!a.CheckType(EAttrType::BOOL, chart->Error)) return true;
-        makeroom.is_set = true;
-        makeroom.value = a.yes;
+        makeroom = a.yes;
         return true;
     }
     if (f_lost) {
@@ -352,12 +343,11 @@ bool MscStyle::AddAttribute(const Attribute &a, Chart *c)
         if (a.StartsWith("x.line"))
             return lsym_line.AddAttribute(a, chart, type);
         if (a.Is("x.size")) {
-            if (a.type == EAttrType::STRING &&
-                Convert(a.value, lsym_size.value)) {
-                lsym_size.is_set = true;
+            if (EArrowSize s; a.type == EAttrType::STRING && Convert(a.value, s)) {
+                lsym_size = s;
                 return true;
             }
-            a.InvalidValueError(CandidatesFor(lsym_size.value), chart->Error);
+            a.InvalidValueError(CandidatesFor(*lsym_size), chart->Error);
             return true;
         }
     }
@@ -490,12 +480,12 @@ bool CshHintGraphicCallbackForSide(Canvas *canvas, CshHintGraphicParam p, CshHin
     LineAttr eLine(ELineType::SOLID, ColorType(0,0,0), 1, ECornerType::NONE, 0);
     MscArrowHeads aa(EArcArrowType::BIGARROW);
     aa.SetLineColor(ColorType(0,32,192)); //blue-green
-    aa.SetType(EArrowEnd::END, {true, EArrowType::SOLID});
-    aa.SetType(EArrowEnd::START, {true, EArrowType::NONE});
+    aa.SetType(EArrowEnd::END, EArrowType::SOLID);
+    aa.SetType(EArrowEnd::START, EArrowType::NONE);
     aa.SetSize(EArrowSize::SPOT);
     aa.CreateArrowHeads();
     ShadowAttr shadow;
-    FillAttr fill(aa.GetLine().color.value.Lighter(0.7), t==ESide::RIGHT ? EGradientType::DOWN : EGradientType::UP);
+    FillAttr fill(aa.GetLine().color->Lighter(0.7), t==ESide::RIGHT ? EGradientType::DOWN : EGradientType::UP);
     std::vector<double> active(2,0.);
     canvas->Transform_Rotate(bounds/2, t==ESide::LEFT ? 0 : t==ESide::RIGHT ? M_PI : M_PI/2);
     canvas->Clip(XY(bounds.x*0.1,1), XY(bounds.x-1, bounds.y-1));
@@ -598,13 +588,13 @@ void msc::MscContext::ApplyContextContent(const MscContext & o)
         defCommentFill = o.defCommentFill;
         defBackground = o.defBackground;
     } else {
-        if (o.hscale.is_set) hscale = o.hscale;
-        if (o.numbering.is_set) numbering = o.numbering;
-        if (o.vspacing.is_set) vspacing = o.vspacing;
-        if (o.indicator.is_set) indicator = o.indicator;
-        if (o.slant_angle.is_set) slant_angle = o.slant_angle;
-        if (o.slant_depth.is_set) slant_depth = o.slant_depth;
-        if (o.auto_heading.is_set) auto_heading = o.auto_heading;
+        if (o.hscale) hscale = o.hscale;
+        if (o.numbering) numbering = o.numbering;
+        if (o.vspacing) vspacing = o.vspacing;
+        if (o.indicator) indicator = o.indicator;
+        if (o.slant_angle) slant_angle = o.slant_angle;
+        if (o.slant_depth) slant_depth = o.slant_depth;
+        if (o.auto_heading) auto_heading = o.auto_heading;
         defCommentLine += o.defCommentLine;
         defCommentFill += o.defCommentFill;
         defBackground += o.defBackground;
@@ -625,13 +615,13 @@ void msc::MscContext::ApplyContextContent(MscContext && o)
         defCommentFill = o.defCommentFill;
         defBackground = o.defBackground;
     } else {
-        if (o.hscale.is_set) hscale = o.hscale;
-        if (o.numbering.is_set) numbering = o.numbering;
-        if (o.vspacing.is_set) vspacing = o.vspacing;
-        if (o.indicator.is_set) indicator = o.indicator;
-        if (o.slant_angle.is_set) slant_angle = o.slant_angle;
-        if (o.slant_depth.is_set) slant_depth = o.slant_depth;
-        if (o.auto_heading.is_set) auto_heading = o.auto_heading;
+        if (o.hscale) hscale = o.hscale;
+        if (o.numbering) numbering = o.numbering;
+        if (o.vspacing) vspacing = o.vspacing;
+        if (o.indicator) indicator = o.indicator;
+        if (o.slant_angle) slant_angle = o.slant_angle;
+        if (o.slant_depth) slant_depth = o.slant_depth;
+        if (o.auto_heading) auto_heading = o.auto_heading;
         defCommentLine += o.defCommentLine;
         defCommentFill += o.defCommentFill;
         defBackground += o.defBackground;
@@ -643,13 +633,13 @@ void MscContext::Empty()
 {
     ContextBase::Empty();
     Procedures.clear();
-    hscale.is_set = false;
-    numbering.is_set = false;
-    vspacing.is_set = false;
-    indicator.is_set = false;
-    slant_angle.is_set = false;
-    slant_depth.is_set = false;
-    auto_heading.is_set = false;
+    hscale.reset();
+    numbering.reset();
+    vspacing.reset();
+    indicator.reset();
+    slant_angle.reset();
+    slant_depth.reset();
+    auto_heading.reset();
     defCommentLine.Empty();
     defCommentFill.Empty();
     defBackground.Empty();
@@ -674,9 +664,9 @@ void MscContext::Empty()
     styles["blockarrow"] =
         MscStyle(EStyleType::DEFAULT, EColorMeaning::FILL, EArcArrowType::BIGARROW,
                  false, true, true, true, true, false,
-                 false, true, true, ESideType::NO, false, false, false, false, 
+                 false, true, true, ESideType::NO, true, false, false, false, 
                  false, false, false, false);
-                 //no alt vline solid side indicator vfill makeroom note loss lsym shape, tag
+                 //no alt vline solid side vfill makeroom note loss lsym shape, tag
     styles["box_collapsed_arrow"] = styles["blockarrow"];
     styles["block->"] = styles["blockarrow"];
     styles["block->"].write().type = EStyleType::DEF_ADD;
@@ -828,25 +818,18 @@ void MscContext::Plain()
 {
     Empty();
     ContextBase::Plain();
-    hscale.is_set = true;
-    hscale.value = 1.0;
-    numbering.is_set = true;
-    numbering.value=false;
-    vspacing.is_set = true;
-    vspacing.value = 0;
-    indicator.is_set = true;
-    indicator.value = true;
-    slant_angle.is_set = true;
-    slant_angle.value = 0;
-    slant_depth.is_set = true;
-    slant_depth.value = 0;
-    auto_heading.is_set = true;
-    auto_heading.value = false;
+    hscale = 1.0;
+    numbering = false;
+    vspacing = 0;
+    indicator = true;
+    slant_angle = 0;
+    slant_depth = 0;
+    auto_heading = false;
     defCommentLine.MakeComplete();
     defCommentFill.MakeComplete();
     defBackground.MakeComplete();
-    defCommentLine.width.value = 3;
-    defCommentFill.color.value.a = 0; //fully transparent
+    defCommentLine.width = 3;
+    defCommentFill.color = ColorType(0,0,0,0); //fully transparent
     defBackground.color = ColorType::white();
 
     MscStyle *p;
@@ -854,64 +837,50 @@ void MscContext::Plain()
 
     p = &styles["arrow"].write();
     p->MakeCompleteButText();
-    p->vspacing.is_set = false;
-    p->numbering.is_set = false;
-    p->line.radius.value = 10;
-    p->line.corner.value = ECornerType::ROUND;
-    p->lost_line.color = {true, faint};
+    p->vspacing.reset();
+    p->numbering.reset();
+    p->line.radius = 10;
+    p->line.corner = ECornerType::ROUND;
+    p->lost_line.color = faint;
     p->lost_arrow.write().SetLineColor(faint);
     //p->lost_text.SetColor(faint);  do not change label in the lost part
 
-    styles["arrow_self"].write().line.radius.is_set = true;
-    styles["arrow_self"].write().line.radius.value = -1;
-    styles["arrow_self"].write().side.is_set = true;
-    styles["arrow_self"].write().side.value = ESide::RIGHT;
+    styles["arrow_self"].write().line.radius = -1;
+    styles["arrow_self"].write().side = ESide::RIGHT;
 
-    styles["->"].write().line.type.is_set = true;
-    styles["->"].write().line.type.value = ELineType::SOLID;
-    styles[">"].write().line.type.is_set = true;
-    styles[">"].write().line.type.value = ELineType::DOTTED;
-    styles[">>"].write().line.type.is_set = true;
-    styles[">>"].write().line.type.value = ELineType::DASHED;
-    styles["=>"].write().line.type.is_set = true;
-    styles["=>"].write().line.type.value = ELineType::DOUBLE;
-    styles["=>>"].write().line.type.is_set = true;
-    styles["=>>"].write().line.type.value = ELineType::DOUBLE;
-    styles["=>>"].write().arrow.write().SetType(EArrowEnd::END, {true, EArrowType::SHARP});
-    styles["=>>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, {true, EArrowType::SHARP});
-    styles["==>"].write().line.type.is_set = true;
-    styles["==>"].write().line.type.value = ELineType::DOUBLE;
+    styles["->"].write().line.type = ELineType::SOLID;
+    styles[">"].write().line.type = ELineType::DOTTED;
+    styles[">>"].write().line.type = ELineType::DASHED;
+    styles["=>"].write().line.type = ELineType::DOUBLE;
+    styles["=>>"].write().line.type = ELineType::DOUBLE;
+    styles["=>>"].write().arrow.write().SetType(EArrowEnd::END, EArrowType::SHARP);
+    styles["=>>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, EArrowType::SHARP);
+    styles["==>"].write().line.type = ELineType::DOUBLE;
 
     p = &styles["blockarrow"].write();
     p->MakeCompleteButText();
-    p->vspacing.is_set = false;
-    p->numbering.is_set = false;
-    p->line.radius.value = -1;
+    p->vspacing.reset();
+    p->numbering.reset();
+    p->line.radius = -1;
     styles["box_collapsed_arrow"] = *p;
-    styles["box_collapsed_arrow"].write().arrow.write().SetType(EArrowEnd::MIDDLE, {true, EArrowType::DOT});
+    styles["box_collapsed_arrow"].write().arrow.write().SetType(EArrowEnd::MIDDLE, EArrowType::DOT);
 
-    styles["block->"].write().line.type.is_set = true;
-    styles["block->"].write().line.type.value = ELineType::SOLID;
-    styles["block>"].write().line.type.is_set = true;
-    styles["block>"].write().line.type.value = ELineType::DOTTED;
-    styles["block>>"].write().line.type.is_set = true;
-    styles["block>>"].write().line.type.value = ELineType::DASHED;
-    styles["block=>"].write().line.type.is_set = true;
-    styles["block=>"].write().line.type.value = ELineType::DOUBLE;
-    styles["block=>>"].write().line.type.is_set = true;
-    styles["block=>>"].write().line.type.value = ELineType::DOUBLE;
-    styles["block=>>"].write().arrow.write().SetType(EArrowEnd::END, {true, EArrowType::SHARP});
-    styles["block=>>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, {true, EArrowType::SHARP});
-    styles["block==>"].write().line.type.is_set = true;
-    styles["block==>"].write().line.type.value = ELineType::DOUBLE;
+    styles["block->"].write().line.type = ELineType::SOLID;
+    styles["block>"].write().line.type = ELineType::DOTTED;
+    styles["block>>"].write().line.type = ELineType::DASHED;
+    styles["block=>"].write().line.type = ELineType::DOUBLE;
+    styles["block=>>"].write().line.type = ELineType::DOUBLE;
+    styles["block=>>"].write().arrow.write().SetType(EArrowEnd::END, EArrowType::SHARP);
+    styles["block=>>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, EArrowType::SHARP);
+    styles["block==>"].write().line.type = ELineType::DOUBLE;
 
     p = &styles["vertical"].write();
     p->MakeCompleteButText();
-    p->vspacing.is_set = false;
-    p->numbering.is_set = false;
-    p->makeroom.value = true;
-    p->side.is_set = false;
-    p->line.radius.value = -1;
+    p->vspacing.reset();
+    p->numbering.reset();
+    p->makeroom = true;
+    p->side.reset();
+    p->line.radius = -1;
     p->text.UnsetWordWrap();
     //We need this otherwise setting the global text style to wrap
     //would cause warnings for all horizontally set verticals which
@@ -919,179 +888,162 @@ void MscContext::Plain()
 
     p = &styles["vertical_range"].write();
     p->MakeCompleteButText();
-    p->vspacing.is_set = false;
-    p->numbering.is_set = false;
-    p->makeroom.value = true;
-    p->side.is_set = false;
-    p->line.radius.value = 8;
+    p->vspacing.reset();
+    p->numbering.reset();
+    p->makeroom = true;
+    p->side.reset();
+    p->line.radius = 8;
     p->text.UnsetWordWrap();
 
     p = &styles["vertical_bracket"].write();
     p->MakeCompleteButText();
-    p->vspacing.is_set = false;
-    p->numbering.is_set = false;
-    p->makeroom.value = true;
-    p->side.is_set = false;
-    p->line.radius.value = 8;
-    p->line.corner.value = ECornerType::NONE;
+    p->vspacing.reset();
+    p->numbering.reset();
+    p->makeroom = true;
+    p->side.reset();
+    p->line.radius = 8;
+    p->line.corner = ECornerType::NONE;
     p->text.UnsetWordWrap();
     
     p = &styles["vertical_pointer"].write();
     p->MakeCompleteButText();
-    p->vspacing.is_set = false;
-    p->numbering.is_set = false;
-    p->makeroom.value = true;
-    p->side.is_set = false;
-    p->line.radius.value = 8;
-    p->line.corner.value = ECornerType::NONE;
+    p->vspacing.reset();
+    p->numbering.reset();
+    p->makeroom = true;
+    p->side.reset();
+    p->line.radius = 8;
+    p->line.corner = ECornerType::NONE;
     p->text.UnsetWordWrap();
-    p->lost_line.color = {true, faint};
+    p->lost_line.color = faint;
     p->lost_arrow.write().SetLineColor(faint);
     //p->lost_text.SetColor(faint); do not change label in lost part
 
     p = &styles["vertical_brace"].write();
     p->MakeCompleteButText();
-    p->vspacing.is_set = false;
-    p->numbering.is_set = false;
-    p->makeroom.value = true;
-    p->side.is_set = false;
-    p->line.radius.value = 8;
-    p->line.corner.value = ECornerType::ROUND;
+    p->vspacing.reset();
+    p->numbering.reset();
+    p->makeroom = true;
+    p->side.reset();
+    p->line.radius = 8;
+    p->line.corner = ECornerType::ROUND;
     p->text.UnsetWordWrap();
 
-    styles["vertical->"].write().line.type.is_set = true;
-    styles["vertical->"].write().line.type.value = ELineType::SOLID;
-    styles["vertical>"].write().line.type.is_set = true; 
-    styles["vertical>"].write().line.type.value = ELineType::DOTTED;
-    styles["vertical>>"].write().line.type.is_set = true;
-    styles["vertical>>"].write().line.type.value = ELineType::DASHED;
-    styles["vertical=>"].write().line.type.is_set = true;
-    styles["vertical=>"].write().line.type.value = ELineType::DOUBLE;
-    styles["vertical=>>"].write().line.type.is_set = true;
-    styles["vertical=>>"].write().line.type.value = ELineType::DOUBLE;
-    styles["vertical=>>"].write().arrow.write().SetType(EArrowEnd::END, {true, EArrowType::SHARP});
-    styles["vertical=>>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, {true, EArrowType::SHARP});
-    styles["vertical==>"].write().line.type.is_set = true;
-    styles["vertical==>"].write().line.type.value = ELineType::DOUBLE;
+    styles["vertical->"].write().line.type = ELineType::SOLID;
+    styles["vertical>"].write().line.type = ELineType::DOTTED;
+    styles["vertical>>"].write().line.type = ELineType::DASHED;
+    styles["vertical=>"].write().line.type = ELineType::DOUBLE;
+    styles["vertical=>>"].write().line.type = ELineType::DOUBLE;
+    styles["vertical=>>"].write().arrow.write().SetType(EArrowEnd::END, EArrowType::SHARP);
+    styles["vertical=>>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, EArrowType::SHARP);
+    styles["vertical==>"].write().line.type = ELineType::DOUBLE;
 
-    styles["vertical--"].write().arrow.write().SetType(EArrowEnd::START, {true, EArrowType::NONE});
-    styles["vertical--"].write().arrow.write().SetType(EArrowEnd::MIDDLE, {true, EArrowType::NONE});
-    styles["vertical--"].write().arrow.write().SetType(EArrowEnd::END, {true, EArrowType::NONE});
-    styles["vertical--"].write().line.type.is_set = true;
-    styles["vertical--"].write().line.type.value = ELineType::SOLID;
+    styles["vertical--"].write().arrow.write().SetType(EArrowEnd::START, EArrowType::NONE);
+    styles["vertical--"].write().arrow.write().SetType(EArrowEnd::MIDDLE, EArrowType::NONE);
+    styles["vertical--"].write().arrow.write().SetType(EArrowEnd::END, EArrowType::NONE);
+    styles["vertical--"].write().line.type = ELineType::SOLID;
     styles["vertical++"] = styles["vertical--"];
-    styles["vertical++"].write().line.type.value = ELineType::DASHED;
+    styles["vertical++"].write().line.type = ELineType::DASHED;
     styles["vertical.."] = styles["vertical--"];
-    styles["vertical.."].write().line.type.value = ELineType::DOTTED;
+    styles["vertical.."].write().line.type = ELineType::DOTTED;
     styles["vertical=="] = styles["vertical--"];
-    styles["vertical=="].write().line.type.value = ELineType::DOUBLE;
+    styles["vertical=="].write().line.type = ELineType::DOUBLE;
 
     p = &styles["divider"].write();
     p->MakeCompleteButText();
-    p->vspacing.is_set = false;
-    p->numbering.is_set = false;
+    p->vspacing.reset();
+    p->numbering.reset();
     p->vline.Empty();
-    p->line.type.value = ELineType::NONE;
+    p->line.type = ELineType::NONE;
 
     p = &styles["---"].write();
-    p->line.type.is_set = true;
-    p->line.type.value = ELineType::DOTTED;
+    p->line.type = ELineType::DOTTED;
     p = &styles["..."].write();
-    p->vline.type.is_set = true;
-    p->vline.type.value = ELineType::DOTTED;
+    p->vline.type = ELineType::DOTTED;
     p->text.Apply("\\mu(10)\\md(10)");
 
     p = &styles["emptybox"].write();
     p->MakeCompleteButText();
-    p->vspacing.is_set = false;
-    p->numbering.is_set = false;
-    p->tag_line.corner.value = ECornerType::BEVEL;
-    p->tag_line.radius.value = 10;
+    p->vspacing.reset();
+    p->numbering.reset();
+    p->tag_line.corner = ECornerType::BEVEL;
+    p->tag_line.radius = 10;
     p->tag_text.Apply("\\B\\-");
     styles["box_collapsed"] = styles["emptybox"];
     styles["box"] = styles["emptybox"];
     styles["box"].write().text.Apply("\\pl");
-    styles["box"].write().line.type.is_set = false;
+    styles["box"].write().line.type.reset();
 
-    styles["--"].write().line.type.is_set = true;
-    styles["--"].write().line.type.value = ELineType::SOLID;
-    styles["++"].write().line.type.is_set = true;
-    styles["++"].write().line.type.value = ELineType::DASHED;
-    styles[".."].write().line.type.is_set = true;
-    styles[".."].write().line.type.value = ELineType::DOTTED;
-    styles["=="].write().line.type.is_set = true;
-    styles["=="].write().line.type.value = ELineType::DOUBLE;
+    styles["--"].write().line.type = ELineType::SOLID;
+    styles["++"].write().line.type = ELineType::DASHED;
+    styles[".."].write().line.type = ELineType::DOTTED;
+    styles["=="].write().line.type = ELineType::DOUBLE;
 
     p = &styles["pipe"].write();
     p->MakeCompleteButText();
-    p->vspacing.is_set = false;
-    p->numbering.is_set = false;
-    p->line.radius.value = 5;
+    p->vspacing.reset();
+    p->numbering.reset();
+    p->line.radius = 5;
 
-    styles["pipe--"].write().line.type.is_set = true;
-    styles["pipe--"].write().line.type.value = ELineType::SOLID;
-    styles["pipe++"].write().line.type.is_set = true;
-    styles["pipe++"].write().line.type.value = ELineType::DASHED;
-    styles["pipe.."].write().line.type.is_set = true;
-    styles["pipe.."].write().line.type.value = ELineType::DOTTED;
-    styles["pipe=="].write().line.type.is_set = true;
-    styles["pipe=="].write().line.type.value = ELineType::DOUBLE;
+    styles["pipe--"].write().line.type = ELineType::SOLID;
+    styles["pipe++"].write().line.type = ELineType::DASHED;
+    styles["pipe.."].write().line.type = ELineType::DOTTED;
+    styles["pipe=="].write().line.type = ELineType::DOUBLE;
 
     styles["entity"].write().MakeCompleteButText();
     styles["entity"].write().arrow.Empty(); //we have to ensure this is empty. aline and atext is left empty by MakeCompleteButText()
     styles["entitygroup_collapsed"].write().MakeCompleteButText();
     styles["entitygroup_collapsed"].write().arrow.Empty(); 
     styles["entitygroup"].write().MakeCompleteButText();
-    styles["entitygroup"].write().line.type.value = ELineType::DASHED;
+    styles["entitygroup"].write().line.type = ELineType::DASHED;
     styles["entitygroup_large"].write().MakeCompleteButText();
-    styles["entitygroup_large"].write().line.type.value = ELineType::NONE;
-    styles["entitygroup_large"].write().fill.color.value = ColorType(192,192,192);
+    styles["entitygroup_large"].write().line.type = ELineType::NONE;
+    styles["entitygroup_large"].write().fill.color = ColorType(192,192,192);
 
     styles["indicator"].write().MakeCompleteButText();
-    styles["indicator"].write().line.width.value = 2;
+    styles["indicator"].write().line.width = 2;
 
     p = &styles["symbol"].write();
     p->MakeCompleteButText();
-    p->makeroom.value = true;
+    p->makeroom = true;
     styles["text"] = styles["symbol"];
-    styles["text"].write().line.type.value = ELineType::NONE;
-    styles["text"].write().fill.color.value = ColorType(0, 0, 0, 0); //fully transparent
+    styles["text"].write().line.type = ELineType::NONE;
+    styles["text"].write().fill.color = ColorType::none(); //fully transparent
     styles["text"].write().text += "\\mn(10)\\ms(6)"; //small font to 6, normal to 10
 
     styles["note"].write().MakeCompleteButText();
-    styles["note"].write().numbering.is_set = false;
+    styles["note"].write().numbering.reset();
     styles["note"].write().text += "\\mn(10)\\ms(6)"; //small font to 6, normal to 10
 
     styles["comment"].write().MakeCompleteButText();
-    styles["comment"].write().numbering.is_set = false;
+    styles["comment"].write().numbering.reset();
     styles["comment"].write().text += "\\mn(10)\\ms(6)\\pl"; //small font to 6, normal to 10, left para
 
     p = &styles["endnote"].write();
     p->MakeCompleteButText();
-    p->numbering.is_set = false;
+    p->numbering.reset();
     p->text += "\\mn(10)\\ms(6)\\pl"; //small font to 6, normal to 10, left para
-    p->side.value = ESide::END;
+    p->side = ESide::END;
 
     //styles["footnote"].write().MakeCompleteButText();
-    //styles["footnote"].write().numbering.is_set = false;
+    //styles["footnote"].write().numbering.reset();
     //styles["footnote"].write().text += "\\mn(10)\\ms(6)\\pl"; //small font to 6, normal to 10, left para
     //styles["footnote"].write().side.value = ESide::END;
 
     p = &styles["title"].write();
     p->MakeCompleteButText();
-    p->vline.type.value = ELineType::NONE;
-    p->line.type.value = ELineType::NONE;
-    p->fill.color.value = ColorType(0,0,0,0); //no fill
+    p->vline.type = ELineType::NONE;
+    p->line.type = ELineType::NONE;
+    p->fill.color = ColorType(0,0,0,0); //no fill
     p->text += "\\mn(28)\\ms(18)\\B";
     styles["subtitle"] = styles["title"];
     styles["subtitle"].write().text += "\\mn(22)\\ms(14)\\B";
 
-    const OptAttr<ColorType> weak(true, ColorType(255, 255, 255, 128, ColorType::OVERLAY));
+    const OptAttr<ColorType> weak = ColorType(255, 255, 255, 128, ColorType::OVERLAY);
     styles["weak"].write().line.color = weak;
-    styles["weak"].write().arrow.write().SetLineColor(weak.value);
-    styles["weak"].write().text.SetColor(weak.value);
+    styles["weak"].write().arrow.write().SetLineColor(*weak);
+    styles["weak"].write().text.SetColor(*weak);
     styles["weak"].write().text.Apply("\\i");
-    OptAttr<double> w2(true, 2.0);
+    const OptAttr<double> w2 = 2.0;
     styles["strong"].write().line.width = w2;
     LineAttr l = styles["strong"].read().arrow.read().GetLine();
     l.width = w2;
@@ -1105,38 +1057,29 @@ void MscContext::MscgenCompat()
     is_full = false;
 
     MscStyle *p = &styles["arrow"].write();
-    p->lost_line.color.is_set = true;
-    p->lost_line.color.value = ColorType(0, 0, 0, 0); //none
-    p->lost_arrow.write().SetLineColor(ColorType(0, 0, 0, 0));
-    p->lsym_line.width.is_set = true;
-    p->lsym_line.width.value = 1;
-    p->lsym_line.color.is_set = true;
-    p->lsym_line.color.value = ColorType(0, 0, 0); //black
+    p->lost_line.color = ColorType::none();
+    p->lost_arrow.write().SetLineColor(ColorType::none());
+    p->lsym_line.width = 1;
+    p->lsym_line.color = ColorType::black();
 
-    styles["->"].write().line.type.is_set = true;
-    styles["->"].write().line.type.value = ELineType::SOLID;
-    styles["->"].write().arrow.write().SetType(EArrowEnd::END, {true, EArrowType::HALF});
-    styles["->"].write().arrow.write().SetType(EArrowEnd::MIDDLE, {true, EArrowType::HALF});
-    styles[">"].write().line.type.is_set = true;
-    styles[">"].write().line.type.value = ELineType::DOTTED;
-    styles[">"].write().arrow.write().SetType(EArrowEnd::END, {true, EArrowType::SOLID});
-    styles[">"].write().arrow.write().SetType(EArrowEnd::MIDDLE, {true, EArrowType::SOLID});
-    styles[">>"].write().line.type.is_set = true;
-    styles[">>"].write().line.type.value = ELineType::DOTTED;
-    styles[">>"].write().arrow.write().SetType(EArrowEnd::END, {true, EArrowType::SOLID});
-    styles[">>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, {true, EArrowType::SOLID});
-    styles["=>"].write().line.type.is_set = true;
-    styles["=>"].write().line.type.value = ELineType::SOLID;
-    styles["=>"].write().arrow.write().SetType(EArrowEnd::END, {true, EArrowType::SOLID});
-    styles["=>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, {true, EArrowType::SOLID});
-    styles["=>>"].write().line.type.is_set = true;
-    styles["=>>"].write().line.type.value = ELineType::SOLID;
-    styles["=>>"].write().arrow.write().SetType(EArrowEnd::END, {true, EArrowType::LINE});
-    styles["=>>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, {true, EArrowType::LINE});
+    styles["->"].write().line.type = ELineType::SOLID;
+    styles["->"].write().arrow.write().SetType(EArrowEnd::END, EArrowType::HALF);
+    styles["->"].write().arrow.write().SetType(EArrowEnd::MIDDLE, EArrowType::HALF);
+    styles[">"].write().line.type = ELineType::DOTTED;
+    styles[">"].write().arrow.write().SetType(EArrowEnd::END, EArrowType::SOLID);
+    styles[">"].write().arrow.write().SetType(EArrowEnd::MIDDLE, EArrowType::SOLID);
+    styles[">>"].write().line.type = ELineType::DOTTED;
+    styles[">>"].write().arrow.write().SetType(EArrowEnd::END, EArrowType::SOLID);
+    styles[">>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, EArrowType::SOLID);
+    styles["=>"].write().line.type = ELineType::SOLID;
+    styles["=>"].write().arrow.write().SetType(EArrowEnd::END, EArrowType::SOLID);
+    styles["=>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, EArrowType::SOLID);
+    styles["=>>"].write().line.type = ELineType::SOLID;
+    styles["=>>"].write().arrow.write().SetType(EArrowEnd::END, EArrowType::LINE);
+    styles["=>>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, EArrowType::LINE);
     //==> is the same as in plain: double line, solid arrow
-    styles[":>"].write().line.type.is_set = true;
-    styles[":>"].write().line.type.value = ELineType::DOUBLE;
-    styles[":>"].write().arrow.write().SetType(EArrowEnd::END, {true, EArrowType::SOLID});
-    styles[":>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, {true, EArrowType::SOLID});
+    styles[":>"].write().line.type = ELineType::DOUBLE;
+    styles[":>"].write().arrow.write().SetType(EArrowEnd::END, EArrowType::SOLID);
+    styles[":>"].write().arrow.write().SetType(EArrowEnd::MIDDLE, EArrowType::SOLID);
 }
 
