@@ -123,20 +123,104 @@ ReadIncludeFile(std::string_view filename, void *, std::string_view included_fro
     return ret;
 }
 
+std::vector<std::string> FileListProc(std::string_view prefix_, std::string_view included_from_, void* app) {
+    std::string included_from__(included_from_);
+    std::string prefix__(prefix_);
+    //convert Windows style to UNIX style
+    std::ranges::replace(included_from__, '\\', '/');
+    std::ranges::replace(prefix__, '\\', '/');
+
+    namespace fs = std::filesystem;
+
+    fs::path included_from = fs::path(included_from__).parent_path();
+    fs::path dir = fs::path(prefix__).parent_path();
+
+    if ((included_from.empty() || included_from.is_relative()) && (dir.empty() || dir.is_relative()))
+        return{}; //cannot determine directory
+    const bool use_included_from = dir.is_relative();
+    if (use_included_from) dir = included_from / dir;
+    std::vector<std::string> ret;
+    try {
+        for (auto& e : fs::directory_iterator(dir))
+            if (e.is_regular_file() || e.is_symlink() || e.is_directory()) {
+                fs::path p = e.path();
+                if (use_included_from) {
+                    fs::path p2;
+                    for (auto i = std::mismatch(p.begin(), p.end(), included_from.begin(), included_from.end()).first;
+                         i!=p.end(); i++)
+                        p2 /= *i;
+                    ret.push_back(p2.native());
+                } else
+                    ret.push_back(p.native());
+            }
+    } catch (const std::filesystem::filesystem_error&) {
+    }
+    return ret;
+}
+
 
 #define RCDIR_NAME ".msc-genrc"
 #define LOAD_FILE_NAME ".msc-generator-load"
 
 #ifdef BUILD_GUI
-std::optional<ImTextureID> TextureFromSurface(cairo_surface_t *rgba_surface);
+TextureID TextureFromSurface(cairo_surface_t *rgba_surface);
 void DestroyTexture(ImTextureID t);
 int MscGenKeyMap[MscGenKey_COUNT];
+const char directory_separator = '/';
 void GUIBeep() { printf("\a"); }
-std::string_view FileNameFromPath(std::string_view full_name) noexcept {
-    const auto last_slash_pos = full_name.rfind('/');
-    if (last_slash_pos == full_name.npos) return full_name;
-    return full_name.substr(last_slash_pos+1);
+
+/** Maps the short name of a documentation file to its full UTF-8 path.*/
+std::map<std::string, std::string> Documentation;
+
+std::tuple<int, std::string> POpen(std::string_view cmd) {
+    std::unique_ptr<FILE, int(*)(FILE*)> fd{popen(cmd.data(), "r"), pclose};
+    using namespace std::string_literals;
+    if (!fd)
+        return {errno, "popen returned "s + std::strerror(errno)};
+    std::string out;
+    while (true) {
+        char buf[1024];
+        if (std::fgets(buf, sizeof buf, fd.get()))
+            out += buf;
+        auto err = errno;
+        if (feof(fd.get()))
+            break;
+        if (ferror(fd.get()))
+            return {err, "problem reading output: "s + std::strerror(err) + " after " + out};
+    }
+    auto r = pclose(fd.release());
+    if (r == -1 && (r = errno))
+        return {r, "pclose returned "s + std::strerror(r) + " with process output: " + out};
+    if (WIFEXITED(r))
+        r = WEXITSTATUS(r);
+    else if (WIFSIGNALED(r))
+        out = "child process terminated with " + std::to_string(WTERMSIG(r)) + " and output: " + out;
+    else
+        out = "child process stopped with output: " + out;
+    return {r, out};
 }
+
+std::optional<std::string> ShowDocumentation(const std::string& doc_name) {
+    const auto i = Documentation.find(doc_name);
+    if (i == Documentation.end())
+        return {};
+    if (auto [ret,err] = POpen("exec bash -c '"
+                               "for i in xdg-open sensible-browser open; do"
+                               " err=`$i $1 2>&1`;"
+                               " ret=$?;"
+                               " case $ret in"
+                               "  126 | 127) ;;"
+                               "  *) echo $err; exit $ret;;"
+                               " esac;"
+                               "done;"
+                               "echo No documentation browser found;"
+                               "exit 1"
+                               "' msc-doc.sh " + i->second);
+            ret)
+        return {err + " (exit status: " + std::to_string(ret) + ')'};
+    return {};
+}
+
 #endif
 
 int main(int argc, char* argv[])
@@ -156,13 +240,13 @@ int main(int argc, char* argv[])
 
     //Generate language packs
     LanguageCollection languages;
-    languages.AddLanguage(&MscChart::Factory, nullptr, nullptr);
-    languages.AddLanguage(&GraphChart::Factory, nullptr, nullptr);
+    languages.AddLanguage(&MscChart::Factory, FileListProc, nullptr);
+    languages.AddLanguage(&GraphChart::Factory, FileListProc, nullptr);
 #ifdef _DEBUG
-    languages.AddLanguage(&XxxChart::Factory, nullptr, nullptr);
-    languages.AddLanguage(&FlowChart::Factory, nullptr, nullptr);
-#endif 
-    languages.AddLanguage(&BlockChart::Factory, nullptr, nullptr);
+    languages.AddLanguage(&XxxChart::Factory, FileListProc, nullptr);
+    languages.AddLanguage(&FlowChart::Factory, FileListProc, nullptr);
+#endif
+    languages.AddLanguage(&BlockChart::Factory, FileListProc, nullptr);
 
     //Load progress balance data
     std::string load_data_fname = hdir + RCDIR_NAME "/" LOAD_FILE_NAME;
@@ -204,20 +288,20 @@ int main(int argc, char* argv[])
         snprintf(buff, sizeof(buff), "SDL v%d.%d.%d",
                  v_comp.major, v_comp.minor, v_comp.patch);
     else
-        snprintf(buff, sizeof(buff), "SDL v%d.%d.%d (compiled for: v%d.%d.%d)", 
+        snprintf(buff, sizeof(buff), "SDL v%d.%d.%d (compiled for: v%d.%d.%d)",
                  v_link.major, v_link.minor, v_link.patch, v_comp.major, v_comp.minor, v_comp.patch);
     languages.libraries[buff];
     languages.libraries["OpenGL v3+"];
 
     const bool display_only = getenv("DISPLAY") && getenv("DISPLAY")[0] && !isatty(STDIN_FILENO);
-#else 
+#else
     const bool display_only = false;
 #endif //BUILD_GUI
 
     //Do it
     auto ret = do_main(args, languages, design_files, OpenNamedFile,
                        ReadIncludeFile, nullptr,
-                       "\\mn(10)", hdir+RCDIR_NAME "/", display_only,
+                       "\\mn(10)", display_only,
                        progress, NULL, &load_data);
     if (std::holds_alternative<int>(ret)) {
         //We have completed a real command-line operation
@@ -277,7 +361,7 @@ int main(int argc, char* argv[])
         std::string last_window_title;
         bool we_are_fullscreen = false;
 #define WINDOW_TITLE_PREFIX "Msc-generator GUI"
-        auto window_flags = SDL_WindowFlags(SDL_WINDOW_OPENGL | SDL_WINDOW_RESIZABLE | SDL_WINDOW_ALLOW_HIGHDPI);
+        auto window_flags = SDL_WindowFlags(SDL_WINDOW_OPENGL | SDL_WINDOW_RESIZABLE | SDL_WINDOW_ALLOW_HIGHDPI | SDL_WINDOW_HIDDEN);
         int width = 1280, height = 720;
         SDL_Rect bounds(0,0,1920,1080);
         if (SDL_GetDisplayUsableBounds(0, &bounds))
@@ -285,7 +369,6 @@ int main(int argc, char* argv[])
         else
             width = bounds.w / 2, height = bounds.h / 2;
         const auto dpi_mul = [](const SDL_Rect &bounds) { return bounds.h/1080.0f; }; //1080p height screen is the multiplier of 1.
-        //TODO: save/load window size and pos to settings
         auto window = SDL_CreateWindow(WINDOW_TITLE_PREFIX, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, width, height, window_flags);
         SDL_GLContext gl_context = SDL_GL_CreateContext(window);
         SDL_GL_MakeCurrent(window, gl_context);
@@ -321,10 +404,11 @@ int main(int argc, char* argv[])
             }
         }
         //Ensure 'settings_dir' exists
+        std::string settings_dir = hdir+RCDIR_NAME "/";
         std::error_code ec;
-        std::filesystem::create_directories(std::get<GUIInit>(ret).settings_dir, ec);
+        std::filesystem::create_directories(settings_dir, ec);
         if (ec)
-            std::cerr<<"Could not create settings directory '"<<std::get<GUIInit>(ret).settings_dir<<"':"<<ec.message()<<std::endl;
+            std::cerr<<"Could not create settings directory '"<<settings_dir<<"':"<<ec.message()<<std::endl;
 
         // Setup Dear ImGui style
         ImGui::StyleColorsLight();
@@ -339,16 +423,80 @@ int main(int argc, char* argv[])
         MscGenKeyMap[MscGenKey_F8] = SDL_SCANCODE_F8;
         MscGenKeyMap[MscGenKey_S] = SDL_SCANCODE_S;
         MscGenKeyMap[MscGenKey_N] = SDL_SCANCODE_N;
+        MscGenKeyMap[MscGenKey_W] = SDL_SCANCODE_W;
         MscGenKeyMap[MscGenKey_O] = SDL_SCANCODE_O;
         MscGenKeyMap[MscGenKey_R] = SDL_SCANCODE_R;
         MscGenKeyMap[MscGenKey_F] = SDL_SCANCODE_F;
         MscGenKeyMap[MscGenKey_H] = SDL_SCANCODE_H;
+        MscGenKeyMap[MscGenKey_E] = SDL_SCANCODE_E;
 
 #ifdef HAS_FONTCONFIG
         std::get<GUIInit>(ret).languages.cshParams->fontnames = GetFontNames();
 #endif
 
-        InitGUI(std::get<GUIInit>(ret));
+#define STRING2(s) #s
+#define STRING(s) STRING2(s)
+
+        const std::vector example_dirs = {
+            //1. Search for a binary compiled in-place
+            //If the executable path is with "XXX/src/unix-msc-gen/msc-gen" then search for "XXX/examples/"
+            [argv]()->std::string {
+                if (!argv[0] || !argv[0][0]) return {};
+                try {
+                    std::filesystem::path p(argv[0]);
+                    p = canonical(p);
+                    if (p.filename()!="msc-gen") return {};
+                    p = p.parent_path();
+                    if (p.filename()!="unix-msc-gen") return {};
+                    p = p.parent_path();
+                    if (p.filename()!="src") return {};
+                    return (p.parent_path() / "examples").native()+"/";
+                } catch (std::filesystem::filesystem_error&) { return {}; }
+            }(),
+#ifdef MSC_GEN_EXAMPLES_DIR
+            //2. automake will compile us with MSC_GEN_EXAMPLES_DIR set to the dir where
+            //the examples will be installed when issuing make install
+            //We dont check but this shall be an absolute path...
+            std::string(STRING(MSC_GEN_EXAMPLES_DIR) "/"),
+#else
+            //If this is being compiled without automake, emit a warning
+            #warning "MSC_GEN_EXAMPLES_DIR is not defined, msc-gen will not find the examples. Do you compile from Makefile.am?"
+#endif
+        };
+
+        //Fill in what documentation files do we have. Search through some dirs
+        std::vector<std::string> doc_dirs;
+        if (example_dirs[0].size()) //for in-place compiled stuff we replace "examples" to "doc"
+            doc_dirs.push_back((std::filesystem::path(example_dirs[0]).parent_path().parent_path() / "doc").native()+"/");
+#ifdef MSC_GEN_DOC_DIR
+        //2. automake will compile us with MSC_GEN_DOC_DIR set to the dir where
+        //the doc will be installed when issuing make install
+        //We dont check but this shall be an absolute path...
+        doc_dirs.push_back(STRING(MSC_GEN_DOC_DIR) "/");
+#else
+        //If this is being compiled without automake, emit a warning
+        #warning "MSC_GEN_DOC_DIR is not defined, msc-gen will not find the documentation. Do you compile from Makefile.am?"
+#endif
+        for (auto [name, file] : std::multimap<const char*, const char*>{{"PDF", "msc-gen.pdf"}, {"HTML", "html/index.html"}, {"HTML", "msc-gen.html/index.html"}})
+            for (const std::string& dir : doc_dirs)
+                if (std::error_code ec; std::filesystem::exists(dir+file, ec)) {
+                    Documentation[name] = dir+file;
+                    break;
+                }
+        std::vector<std::string> doc_names;
+        for (const auto& [name, _] : Documentation)
+            doc_names.push_back(name);
+
+        GUIWindowGeometry w = InitGUI(std::get<GUIInit>(ret), 
+                                      settings_dir, std::filesystem::current_path().native(), 
+                                      example_dirs, std::move(doc_names));
+        SDL_DisplayMode DM{0,0,0,0,0};
+        SDL_GetCurrentDisplayMode(0, &DM);
+        if (DM.w>0 && DM.h>0)
+            w.ensure_fit_in(DM.w, DM.h);
+        if (w.has_size()) SDL_SetWindowSize(window, w.cx, w.cy);
+        if (w.has_pos()) SDL_SetWindowPosition(window, w.x, w.y);
+        SDL_ShowWindow(window);
         SDL_EventState(SDL_DROPFILE, SDL_ENABLE);
 
         // Main loop
@@ -372,7 +520,7 @@ int main(int argc, char* argv[])
                 }
             }
             if (!SDL_GetDisplayUsableBounds(SDL_GetWindowDisplayIndex(window), &bounds))
-                if (PreFrameGUI(dpi_mul(bounds))) {  
+                if (PreFrameGUI(dpi_mul(bounds))) {
                     ImGui_ImplOpenGL3_DestroyFontsTexture();
                     ImGui_ImplOpenGL3_CreateFontsTexture();
                 }
@@ -404,17 +552,20 @@ int main(int argc, char* argv[])
                 last_window_title = res.window_title;
                 SDL_SetWindowTitle(window, (WINDOW_TITLE_PREFIX " - " + last_window_title).c_str());
             }
-            if (we_are_fullscreen!=res.full_screen) {
-                if (SDL_SetWindowFullscreen(window, res.full_screen ? SDL_WINDOW_FULLSCREEN : 0))
+            if (we_are_fullscreen!=(res.action == GUIReturn::FullScreen)) {
+                if (SDL_SetWindowFullscreen(window, res.action == GUIReturn::FullScreen ? SDL_WINDOW_FULLSCREEN : 0))
                     std::cerr << "Could not change fullscreen status: " << SDL_GetError() << std::endl;
                 else
-                    we_are_fullscreen = res.full_screen;
+                    we_are_fullscreen = res.action == GUIReturn::FullScreen;
             }
 
-            if (res.exit) break;
+            if (res.action == GUIReturn::Exit) break;
+            if (res.action == GUIReturn::Focus) ::SDL_RaiseWindow(window);
         }
 
-        ShutdownGUI();
+        SDL_GetWindowPosition(window, &w.x, &w.y);
+        SDL_GetWindowSize(window, &w.cx, &w.cy);
+        ShutdownGUI(w);
 
         // Cleanup
         ImGui_ImplOpenGL3_Shutdown();
@@ -447,7 +598,7 @@ int main(int argc, char* argv[])
 
 #ifdef BUILD_GUI
 
-std::optional<ImTextureID> TextureFromSurface(cairo_surface_t *rgba_surface) {
+TextureID TextureFromSurface(cairo_surface_t *rgba_surface) {
     GLenum err;
     while ((err = glGetError()) != GL_NO_ERROR) { _ASSERT(0); } //purge error queue
     // Load from disk into a raw RGBA buffer
@@ -475,7 +626,7 @@ std::optional<ImTextureID> TextureFromSurface(cairo_surface_t *rgba_surface) {
     bool had_error = false;
     while ((err = glGetError()) != GL_NO_ERROR) had_error = true;
 
-    if (!had_error)
+    if (!had_error && image_texture)
         return (ImTextureID)(size_t)image_texture;
     return {};
 }
@@ -514,4 +665,49 @@ std::set<std::string> GetFontNames() {
 }
 #endif //HAS_FONTCONFIG
 
+#ifdef __APPLE__
+
+#include "clipboard.h"
+//On the Mac, it is in clipboard.mm
+std::pair<ClipboardFormat, std::string> GetClipboard() {
+    MacClipboardFormat format;
+    unsigned len;
+    char* content = MacGetClipboard(&format, &len);
+    std::pair<ClipboardFormat, std::string> ret;
+    if (content && len) {
+        switch (format) {
+        case MacClipboardFormat::Art_GVML: ret.first = ClipboardFormat::Art_GVML; break;
+        case MacClipboardFormat::Escaped_Text: ret.first = ClipboardFormat::Escaped_Text; break;
+        default: return ret;
+        }
+        ret.second.assign(content, len);
+        free(content);
+    }
+    return ret;
+}
+
+bool CopyToClipboard(ClipboardFormat format, const std::string& content, std::string_view type) {
+    std::string mac_type(type);
+    const MacClipboardFormat mac_format = [format]() {
+        switch (format) {
+        case ClipboardFormat::Art_GVML: return MacClipboardFormat::Art_GVML;
+        case ClipboardFormat::Escaped_Text: return MacClipboardFormat::Escaped_Text;
+        default: _ASSERT(0); return (MacClipboardFormat)format;
+        }
+    }();
+    return 0!=MacCopyToClipboard(mac_format, content.data(), content.length(), mac_type.c_str());
+}
+
+#else
+//On Linux we have no clipboard support for now
+std::pair<ClipboardFormat, std::string> GetClipboard() {
+    return {};
+}
+
+bool CopyToClipboard(ClipboardFormat format, const std::string& content, std::string_view type) {
+    return true; //dont try again
+}
+
+#endif //ifndef(__APPLE__)
+
 #endif // BUILD_GUI
